/*
 *  $Revision: 1.33 $
 *
 *  This file implements the system command.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


/*\vscmuserDocpage vcWin.System -- Spawn a subprocess, perhaps in a window.

SYNOPSIS
     vcSysVT vcWin.System(win, cmd, env, wait, output, error_proc,
                          completion_proc, cancel_proc, host)
         vcWindowVT     win;
         vlStringVT     cmd;
         consVT         env;
         boolVT         wait;
         vcSysOutputVT  output;
         closureVT      error_proc;
         closureVT      completion_proc;
         closureVT      cancel_proc;
         vlStringVT     host;

DESCRIPTION
     This function can be called to run a command in a subprocess.

     The argument "win" is used for displaying status messages about the
     subprocess.

     The "cmd" argument is passed to the operating system.
     For Unix, we use /bin/csh to run the command.
     
     The "env" argument is used to pass an environment to the subprocess.
     If "nil" is specified, the current process' environment is used.
     Otherwise, a list of strings of the form NAME=VALUE can be supplied.
     For example:  list ("WPROJECT=prjname", "WDIR=/asdf/.../")

     If "wait" is TRUE, and "win" is non-nil, the window will be
     placed in busy mode, and user input into the window will be blocked,
     until the subprocess terminates or is killed.  This will not affect
     the responsiveness of other windows owned by the process.

     The output from the command is controlled by the "output" argument,
     which can be one of:
          vcSysOutput.Viewtext
                A Viewtext window, which is adorned with 2 buttons to
                dismiss the window and kill the process.  The advantages
                of this choice is that the user can save the output
                to a log file, scroll back and forth, and can kill the
                subprocess with a mouse button click.  The disadvantages
                is that at this point, Viewtext is not very efficient
                at logging large amounts of output, and cannot deal
                with programs that do direct cursor addressing.
                The "
          vcSysOutput.Terminal
                A terminal window (e.g. xterm), which can be set up
                to disappear on termination of the subprocess, or to
                wait for the user to confirm it with a carriage return.
                The advantage of this choice is that it is a real
                terminal emulator, and is fairly fast.  The disadvantage
                is that the scroll back and logging capabilities may
                be more limited, and the user must know to hit ^C to
                interrupt the subprocess.
          vcSysOutput.TerminalNoPause
                Same as vcSysOutput.Terminal, but the window disappears
                as soon as the subprocess terminates.
          vcSysOutput.NoOutput
                Output is suppressed.  The advantage is
                that if the program output is irrelevant, no boring windows
                pop-up on the screen.  The disadvantage is that there
                is no control of the subprocess, without the user opening
                up another window, doing a `ps', and killing it manually.
                This is not recommended for "wait" mode subprocesses,
                because it can hang up the original window without indicating
                why, or how to free it up.
          A function that takes one string argument
                This function is called whenever output is available
                from the subprocess.  You can use the output in whatever
                manner you choose.

     completion_proc, if not nil, is called with two arguments when the
     subprocess completes.  The two arguments are the "mode" of completion,
     which can be one of:  vcSysMode.Exited, vcSysMode.Signaled, or
     vcSysMode.Stopped.  The code is either the exit code of the subprogram,
     which is the case if the mode is vcSysMode.Exited, or the signal
     number with which it was stopped.

     cancel_proc, if not nil, is called with the same two arguments when the
     subprocess is killed by the user, using the button provided by
     Core, when output==vcSysOutput.Viewtext.  If cancel_proc is called,
     completion_proc will not be.

     The "host" argument is used as a machine name for remote process
     invocation.  For the moment, "rsh" is used on Unix to implement this
     functionality, and some user environments may not support rsh.  For
     those that do, however, we give easy access to it here.  When we
     support remote process invocation directly, this same interface will
     be supported, and work more reliably and more universally.

NOTE
     This functionality is also available from the user interface, through
     the Red Square menu, but there you can't supply a completion_proc or a
     cancel_proc.  You have to make a direct ViewScript call to this routine
     for that to work.  Similarly, the `system' command invoked from the
     command line editor has this function as its action routine.

MACRO RECORDING
     By default, the macro system will not wait for the completion of a
     spawned subsystem.  This would make it impossible to replay cross-process
     macros.  This function is an exception to this rule and has been modified
     specifically to work closely with the macro system.  If this function is
     used in a macro recordable command AND THE WAIT ARGUMENT IS TRUE, the
     macro system will wait for the completion of the spawned subprocess
     before resuming macro execution.  As mentioned above, the `system'
     command available from the Red Square menu uses this function as the
     action routine, thus all the user interactions involving the system
     commands will be properly replayed in macros.

     While replaying commands that use the vcSys.(man) functions, the macro
     system WILL NOT wait for the subprocesses to finish.  Only this function
     will cause the macro system to wait.

SEE ALSO
     vcSys.Program, vcSys.Command
*/          
function System(win, cmd, env, wait, output, error_proc,
                completion_proc, cancel_proc, host)
{
    /* initialize some variables */
    sys := nil;      /* Spawned system object, needed to kill it */
    output_handler := nil;

    cmd = vcCmdtable.Substitute (cmd);  /* substitute for $varibles */

    if ((output == vcSysOutput.TerminalNoPause) ||  /* NEED TO HANDLE DOS!!! */
        (output == vcSysOutput.TerminalPause))
    {
        /*
         * IMPORTANT!  ACHTUNG!
         *
         * Here you can modify the ViewSys window to use 'shelltool'
         * instead of 'xterm' if you like.  There are a few very
         * important things to note, however.
         *
         * ESCAPED DOUBLE QUOTES must be used to surround the
         * command sent to '/bin/csh'.  Follow the example.
         *
         * The '-e' switch to 'xterm' causes the command line
         * following it to be run in the 'xterm' window.  If you use
         * 'shelltool', DO NOT USE '-I' just put the command line
         * there.  Follow the example.
         *
         * You may substitute '/bin/sh -c' for '/bin/csh -fc'.
         *
         * The '-n' and '-T' switches to 'xterm' and the '-title'
         * switch to 'shelltool' serve only to change the icon name
         * and window title.
         */
        if (string_eq (cmd, ""))
            cmd = "xterm -n ViewSys";
            /* OR THIS // cmd = "shelltool -title ViewSys"; */
        else {
            if (output == vcSysOutput.TerminalPause)
                cmd = cmd + "; viewsys 1";
            cmd = sprintf ("xterm -n ViewSys -T ViewSys -e /bin/csh -fc \"%s\"", cmd);
            /* OR THIS // cmd = sprintf ("shelltool -title ViewSys /bin/csh -fc \"%s\"", cmd); */
        }
    }
    else if (output == vcSysOutput.Viewtext) {
        fname := "viewsys.log";
        table := vcTable.Create (fname, 2, 1);
        args  := list (kill_on_close <- FALSE);
        if (host != nil)
            args = cons (host <- host, args);
        if (env != nil)
            args = cons (env <- env, args);
        sh := vcRegion.Create1 (fname, vcShell.fac, cmd, args);
        sys = vcShell.Sys (sh);
        b1 := vcWv.LabelButtonCreate ("Proceed",
                                      "Press button to Dismiss window",
                                      nil, vcWin.wclose, nil);
        vcRegion.Enable (b1, FALSE, FALSE);

        killed := FALSE;

        function kill_proc() {
            vcSys.Kill (sys, killed ? vcSysKill.Severe : vcSysKill.Normal);
            killed = TRUE;
        }

        b2 := vcWv.LabelButtonCreate ("Kill",
                                      "Kill subprocess",
                                      nil, kill_proc, list (macro_on <- TRUE));
        vcRegion.Align (b1, vgAlign.MiddleCenter);
        vcRegion.Align (b2, vgAlign.MiddleCenter);
        button_table := vcTable.Create ("buttons", 1, 2);
        vcTable.Add (button_table, b1);
        vcTable.Add (button_table, b2);
        vcTable.Add(table, button_table);
        vcTable.Add(table, sh);
        vcRegion.SetFocus(sh, sh);
        w := vcWin.Create (table, vcWin.NoCmdline + vcWin.NoStatus);
        vcWin.SetConfirmMessage (w, "
                 The window is attached to a running process. 

 You may close the window now, in which case the subprocess will 
 continue to run until it tries to read user input, at which point 
 it will probably terminate.  Or you can kill the subprocess first, 
 and then close the window.

");
        vcWin.Modified (w, TRUE);
        vcWin.DestroyNotify (w, func(win) {b2 = nil; w = nil;});

        old_term_proc := completion_proc;
        completion_proc = func(mode, code) {
            if (w != nil) {
                vcWin.Modified (w, FALSE);
                unwait_proc ();
                vcRegion.Enable (b1, TRUE, TRUE);
            }

            if (b2 != nil)
                vcRegion.Enable (b2, FALSE, TRUE);
            if (killed && (cancel_proc != nil))
                cancel_proc (mode, code);
            else if (!killed && (old_term_proc != nil))
                old_term_proc (mode, code);
        };
        vcSys.CallOnExit (sys, completion_proc);
    }
    else if ((output != nil) && (output != vcSysOutput.NoOutput))
        output_handler = output;
        
    function unwait_proc() {
         if (wait && (win != nil) && vcWin.IsAlive (win)) {
             vcWin.Busy1 (win, FALSE);
             vcWin.UngrabData (win);
         }
     }

    if (wait && (win != nil)) {
        vcWin.Busy1 (win, TRUE);
        vcWin.GrabData (win, vcBind.Create (), nil);
    }

    if (win != nil) {
        vcWin.ClearStatus (win);
        vcWin.AddStatus (win, sprintf ("Spawning command `%s'", cmd));
    }

    if (output != vcSysOutput.Viewtext) {
        /*
         * If there's nothing looking out the output, the error code, or
         * the exit handler, then we better hang onto stderr and show errors
         * so mysterious failures can be debugged.
         */
        if ((output_handler == nil) && (error_proc == nil) &&
            (completion_proc == nil))
        {
            error_string := nil;
            error_proc = func(str) {
                if (error_string == nil)
                    error_string = ":\n";
                error_string += str;
            };
            completion_proc = func(mode, code) {
                if ((mode != vcSysMode.Exited) || (code != 0))
                    vcWv.Delay (wv, func() {
                        vlEx.Throw2 (vlEx.FAILURE, "vcSystem",
                                     vlErrSev.Error,
                                     sprintf ("`%s' %s, code %d%s", cmd,
                                              convert (string, mode), code,
                                              error_string));
                    });

                else if ((win != nil) && vcWin.IsAlive (win)) {
                    vcWin.ClearStatus (win);
                    vcWin.AddStatus (win, sprintf ("Command `%s' completed.",
                                                   cmd));
                } /* else if */
            };
        } /* if */

        sys = vcSys.Command (cmd, env, output_handler, error_proc,
                             func(mode, code) {
                                 unwait_proc ();
                                 if (completion_proc != nil)
                                     completion_proc (mode, code);
                             },
                             host);

    }

    /*
     * The vcWin.System function is the only subprocess spawning function
     * which will wait for the subprocess to finish before resuming macro
     * playback.
     */
    if (wait && vcMac.IsReplayingOn ())
        vcMac.SubprocessHold (sys, 0);

    sys;
}


/*
 * This is the function that is run when the user types "sys ...." at the
 * command line.  It is also the function that is used for the command that
 * gets macro-recorded.  This function, in turn, calls the System function to
 * do the actual job.
 */
function sys(window, event, restofline) {
    wait    := FALSE;
    output  := vcSysOutput.Viewtext;
    host    := nil;

    /* convert the argument restofline to a string for string manipulation */
    cmd := convert(string,restofline);

    declare option;  /* option string */

    /* is first character a dash, signifying an option */
    while ((string_length (cmd) > 1) && (string_ref (cmd, 0) == '-')) {
        cmd = string_token (cmd, &option, " ");
        if (string_eqi (option, "-nopause"))
            output = vcSysOutput.TerminalNoPause;
        else if (string_eqi (option, "-pause"))
            output = vcSysOutput.TerminalPause;
        else if (string_eqi (option, "-nowindow"))
            output = nil;
        else if (string_eqi (option, "-wait"))
            wait = TRUE;
        else if (string_eqi (option, "-nowait"))
            wait = FALSE;
        else if (string_eqi (option, "-text"))
            output = vcSysOutput.Viewtext;
        else if (string_eqi (option, "-host"))
            cmd = string_token (cmd, &host, " ");
        else
            vlEx.Throw2 (vlEx.FAILURE, "vcSystem", vlErrSev.Error,
                         sprintf ("Unknown option `%s'", option));

        /* Advance past any leading spaces */
        len := string_length (cmd);
        if (len > 0) {
            for (i := 0; (i < len) && (cmd [i] == ' '); ++i)
                ;
            cmd = substring (cmd, i, len);
        }
    }

    System (window, cmd, nil /* env */, wait, output, nil, nil, nil, host);
}

sys_cmd := vcWv.Cmd (vcWin.fac, sys, "sys", "System",
                     vcWv.Param ("System Command Line", vlStringRestOfLineVT,
                                 vcWv.Sticky, ""));

/*
 * This is the function that gets run when the user invokes the System
 * window dialog box from the Red Square menu.  It generates a command
 * line and then runs the "sys" function, which is macro-recordable.
 */
function system(window, event, cmd, wait, output, host) {
    /* Only record changes from the default... */
    cmdline := "";

    if (wait == vlOnOff.On)
        cmdline = "-wait ";
    if (output == vcSysOutput.TerminalNoPause)
        cmdline = cmdline + "-nopause ";
    else if (output == vcSysOutput.TerminalPause)
        cmdline = cmdline + "-pause ";
    else if (output == vcSysOutput.NoOutput)
        cmdline = cmdline + "-nowindow ";
    if (!string_eqi (host, ""))
        cmdline = cmdline + "-host " + host + " ";

    cmdline = cmdline + convert (string, cmd);
    vcCmd.Run (sys_cmd, window, event, list (cmdline), TRUE);
}

vcWv.MakeEnum ("vcSysOutputVT", list ("Viewtext", "TerminalPause",
                                      "TerminalNoPause", "NoOutput"));

system_cmd := vcWv.Cmd (vcWin.fac, system, "system", "System",
                        vcWv.Param ("System Command Line",
                                    vlStringRestOfLineVT, vcWv.Sticky, ""),
                        vcWv.Param ("Wait for completion", vlOnOffVT,
                                    vcWv.Sticky, vlOnOff.Off, vcWv.Secondary),
                        vcWv.Param ("Show Output", vcSysOutputVT,
                                    vcWv.Sticky, vcSysOutput.Viewtext,
                                    vcWv.Secondary),
                        vcWv.Param ("Host Name", vlStringVT,
                                    vcWv.Sticky, "", vcWv.Secondary));
vcCmd.EnableMacro (system_cmd, vlOnOff.Off);
