/*\vscmuserSection vcFlagEdit. -- Edit flags for a command.
 *
 * DESCRIPTION
 *      This facility is used mainly as a command parameter editor,
 *      via vcWv.Param (see EXAMPLES, below).  It allows the selection
 *      of independent boolean flags.  It is an alternative to using
 *      a mask parameter (see define_mask) with vcPickenum.  It has two
 *      advantages:
 *              1.      It allows help on each boolean flag, which
 *                      appears in the dialog box status line whenever
 *                      the user clicks on an entry.
 *              2.      It doesn't require button strings to conform
 *                      to Viewscript variable syntax.  In fact, icons
 *                      or other arbitrary regions can be used as the
 *                      visual inside each button.
 *
 *      The facilty data is a string of tokens, separated by newlines.
 *      This is what is expected as the default value, and is what is
 *      emitted by _Get.
 *
 *      The allowable set of tokens is defined by the flags preference,
 *      which is a list of the form:
 *              list (list ("Entry 1", "Help 1", "true1", nil),
 *                    list ("Entry 2", "Help 2", nil, "false2"),
 *                    list ("Entry 3", "Help 3", "true3", nil)...)
 *
 *      The entry string can alternatively be a function which returns a
 *      region, allowing the use of vcWv.Icon, etc.  Help is, of course,
 *      the string printed in the status line of the dialog box when the
 *      user clicks on an entry.
 *
 *      The last two entries refer to how the input and output strings
 *      get tokenized in the data.  Most flags are specified positively
 *      (e.g. -writestate), but some are specified negatively
 *      (e.g. -nooutput).  If a flag is specified postively, then the
 *      3rd element of its descriptor list should be the token that
 *      you'd put on the command line, and the 4th element should be nil.
 *      If it's specified negatively, use nil for the 3rd argument and
 *      the appropriate flag for the 4th argument.
 *
 * SEE ALSO
 *      vcPickenum.
 *
 * FACILITY DATA
 *      vlStringVT -- newline-separated string of tokens
 *
 * PREFERENCES
 *      columns     -- vlIntVT, number of columns in each row of the picklist
 *      flags       -- vscmVT, list of descriptor-lists (see above).
 *
 * EXAMPLES
 *      ReadErrorFileCmd :=
 *          vcWv.Cmd (fac, SomeFunctionName,
 *                    "TrackErrors", "Track (visit) errors from file",
 *                    vcWv.Param("Error file", vlStringVT, vcWv.Sticky, "",
 *                               vcFileSelect.fac,
 *                               filter <- "*.err",
 *                               read_now <- TRUE));
 *      
 */
/*
 * This kludge allows vcFlagEdit.vs to be used from a Core 5.1
 * executable, which is required for IR 1.0.  We don't need to
 * document its existence, or why it's required.  At some point
 * we should get rid of it.
 */
function eval_if_str(str)
    (istype (str, vlStringVT)
     ? vcWin.EvalPrint (str + ";", func(args...);)
     : str);

/*
 * Format for each picklist entry:
 *      list (entry_string, help_string, true_token, false_token);
 */
function _Create(token_list, region) {
    format := eval_if_str (vcRegion.PrefGet (region, "flags"));
    picklist := vcRegion.Create (vcRegion.Name (region), vcPicklist.fac, nil,
                                 exclusive <- FALSE,
                                 columns <- vcRegion.PrefGet (region,
                                                              "columns"));
    tokens := vlSym.StrCreate (101);
    foreach (token; string_loop (token_list, "\n"))
        vlSym.Store (tokens, token, TRUE);

    i := 0;
    foreach (entry; vscript.list_iter (format)) {
        vcPicklist.Add (picklist, car (entry), cdr (entry));
        true_token := caddr (entry);
        false_token := cadddr (entry);
        if (((true_token != nil) &&
             (vlSym.Fetch (tokens, true_token) != nil)) ||
            ((false_token != nil) &&
             (vlSym.Fetch (tokens, false_token) == nil)))
            vcPicklist.SetPick (picklist, i);
        ++i;
    } /* foreach */

    vcPicklist.HighlightNotify (picklist, func(entry, on_off, index) {
        w := vcRegion.Window (picklist);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, car (entry));
    });
    vcRegion.AddChild (region, picklist);
    picklist;
} /* function _Create */

function _Get(picklist, region, type) {
    format := eval_if_str (vcRegion.PrefGet (region, "flags"));
    i := 0;
    tokens := "";
    foreach (entry; vscript.list_iter (format)) {
        token := ((vcPicklist.IsSet (picklist, i)) ? caddr : cadddr) (entry);
        if (token != nil)
            tokens += token + "\n";
        ++i;
    } /* foreach */
    tokens;
} /* function _Get */

mkpref := isbound (`Replace, vlPref) ? vlPref.Replace : vlPref.Add;
list_of_flags_type := isbound (`CompatibleVersion, vcWv) ? vscmVT : vlStringVT;

mkpref (prefs, "flags", list_of_flags_type, "~~list of flags in the form:
(entry_string, help_string, true_token, false_token)", nil);
mkpref (prefs, "columns", vlPositiveIntVT, "number of columns", 4);
