/*
 *  $Revision: 1.24.1.1 $   $Date: 1994/11/07 21:11:51 $
 *
 *  Facility to display information about active processes and so on, to allow
 *  the user to decide whether to quit. 
 *
 *  Copyright (c) 1992,1994 by Viewlogic Systems, Inc.
 */


/*\vscmuserSection vcQuit. -- last ditch effort
 *
 * DESCRIPTION
 *      The vcQuit callbacks allow the user to modify and/or to be notified
 *      about the impending demise of the Powerview process.  This is not a
 *      Core facility, in that you cannot instantiate it via a call to the
 *      vcRegion.Create function.
 */
 

/*
 * Global variables holding the quit handlers.
 */
quit_functions  := nil;
call_on_quit    := nil;


/*
 * Maintain a list of active subprocesses by hooking into the create
 * notification and exit notification of vcSys.  This list will come in handy
 * when the Quit dialog box is to be composed: a list of the active process
 * will be displayed for the user.
 */
active_subprocesses := nil;
vcSys.CallOnCreate (func(sys) {
    active_subprocesses = cons (sys, active_subprocesses);
    vcSys.CallOnExit (sys, func(mode, code)
        active_subprocesses =
                      vscript.delete_list_element (active_subprocesses, sys););
});


/*\vscmuserDocpage vcQuit.IgnoreSubprocess -- don't report info on a subprocess when quitting
 *
 * SYNOPSIS
 *      vcQuit.IgnoreSubprocess(sys)
 *          vcSysVT sys;
 *
 * DESCRIPTION
 *      Ordinarily, all active subprocesses are noted when a Powerview process
 *      terminates.  This function prevents `sys' from being displayed in the
 *      Quit dialog box.
 */
function IgnoreSubprocess(sys)
    active_subprocesses =
        vscript.delete_list_element (active_subprocesses, sys);


/*
 * Build the modified windows string with the titles of all the modifed AND
 * mapped windows.
 */
function ModifiedWindowsInfo()
{
    modified := "";

    foreach (w; vcWin.Loop ())
        if (vcWin.IsModified (w) && vcWin.IsMapped (w))
            modified = modified + "     " + vcWin.Title (w) + "\n";

    if (!string_eq (modified, ""))
        modified = "The following windows have been modified:\n" +
            modified + "\n";

    modified;
}


/*
 * Build the modified subprocess string with the names of all the currently
 * active subprocesses.
 */
function ActiveProcessInfo()
{
    modified := "";

    if (active_subprocesses != nil)
    {
        modified = modified + "The following subprocesses are still alive:\n";

        for (a := active_subprocesses; a != nil; a = cdr (a))
            modified = modified + "     " + vcSys.Name (car (a)) + "\n";

        modified = modified + "\n";
    }

    modified;
}


/*\vscmuserDocpage vcQuit.CallOnQuit -- register a quit handler
 *
 * SYNOPSIS
 *      vcQuit.CallOnQuit(proc)
 *          closureVT   proc;
 *
 * DESCRIPTION
 *      Registers a function which will be called when the "quit" command is
 *      executed.
 *
 * NOTE
 *      In all the Powerview releases 5.2 and earlier the handler was always
 *      called with no arguments.  In post-5.2 releases, this function will
 *      check if the handler takes 0 or 1 argument.  If the handler takes an
 *      argument, it will be called with the event data structure.
 *
 * CAUTION
 *      Similarly to the vcQuit.AddFunction function, the quit handler will be
 *      called only if the "quit" command has been executed.  If the process
 *      has been killed using operating system commands, there's no quarantee
 *      that the command will be executed.
 *
 *      If you record a macro that registers the "quit" command, all the quit
 *      handlers will be called during macro playback.
 *
 * SEE ALSO
 *      vcQuit.AddFunction
 */
function CallOnQuit(proc)
    if (isprocedure (proc))
    {
        if (((nargs := vscript.Nargs (proc)) == 0) || (nargs == 1))
            call_on_quit = cons (proc, call_on_quit);
        else
            vlEx.Throw2 (vlEx.FAILURE, "vcQuit", vlErrSev.Error,
                         sprintf ("Quit function should take 0 or 1 argument.\n\t%s takes %d arguments.",
                                  proc, nargs));
    }
    else
        vlEx.Throw2 (vlEx.FAILURE, "vcQuit", vlErrSev.Error,
          sprintf ("Illegal argument type--expected a procedure, received %s",
                   proc));


/*\vscmuserDocpage vcQuit.AddFunction -- add routine to be called on Quit dialog
 *
 * SYNOPSIS
 *      vcQuit.AddFunction(proc)
 *          closureVT   proc;
 *
 * DESCRIPTION
 *      This will add a function to be called during the creation of the "OK
 *      to Quit?" dialog box.  Its purpose is to return either a text message
 *      describing a modified window, or a region containing controls to alter
 *      the behavior of the quit dialog.  It is called without arguments:
 *
 *          f ();
 *
 * CAUTIONS
 *      This capability is only available in the presence of the quit dialog
 *      box.  It is not available when replaying a macro which has recorded
 *      the quit command, and therefore should not be used to implement
 *      capability which is always required prior to quitting; for that, use
 *      vcQuit.CallOnQuit.
 *
 * SEE ALSO
 *      vcQuit.CallOnQuit
 */
function AddFunction(proc)
    if (isprocedure (proc))
    {
        if ((nargs := vscript.Nargs (proc)) == 0)
            quit_functions = cons (proc, quit_functions);
        else
            vlEx.Throw2 (vlEx.FAILURE, "vcQuit", vlErrSev.Error,
                         sprintf ("Quit dialog box function should take 0 arguments.\n\t%s takes %d arguments.",
                                  proc, nargs));
    }
    else
        vlEx.Throw2 (vlEx.FAILURE, "vcQuit", vlErrSev.Error,
          sprintf ("Illegal argument type--expeced a procedure, received %s",
                   proc));


/*
 * Action routine for the Quit command, which closes all the windows.  Note
 * that the "quit" command is defined in the vcWin.vs file.
 */
function Quit(w, e, arg)
{
    foreach (proc; vscript.list_iter (call_on_quit))
        catch (vlEx.FAILURE)
        {
            if ((nargs := vscript.Nargs (proc)) == 0)
                proc ();
            else if (nargs == 1)
                proc (e);
            else
                vlEx.Throw2 (vlEx.FAILURE, "vcQuit", vlErrSev.Error,
     sprintf ("Quit handlers can either take 0 or 1 argument.\n\t%s takes %d.",
              proc, nargs));
        }
        exception
        {
            err := vlErr.Create2 ("vcQuit", vlErrSev.Error,
                                  "Caught exception during quit processing.");
            vlErr.Report (nil, vlErr.Append (err, vlErr.CurrentError ()));
        }

    foreach (w; vcWin.Loop ())
        catch (vlEx.FAILURE)
            vcWin.Destroy (w);
        exception
        {
            err := vlErr.Create2 ("vcQuit", vlErrSev.Error,
                                  "Caught exception during quit processing.");
            vlErr.Report (nil. vlErr,Append (err, vlErr.CurrentError ()));
        }
}


/*
 * There's a sense of recursiveness here, because this is a _Create method for
 * the vcQuit facility, which is used as the editor facility for the "quit"
 * command defined in the vcWin.vs file.  The action routine for the "quit"
 * command is defined, guess what?, right above!
 */
function _Create(dflt, region)
{
    strings         := ModifiedWindowsInfo () + ActiveProcessInfo ();
    extra_strings   := "";
    regions         := nil;

    foreach (proc; vscript.list_iter (quit_functions))
    {
        val := proc ();

        if (istype (val, vcRegionVT))
            regions = cons (val, regions);
        else if (val != nil)
        {
            if (!istype (val, vlStringVT))
                val = convert (vlStringVT, val);
            if (!string_eq (val, ""))
                extra_strings = val + "\n\n" + extra_strings;
        }
    }

    n_rows := 2 + length (regions);

    strings = strings + extra_strings;
    if (string_eq (strings, ""))
    {
        strings = nil;
        --n_rows;
    }

    t := vcTable.Create ("QuitDialog", n_rows, 1);
    if (strings != nil)
        vcTable.Add (t, vcRegion.Create ("", vcStaticText.fac, strings));

    foreach (r; vscript.list_iter (regions))
        vcTable.Add (t, r);

    if (vcWv.UI (wv) == vcUi.OpenLook)
        text := "Quit " + vcWin.ProgramName + "?";
    else
        text := "OK to quit " + vcWin.ProgramName + "?";

    vcTable.Add (t, vcRegion.Create ("", vcStaticText.fac, text));

    tbl := vcTable.Create ("quit_tbl", 1, 4);
    vcTable.Add (tbl, vcRegion.Create ("quit_icon", vcIcon.fac, "motif_head"));
    vcTable.Add (tbl, vcRubber.Create ("", 0, 0));
    vcTable.Add (tbl, t);
    vcTable.Add (tbl, vcRubber.Create ("", 0, 0));

    margin := vcRegion.Create ("quit_margin", vcMargin.fac, tbl,
                               left_margin   <- 0,
                               right_margin  <- 10,
                               top_margin    <- 10,
                               bottom_margin <- 5);

    vcRegion.AddChild (region, margin);

    nil;
}


function _Get(arg1, arg2, arg3)
    "yes";
