/*  $Revision: 1.36.2.1 $   $Date    : 94/01/24 14:53:43 $
 *
 *  GUI definitions for the VGE facility.
 *
 *  Copyright (c) 1993,1994 by Viewlogic Systems, Inc.
 */


/*
 * READ THIS FIRST, BEFORE YOU MAKE ANY MODIFICATIONS TO THIS CODE.
 *
 * In the Powerview architecture each region has its own binding table, but
 * there's only one command table per facility.  Thus, two regions of the same
 * facility can have different bindings enabled/disabled, but the the commands
 * are either enabled or disabled in both at the same time.
 *
 * This poses a problem to the applications that build on top of VGE.  Some,
 * like ViewFSM, want to disable all the VGE bindings and commands so that a
 * user would not be able to, for example, move a transition circle, without
 * the application knowing about it and updating the semantics accordingly.
 * On the other hand, some applications don't care and want all the VGE
 * bindings and commands enabled.  A third kind of applications, built on top
 * of the previous two, would like to have both at the same time, in the same
 * window.  How are we going to allow this?
 *
 * The answer is that there are two sets of commands defined in this file.
 * The normal commands following the standard naming convention have their
 * name ending in `Cmd'.  The second set of commands shadow the original set,
 * but those commands are defined without a name string.  Their names end in
 * `Cmd2'.  In addition, the VGE bindings are bound to the `Cmd2' shadow
 * commands.  What does this buy us?
 *
 * Scenario 1: the vge.set_bindings is set to TRUE (default)
 *
 * In the VGE region, the user can use either the key bindings to create or
 * modify drawings or can invoke the vge commands by typing their names at the
 * command line editor prompt.
 *
 * Scenario 2: the vge.set_bindings is set to FALSE
 *
 * An application like ViewFSM is not worried about any user either
 * accidentally or on purpose invoking any VGE commands because the VGE key
 * bindings are disabled and all the named commands which can be invoked
 * through the command line editor are disabled as well.
 *
 * Scenario 3: the vge.set_bindings is set to FALSE and there are two regions
 * in the same application: one controlled by the ViewFSM and the other one by
 * VGE.
 *
 * In the region controlled by the ViewFSM, the situation is unchanged from
 * the Scenario 2.  In the region controlled by the VGE the situation is
 * similar to the Scenario 1, but not quite.  The user can use the VGE
 * bindings and menu commands, but cannot invoke any commands by name.
 *
 *
 * Implementation note
 * -------------------
 *
 * To implement two sets of commands, I took advantage of the multiple value
 * return and assignment capabilities of ViewScript.  Whenever you see syntax
 * like this:
 *
 *      <var1, var2> := foo (...);
 *
 * it means that two variables, var1 and var2 are defined and assigned new
 * values.  The function foo returns two values which will be assigned to the
 * variables sequentially.
 */


/*\Docpage vge_cmds -- build two commands in one shot
 *
 * DESCRIPTION
 *      Returns two values: a named command and an unnamed command.  See the
 *      comment at the beginning of this file for more information on the
 *      shadow commands.
 */
function vge_cmds(args...)
{
    action  := car (args);  args = cdr (args);
    name    := car (args);  args = cdr (args);
    title   := car (args);  args = cdr (args);
    params  := args;


    named_cmd   := vcCmd.Create (fac, action, name, title, params);
    unnamed_cmd := vcCmd.Create (fac, action, nil, nil, params);

    list (named_cmd, unnamed_cmd);
}


/*\Docpage open_vgedit -- "vgedit" action routine
 *
 * DESCRIPTION
 *      Defines the `vgedit' command which creates a new instance of the vge
 *      editor.  Note that this command is added to the vcWv command table to
 *      be available from any Powerview window.
 */
function open_vgedit(data, event, name)
{
    win := vcWin.Create (vcRegion.Create ("vgEdit", vge.fac, name));
    vcWin.SetTitle (win, "vgeEdit - " + name);
}


<EditCmd, EditCmd2> := vge_cmds (open_vgedit, "vgedit", "Edit Drawing",
                                 vcWv.Param ("", vlFileNameVT, vcWv.Sticky,
                                             vcWv.NoDefault, vcFileSelect.fac,
                                       "Name of a Viewlogic metafile to edit",
                                             filter   <- "*.mf",
                                             read_now <- TRUE));

vcCmdtable.Add (vcWv.commands, "vgedit", EditCmd);


/*\Docpage make_pline -- create a poly line drag command
 *
 * DESCRIPTION
 *      The pline command is implemented a little bit differently from the
 *      rest of the commands.  This function provides most of the interactive
 *      implementation for defining, undoing and cancelling of a poly line.
 *      In addition, it is not used only to create poly lines, but also
 *      splines.
 */
function make_pline(xy_create_proc, name)
{
    pline_table := vcBind.Create ();
    declare points, viewport, old_color;
    declare last_point, last_arrow;
    declare xy_create_arguments;

    /*
     * When done, erase the arrow, create the object and add it to the
     * drawing.  Ah, and cancel the grab, of course.  Note that the there are
     * any xy_create_arguments, they are appended to the list of points.
     */
    function fin(d, e)
    {
        erase_arrow ();
        Ungrab (d);

        if (!null (points))
        {
            points = reverse_destructive (points);

            if (!null (xy_create_arguments))
                xy_create_arguments = append (xy_create_arguments,
                                              list (points));
            else
                xy_create_arguments = list (points);

            object := apply (xy_create_proc, xy_create_arguments);
            draw   := GetDrawing (vcEvent.Region (e));
            DrawAddObject (draw, object);
            DrawRefreshDisplays (draw);
        }
    }
    

    /*
     * Remove the last added point.  If there are no more points left on the
     * list, this means that it's a true cancel: cancel the grab.
     */
    function undo(d, e)
    {
        old_last_point := last_point;
        points          = cdr (points);
        last_point      = car (points);

        erase_arrow ();

        if (null (points))
            Ungrab (d);
        else
        {
            function erase_line(fx, fy, tx, ty)
            {
                oldmode := vgU.Rasterop (viewport, vgRasterop.Xor);
                vgU.Line (viewport, fx, fy, tx, ty);
                vgU.Rasterop (viewport, oldmode);
            }


            fx := car (last_point);
            fy := cadr (last_point);

            last_arrow [0] = fx;
            last_arrow [1] = fy;
            last_arrow [2] = vcEvent.UserX (e);
            last_arrow [3] = vcEvent.UserY (e);

            erase_arrow ();
            erase_line (car (old_last_point), cadr (old_last_point),
                        car (last_point), cadr (last_point));
        }
    }
    

    /*
     * Draw a line from the last point to the current mouse location.  Save
     * the current location in the points list.
     */
    function con(d, e)
    {
        fx      := car (last_point);
        fy      := cadr (last_point);
        new_x   := vcEvent.UserX (e);
        new_y   := vcEvent.UserY (e);

        vgU.Line (viewport, fx, fy, new_x, new_y);
        
        last_point = list (new_x, new_y);
        points     = cons (last_point, points);
    }


    /*
     * Erase the current arrow by re-drawing it in the XOR mode.
     */
    function erase_arrow()
        if (last_arrow [0] != nil)
        {
            oldmode := vgU.Rasterop (viewport, vgRasterop.Xor);

            vgU.Arrow (viewport,
                       last_arrow [0], last_arrow [1],
                       last_arrow [2], last_arrow [3],
                       vgArrow.Point2);
            vgU.Rasterop (viewport, oldmode);
        }
    

    /*
     * While the mouse is moving: display the current coordinates in the
     * status window, erase the old arrow, draw a new one and save the
     * coordinates of the new arrow in the last_arrow vector.
     */
    function move(d, e)
    {
        declare fx, fy;
        declare tx, ty;
        
        w := vcEvent.Window (e);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, sprintf ("X=%d, Y=%d", vcEvent.UserX (e),
                                     vcEvent.UserY (e)));
        
        fx = car (last_point);
        fy = cadr (last_point);
        
        erase_arrow ();
        tx = vcEvent.UserX (e);
        ty = vcEvent.UserY (e);

        vgU.Arrow (viewport, fx, fy, tx, ty, vgArrow.Point2);

        last_arrow [0] = fx;
        last_arrow [1] = fy;
        last_arrow [2] = tx;
        last_arrow [3] = ty;
    }
    

    function start(d, e, args)
    {
        xy_create_arguments = args;

        UnselectAll (vcEvent.Region (e));

        set_default_binding (begin_cmd);
        Grab (d, pline_table);
        
        last_point  = list (vcEvent.UserX (e), vcEvent.UserY (e));
        points      = list (last_point);
        last_arrow  = make_vector (4);
        viewport    = GetViewport (vcEvent.Region (e));
        
        vcBind.Set (pline_table, '\move',       move_cmd);
        vcBind.Set (pline_table, '\leftdown',   cont_cmd);
        vcBind.Set (pline_table, '\middledown', finish_cmd);
        vcBind.Set (pline_table, '\rightdown',  undo_cmd);

        vgU.ArrowSetWidth (viewport,  10);
        vgU.ArrowSetLength (viewport, 20);
    }
    
    
    begin_cmd   := vcWv.Cmd (fac, start, "Start" + name, "Start" + name,
                             vcWv.Param ("Args", vscmVT, vcWv.Sticky, nil));
    cont_cmd    := vcWv.Cmd (fac, con,   "Cont" + name);
    undo_cmd    := vcWv.Cmd (fac, undo,  "Undo" + name);
    finish_cmd  := vcWv.Cmd (fac, fin,   "Finish" + name);
    move_cmd    := vcWv.Cmd (fac, move);

    /*
     * The only macro-recordable command is the one that actually places the
     * poly line.
     */
    vcCmd.EnableMacro (begin_cmd,   vlOnOff.Off);
    vcCmd.EnableMacro (cont_cmd,    vlOnOff.Off);
    vcCmd.EnableMacro (undo_cmd,    vlOnOff.Off);
    vcCmd.EnableMacro (finish_cmd,  vlOnOff.On);

    begin_cmd;
}


function pline_action(data, event)
    vcCmd.Run (PlineAuxCmd, data, event, list (nil), FALSE);

PlineAuxCmd := make_pline (PlineCreateXY, "Pline");
<PlineCmd, PlineCmd2> := vge_cmds (pline_action, "make_pline", "Poly Line");


function spline_action(data, event, order, type,
         granularity, arrow_point, arrow_width, arrow_length, arrow_fill)
    vcCmd.Run (SplineAuxCmd, data, event,
               list (list (order, type, granularity, arrow_point,
                           arrow_width, arrow_length, arrow_fill)), FALSE);

SplineAuxCmd := make_pline (SplineCreateXY, "Spline");
<SplineCmd, SplineCmd2> := vge_cmds (spline_action, "make_spline", "Spline",
                          vcWv.Param ("Order", vgSplineOrderVT,
                                      vcWv.Sticky, vgSplineOrder.Smooth),
                          vcWv.Param ("Type", vgSplineTypeVT,
                                      vcWv.Sticky, vgSplineType.Open),
                          vcWv.Param ("Granularity", vlPositiveIntVT,
                                      vcWv.Sticky, 10),
                          vcWv.Param ("Arrow Point", vgArrowVT,
                                      vcWv.Sticky, vgArrow.Point2),
                          vcWv.Param ("Arrow Width", vlPositiveIntVT,
                                      vcWv.Sticky, 3),
                          vcWv.Param ("Arrow Length", vlPositiveIntVT,
                                      vcWv.Sticky, 5),
                          vcWv.Param ("Arrow Fstyle", vgFillVT,
                                      vcWv.Sticky, vgFill.Solid));


/*\Docpage make_drag -- make a basic drag command
 *
 * DESCRIPTION
 *      Serves as a basis for all the interactive placement, move, copy, and
 *      stretch functions which are all built on the drag paradigm: begin,
 *      continue, finish or cancel.
 */
function make_drag(begin, con, finish, cancel, init_name, init_params)
{
    drag_table := vcBind.Create ();
    start_cmd  := vcCmd.Create (fac, begin,
                                (null (init_name) ?
                                 init_name : "_Begin" + init_name),
                                init_name, init_params);
    vcCmd.EnableMacro (start_cmd, vlOnOff.Off);

    function beg(args...)
    {
        vcCmd.Run (start_cmd, car (args), cadr (args),
                   cddr (args), TRUE);
        Grab (car (args), drag_table);
    }
    
    function fin(d, e)
    {
        finish (d, e);
        Ungrab (d);
    }

    function can(d, e)
    {
        cancel (d, e);
        Ungrab (d);
    }

    function startup(args...)
    {
        apply (beg, args);
        set_default_binding (begin_cmd);

        vcBind.Set (drag_table, '\move',        cont_cmd);
        vcBind.Set (drag_table, '\middledown',  finish_cmd);
        vcBind.Set (drag_table, '\rightdown',   cancel_cmd);
    }

    begin_cmd   := vcCmd.Create (fac, startup, init_name, init_name,
                                 init_params);
    cont_cmd    := vcWv.Cmd (fac, con);
    finish_cmd  := vcCmd.Create (fac, fin,
                                 (null (init_name) ?
                                  init_name : "_Finish" + init_name),
                                 nil, nil);
    cancel_cmd  := vcWv.Cmd (fac, can,
                             (null (init_name) ?
                              init_name : "_Cancel" + init_name));

    begin_cmd;
}


MoveCmd     := make_drag (BeginMove, ContinueMove, FinishMove, CancelMove,
                          "move", nil);
MoveCmd2    := make_drag (BeginMove, ContinueMove, FinishMove, CancelMove,
                          nil, nil);
CopyCmd     := make_drag (BeginCopy, ContinueCopy, FinishCopy, CancelCopy,
                          "copy", nil);
CopyCmd2    := make_drag (BeginCopy, ContinueCopy, FinishCopy, CancelCopy,
                          nil, nil);
StretchCmd  := make_drag (BeginStretch, ContinueStretch, FinishStretch,
                          CancelStretch, "stretch", nil);
StretchCmd2 := make_drag (BeginStretch, ContinueStretch, FinishStretch,
                          CancelStretch, nil, nil);


function make_stretch(init, name, init_params)
{
    named_cmd   := make_drag (init, ContinueStretch, FinishStretch,
                              CancelStretch, name, init_params);
    unnamed_cmd := make_drag (init, ContinueStretch, FinishStretch,
                              CancelStretch, nil, init_params);

    list (named_cmd, unnamed_cmd);
}

function get_arrow_length(data)
    vgU.ArrowGetLength (GetViewport (GetRegion (data)));
function get_arrow_width(data)
    vgU.ArrowGetWidth (GetViewport (GetRegion (data)));
function get_arrow_point(data)
    vgU.ArrowGetPoint (GetViewport (GetRegion (data)));

<ArcCmd, ArcCmd2>           := make_stretch (ArcCreate,     "arc",      nil);
<BoxCmd, BoxCmd2>           := make_stretch (BoxCreate,     "box",      nil);
<CircleCmd, CircleCmd2>     := make_stretch (CircleCreate,  "circle",   nil);
<EllipseCmd, EllipseCmd2>   := make_stretch (EllipseCreate, "ellipse",  nil);
<LineCmd, LineCmd2>         := make_stretch (LineCreate,    "line",     nil);
<VectorCmd, VectorCmd2>     :=
    make_stretch (VectorCreate,  "vector",
                  list (vcWv.Param ("Arrow Point", vgArrowVT, vcWv.Dynamic,
                                    get_arrow_point),
                        vcWv.Param ("Arrow Width", vlIntVT, vcWv.Dynamic,
                                    get_arrow_width, vcScalebar.fac,
                                    (min <- 1), (max <- 30)),
                        vcWv.Param ("Arrow Length", vlIntVT, vcWv.Dynamic,
                                    get_arrow_length, vcScalebar.fac,
                                    (min <- 1), (max <- 30))));

function make_place(args...)
{
    init        := car (args);  args = cdr (args);
    init_name   := car (args);  args = cdr (args);
    init_params := args;

    named_cmd   := make_drag (init, ContinuePlace, FinishPlace, CancelPlace,
                              init_name, init_params);
    unnamed_cmd := make_drag (init, ContinuePlace, FinishPlace, CancelPlace,
                              nil, init_params);

    list (named_cmd, unnamed_cmd);
}

<BitCmd, BitCmd2>   := make_place (BitmapCreate, "bitmap",
                                   vcWv.Param ("Bitmap Filename",
                                               vlStringVT, vcWv.Sticky,
                                               vcWv.NoDefault));
vcCmd.EnableApply (BitCmd, vlOnOff.On);


<BorderCmd, BorderCmd2> :=
    make_place (BorderCreate, "border",
                vcWv.Param ("X size", vlPositiveIntVT, vcWv.Sticky, 100),
                vcWv.Param ("Y size", vlPositiveIntVT, vcWv.Sticky, 60),
                vcWv.Param ("Width", vlPositiveIntVT, vcWv.Sticky, 12),
                vcWv.Param ("Background", vgColorVT, vcWv.Sticky,
                            vcWv.NoDefault),
                vcWv.Param ("Highlight", vgColorVT, vcWv.Sticky,
                            vcWv.NoDefault),
                vcWv.Param ("Shadowed", vgColorVT, vcWv.Sticky,
                            vcWv.NoDefault),
                vcWv.Param ("Fill Style", vgFillVT, vcWv.Dynamic,
                            func(d) GetFill (GetRegion (d));),
                vcWv.Param ("Border Style", vcBorderStyleVT,
                            vcWv.Sticky, vcBorderStyle.ShadowOut));
vcCmd.EnableApply (BorderCmd, vlOnOff.On);

                                   
<InstCmd, InstCmd2> :=
    make_place (InstanceCreate, "instance",
                vcWv.Param ("Drawing", vlFileNameVT, vcWv.Sticky,
                            vcWv.NoDefault,
                            "Name of the Viewlogic metafile to instantiate"),
                vcWv.Param ("Angle", vgeAngleVT, vcWv.Sticky, 0,
                            vcScalebar.fac,
                       "Angle at which to draw the instantiation, in degrees"),
                vcWv.Param ("Mirroring", vlYesNoVT, vcWv.Sticky, `no),
                vcWv.Param ("Scale", vlFloatVT, vcWv.Sticky, 1.0,
                            vcScalebar.fac,
                            (min <- 0.1), (max <- 10.0), (increment <- 0.1)));
vcCmd.EnableApply (InstCmd, vlOnOff.On);


function RasterCreate1(d, ev, str, txsize, align, bold, italic, fixed)
{
    declare font;

    if (!bold && !italic && !fixed)
        font = vgFont.medium;
    else if (!bold && !italic && fixed)
        font = vgFont.mediumf;
    else if (!bold && italic && !fixed)
        font = vgFont.mediumi;
    else if (!bold && italic && fixed)
        font = vgFont.mediumif;
    else if (bold && !italic && !fixed)
        font = vgFont.mediumb;
    else if (bold && !italic && fixed)
        font = vgFont.mediumbf;
    else if (bold && italic && !fixed)
        font = vgFont.mediumbi;
    else if (bold && italic && fixed)
        font = vgFont.mediumbif;

    RasterCreate (d, ev, str, txsize, align, font);
}

<RasterCmd, RasterCmd2> :=
    make_place (RasterCreate1, "raster",
                vcWv.Param ("Raster String", vlStringVT, vcWv.Sticky, ""),
                vcWv.Param ("Size", vgeTxSizeVT, vcWv.Sticky, 10),
                vcWv.Param ("Alignment", vgAlignVT, vcWv.Sticky,
                            vgAlign.LowerLeft),
                vcWv.Param ("Bold", vlTrueFalseVT, vcWv.Sticky, FALSE),
                vcWv.Param ("Italic", vlTrueFalseVT, vcWv.Sticky, FALSE),
                vcWv.Param ("Fixed", vlTrueFalseVT, vcWv.Sticky, FALSE));
vcCmd.EnableApply (RasterCmd, vlOnOff.On);


<TextCmd, TextCmd2> :=
    make_place (TextCreate, "text",
                vcWv.Param ("Text String", vlStringVT, vcWv.Sticky, ""),
                vcWv.Param ("Angle", vgeAngleVT, vcWv.Sticky, 0,
                            vcScalebar.fac),
                vcWv.Param ("Alignment", vgAlignVT, vcWv.Sticky,
                            vgAlign.LowerLeft),
                vcWv.Param ("Scoring", vgeScoreVT, vcWv.Sticky, vgeScore.None),
                vcWv.Param ("Size", vgeTxSizeVT, vcWv.Sticky, 10),
                vcWv.Param ("Font", vgStkVT, vcWv.Sticky, `roman));
vcCmd.EnableApply (TextCmd, vlOnOff.On);

<TEXTCmd, TEXTCmd2> :=
    make_place (TextCreate, "UTEXT",
                vcWv.Param ("Text String", vlStringVT, vcWv.Sticky, "",
                            (case <- vcCmdlineCase.Upper)),
                vcWv.Param ("Angle", vgeAngleVT, vcWv.Sticky, 0,
                            vcScalebar.fac),
                vcWv.Param ("Alignment", vgAlignVT, vcWv.Sticky,
                            vgAlign.LowerLeft),
                vcWv.Param ("Scoring", vgeScoreVT, vcWv.Sticky,
                            vgeScore.None),
                vcWv.Param ("Size", vgeTxSizeVT, vcWv.Sticky, 10),
                vcWv.Param ("Font", vgStkVT, vcWv.Sticky, `roman));
vcCmd.EnableApply (TEXTCmd, vlOnOff.On);

<KanjiCmd, KanjiCmd2> :=
    make_place (KanjiCreate, "kanji",
                vcWv.Param ("Text String", vlStringVT, vcWv.Sticky, "",
                            vjEditor.fac),
                vcWv.Param ("Angle", vgeAngleVT, vcWv.Sticky, 0,
                            vcScalebar.fac),
                vcWv.Param ("Alignment", vgAlignVT, vcWv.Sticky,
                            vgAlign.LowerLeft),
                vcWv.Param ("Scoring", vgeScoreVT, vcWv.Sticky, vgeScore.None),
                vcWv.Param ("Size", vgeTxSizeVT, vcWv.Sticky, 10));
vcCmd.EnableApply (KanjiCmd, vlOnOff.On);


function text_ring(d, e, str, delta, origin, size, font)
{
    if ((delta < 1) || (delta > 359))
        vlEx.Throw2 (vlEx.FAILURE, "vge", vlErrSev.Error,
                     sprintf ("Angle %s is out of range!", angle));

    for (angle := 0; angle < 360; angle += delta)
    {
        if (vcWin.Interrupt (vcEvent.Window (e), fac))
            break;
        TextCreate (d, e, str, angle, origin, vgeScore.None, size, font);
        FinishPlace (d, e);
    }

    for (angle := 0; angle < 360; angle += delta)
        Select1 (d, e);

    vcCmd.Run (MoveCmd, d, e, nil, FALSE);
}


<TextRingCmd, TextRingCmd2> :=
    vge_cmds (text_ring, "textring", "Text Ring",
              vcWv.Param ("Text String", vlStringVT, vcWv.Sticky, ""),
              vcWv.Param ("Angle Delta", vgeAngleVT, vcWv.Sticky, 10,
                          vcScalebar.fac),
              vcWv.Param ("Alignment", vgAlignVT, vcWv.Sticky,
                          vgAlign.LowerLeft),
              vcWv.Param ("Size", vgeTxSizeVT, vcWv.Sticky, 10,
                          vcScalebar.fac),
              vcWv.Param ("Font", vgStkVT, vcWv.Sticky, `roman));


function select(d, e)
{
    UnselectAll (vcEvent.Region (e));
    Select1 (d, e);
}


function set_default_binding(cmd)
    vcGesture.Set (gestures, vcGesture.Default, cmd);


<WriteCmd, WriteCmd2>           := vge_cmds (func(d, e) Write (e);, "write");
<WriteToCmd, WriteToCmd2>       := vge_cmds (func(d, e, filename)
                                             WriteTo (e, filename);,
                                             "writeto", "New Filename",
                                             vcWv.Param ("New filename",
                                                         vlFileWriteVT,
                                                         vcWv.Sticky,
                                                         vcWv.NoDefault));
<DeleteCmd, DeleteCmd2>         := vge_cmds (func(d, e)
                                             Delete (vcEvent.Region (e));,
                                             "delete");
<OdetailCmd, OdetailCmd2>       := vge_cmds (ObjectDetail, "odetail");
<SelectCmd, SelectCmd2>         := vge_cmds (select, "select");
<SelectAddCmd, SelectAddCmd2>   := vge_cmds (Select1, "selectadd");
<ActivateCmd, ActivateCmd2>     := vge_cmds (Activate, "activate");
<AltActivateCmd, AltActivateCmd2>  := vge_cmds (AltActivate, "altactivate");


/*
 * The AltActivateCmd may be bound by a parent region if the developer feels a
 * need for it. There is no binding built in for it.
 */


/*
 * Command for changing the default color, fill style and line style.
 */
function style(data, event, col, fill, pat, cap, join, thick)
{
    vge_region := vcEvent.Region (event);

    SetColor  (vge_region, col);
    SetFill   (vge_region, fill);
    SetLStyle (vge_region, pat, cap, join, thick);
}

<StyleCmd, StyleCmd2> :=
    vge_cmds (style, "style", "Color, Fillstyle, Linestyle",
              vcWv.Param ("Color", vgColorVT, vcWv.Dynamic,
                          func(d) GetColor (GetRegion (d));),
              vcWv.Param ("Fillstyle", vgFillVT, vcWv.Dynamic,
                          func(d) GetFill (GetRegion (d));),
              vcWv.Param ("Line Pattern", vgLinePatternVT, vcWv.Dynamic,
                          func(d) GetLPattern (GetRegion (d));),
              vcWv.Param ("Line Cap Style", vgLineCapVT, vcWv.Dynamic,
                          func(d) GetLCap (GetRegion (d));),
              vcWv.Param ("Line Join Style", vgLineJoinVT, vcWv.Dynamic,
                          func(d) GetLJoin (GetRegion (d));),
              vcWv.Param ("Line Thickness", vlIntVT, vcWv.Dynamic,
                          func(d) GetLThickness (GetRegion (d));,
                          vcScalebar.fac, (min <- 0), (max <- 100)));
vcCmd.EnableApply (StyleCmd, vlOnOff.On);


<PopupEditCmd, PopupEditCmd2> :=
    vge_cmds (func (d, e) vcMenuPop.Create (EditMenu, e););


vcWv.Interrupts (fac, '\F5', '\F6', '\F7', '\F8', '\F9', '\F0');


/*
 * Command definition to enable reception of IPC exec messages.
 */
IpcExecMsgType := "vgeCommand";

function enable_ipc(data, event)
{
    win         := vcEvent.Window (event);
    src_cntx    := vipc.GetCntxDflt ();
    dest_cntx   := vitc.GetCntxFid ();


    function ipc_exec(mid, src, match, data)
    {
        if (string_eq (mid, IpcExecMsgType))
            vcWin.RunCmdline (win, data, event);
    }

    vitc.Register (IpcExecMsgType, src_cntx, dest_cntx, ipc_exec);
}

<IpcCmd, IpcCmd2> := vge_cmds (enable_ipc, "ipc");


function push(data, event, filename)
{
    vge_region := vcEvent.Region (event);

    PushDrawing (vge_region, DrawCreate (filename));

    vcCmd.Run (vcVPCanvas.FullCmd, vge_region, event, nil, FALSE);
}

<PushCmd, PushCmd2> :=
    vge_cmds (push, "push", "Push New Drawing",
              vcWv.Param ("Metafile", vlFileNameVT, vcWv.Sticky,
                          vcWv.NoDefault));

function pop(date, event)
{
    vge_region := vcEvent.Region (event);

    if (PopDrawing (vge_region))
        vcCmd.Run (vcVPCanvas.FullCmd, vge_region, event, nil, FALSE);
}

<PopCmd, PopCmd2> := vge_cmds (pop, "pop");


/*\Docpage quit_function
 * 
 * DESCRIPTION
 *      Register a function which will determine if there are any modified
 *      drawings that might be worth saving before executing the quit command.
 */
function quit_function()
{
    modified := "";

    foreach (name; ModifiedDrawings ())
        modified = sprintf ("%s\n\t%s", modified, name);

    if (string_eq (modified, ""))
        modified = nil;
    else
        modified = "The following drawings are modified:\n" + modified;
}

vcQuit.AddFunction (quit_function);


/****************************************************************************/
/*                              Gestures.                                   */
/****************************************************************************/

vcGesture.Set (gestures, vcGesture.M,       MoveCmd2);
vcGesture.Set (gestures, vcGesture.C,       CopyCmd2);
vcGesture.Set (gestures, vcGesture.Z,       vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.Line,    vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.R,       EditCmd2);
vcGesture.Set (gestures, vcGesture.UpsideDownCheck, vcVPCanvas.FullCmd);
vcGesture.Set (gestures, vcGesture.P,       vcVPCanvas.PanCmd);
vcGesture.Set (gestures, vcGesture.d,       DeleteCmd2);


/****************************************************************************/
/*                          Command help strings.                           */
/****************************************************************************/

vcCmd.SetHelp (ArcCmd2,      "Enter `arc' mode");
vcCmd.SetHelp (BitCmd2,      "Enter `bitmap' mode");
vcCmd.SetHelp (BorderCmd2,   "Enter `border' mode");
vcCmd.SetHelp (BoxCmd2,      "Enter `box' mode");
vcCmd.SetHelp (CircleCmd2,   "Enter `circle' mode");
vcCmd.SetHelp (CopyCmd2,     "Copy selected object(s)");
vcCmd.SetHelp (DeleteCmd2,   "Delete selected object(s)");
vcCmd.SetHelp (EditCmd2,     "Open another instance of the graphical editor");
vcCmd.SetHelp (EllipseCmd2,  "Enter `ellipse' mode");
vcCmd.SetHelp (InstCmd2,     "Enter `instance' mode");
vcCmd.SetHelp (IpcCmd2,      "Enable reception of IPC exec messages");
vcCmd.SetHelp (KanjiCmd2,    "Enter `Kanji' mode");
vcCmd.SetHelp (LineCmd2,     "Enter `line' mode");
vcCmd.SetHelp (MoveCmd2,     "Move selected object(s)");
vcCmd.SetHelp (OdetailCmd2,  "List object details");
vcCmd.SetHelp (PlineCmd2,    "Enter `pline' mode");
vcCmd.SetHelp (PopCmd2,      "Remove the current drawing from display");
vcCmd.SetHelp (PushCmd2,     "Add a new drawing to the display");
vcCmd.SetHelp (RasterCmd2,   "Enter `raster' mode");
vcCmd.SetHelp (SelectAddCmd2,"Select an additional object");
vcCmd.SetHelp (SelectCmd2,   "Select an object");
vcCmd.SetHelp (SplineCmd2,   "Enter `spline' mode");
vcCmd.SetHelp (StretchCmd2,  "Stretch one selected object");
vcCmd.SetHelp (StyleCmd2,    "Set common style parameters");
vcCmd.SetHelp (TextCmd2,     "Enter `text' mode");
vcCmd.SetHelp (TEXTCmd2,     "Enter `uppercase text' mode");
vcCmd.SetHelp (VectorCmd2,   "Enter `vector' mode");
vcCmd.SetHelp (WriteCmd2,    "Save all object");
vcCmd.SetHelp (WriteToCmd2,  "Save all objects in a new metafile");


/****************************************************************************/
/*                              Bindings.                                   */
/****************************************************************************/

/*
 * NOTE
 * The vge bindings are controlled by the setting of the vge.set_bindings
 * preference.  By default it's set to TRUE meaning that all the applications
 * that build on top of the VGE facility will inherit its bindings.  If you
 * don't want to see the bindings defined below, set the preference to FALSE
 * either on the command line or in your powerview.ini file.
 */

vcBind.Set (bindings, 'A',          ArcCmd2);
vcBind.Set (bindings, 'B',          BoxCmd2);
vcBind.Set (bindings, 'C',          CircleCmd2);
vcBind.Set (bindings, 'E',          EllipseCmd2);
vcBind.Set (bindings, 'I',          InstCmd2);
vcBind.Set (bindings, 'K',          KanjiCmd2);
vcBind.Set (bindings, 'L',          LineCmd2);
vcBind.Set (bindings, 'O',          BorderCmd2);
vcBind.Set (bindings, 'P',          PlineCmd2);
vcBind.Set (bindings, 'R',          RasterCmd2);
vcBind.Set (bindings, 'T',          TEXTCmd2);
vcBind.Set (bindings, 'W',          WriteToCmd2);
vcBind.Set (bindings, 'V',          VectorCmd2);
vcBind.Set (bindings, 'X',          BitCmd2);
vcBind.Set (bindings, 'Z',          SplineCmd2);

vcBind.Set (bindings, 'c',          CopyCmd2);
vcBind.Set (bindings, 'd',          DeleteCmd2);
vcBind.Set (bindings, 'm',          MoveCmd2);
vcBind.Set (bindings, 's',          StretchCmd2); 
vcBind.Set (bindings, 't',          TextCmd2);
vcBind.Set (bindings, 'w',          WriteCmd2);
vcBind.Set (bindings, 'z',          vcVPCanvas.ZoomCmd);

vcBind.Set (bindings, '\_p',        PopCmd2);
vcBind.Set (bindings, '\_n',        PushCmd2);
vcBind.Set (bindings, '\_s',        StyleCmd2);                        
vcBind.Set (bindings, '\_r',        TextRingCmd2);
vcBind.Set (bindings, '\middledown',vcGesture.StartCmd);
vcBind.Set (bindings, '\delete',    DeleteCmd2);
vcBind.Set (bindings, '\leftdown',  SelectCmd2);
vcBind.Set (bindings, '\rightdown', PopupEditCmd2);
vcBind.Set (bindings, '\^\leftdown',SelectAddCmd2);
vcBind.Set (bindings, '\leftdbl',   ActivateCmd2);

function arrow(key, dx, dy)
    vcBind.Set (bindings, key,
                vcWv.Cmd (fac, func (d, e) MoveCursor (d, dx, dy);));

arrow ('\left',    -1,  0);
arrow ('\right',    1,  0);
arrow ('\up',       0,  1);
arrow ('\down',     0, -1);

arrow ('\^\left', -10,   0);
arrow ('\^\right', 10,   0);
arrow ('\^\up',     0,  10);
arrow ('\^\down',   0, -10);


/****************************************************************************/
/*                          Disable all commands                            */
/****************************************************************************/

/*
 * Just like with bindings, the commands are also disabled if the set_bindings
 * preference value is set to FALSE.  This is to allow applications developed
 * on top of the VGE facility the freedom from interference by the user who
 * inadvertently invokes a VGE command.
 */
if (!vlPref.Val (prefs, "set_bindings"))
{
    vcCmd.Enable (EditCmd,          vlOnOff.Off);
    vcCmd.Enable (PlineCmd,         vlOnOff.Off);
    vcCmd.Enable (PlineAuxCmd,      vlOnOff.Off);
    vcCmd.Enable (SplineCmd,        vlOnOff.Off);
    vcCmd.Enable (SplineAuxCmd,     vlOnOff.Off);
    vcCmd.Enable (TextRingCmd,      vlOnOff.Off);
    vcCmd.Enable (WriteCmd,         vlOnOff.Off);
    vcCmd.Enable (WriteToCmd,       vlOnOff.Off);
    vcCmd.Enable (DeleteCmd,        vlOnOff.Off);
    vcCmd.Enable (OdetailCmd,       vlOnOff.Off);
    vcCmd.Enable (SelectCmd,        vlOnOff.Off);
    vcCmd.Enable (SelectAddCmd,     vlOnOff.Off);
    vcCmd.Enable (ActivateCmd,      vlOnOff.Off);
    vcCmd.Enable (AltActivateCmd,   vlOnOff.Off);
    vcCmd.Enable (StyleCmd,         vlOnOff.Off);
    vcCmd.Enable (PopupEditCmd,     vlOnOff.Off);
    vcCmd.Enable (IpcCmd,           vlOnOff.Off);
    vcCmd.Enable (PushCmd,          vlOnOff.Off);
    vcCmd.Enable (PopCmd,           vlOnOff.Off);
    vcCmd.Enable (MoveCmd,          vlOnOff.Off);
    vcCmd.Enable (CopyCmd,          vlOnOff.Off);
    vcCmd.Enable (StretchCmd,       vlOnOff.Off);
    vcCmd.Enable (ArcCmd,           vlOnOff.Off);
    vcCmd.Enable (BoxCmd,           vlOnOff.Off);
    vcCmd.Enable (CircleCmd,        vlOnOff.Off);
    vcCmd.Enable (EllipseCmd,       vlOnOff.Off);
    vcCmd.Enable (LineCmd,          vlOnOff.Off);
    vcCmd.Enable (VectorCmd,        vlOnOff.Off);
    vcCmd.Enable (BitCmd,           vlOnOff.Off);
    vcCmd.Enable (BorderCmd,        vlOnOff.Off);
    vcCmd.Enable (InstCmd,          vlOnOff.Off);
    vcCmd.Enable (RasterCmd,        vlOnOff.Off);
    vcCmd.Enable (TextCmd,          vlOnOff.Off);
    vcCmd.Enable (TEXTCmd,          vlOnOff.Off);
    vcCmd.Enable (KanjiCmd,         vlOnOff.Off);
    vcCmd.Enable (TextRingCmd,      vlOnOff.Off);
}


/****************************************************************************/
/*                          Menu definitions.                               */
/****************************************************************************/

function set_mode(c)
{
    cc := vcWv.Cmd (fac, func (d, e) set_default_binding (c););
    vcCmd.SetHelp (cc, "Set the current `mode' to " + vcCmd.Name (c));
    vcCmd.SetKey (cc, vcCmd.Key (c));
    cc;
}


FileMenu := vcWv.MakeMenu ("File Commands",
                           vcWv.Pin (),
                           list ("Open",    EditCmd2),
                           list ("Write",   WriteCmd2),
                           list ("WriteTo", WriteToCmd2));


AddMenu := vcWv.MakeMenu ("Add Objects",
                          vcWv.Pin (),
                          list ("Arc",      set_mode (ArcCmd2)),
                          list ("Bitmap",   set_mode (BitCmd2)),
                          list ("Border",   set_mode (BorderCmd2)),
                          list ("Box",      set_mode (BoxCmd2)),
                          list ("Circle",   set_mode (CircleCmd2)),
                          list ("Ellipse",  set_mode (EllipseCmd2)),
                          list ("Line",     set_mode (LineCmd2)),
                          list ("Kanji",    set_mode (KanjiCmd2)),
                          list ("Instance", set_mode (InstCmd2)),
                          list ("Pline",    set_mode (PlineCmd2)),
                          list ("Raster",   set_mode (RasterCmd2)),
                          list ("Spline",   set_mode (SplineCmd2)),
                          list ("Text",     set_mode (TextCmd2)),
                          list ("TEXT",     set_mode (TEXTCmd2)),
                          list ("TextRing", set_mode (TextRingCmd2)),
                          list ("Vector",   set_mode (VectorCmd2)));

EditMenu := vcWv.MakeMenu ("Edit Commands",
                           vcWv.Pin (),
                           list ("Move",    set_mode (MoveCmd2)),
                           list ("Copy",    set_mode (CopyCmd2)),
                           list ("Stretch", set_mode (StretchCmd2)),
                           list ("Style",   StyleCmd2),
                           list ("Delete",  DeleteCmd2));

ViewMenu := vcWv.MakeMenu ("View Commands",
                           vcWv.Pin (),
                           list ("Refresh", vcVPCanvas.RefreshCmd),
                           list ("In",      vcVPCanvas.InCmd),
                           list ("Out",     vcVPCanvas.OutCmd),
                           list ("Pan",     set_mode (vcVPCanvas.PanCmd)),
                           list ("Full",    vcVPCanvas.FullCmd),
                           list ("Objects", OdetailCmd2),
                           list ("Push",    PushCmd2),
                           list ("Pop",     PopCmd2));

MainMenu := vcWv.MakeMenu ("Main Menu",
                           vcWin.RedSquareEntry,
                           list ("File",  FileMenu),
                           list ("View",  ViewMenu),
                           list ("Add",   AddMenu),
                           list ("Edit",  EditMenu));

vcFac.SetMenus (fac, MainMenu);
