/*
 *  $Revision: 1.8.7.1 $
 *
 *  Viewscript UI definitions for the Viewtext incremental search.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/*
 * This is an incremental search package for Viewtext.  It is defined
 * inside a massive package statement to avoid symbol name conflicts
 * with whatever this package is loaded in.  Nobody needs to reference
 * names in this package, since when its loaded, it directly modifies
 * the viewtext binding table.
 */

package incremental_search() {
    /* Format of the stack:
     *  stack is a list, each element of which has the format:
     *  (line column string failure_flag)
     */
    stack := nil;
    function push(line, column, str, direction, failure_flag)
        stack = cons (list (line, column, str, direction, failure_flag),
                      stack);
    line        := func() caar (stack);;
    column      := func() cadar (stack);;
    search_str  := func() caddar (stack);;
    direction   := func() car (cdddar (stack));;
    failure     := func() cadr (cdddar (stack));;
    function pop()
        stack = cdr (stack);
    function empty()
        cdr (stack) == nil;
        
    /* State variable can be `init or `in_action */
    state := `init;
    default_string := "";

    /* Incremental-search binding table */
    isearch_bindings := vcBind.Create ();

    save_bindings := nil;

    function init(vt, event) {
        r := vtx.Region (vt);
        save_bindings = vcRegion.Bindings (r);
        vcRegion.SetBindings (r, isearch_bindings);
        vcRegion.SetGrab (r, r);
    } /* function init */
        
    function term(vt, event) {
        default_string = search_str ();
        r := vtx.Region (vt);
        vcRegion.SetBindings (r, save_bindings);
        vcRegion.SetGrab (r, nil);
        vcWin.ClearStatus (vcEvent.Window (event));
    } /* function term */

    function default_valid()
        ((state == `init) && !string_eq (default_string, ""));

    function update(vt, event) {
        w := vcEvent.Window (event);
        vcWin.ClearStatus (w);
        if (failure ())
            vcWin.AddStatus (w, "Failing ");
        vcWin.AddStatus (w, "I-search");
        if (direction () == `reverse)
            vcWin.AddStatus (w, " backward");
        if (default_valid ())
            vcWin.AddStatus (w, " [" + default_string + "]");
            
        vcWin.AddStatus (w, ":");
        vcWin.AddStatus (w, search_str ());
    } /* function update */

    function restart_forward(vt, event) {
        init (vt, event);
        state = `in_action;
        vtx.Move (vt, line (), column ());
        update (vt, event);
    } /* function restart_forward */
        
    function start_forward(vt, event) {
        init (vt, event);
        state = `init;
        stack = nil;
        push (vtx.CurrentLine (vt), vtx.CurrentCol (vt),
              "", `forward, FALSE);
        update (vt, event);
    } /* function start_forward */

    function try_search(vt, event, str) {
        state = `in_action;
        failed := TRUE;
        if (!failure ()) {
            if (!string_eq (search_str (), str))
                vtx.Move (vt, line (),
                          column() - string_length (search_str ()));
            failed = !vtx.SearchFwd (vt, str);
            if (failed)
                vtx.Move (vt, line (), column ());
        } /* if */
        push (vtx.CurrentLine (vt), vtx.CurrentCol (vt),
              str, `forward, failed);
        update (vt, event);
    } /* function try_search */

    function forward(vt, event) {
        if (!failure ())
            try_search (vt,
                        event,
                        default_valid () ? default_string : search_str ());
    } /* function forward */

    function add_ch(vt, event) {
        try_search (vt,
                    event,
                    search_str () + sprintf ("%c", vcEvent.Ch (event)));
    } /* function add_ch */

    function backup(vt, event) {
        state = `in_action;
        if (!empty ())
            pop ();
        vtx.Move (vt, line (), column ());
        update (vt, event);
    } /* function backup */

    function abort(vt, event) {
        state = `in_action;
        if (failure ()) {
            while (failure () && !empty ())
                pop ();
            vtx.Move (vt, line (), column ());
            update (vt, event);
        } /* if */
        else {
            while (!empty ())
                pop ();
            vtx.Move (vt, line (), column ());
            term (vt, event);
        } /* else */
    } /* function abort */

    function term_and_do(vt, event) {
        term (vt, event);
        cmd := vcBind.Get (vtx.bindings, vcEvent.Ch (event));
        if (cmd != nil)
            vcCmd.Run (cmd, vt, event, nil, FALSE);
    } /* function term_and_do */

    vcBind.Set (vtx.bindings, '\^S', vcWv.Cmd (vtx.fac, start_forward,
                                               "i-search-forward"));
    vcBind.Set (vtx.bindings, '\_\^S', vcWv.Cmd (vtx.fac, restart_forward,
                                                 "i-search-restart"));

    term_and_do_cmd := vcWv.Cmd (vtx.fac, term_and_do, "isearch-term-and-do");
    for (i := 0; i < 32; ++i)
        vcBind.Set (isearch_bindings, int2char (i), term_and_do_cmd);

    vcBind.Set (isearch_bindings, '\^S',
                vcWv.Cmd (vtx.fac, forward, "isearch-forward"));
    vcBind.Set (isearch_bindings, '\e',
                vcWv.Cmd (vtx.fac, term, "isearch-term"));
    backup_cmd := vcWv.Cmd (vtx.fac, backup, "isearch-backup");
    vcBind.Set (isearch_bindings, '\177', backup_cmd);
    vcBind.Set (isearch_bindings, '\^H', backup_cmd);

    abort_cmd := vcWv.Cmd (vtx.fac, abort, "isearch-abort");
    vcBind.Set (isearch_bindings, '\^G', abort_cmd);

    add_cmd := vcWv.Cmd (vtx.fac, add_ch, "isearch-add-ch");
    foreach (ch; vcWv.Printable ())
        vcBind.Set (isearch_bindings, ch, add_cmd);
} /* package incremental_search */

/*
 *    function restart_reverse(vt, event) {
 *        init (vt, event);
 *        state = `in_action;
 *    }
 *
 *    function start_reverse(vt, event) {
 *        init (vt, event);
 *        state = `init;
 *        stack = nil;
 *        push (vtx.CurrentLine (vt), vtx.CurrentCol (vt),
 *              "", `reverse, FALSE);
 *        update (vt, event);
 *    }
 */
