/*
 *  $Revision: 1.4.2.7 $  $Date: 1995/05/02 21:47:47 $
 *
 *  Routines for parsing up a version 4 or 5 RCS file.  For more info
 *  read rcsfile(5).
 *
 *  Copyright (c) 1994,1995 by Viewlogic Systems, Inc.
 */



/* The structure which represents the entire RCS file. */
struct
rcs_text_s
{
    vlStringVT  head;
    vlStringVT  default;
    vlStringVT  branch;
    vscmVT      symbols;
    vscmVT      locks;
    vscmVT      deltas;

    vlIntVT     hits;
    vlIntVT     time;
    vlIntVT     size;
};



/* The structure which represents a single version delta. */
struct
rcs_delta_s
{
    vlStringVT  version;
    vlStringVT  date;
    vlStringVT  author;
    vlStringVT  state;
    vscmVT      branches;
    /* Yeah, I ignore some other stuff. */
};


rcs_cache_size  := 71;
rcs_cache       := vlSym.StrCreate (rcs_cache_size);


/*\intFunction rcs_alloc_text -- Allocate a new rcs_text_s struct.
 */
function rcs_alloc_text()
{
    rcs_text        := struct rcs_text_s { "", "","", nil, nil, nil, 0, 0, 0 };


    /* All constants are arbitrary. */
    rcs_text.symbols = vlSym.StrCreate (71);
    rcs_text.locks   = vlSym.StrCreate (5);
    rcs_text.deltas  = vlSym.StrCreate (71);

    vlSym.Dynamic (rcs_text.symbols, 2.0, 2.0);
    vlSym.Dynamic (rcs_text.locks, 2.0, 2.0);
    vlSym.Dynamic (rcs_text.deltas, 2.0, 2.0);

    rcs_text;
}



/*\intFunction rcs_alloc_delta -- Allocate a new rcs_delta_s struct.
 */
function rcs_alloc_delta()
    rcs_delta := struct rcs_delta_s { "", "", "", "", nil };



/*\intFunction rcs_parse_admin -- Parse up the administration section.
 * 
 * SYNOPSIS
 *      rcs_parse_admin(fp, rcs)
 *          vlFileVT            fp;
 *          struct rcs_text_s   rcs;
 *
 * DESCRIPTION
 *      This routine parses up the admin section of an RCS file. For our
 *      purposes, the administration section consists only of those fields
 *      which are required to be present by RCS. These fields are the head,
 *      the symbols list, and the locks list. The remainder of the admin
 *      section is ignored.
 */
function rcs_parse_admin(fp, rcs)
{
    declare line;
    declare done_with_admin;

    done_with_admin = FALSE;

    while (!null (line = fgetline (fp)) && !done_with_admin)
        if (string_search (line, "head") == 0)
            rcs_parse_head (line, rcs);

        else if (string_search (line, "branch") == 0)
            rcs_parse_branch (line, rcs);

        else if (string_search (line, "symbols") == 0)
            rcs_parse_symbols (line, rcs, fp);

        else if (string_search (line, "locks") == 0)
        {
            rcs_parse_locks (line, rcs, fp);
            done_with_admin = TRUE;
        }
}



/*\srcFunction rcs_parse_head -- Parse up the head.
 */
function rcs_parse_head(line, rcs)
{
    declare token;

    line = string_token (line, &token, " \t");
    line = string_token (line, &token, " \t\n;");

    rcs.head = token;
}



/*\srcFunction rcs_parse_branch -- Parse up the branch.
 */
function rcs_parse_branch(line, rcs)
{
    declare token;

    line = string_token (line, &token, " \t");
    line = string_token (line, &token, " \t\n;");

    rcs.branch = (token == nil) ? "" : token;
}



/*\srcFunction rcs_parse_symbols -- Parse up the symbols list.
 *
 * DESCRIPTION
 *      The symbol list has the following syntax:
 *
 *          symbols    {id : num}*;
 */
function rcs_parse_symbols(line, rcs, fp)
{
    declare token;
    declare symbol;
    declare version;

    done       := FALSE;
    str        := "";

    while (!done)
    {
        str  = str + line;
        done = rindex (line, ';');
        if (!done)
            line = fgetline (fp);
    }

    str = substring (str, 8, 0);
    while (!null (str = string_token (str, &symbol, " \t\n:;")))
    {
        str = string_token (str, &version, " \t\n:;");
        vlSym.Store (rcs.symbols, symbol, version);
    }
}        



/*\srcFunction rcs_parse_locks -- Parse up the locks list.
 *
 * DESCRIPTION
 *      The lock list has the following syntax:
 *
 *          locks      {id : num}*;  {strict  ;}
 */
function rcs_parse_locks(line, rcs, fp)
{
    declare user;
    declare version;
    declare index;

    done       := FALSE;
    str        := "";

    while (!done)
    {
        str  = str + line;
        done = rindex (line, ';');
        if (!done)
            line = fgetline (fp);
    }

    str = substring (str, 5, 0);

    index = string_search (str, "strict;");
    if (!null (index))
        str = substring (str, 0, index);

    /*
     * NOTE: It's the version that is serves as the keyword in the rcs.locks
     * table, not the user, because the same user can have locks on multiple
     * versions.
     */
    while (!null (str = string_token (str, &user, " \t\n:;")))
    {
        str = string_token (str, &version, " \t\n:;");
        vlSym.Store (rcs.locks, version, user);
    }

}



/*\intFunction rcs_parse_delta -- Parse up the deltas.
 * 
 * SYNOPSIS
 *      rcs_parse_delta(fp, rcs)
 *          vlFileVT            fp;
 *          struct rcs_text_s   rcs;
 *
 * DESCRIPTION
 *      This routine parses up all the delta sections found in an RCS file.
 *      It allocates new delta records and adds them to the rcs structure
 *      which is being built.
 */
function rcs_parse_delta(fp, rcs)
{
    declare line;
    declare token;
    declare i;

    done       := FALSE;
    first_time := TRUE;


    while ((!done) && (!null (line = fgetline (fp))))
    {
        /* The first time we get here, there may be some junk left over
         * from the admin section which we don't want. So, we just throw
         * away everything until we encounter a numeric digit in the first
         * column which indicates that the delta section has really begun.
         */
        while (first_time)
        {
            i = char2int (line[0]);

            if ((i >= 48) && (i <= 57))
                first_time = FALSE;
            else if (null (line = fgetline (fp)))
                /* Return used on purpose. */
                return nil;                
        }

        i = char2int (line[0]);

        if ((i >= 48) && (i <= 57))
        {
            rcs_delta := rcs_alloc_delta ();

            rcs_delta.version = string_trim (line);
            /* printf ("version: %s\n", rcs_delta.version); */

            line = fgetline (fp);

            line = substring (line, 4, 0);
            line = string_token (line, &token, " \t;");            
            rcs_delta.date = token;
            /*printf ("date: %s\n", rcs_delta.date);*/

            line = string_token (line, &token, " \t;");
            line = string_token (line, &token, " \t;");            
            rcs_delta.author = token;
            /* printf ("author: %s\n", rcs_delta.author); */

            line = string_token (line, &token, " \t;");
            line = string_token (line, &token, " \t;");            
            rcs_delta.state = token;
            /* printf ("state: %s\n", rcs_delta.state); */

            line = fgetline (fp);

            vlSym.Store (rcs.deltas, rcs_delta.version, rcs_delta);
        }

        /* The delta section is over. */
        if (string_search (line, "desc") == 0)
            done = TRUE;
    }
}



/*\Function rcs_parse -- Parse the specified RCS file.
 * 
 * SYNOPSIS
 *      struct rcs_file_s
 *      rcs_parse(filename)
 *          vlStringVT      filename;
 *          
 * DESCRIPTION
 *      This routine opens the file specified by ``filename'' and parses
 *      it and loads some of the interesting info from this structure into
 *      the structure which it allocates.
 *
 * CAUTION
 *      It raises an exception if the file does not exist, or cannot be opened
 *      for reading.
 *
 * RETURNS
 *      A data structure containing interesting bits from a parsed up RCS
 *      file.
 */
function rcs_parse(filename)
{
    rcs := rcs_cache_fetch (filename);

    if (null (rcs))
    {
        fp  := fopen (filename, "r");
        rcs  = rcs_alloc_text ();

        rcs_parse_admin (fp, rcs);
        rcs_parse_delta (fp, rcs);

        fclose (fp);

        rcs_cache_store (rcs, filename);
    }

    rcs;
}




/*\intFunction rcs_rcstime_to_longtime -- Convert RCS time format to long.
 * 
 * DESCRIPTION
 *      RCS stores its date in the following format:
 *
 *          yy.mm.dd.h.m.s
 *
 *      This routine converts it from this format to the standard long format
 *      used by most time based routines. This routine does not perform any
 *      adjustment based on time zones.
 */
function rcs_rcstime_to_longtime(rcstime)
{
    declare longtime;

    yy  := string2number ("19" + substring (rcstime, 0, 2));
    mm  := string2number (substring (rcstime, 3, 5));
    dd  := string2number (substring (rcstime, 6, 8));
    h   := string2number (substring (rcstime, 9, 11));
    m   := string2number (substring (rcstime, 12, 14));
    s   := string2number (substring (rcstime, 15, 17));

    longtime = 0;

    /* Add the years. */
    for (i := 1970; i < yy; ++i)
        if (rcs_leapyear_p (i))
            dd += 366;
        else
            dd += 365;

    /* Add the months. */
    for (i = 0; i < (mm - 1); ++i)
        dd += rcs_rcstime_months[i];

    /* Add the days, hours, minutes, seconds. */
    longtime += ((dd - 1) * 86400) + (h * 3600) + (m * 60) + s;

    longtime;
}


rcs_rcstime_months := [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];



/*\intFunction rcs_leapyear_p -- Is the given year a leapyear?
 * 
 * DESCRIPTION
 *      Return TRUE ``year'' is a leapyear.
 */
function rcs_leapyear_p(year)
    (((((year) % 4) == 0) && ((year) % 100 != 0)) || (((year) % 400) == 0));



function rcs_branch_is_version(branch)
{
    count := 0;

    /*printf ("rcs_branch_is_version\n");*/
    /*
     * Break the version string into tokens, ie 1.2.1.1 is 4 tokens, 1, 2, 
     * 1, & 1.
     */
    foreach (tok; string_loop (branch, "."))
        ++count;

    /*
     * If there are an even number of tokens, it's a version, else it's a 
     * branch.
     */
    mod (count, 2) == 0;
}



function rcs_version_get_root_and_leaf(version)
{
    /*printf ("rcs_get_root_and_leaf\n");*/
    if ((posnDot := rindex (version, '.')) == FALSE)
    {
        vlErr.Out2 ("rcslib", vlErrSev.Warning,
                    sprintf ("Expected a '.' in the version string", version));
        list (nil, nil);
    }
    else
    {
        root   := substring (version, 0, posnDot);
        suffix := substring (version, posnDot + 1, 0);

        list (root, suffix);
    }
}



/*\intDocpage rcs_default_version -- Get the default verion.
 */
function rcs_default_version(rcs)
{
    /* printf ("rcs_default_version\n"); */
    if (string_eq (rcs.default, ""))
    {
        if (string_eq (rcs.branch, ""))
            rcs.default = rcs.head;
        else
        {
            if (rcs_branch_is_version (rcs.branch))
                rcs.default = rcs.branch;
            else
            {
                highest := 0;

                foreach (vers, delta; vlSym.Loop (rcs.deltas))
                {
                    <root, suffix> := rcs_version_get_root_and_leaf (vers);

                    if ((root != nil) && (suffix != nil) &&
                        string_eq (root, rcs.branch))
                        highest = max (highest, string2number (suffix));
                }

                rcs.default = rcs.branch + "." + number2string (highest);
            }
        }
    }


    rcs.default;
}



/*\intDocpage rcs_cache_store -- Store this record in the cache.
 *
 *  DESCRIPTION
 *      Store the rcs record in the cache. Boot somebody out if
 *      the cache is already full. The record with the fewest hits
 *      is booted.
 */
function rcs_cache_store(rcs, key)
{
    declare loser;
    declare min_hits;

    min_hits := 1000000000;

    if (vlSym.Size (rcs_cache) == rcs_cache_size)
    {
        foreach (n, r; vlSym.Loop (rcs_cache))
            if (r.hits < min_hits)
            {            
                min_hits = r.hits;
                loser    = n;
            }

        vlSym.Remove (rcs_cache, loser);
    }

    info := vscript.vsInfo (key);
    rcs.time = car (info);
    rcs.size = car (cdr (info));
    rcs.hits = 1;

    vlSym.Store (rcs_cache, key, rcs);
}



/*\intDocpage rcs_cache_fetch -- Fetch a record from the cache.
 *
 *  DESCRIPTION
 *      Fetch the given record from the cache. Before handing
 *      it back, make sure it is still up to date.
 */
function rcs_cache_fetch(key)
{
    rcs := nil;    

    rcs = vlSym.Fetch (rcs_cache, key);
    if (!null (rcs))
    {
        info := vscript.vsInfo (key);
        if ((!null (info)) &&
            (rcs.time == car (info)) &&
            (rcs.size == car (cdr (info))))
            rcs.hits = rcs.hits + 1;
        else
            rcs = nil;        
    }

    rcs;
}

