/* 
 * $Id: vcMultOpts.vs,v 1.1.1.4 1995/03/29 20:46:37 akoren Stab $
 *
 * vcMultOpts facility 
 */

/*\vscmuserSection vcMultOpts. - multiple option facility
 * 
 * DESCRIPTION 
 *     This facility is useful for allowing the user to select from
 *     a large (or unlimited) number of options, while making a 
 *     few common options easily available.
 *     In order to use this facility, it must be loaded with 
 *     vcWv.RequireFac(man).
 *
 *     The facility appears as a radio box, with several options.
 *     The first option, instead of having a label, has a command
 *     line, into which users may enter whatever they want.
 *
 *     In addition, the developer may optionally provide a 'browse' 
 *     button by providing a function that will be called when
 *     that button is pressed. This function takes a single argument,
 *     a vcCmdline region, which the function may get and set.
 *     Some functions listed below can help the developer create her
 *     own browse functions; see also vcMultOpts.Changed(man).
 *     
 *     Command-line completion, when the user presses the space bar,
 *     may be provided if the completion preference is set.
 *
 *     vcRegion.Get may be used to get the value in the region.
 *
 * INITIALIZATION DATA
 *     vlStringVT - the inital value
 *
 * PREFERENCES
 *      labels - list of labels for the radio buttons
 *      browse - the browse function (described above)
 *      border - whether to put a border around the region
 *      border_label - optional label to attach to border
 *      completion - list of command-line completion strings
 */
 
armed_bindings := vcBind.Create ();
empty_bindings := vcBind.Create ();

/* gets the toggle button from a numerical index */
function index_to_toggle (opt, ix)
    opt.ToggleList[ix];

/* gets a numerical index from a toggle button */
function toggle_to_index(opt, tog) {
    toglist := opt.ToggleList;
    ix := 0;
    found := FALSE;
    while (!null (toglist) && !found) {
        if (car (toglist[ix]) == tog)
            found := TRUE;
        else
            ++ix;
    }
    ix;
}

/* enable the command line, and also the browse button  */
function enable(opt, enable_flg, refresh_flg) {
    vcRegion.Enable (opt.Cmdline, enable_flg, refresh_flg);
    if (opt.Browse != nil)
        vcRegion.Enable (opt.Browse, enable_flg, refresh_flg);
}

/* show the right action when the focus cursor moves */
function change_current (opt, new_current) {
    ev := vcEvent.Create (0, 0, ' ', nil, vcRegion.Window (opt.rgn), 
                          opt.rgn, nil);
    _UnshowFocus (opt, ev);
    opt.Current = new_current;
    _ShowFocus (opt, ev);
}

function do_call_on_change (opt)
    if (opt.CallOnChange != nil)
        opt.CallOnChange ();

/*\vscmuserDocpage vcMultOpts.Changed - indicate the value changed
 *
 * DESCRIPTION
 *     This function should be called by a browse function to
 *     indicate that the value changed.
 *
 * SYNOPSIS
 *     vlVoidVT vcMultOpts.Changed (vcRegionVT);
 */
function Changed (rgn)
    do_call_on_change (vcRegion.GetData (rgn));

function _Create (initial, rgn) {

    labs := vcRegion.PrefGet (rgn, "labels");

    opt := struct {
             rgn;
             Current;
             Select;
             ToggleList; /* list of list (toggle, stattext, value) */
             max;
             CmdToggle;
             Cmdline;
            
             Action;
             Browse;
             BrowseFunc;
             CompletionList;
             CallOnChange;

             ArmedToggle;       /* used for handling mouse clicks */
             ArmedArea;
             ArmedToggleData
         };

    browse_proc := vcRegion.PrefGet (rgn, "browse");

    /* one for each label + the cmdline */
    /* this vector actually contains lists that look like this:
     *   (toggle region, label region, label value)
     */
    opt.ToggleList = make_vector (length (labs) + 1);

    /* produce an action function to turn off all other togglers */
    function action(ev, flag) {
        toggle := vcEvent.Region (ev);
        /* turn off all togglers except one that was just turned on */
        foreach (tog; vscript.iter (opt.ToggleList))
            if (car (tog) != toggle) {
                vcToggleButton.Unset (car (tog));
                vcRegion.QueueForRefresh (car (tog));
            }
        /* move the focus cursor, enable right things */
        enable (opt, (opt.CmdToggle == toggle), TRUE);
        newcur := toggle_to_index (opt, toggle);
        change_current (opt, newcur);
        opt.Select = newcur;
        do_call_on_change (opt);        
    }

    /* do this when browse button is pushed */
    function browse(ev)
        browse_proc(opt.rgn, opt.Cmdline);
    opt.BrowseFunc = browse;

    /* set up the first entry: the one with the command line
     * an maybe a browse button 
     */
    t := vcTable.Create ("", length (labs) + 1, 2);
    vcTable.Row (t, vcTable.All, vcTable.Border.Invisible, 1, 1, 3, 
                    vgAlign.MiddleLeft);
    opt.CmdToggle = vcRegion.Create ("", vcToggleButton.fac, FALSE, 
                                    style <- vcToggleButtonStyle.radio,
                                    action <- action); 
    vcRegion.SetBindings (opt.CmdToggle, empty_bindings);
    opt.Cmdline = vcRegion.Create ("", vcCmdline.fac, "", width <- 35);
    vcTable.Add (t, opt.CmdToggle);
    vcCmdline.CallOnChange (opt.Cmdline, 
                          func (a) do_call_on_change (opt););

    declare cmdline_entry;
    if (browse_proc == nil) 
        cmdline_entry = opt.Cmdline;
    else {
       opt.Browse = vcArrow.ButtonCreate ("", browse, Orient.down, 0);
       vcRegion.FocusInhibit (opt.Browse);
       border := vcRegion.Create ("", vcBorder.fac, opt.Browse);
       cmdline_entry := vcTable.Make ("", 1, 2, opt.Cmdline, border);
   }
    vcTable.Add (t, cmdline_entry);
    elem := list (opt.CmdToggle, cmdline_entry, nil);
    opt.ToggleList[0] = elem;

    /* now add the rest of the toggle buttons, with labels */
    index := 0;
    foreach (label; vscript.iter (labs)) {
        tog := vcRegion.Create (label, vcToggleButton.fac, FALSE, 
                                        style <- vcToggleButtonStyle.radio,
                                        action <- action);
        vcRegion.SetBindings (tog, empty_bindings);
        lab := vcRegion.Create ("", vcStaticText.fac, " " + label);
        vcRegion.Font (lab, vgFont.medium);
        elem := list (tog, lab, label);
        opt.ToggleList[++index] = elem;
        vcTable.Add (t, tog);
        vcTable.Add (t, lab);
    }
    
    foreach (tog; vscript.iter (opt.ToggleList))
        vcRegion.FocusInhibit (car (tog));
    vcRegion.FocusInhibit (opt.Cmdline);

    top := vcRegion.Create ("", vcMargin.fac, t, 
                                margin_size <- 2);

    /* if we have to, provide the border or group box */
    if (vcRegion.PrefGet (rgn, "border")) {
         border_lab := vcRegion.PrefGet (rgn, "border_label");
         if (equal (border_lab, ""))
             top = vcRegion.Create ("", vcBorder.fac, top, 
                                     border_style <- vcBorderStyle.EtchedIn);
         else
             top = vcRegion.Create ("", vcGroupBox.fac, top,
                                     label <- border_lab);
    }
    vcRegion.AddChild (rgn, top);


    opt.rgn = rgn;
    opt.max = length (labs);
    opt.Action = action;

    /* now lets deal with the initial value */
    /* try to find a label that matches */
    set (opt, initial, FALSE);

    opt.CompletionList = vcRegion.PrefGet (rgn, "completion");
    opt;
}

/* do the set of the the region */
function set (opt, val, refresh) {
    opt.Select = -1;
    for (i := 1; i <= opt.max && opt.Select == -1; ++i) {
        lab := caddr (index_to_toggle (opt, i));
        if (equal (lab, val))
            opt.Select = i;
    }
    /* punt: put it in the command line */
    if (opt.Select == -1) {
        opt.Select = 0;
        vcCmdline.Set (opt.Cmdline, val);
    }
    for (i := 0; i <= opt.max; ++i)                               
        vcToggleButton._Set (car (index_to_toggle (opt, i)),
                             i == opt.Select);
    opt.Current = opt.Select;
    enable (opt, opt.Select == 0, FALSE);
    if (refresh)
        vcRegion.QueueForRefresh (opt.rgn);
}

/*\vscmuserDocpage vcMultOpts.Set - set the value 
 *
 * SYNOPSIS
 *    vlVoidVT vcMultOpts.Set (vcRegionVT, vlStringVT);
 */
function Set(rgn, val)     
    set (vcRegion.GetData (rgn), val, TRUE);

/* the pixel area taken up by a given element */
function elem_area (opt, ix) {
    elem := index_to_toggle (opt, ix);
    r := vgRect.Create (0, 0, 0, 0);
    vgRect.Max (vcRegion.Area (car (elem)), 
                vcRegion.Area (cadr (elem)), r);
    vgRect.Inset (r, -2, -2);
    r;
}

/* the focus cursor stuff */
function _ShowFocus (opt, ev) {
    if (vcRegion.ShouldShowFocusCursor (opt.rgn)) {
        r := elem_area (opt, opt.Current);
        vcRegion.FocusCursorRender (opt.rgn, r, TRUE);
    }
    if (opt.Current == 0 && opt.Select == 0)
        vcCmdline._FocusIn (opt.Cmdline, ev);
    else
        vcCmdline._FocusOut (opt.Cmdline, ev);
    TRUE;
}

function _UnshowFocus (opt, ev) {
    tog := index_to_toggle (opt, opt.Current);
    r := elem_area (opt, opt.Current);
    vcRegion.FocusCursorRender (opt.rgn, r, FALSE);
    vcCmdline._FocusOut (opt.Cmdline, ev);
    TRUE;
}


/* we changed the selection: update everything.
 * this is done for programmatic or keyboard interfaces 
 */
function select (opt, index) {
    tog := car (index_to_toggle (opt, index));
    vcToggleButton.Set (tog);
    vcRegion.QueueForRefresh (tog);
    ev := vcEvent.Create (0, 0, ' ', nil, vcRegion.Window (tog), tog, nil);
    opt.Action (ev, TRUE);
    if (index == 0)
        vcCmdline._FocusIn (opt.Cmdline, ev);
}

/*\vscmuserDocpage vcMultOpts.Select - set the toggle button
 *
 * DESCRIPTION
 *     This function sets the radio button corresponding to the
 *     index given.
 *
 * SYNOPSIS
 *    vlVoidVT vcMultOpts.Select (vcRegionVT rgn, vlIntVT index)
 */

function Select(rgn, index)
    select (vcRegion.GetData (rgn), index);

/* keyboard interface */
function down (opt, ev)
    if (opt.Current == opt.max)
        change_current (opt, 0);
    else
        change_current (opt, opt.Current + 1);

function up (opt, ev)
    if (opt.Current == 0)
        change_current (opt, opt.max);
    else
        change_current (opt, opt.Current - 1);

/* try to do a completion */
function do_complete (opt) {
    str := vcCmdline.Get (opt.Cmdline);
    newstr := Complete (str, opt.CompletionList);
    if (null (newstr))
        vcWin.VisualBell (vcRegion.Window (opt.rgn));
    else
        vcCmdline.Set (opt.Cmdline, newstr);
}
/* space bar might select a toggle button, do a completion,
 * or insert a space.
 */
function space (opt, ev)
    if (opt.Current == 0 && opt.Select == 0)
        if (null (opt.CompletionList))
            vcCmd.Run (vcCmdline.ins_char_cmd, 
                     vcRegion.GetData (opt.Cmdline), 
                     ev, nil, TRUE);
        else 
            do_complete  (opt);
    else
         select (opt, opt.Current);

/* we implement arming & firing ourselves, so clicking over a label
 * or an inactive commandline still does a selection; you don't have
 * to actually hit the toggle button region itself.
 */
function arm(opt, ev) {
    found := FALSE;
    x := vcEvent.X (ev);
    y := vcEvent.Y (ev);
    for (i := 0; i <= opt.max && !found; ++i) {
        r := elem_area (opt, i);
        if (vgRect.Inside (x, y, r)) {
            found := TRUE;
            vcRegion.SetBindings (opt.rgn, armed_bindings);
            vcRegion.SetGrab (opt.rgn, opt.rgn);
            /* keep track of stuff we will need to use several times */
            opt.ArmedArea = r;
            opt.ArmedToggle = car (index_to_toggle (opt, i));
            opt.ArmedToggleData = vcRegion.GetData (opt.ArmedToggle);
            vcToggleButton.arm (opt.ArmedToggleData, ev);
        }
    }
}

function move (opt, ev)
    if (vgRect.Inside (vcEvent.X (ev), vcEvent.Y (ev), opt.ArmedArea))
        vcToggleButton.arm (opt.ArmedToggleData, ev);
    else
        vcToggleButton.disarm (opt.ArmedToggleData, ev);

function fire (opt, ev) {
    vcRegion.SetGrab (opt.rgn, nil);
    vcRegion.SetBindings (opt.rgn, bindings);
    vcEvent.SetRegion (ev, opt.ArmedToggle);
    if (vgRect.Inside (vcEvent.X (ev), vcEvent.Y (ev), opt.ArmedArea))
        vcToggleButton.fire (opt.ArmedToggleData, ev);
    else
        vcToggleButton.disarm (opt.ArmedToggleData, ev);
}

/* keyboard browse interface */
function do_browse (opt, ev)
    if (opt.Select == 0)
        opt.BrowseFunc (ev);

UpCmd := vcWv.Cmd (fac, up, "up");
DownCmd := vcWv.Cmd (fac, down, "down");
SpaceCmd := vcWv.Cmd (fac, space, "space");
ArmCmd := vcWv.Cmd (fac, arm, "arm");
MoveCmd := vcWv.Cmd (fac, move, "move");
FireCmd := vcWv.Cmd (fac, fire, "fire");
BrowseCmd := vcWv.Cmd (fac, do_browse, "Browse");

vcBind.Set (bindings, '\up', UpCmd);
vcBind.Set (bindings, '\down', DownCmd);
vcBind.Set (bindings, ' ', SpaceCmd);
vcBind.Set (bindings, '\leftdown', ArmCmd);
vcBind.Set (bindings, '?', BrowseCmd);

vcBind.Set (armed_bindings, '\move', MoveCmd);
vcBind.Set (armed_bindings, '\leftup', FireCmd);


/*\vscmuserDocpage vcMultOpts.CreateBrowseFunc - help create browse function
 *
 * DESCRIPTION
 *     This function helps do some of the grunt work in creating
 *     a browse function. 
 *     This function takes as its second argument a function that 
 *     should take a command line region, and return an editor function.
 *     The return value of this function is a function that may be
 *     passed to the browse preference.
 *
 *     When the browse button is pressed, your function will be called
 *     to obtain a region, which will be placed in a window with
 *     a button row: "Ok" "Cancel".
 *
 * EXAMPLE
 *     function create_pick (cline)
 *          vcRegion.Create ("", vcPicklist...);
 *
 *     browse_proc := vcMultOpts.CreateBrowseFunc ("Look for it",
 *                                                 create_pick);
 *     r := vcRegion.Create ("", vcMultOpts.fac, "initial val",
 *                                  browse <- browse_proc);
 *
 * SYNOPSIS
 *     closureVT vcMultOpts.CreateBrowseFunc (vlStringVT title,
 *                                           closureVT proc)
 */
function CreateBrowseFunc (title, regionproc)
    function browse (opt, cline) {
       print (opt);
       rgn := regionproc (cline);
       function ok(ev) {
           str := vcRegion.Get (rgn, vlStringVT);
           vcCmdline.Set (cline, str);
           Changed (opt);
       }
       brow := vcRegion.Create ("", vcButnRow.fac, 
         list (list ("", "Accept", ok, list (close_window <- TRUE)),
               list ("", "Cancel", func(ev)nil;, 
                                       list (close_window <- TRUE))));
   
       parent := vcRegion.Window (cline);

       /* put the developer's region in a window with the buttonrow */
       t := vcTable.Make (title, 2, 1, rgn, brow);
       vcTable.Row (t, 1, vcTable.Border.Visible, 1, 1, 1, 
                      vgAlign.MiddleCenter);
       w := vcWin.Popup (parent, t, rgn, vcWin.NoBindings);
       vcWin.Forward (parent, w);
   }

/*\vscmuserDocpage vcMultOpts.CreatePickListFunc - create browse proc from list
 *
 * DESCRIPTION
 *     Given a list of strings, this function creates a browse function
 *     that may be passed to the browse preference.
 *
 * EXAMPLE
 *     opts := list ("first", "second", "third");
 *     mo := vcRegion.Create ("", vcMultOpts.fac, "first",
 *                 browse <- vcMultOpts.CreatePicklistFunc ("Find it", opts),
 *                 completion <- browse);
 *
 * SYNOPSIS
 *     closureVT vcMultOpts.CreatePicklistFunc (vlStringVT title,
 *                                              vscmVT list)
 */
function CreatePicklistFunc(title, list)
    CreateBrowseFunc (title, func(c) {
        pickdata := nil;
        foreach (elem; vscript.iter (reverse (list)))
             pickdata = cons (cons (elem, elem), pickdata);
        p := vcRegion.Create ("", vcPicklist.fac, pickdata, 
                         will_scroll <- TRUE);
        if (length (list) < 6)
            s := vcPicklist.Scroll ("", p, 3, 5, 10);
        else
            s := vcPicklist.Scroll ("", p, 5, 10, 25);
        s;
    });
   
function _Get(opt, rgn, ts)
    if (opt.Select == 0)
        vcCmdline.Get (opt.Cmdline);
    else
        caddr (index_to_toggle (opt, opt.Select));


/*\vscmuserDocpage vcMultOpts.CallOnChange - set call_on_change action
 *
 * DESCRIPTION
 *     Sets the call on change routine of the region. The function
 *     should take no arguments.
 *
 * SYNOPSIS
 *     vlVoidVT vcMultOpts.CallOnChange (vcRegionVT, closureVT)
 */
function CallOnChange (rgn, proc) {
    opt := vcRegion.GetData (rgn);
    opt.CallOnChange = proc;
}

/* compares s1 and s2 and returns the first index where they don't match
 * starts the search at 'start', which promises that the strings
 * are already known to be equal up to that point
 */
function equal_to (s1, s2, start) {
    declare i;
    for (i = start; 
         i < length (s1) && i < length (s2) && s1[i] == s2[i];
         ++i);
    i;
}


/*\vscmuserDocpage vcMultOpts.Complete - attempt to complete the string
 *
 * DESCRIPTION
 *     This function performs completion in the command line. It is
 *     provided here to allow others to use the same functionality.
 *
 *     It returns the string that matches as much as possible, or nil
 *     if no matching beyond what has been entered is possible.
 *
 * SYNOPSIS
 *     vlStringVT vcMultOpts.Complete (vlStringVT, vscmVT)
 */
function Complete(str, list) {
    declare possibilities;
    l := length (str);
    /* collect all the ones that match */
    foreach (elem; vscript.iter (list))
        if (l <=  length (elem) && equal (str, substring (elem, 0, l)))
             possibilities = cons (elem, possibilities);

    /* shrink back the best_match to whatever everyone can match */
    declare ret;
    if (possibilities != nil) {
        first_match := car (possibilities);
        possibilities = cdr (possibilities);
        best_match := length (first_match);
        while (possibilities != nil && best_match > l) {
            elem := car (possibilities);
            possibilities = cdr (possibilities);
            match := equal_to (elem, first_match, l);
            if (match < best_match)
                best_match = match;
        }
        ret = substring (first_match, 0, best_match);
        if (equal (ret, str))
            ret = nil;
    } 
    ret;
}
         
    
vlPref.Replace (prefs, "labels", vscmVT, "~~labels for common choices", nil);
vlPref.Replace (prefs, "browse", vscmVT, "~~browse button function", nil);
vlPref.Replace (prefs, "border", vlTrueFalseVT, "~~display border", TRUE);
vlPref.Replace (prefs, "border_label", vlStringVT, "~~label with border", "");
vlPref.Replace (prefs, "completion", vscmVT, "~~completetion list", nil);

