/*
 *  $Revision: 2.1.1.15 $  $Date    : 94/06/24 23:36:15 $
 *
 *  Additional GUI utilities for the vdata facility.  This file is loaded from
 *  the vdata.vs file.
 *
 *  Copyright (c) 1995 by Viewlogic Systems, Inc.
 */



/*\Docpage GetDefaultContext -- Get the current VDM session context.
 *
 * DESCRIPTION
 *      Some commands like CreateCollectionCmd and NewObjectCmd require a VDM
 *      context for some of their parameter editing facilities.  Given a
 *      ViewData structure, this function returns its VDM context.
 */
function GetDefaultContext(dv)
    dv.cntx;



/*\Docpage GetDv -- Get the current ViewData facility data structure.
 *
 * DESCRIPTION
 *      Some commands like RestoreCheckpointCmd require a ViewData facility
 *      data structure for its parameter editing facilities.
 */
function GetDv(dv)
    dv;


/*\Docpage GetDefaultBrowseData -- Get data for the vdmLocSelect facility.
 * 
 * DESCRIPTION
 *      The vdmLocSelect facility expects as its initialization data a pair
 *      composed of a VDM session context and a default location, if any.
 *      ViewData can provide both, if a location node is selected.  If it's
 *      not, nil is used.
 */
function GetDefaultBrowseData(dv)
{
    default_location := nil;

    foreach (obj, class; vlSym.Loop (dv.selected_table))
        if (match_type_or_class (vpmLocationVT, obj, class))
        {
            default_location = obj;
            break;
        }

    cons (dv.cntx, default_location);
}



/*\Docpage InsertNewObjectAux -- Insert a new object into display.
 * 
 * DESCRIPTION
 *      A companion function to the vdata.InsertNewObject API function.  It
 *      inserts a new object into the display hierarchy.  The location node is
 *      expanded, if it isn't already.  If the object is of a class that does
 *      not exist yet in this location, a new location class node is created.
 *
 *      If the location has a class filter and the class of the new object is
 *      not in the filter, the user is prompted whether the class should be
 *      added to the filter.
 */
function InsertNewObjectAux(dv, event, location, class, object)
{
    /*
     * Make sure that the location object is expanded.  Otherwise, the
     * vcBrowse.ObjLoop will call the children method anyway, but won't create
     * any children and we will have to call the children method again.
     */
    vcBrowse.ObjExpand (dv.loc_obj_browser, location);

    loc_class := nil;
    foreach (lc, c; vcBrowse.ObjLoop (dv.loc_obj_browser, location))
        if (cadr (lc) == class)
        {
            loc_class = lc;
            break;
        }


    /*
     * If there are no instances of this object type in this location,
     * there won't be a node for the object's class.  Create the class node
     * in such a case, before inserting the collection object.
     */
    if (loc_class == nil)
    {
        loc_class = list (location, class, list (object));
        vcBrowse.ObjInsert (dv.loc_obj_browser, location, loc_class,
                            dv.lclass_class);
        vcBrowse.ObjExpand (dv.loc_obj_browser, loc_class);
        vcBrowse.ObjUnSelect (dv.loc_obj_browser, loc_class);
    }
    else
    {
        vcBrowse.ObjExpand (dv.loc_obj_browser, loc_class);

        /*
         * Don't insert the object if it's already in the hierarchy.  This can
         * happen when we start cold (no location node was expanded and the
         * vcBrowse.ObjLoop function above expands all the location nodes and
         * in the process picks up the presense of a new object and its class
         * node, thus effectively "inserting" them before I get a chance to do
         * it here.
         */
        if (! vcBrowse.ObjIsPresent (dv.loc_obj_browser, object))
            vcBrowse.ObjInsert (dv.loc_obj_browser, loc_class, object,
                                dv.obj_class);
    }

    vcBrowse.ObjSelect (dv.loc_obj_browser, object);

    /*
     * Make sure that default collection (vdmCollection) becomes part of the
     * location's class filter.  If the location has no class filter, ignore
     * the whole thing.
     */
    class_filter := vpm.LocationGetClassFilter (location);
    class_name   := vdm.ClassGetName (class);



    /*\Docpage is_class_filter_member -- Is the class in the class filter?
     * 
     * DESCRIPTION
     *      Remember that a class filter can contain drawer definitions and
     *      those need to be checked recusively as well.
     */
    function is_class_filter_member(class_name, class_filter)
    {
        result := FALSE;

        foreach (class_or_drawer; vscript.list_iter (class_filter))
            if ((drawer := vpm.ProcessGetDrawerDef (class_or_drawer)) != nil)
                result = is_class_filter_member (class_name, drawer);
            else if (string_eq (class_name, class_or_drawer))
            {
                result = TRUE;
                break;
            }
             
        result;
    }


    if (! null (class_filter) &&
        (! is_class_filter_member (class_name, class_filter)))
        vcCmd.Run (AddToClassFilterCmd, dv, event, list (class_name, location),
                   TRUE);
}



/*\Docpage sm_select -- Storage mgr parameter editor.
 *
 * DESCRIPTION
 *      Paramater editor facility to select from the known storage managers.
 */
package sm_select(_Create, _Get)
{
    declare pick, loc_strings;

    /*\Docpage prettyfy -- Make the field name pretty.
     * 
     * DESCRIPTION
     *      Usually, the field name (which is a symbol!) will be composed
     *      of more than one token, each connected by an underline.  This
     *      function breaks the tokens apart, inserts space and
     *      capitalizes each token.  It also inserts a trailing semicolon.
     */
    function prettyfy(sym)
    {
        new_str := "";
        foreach (token; string_loop (symbol2string (sym), " _"))
        {
            token [0] = char_upper (token [0]);
            new_str = new_str + token + " ";
        }

        new_str [rindex (new_str, ' ')] = ':';
        new_str;
    }


    /*\Docpage fill_default_info -- Fill the default location string info.
     * 
     * DESCRIPTION
     *      If the default SM information has been provided, this function
     *      will be called to tokenize the default location string and set the
     *      vcCmdline entries.
     */
    function fill_default_info(no_of_fields, default_sm_str, loc_str_table)
    {
        index := 0;

        foreach (token; string_loop (default_sm_str, " "))
        {
            cmd := vcTable.Get (loc_str_table, index, 1);
            vcCmdline.Set (cmd, token);
            ++index;
        }
    }


    /*
     * You may specify a default storage manager, which will be the last
     * manager chosen.  The default infor must be a list of two elements:
     *
     *      (sm_specification location_string)
     */
    function _Create(default_sm_info, region)
    {
        default_sm_name := car (default_sm_info);
        default_sm_str  := cadr (default_sm_info);

        /*
         * First count the number of storage manager specifications.  If there
         * are none, raise an exception: we can't create a location without at
         * least one storage manager specification.  The number will be useful
         * later on when creating the picklist.
         */
        n_storage_mgrs := 0;

        foreach (print_name, filename, pkg, pkg_name;
                 vpm.ProcessLoopStorageManagers ())
            ++n_storage_mgrs;

        if (n_storage_mgrs == 0)
            dv_err (
 "No storage information found in the `vpm' or `vpmus' initialization files.
Add an AddStorageMgr entry in either file and reinvoke.");


        /*
         * Now we know that we have at least one storage manager
         * specification.  Build a picklist with the storage manager's print
         * names.  If the default SM name was specified and it matches one of
         * the choices, selected it in the picklist.  In addition, build the
         * tables for the location strings.  Those tables are handled through
         * the vcOverlay facility.
         */
        pick = vcRegion.Create ("SM", vcPicklist.fac, nil,
                                rows    <- 1,
                                columns <- n_storage_mgrs);

        loc_strings = vcRegion.Create ("types", vcOverlay.fac,
                                       vgAlign.UpperLeft);
        
        index    := 0;
        sm_name  := nil;
        str_name := nil;

        foreach (print_name, filename, pkg, pkg_name;
                 vpm.ProcessLoopStorageManagers ())
        {
            type := vpm.ProcessGetLocationType (filename, pkg, pkg_name);

            no_of_fields := 0;
            foreach (name, type; struct_field_iter (type))
            {
                str_name = symbol2string (name);
                if (str_name[0] != '_')
                    ++no_of_fields;
            }

            loc_str_table := vcTable.Create ("loc_strings", no_of_fields, 2);

            foreach (name, type; struct_field_iter (type))
            {
                str_name = symbol2string (name);
                if (str_name[0] != '_')
                {
                    vcTable.Add (loc_str_table,
                                 vcRegion.Create ("", vcLabel.fac,
                                                  prettyfy (name)));
                    vcTable.Add (loc_str_table,
                                 vcRegion.Create ("", vcCmdline.fac, ""));
                }
            }

            /*
             * Each location table must be added to the vcOverlay facility, so
             * that it can figure out their common size.
             */
            vcRegion.AddChild (loc_strings, loc_str_table);

            vcPicklist.Add (pick, print_name,
                            list (loc_str_table,
                                  list (filename, pkg, pkg_name)));

            /*
             * Check if the default SM name hasn't been specified.  If so,
             * pick off the default info and set the picklist.
             */
            sm_name = (filename != nil ? filename : pkg_name);
            if ((default_sm_name != nil) &&
                string_eq (default_sm_name, sm_name))
            {
                vcPicklist.SetPick (pick, index);
                fill_default_info (no_of_fields, default_sm_str,
                                   loc_str_table);
                vcOverlay.SetCurrent (loc_strings, loc_str_table);
            }

            ++index;
        }

        /*
         * If there was no default SM info, choose the first SM specification
         * as the default one.
         */
        if (vcOverlay.GetCurrent (loc_strings) == nil)
        {
            vcOverlay.SetCurrent (loc_strings,
                                  cadr (vcPicklist.Get (pick, 0, 0)));
            vcPicklist.SetPick (pick, 0);
        }


        /*
         * Selecting the SM specifications just flips through the overlay
         * regions.
         */
        function show_location_string(pick_data, is_on)
            if (is_on)
                vcOverlay.SetCurrent (loc_strings, car (pick_data));


        vcPicklist.Notify (pick, show_location_string);


        table := vcTable.Create ("type", 2, 1);
        vcTable.Add (table, pick);
        vcTable.Add (table, loc_strings);

        vcRegion.AddChild (region, table);
        table;
    }


    /*\Docpage _Get -- Core's method.
     *
     * DESCRIPTION
     *      The value returned is composed from the the picklist entries and
     *      the values typed by the user in the command line editors
     *
     *      If any of the location values is empty, throw an exception.
     */
    function _Get(table, region, ts)
    {
        declare rows, cols, str;
        loc_str_table := vcOverlay.GetCurrent (loc_strings);

        if (loc_str_table != nil)
        {
            vcTable.Dimensions (loc_str_table, &rows, &cols);

            loc_str    := "";
            first_time := TRUE;

            for (i := 0; i < rows; ++i)
            {
                str = vcRegion.Get (vcTable.Get (loc_str_table, i, 1), ts);

                if ((str == nil) || (string_length (str) == 0))
                    dv_err ("You must provide all the required information.");

                if (first_time)
                {
                    loc_str    = str;
                    first_time = FALSE;
                }
                else
                    loc_str = loc_str + " " + str;
            }
                
            sm_info := cadr (vcRegion.Get (pick, ts));

            cons (loc_str, sm_info);
        }
        else
            nil;
    }
}

sm_select_fac := vcFac.CreateFromPackage ("sm_selector", vscmVT, sm_select);



/*\Docpage class_filter_select -- Editor facility for class filter.
 *
 * DESCRIPTION
 *      The original vcListEdit facility wasn't so hot to use and users
 *      requested that a list of the available class definitions be also
 *      provided so that they don't have to remember how the class names were
 *      spelled.
 */
package class_filter_select(_Create, _Get)
{
    function italicize(str) "\011" + str;
    function bold(str) "\002" + str;
    function un_italicize(str)
        if ((string_length (str) > 0) && (str[0] == '\011'))
            substring (str, 1, 0);
        else
            str;

    /*
     * Build the region for the new facility.  The region is a 2-2 table.  The
     * top row contains headers.  The bottow row contains two picklists.  On
     * the left is the picklist with the available class definitions.  On the
     * right is the picklist for new class filter.
     */
    function _Create(dflt, region)
    {
        dv         := get_dv ();
        class_list := nil;
        new_filter := vlSym.StrCreate (11);

        /*
         * Collect the class names and sort them alphabetically into a list of
         * dotted pairs.  Also, don't forget about the class drawers.
         */
        foreach (class_file, name; vdm.CntxLoopClassFiles (dv.cntx))
            foreach (class, class_name; vdm.ClassFileLoop (class_file))
                class_list = cons (cons (class_name, class_name), class_list);

        foreach (drawer_name, drawer_def; vpm.ProcessLoopDrawers ())
        {
            drawer_name = italicize (drawer_name);
            class_list  = cons (cons (drawer_name, drawer_name), class_list);
        }

        /*
         * Sort the classes.  Note that we need to ignore the format character
         * which will make all the drawers be listed first otherwise.
         */
        function compare_classes(class1, class2)
            string_le (un_italicize (car (class1)),
                       un_italicize (car (class2)));

        class_list = vscript.Sort (class_list, compare_classes);


        /*
         * Build a picklist listing all the class definitions available in
         * this context.
         */
        old_pick := vcRegion.Create ("old_pick", vcPicklist.fac, class_list,
                                     enhanced_notify    <- TRUE,
                                     exclusive          <- FALSE,
                                     std_select         <- FALSE,
                                     formatted          <- TRUE,
                                     will_scroll        <- TRUE);
        old_pick  = vcPicklist.Scroll ("old_pick", old_pick, 3, 5, 0);

        old_msg  := vcRegion.Create ("old_msg", vcStaticText.fac,
                                     bold ("Available classes"),
                                     align          <- vgAlign.MiddleLeft,
                                     formatted      <- TRUE,
                                     recessed_text  <- TRUE);

        vcRegion.FocusInhibit (old_msg);


        /*
         * Build the picklist where the class filter will show up.
         */
        n_rows := quotient (length (class_list), 4);
        new_pick := vcRegion.Create ("new_pick", vcPicklist.fac, nil,
                                     columns            <- 1,
                                     enhanced_notify    <- TRUE,
                                     exclusive          <- TRUE,
                                     formatted          <- TRUE,
                                     rows               <- n_rows,
                                     will_scroll        <- TRUE);
        new_pick  = vcPicklist.Scroll ("new_pick", new_pick, 3, 5, 0);

        new_msg  := vcRegion.Create ("old_msg", vcStaticText.fac,
                                     bold ("Class filter"),
                                     align          <- vgAlign.MiddleLeft,
                                     formatted      <- TRUE,
                                     recessed_text  <- TRUE);

        vcRegion.FocusInhibit (new_msg);

        /*
         * Check if there's no default facility data.  If so, they constitute
         * the existing class filter.  Italicize drawer names.
         */
        foreach (class; vscript.list_iter (dflt))
        {
            if (!null (vpm.ProcessGetDrawerDef (class)))
                class = italicize (class);

            vcPicklist.Add (new_pick, class, class);
            vcPicklist.SetByString (old_pick, class);
            vlSym.Store (new_filter, class, TRUE);
        }


        /*
         * Only the old picklist has a notification function.  If the
         * selection is new, it's added to the new picklist.  If it's already
         * there, it's removed.
         */
        do_select_from_new := TRUE; /* mutual recursion avoidancer */

        function select_from_old(data, on_or_off, index)
        {
            in_filter := ! null (vlSym.Fetch (new_filter, data));

            if (on_or_off && !in_filter)
            {
                vlSym.Store (new_filter, data, TRUE);
                vcPicklist.Add (new_pick, data, data);
            }

            else if (!on_or_off && in_filter)
            {
                do_select_from_new = FALSE;
                vcPicklist.SetByString (new_pick, data);
                do_select_from_new = TRUE;
            }
        }

        vcPicklist.Notify (old_pick, select_from_old);


        /*
         * Now, just for the good measure, add a selection function for the
         * new picklist, just so that the user can click and remove an entry
         * in the class filter.  I saw a need for it when I removed some
         * class drawers, but couldn't edit locations that contained those
         * drawers in their class filter.
         */
        function select_from_new(data, on_or_off, index)
            if (on_or_off)
            {
                vcPicklist.DeleteRows (new_pick, index, 1);
                vlSym.Remove (new_filter, data);

                if (do_select_from_new)
                    vcPicklist.SetByString (old_pick, data);
            }

        vcPicklist.Notify (new_pick, select_from_new);


        table := vcTable.Create ("class_filter", 2, 2);
        vcTable.Add (table, old_msg);
        vcTable.Add (table, new_msg);
        vcTable.Add (table, old_pick);
        vcTable.Add (table, new_pick);

        vcRegion.AddChild (region, table);

        table;
    }


    /*
     * The value of the selector is the contents of the new picklist.
     */
    function _Get(table, region, ts)
    {
        declare rows, cols, class_name;

        new_pick := vcTable.Get (table, 1, 1);
        filter   := nil;

        vcPicklist.Dimensions (new_pick, &rows, &cols);

        for (i := 0; i < rows; ++i)
            filter = cons (un_italicize (car (vcPicklist.Get (new_pick,
                                                              i, 0))),
                           filter);

        filter;
    }
}

class_filter_fac := vcFac.CreateFromPackage ("class_filter_select", vscmVT,
                                             class_filter_select);



/*\Docpage OneCollectionSelect -- Choose between multiple collections.
 * 
 * DESCRIPTION
 *      When working with collections, either adding or removing their
 *      members, it's possible that a member might be another collection.  In
 *      such a case, ViewData has no clue which collection object is being
 *      modified, because it works off of the list of selected nodes.  This
 *      facility is used as a parameter editor facility for the
 *      AddToCollection and RemoveFromCollection commands.  If more then one
 *      collection object is selected, the user is presented with a tree of
 *      the selected collections only and asked to choose which one should be
 *      modified, i.e. to which does the add or remove command applies.
 */
package OneCollectionSelect(_Create, _Get)
{

    function _Create(dv, region)
    {
        last_v  := nil;


        function sort_objects(obj1, obj2)
            string_le (vdm.ObjGetName (obj1), vdm.ObjGetName (obj2));


        /*
         * Find all the collections and order them.
         */
        composable_only := vcRegion.PrefGet (region, "composable_only");
        obj_list        := nil;

        foreach (obj; get_versioned_objects (dv))
            if (vdm.ObjIsCollection (obj) &&
                (! composable_only || is_collection_composable (obj)))
                obj_list = cons (obj, obj_list);

        obj_list = vscript.Sort (obj_list, sort_objects);


        size  := vgSize.Create (100, 100, 100, 250, 10000, 10000);

        prefs := nil;
        prefs  = cons (compass          <- vgCompass.East, prefs);
        prefs  = cons (exclusive        <- TRUE, prefs);
        prefs  = cons (even_indent      <- TRUE, prefs);
        prefs  = cons (follow_focus     <- TRUE, prefs);
        prefs  = cons (group            <- FALSE, prefs);
        prefs  = cons (layout           <- "Narrow", prefs);
        prefs  = cons (level_space      <- 20, prefs);
        prefs  = cons (interactive      <- TRUE, prefs);
        prefs  = cons (scalable         <- FALSE, prefs);
        prefs  = cons (sibling_space    <- 1, prefs);
        prefs  = cons (size             <- size, prefs);
        prefs  = append (vcRegion.PrefCopyChanged (dv.loc_obj_browser, 
                                                   vcTree.fac),
                         prefs);

        tree := vcRegion.Create1 ("Tree", vcTree.fac, nil, prefs);

        /* !!!  These are hard coded. This is lame, but I don't know how
         *      or where to get the colors from vcBrowse.
         */
        l_color := vgColor.Cyan;
        c_color := vgColor.Green;
        o_color := vgColor.Yellow;

        l_label := nil;     c_label := nil;     o_label := nil;
        l_name  := nil;     c_name  := nil;     o_name  := nil;

        l_table := vlSym.StrCreate (11); vlSym.Dynamic (l_table, 2, 2);
        c_table := vlSym.StrCreate (11); vlSym.Dynamic (l_table, 2, 2);

        v     := nil;
        vlist := nil;
        loc   := nil;
        cls   := nil;

        foreach (obj; vscript.list_iter (obj_list))
        {

            /* !!!  There must be some way to smash this into less code. */
            /* Location. */
            l_name = vpm.LocationGetAlias (vdm.ObjGetLocation (obj));

            loc = vlSym.Fetch (l_table, l_name);
            if (null (loc))
            {
                v = vcTree.VrtxCreate1 ("Loc Vertex", vcTree.VrtxType.Text, 
                                        l_name, nil, c_table,
                                        list (raster_font <- vgFont.small,
                                              text_color  <- l_color));

                vlSym.Store (l_table, l_name, v);
                vlist = append (vlist, cons (v, nil));
                loc = v;
            }

            else
                c_table = vcTree.VrtxGetItem (loc);
                

            /* Class. */
            c_name = vdm.ClassGetName (vdm.ObjGetClass (obj));

            cls = vlSym.Fetch (c_table, c_name);
            if (null (cls))
            {
                v = vcTree.VrtxCreate1 ("Class Vertex", vcTree.VrtxType.Text, 
                                        c_name, loc, nil,
                                        list (raster_font <- vgFont.small,
                                              text_color  <- c_color));

                vlSym.Store (c_table, c_name, v);
                cls = v;
            }


            /* Object. */
            v = vcTree.VrtxCreate1 ("Obj Vertex", vcTree.VrtxType.Text, 
                                    vdm.ObjGetName (obj) + " " +
                                    obj_version (obj), cls, obj,
                                    list (raster_font <- vgFont.small,
                                          text_color  <- o_color));
        }
        vcTree.SetRoot (tree, vlist);



        /* Start with the location overlay, because that's the most often used
         * display tree.
         */
        function select(v, is_on)
            if (is_on)
            {
                if (istype (vcTree.VrtxGetItem (v), vdmObjVT))
                    last_v = v;
                else
                {
                    if (!null (last_v))
                    {
                        vcTree.SelectVrtx (tree, last_v);
                        vcRegion.QueueForRefresh (tree);
                    }
                    else
                        vcTree.DeselectVrtx (tree, v);
                }
            }
        vcTree.CallOnSelect (tree, select);


        /* Create instructions. */
        msg :=
           "Multiple collections selected.\nSelect the one you want modified.";
        directions  := vcRegion.Create ("directions", vcStaticText.fac, msg,
                                        align         <- vgAlign.MiddleCenter,
                                        recessed_text <- TRUE);
        vcRegion.Font (directions, vgFont.mediumb);


        /* Create table. */
        table := vcTable.Create ("Select Collection", 2, 1);
        vcTable.Add (table, directions);
        vcTable.Add (table, tree);


        vcRegion.AddChild (region, table);
        tree;
    }


    function _Get(tree, region, ts)
    {
        vertex := vcTree.SelectedVrtx (tree);

        if (null (vertex))
            dv_err ("No collection has been selected.");

        vcTree.VrtxGetItem (vertex);
    }
}


OneCollectionSelectFac :=
    vcFac.CreateFromPackage ("OneCollectionSelect", vcRegionVT,
                             OneCollectionSelect);

vlPref.Add (OneCollectionSelect.prefs, "composable_only", vlTrueFalseVT,
            "~", FALSE);



/*\Docpage relation_select -- Parameter editor for selecting relations.
 * 
 * DESCRIPTION
 *      Used by the AddRelationshipCmd to provide a list of relationships that
 *      can be added to the given object.
 */
package relation_select(_Create, _Get)
{
    function _Create(dflt_relation, region)
    {
        dv          := get_dv ();
        obj         := get_one_obj (dv, vdmObjVT, "AddRelationship");
        relations   := object_spare_relations (obj);
        relations    = vscript.Sort (relations, string_lei);
        pl          := vcRegion.Create ("", vcPicklist.fac, nil,
                                        columns <- length (relations));
        index       := 0;

        foreach (rel; vscript.list_iter (relations))
        {
            vcPicklist.Add (pl, rel, rel);
            if (string_eq (dflt_relation, rel))
                vcPicklist.SetPick (pl, index);

            ++index;
        }

        if (index == 1)
            vcPicklist.SetPick (pl, 0);

        vcRegion.AddChild (region, pl);
        pl;
    }


    function _Get(pl, region, ts)
        vcRegion.Get (pl, ts);
}

relation_select_fac :=
    vcFac.CreateFromPackage ("relation_selector", vscmVT, relation_select);



/*\Docpage ClassSelectFac -- Class type selection facility.
 *
 * DESCRIPTION
 *      When a user wants to instantiate a collection, they need to select the
 *      type of collection that they are interested in.  That would be simple
 *      if that's all that was to it.  However, beside selection the type, the
 *      user then needs to specify all the required attributes needed to
 *      instantate a collection object of the given type.  That's where this
 *      facility comes in handy.
 *
 *      It consists of two main regions: a picklist and an overlay region.
 *      The picklist lists all the available collection class types.  The
 *      overlay displays tables which lists all the required attributes for a
 *      given collection class type.  The link between the picklist and the
 *      overlay is that the picklist's element data are the attribute tables
 *      displayed by the overlay.
 *
 *      The _Get method returns a two element list.  The first element is the
 *      class Id selected by the user.  The second list is a list of attribute
 *      and value pairs in format ready to be passed in to the vdm.ObjCreate
 *      function.
 *
 * FACILITY DATA
 *      vdmCntxVT - a VDM session context
 *
 * PREFERENCES
 *      class_filter - closureVT    - predicate function that serves as a
 *                                    class filter.  If it returns TRUE, the
 *                                    class is included.
 */
package ClassSelect(_Create, _Get)
{
    /*
     * These variables are used by both the _Create and the _Get functions.
     * They hold the main picklist and overlay regions of this facility.
     */
    declare type_pick, attr_pick;


    function _Create(vdm_cntx, region)
    {
        class_filter := vcRegion.PrefGet (region, "class_filter");

        /*
         * First, build a list of the available collection types.  If there
         * are no collection types available, raise an error.
         */
        type_list := nil;

        foreach (bunch, bunch_name; vdm.CntxLoopClassFiles (vdm_cntx))
            foreach (class, class_name; vdm.ClassFileLoop (bunch))
                if ((class_filter == nil) || class_filter (class))
                    type_list = cons (cons (class_name, class), type_list);

        if (null (type_list))
          dv_err ("There are no appropriate class types to form a selection.");


        /*
         * Sort the type list alphabetically.
         */
        function type_list_eq(elem1, elem2)
            string_le (car (elem1), car (elem2));

        type_list = vscript.Sort (type_list, type_list_eq);


        /*
         * If there are 10 or less class types, arrange them horizontal, like
         * radio buttons.  If there are more, create a scrollable picklist.
         */
        class_count := length (type_list);

        type_pick = vcRegion.Create ("Types", vcPicklist.fac, nil,
                                     rows           <- class_count,
                                     columns        <- 1,
                                     exclusive      <- TRUE,
                                     will_scroll    <- TRUE);

        type_pick = vcPicklist.Scroll ("Types", type_pick, 5, 10, 15);

        /*
         * Second, create an overlay space to hold the table which will
         * contain the prompts for the class attribute values.
         */
        attr_pick = vcRegion.Create ("Attributes", vcOverlay.fac,
                                     vgAlign.UpperLeft);

        /*
         * Now that we have placeholders for both the class type picklist and
         * the attribute overlay, let's populate them.  For each class type in
         * the type_list, extract its required arguments, build a table
         * composed of a label (attribute name) and command line (attribute
         * value) and store the label as a picklist value.  The class name
         * will serve as the picklist visual.
         */
        foreach (pair; vscript.list_iter (type_list))
        {
            class_name := car (pair); class := cdr (pair);
            attributes := vdm.ClassGetAttrList (class);

            attr_table := vcTable.Create ("AttrTable", length (attributes), 2);

            foreach (attr; vscript.list_iter (attributes))
            {
                label   := vcRegion.Create ("", vcLabel.fac, attr);
                cmdline := vcRegion.Create ("", vcCmdline.fac, "");
                vcTable.Add (attr_table, label);
                vcTable.Add (attr_table, cmdline);
            }

            /*
             * Now we have a single collection class and a table listing all
             * of its required attributes.
             */
            vcPicklist.Add (type_pick, class_name, cons (class, attr_table));
            vcRegion.AddChild (attr_pick, attr_table);
        }

        /*
         * Choose the first class type as default.
         */
        vcOverlay.SetCurrent (attr_pick,
                              cddr (vcPicklist.Get (type_pick, 0, 0)));
        vcPicklist.SetPick (type_pick, 0);


        /*
         * After all the tables and regions have been defined, register a
         * notification function to switch the attribute tables, as the user
         * changes the picklist.
         */
        function show_attributes(picklist_data, is_on)
            if (is_on)
                vcOverlay.SetCurrent (attr_pick, cdr (picklist_data));

        vcPicklist.Notify (type_pick, show_attributes);


        table := vcTable.Create ("Attributes", 2, 1);
        vcTable.Add (table, type_pick);
        vcTable.Add (table, attr_pick);

        vcRegion.AddChild (region, table);
        table;
    }



    /*\Docpage function _Get -- The "get" method.
     * 
     * DESCRIPTION
     *      Returns a list of two elements: the collection class Id and a
     *      list of attributes and their values arranged in such a way that
     *      they can be passed directly to the vdm.ObjCreate function.
     */
    function _Get(table, region, ts)
    {
        declare rows, cols, attributes, attr, value;

        attr_table := vcOverlay.GetCurrent (attr_pick);

        vcTable.Dimensions (attr_table, &rows, &cols);

        for (i := 0; i < rows; ++i)
        {
            attr  = vcRegion.Get (vcTable.Get (attr_table, i, 0), vlStringVT);
            value = vcRegion.Get (vcTable.Get (attr_table, i, 1), ts);

            if ((value == nil) || (string_length (value) == 0))
                dv_err ("You must provide all the required information.");

            attributes = cons (cons (string2symbol (attr), value), attributes);
        }

        list (car (vcRegion.Get (type_pick, ts)), attributes);
    }
}



ClassSelectFac :=
    vcFac.CreateFromPackage ("ClassSelector", vscmVT, ClassSelect);


vlPref.Replace (ClassSelect.prefs, "class_filter", vscmVT,
                "Class filter predicate", nil);



/*\Docpage VersionSelect -- Parameter editor facility for object versions.
 *
 * DOCPAGE
 *      Used by the RestoreCheckpoint command to display a list of version
 *      tags on a particular object.
 */
package VersionSelect(_Create, _Get)
{
    function _Create(dv, region)
    {
        obj  := get_one_obj (dv, vdmObjVT, "VersionSelector");
        <log, locks, versions> := vdm.ObjQuery (obj, FALSE, nil);

        pick := vcRegion.Create ("Versions", vcPickEdit.fac, versions,
                                 height     <- 10,
                                 exclusive  <- TRUE,
                                 scroll     <- TRUE);

        /*
         * By default, the first entry in the versions list will be selected
         * and highlighted.  Because this facility can be used as an editor
         * for secondary parameters, I don't want any selections to be present
         * when it's instantiated.
         */
        vcPicklist.DeselectAll (pick);

        vcRegion.AddChild (region, pick);
        pick;
    }


    function _Get(pick, region, ts)
        vcRegion.Get (pick, ts);
}


VersionSelectFac :=
    vcFac.CreateFromPackage ("VersionSelector", vscmVT, VersionSelect);



/*\Docpage verify_versions -- Verify objects' versions.
 * 
 * DESCRIPTION
 *      Note that with storage managers that support versioning (a requirement
 *      really), it is possible for a user to blow away his or her working
 *      version data during a fetch or a check out operation.  This function
 *      attempts to warn the user when it detects that an object has a working
 *      version that does not match the default storage version.
 *
 *      This function is used by the Lock, Checkout and CheckoutNoLock
 *      commands.
 */
function verify_versions(dv, event, op_name, op, args...)
{
    /*
     * This function will be executed if there are no version conflicts, or
     * when the user presses the Ok button authorizing the overwrite.
     */
    function do_it()
    {
        sm_args := list (dv, op);

        foreach (arg; vscript.list_iter (args))
            sm_args = cons (args, sm_args);

        sm_args = reverse_destructive (sm_args);

        vcWin.WithBusy (vcEvent.Window (event),
                        func()
                        apply (perform_sm_action, sm_args););
    }


    /*
     * Gather all the selected objects whose working version is not the same
     * as the default storage version.  Ignore all objects with the version
     * set to vdm.VersLatest: they match whatever is in the storage area.
     */
    obj_list := nil;

    foreach (obj; get_versioned_objects (dv))
    {
        working_version := vdm.ObjGetWorkingVersion (obj);
        storage_version := vdm.ObjGetVersion (obj);

        if ((! string_eq (storage_version, vdm.VersLatest))     &&
            (! string_eq (working_version, storage_version)))
            obj_list = cons (obj, obj_list);
    }


    /*
     * If the list with different versions is not empty, display them in a
     * picklist and ask the user whether the operation should continue.
     */
    if (obj_list == nil)
        do_it ();
    else
    {
        /*
         * First, build the message region, which will tell the user what's
         * going on.
         */
        str := sprintf ("The following objects have different working and storage versions.\nContinue with the %s operation?", op_name);

        msg  := vcRegion.Create ("msg", vcStaticText.fac, str,
                                 align          <- vgAlign.MiddleCenter,
                                 recessed_text  <- TRUE);


        /*
         * Next, build three picklists.  The first will list the object's
         * verbose name.  The second will list the working version number.
         * The third will list the storage version number.
         */
        no_objects := length (obj_list);

        pick1 := vcRegion.Create ("pick1", vcPicklist.fac, nil,
                                  columns       <- 1,
                                  exclusive     <- FALSE,
                                  rows          <- no_objects,
                                  will_scroll   <- TRUE);
        pick2 := vcRegion.Create ("pick2", vcPicklist.fac, nil,
                                  columns       <- 1,
                                  exclusive     <- FALSE,
                                  rows          <- no_objects,
                                  will_scroll   <- TRUE);
        pick3 := vcRegion.Create ("pick3", vcPicklist.fac, nil,
                                  columns       <- 1,
                                  exclusive     <- FALSE,
                                  rows          <- no_objects,
                                  will_scroll   <- TRUE);

        /*
         * None of the picklists should be in the focus ring.
         */
        vcRegion.FocusInhibit (pick1);
        vcRegion.FocusInhibit (pick2);
        vcRegion.FocusInhibit (pick3);

        foreach (obj; vscript.list_iter (obj_list))
        {
            vcPicklist.Add (pick1, vdm.ObjGetVerboseLogicalName (obj), obj);
            vcPicklist.Add (pick2, vdm.ObjGetWorkingVersion (obj), obj);
            vcPicklist.Add (pick3, vdm.ObjGetVersion (obj), obj);
        }

        /*
         * Give each picklist a title.
         */
        msg1 := vcRegion.Create ("msg1", vcStaticText.fac, "Object Name",
                                 align  <- vgAlign.MiddleCenter);
        msg2 := vcRegion.Create ("msg1", vcStaticText.fac, "Working",
                                 align  <- vgAlign.MiddleLeft);
        msg3 := vcRegion.Create ("msg1", vcStaticText.fac, "Storage",
                                 align  <- vgAlign.MiddleLeft);

        pick_table := vcTable.Make ("pick table", 2, 3);
        vcTable.Add (pick_table, msg1);
        vcTable.Add (pick_table, msg2);
        vcTable.Add (pick_table, msg3);
        vcTable.Add (pick_table, pick1);
        vcTable.Add (pick_table, pick2);
        vcTable.Add (pick_table, pick3);

        /*
         * Make all three picklists scrollable at the same time.
         */
        pick_table = vcTable.Scroll ("pick_table", pick_table, pick_table,
                                     1, 3, 5);

        vcTable.Row (pick_table, vcTable.All, vcTable.Border.Visible, 2, 3, 4,
                     vgAlign.MiddleCenter);

        buttons := vcRegion.Create ("buttons", vcButnRow.fac,
                                    list (list ("ok", "Continue", do_it,
                                                list (close_window <- TRUE)),
                                          list ("cancel", "Cancel",
                                                vcWin.wclose, nil)));

        table := vcTable.Make ("Verify Versions", 3, 1);
        vcTable.Add (table, msg);
        vcTable.Add (table, pick_table);
        vcTable.Add (table, buttons);

        vcWin.Create (table, vcWin.NoCmdline + vcWin.NoMacros +
                      vcWin.NoStatus);
    }
}
