
/*
   File:     symxlate.vs

   Facility: Synopsys Integration

   Author:   Steve Paluzzi

   Date:     June 1, 1994

   Drives syms2sch tool to translate a library's symbols onto one schematic,
   move the schematic into Synopsys via edif, and compare it against their 
   technology file.

Rev    Who    Date     Modification
---    ---    ----     ------------
1.0    SDP   6/01/94   Initial release


*/

flat_eval(TRUE);
set_stdout(TRUE);


/* Globals */
declare arg_count;
declare cmd_count;

/* Function to create a label string (the label (prompt) of the entry line) */
function label(str) vcRegion.Create1(str, vcLabel.fac, str, nil);


/* Function to create a command entry line (the blank line to accept input) */
function cmdline(title, default, size)
{
   vcRegion.Create1(title, vcCmdline.fac, default, list(cons(`max_width, size),
                                                        cons(`width, 8),
                                                        cons(`macro_on, TRUE),
                                                        cons(`max_size, 64)));
}


/* Function to count the number of args on the given input line */
function count_args(str_list)
{
   /* Count the number of alias arguments supplied */
   count := 0;
   foreach(i; vscript.list_iter(str_list)) {
      if (i != nil) {
         /* printf(i);
         printf("\n"); */
         count = count + 1;
         /* printf("alias is: " + i);
         printf("\n"); */
       }
   }
   /* printf("The value of count is %s\n",count); */
   count;
}


/* Function to write output to stdout */
function out_handler(str)
{
   /* printf("The system command output is\n%s",str); */
   printf("%s",str);
}


/* Function to check the status after each command and abort if unsuccessful */
/* Only print a "program completed" message for the last command, #3 (or #2 if 
   no compare_lib being performed */
function exit_handler(mode,code)
{
   cmd_count = cmd_count + 1;
   /* printf("%s\n", cmd_count); */
   if (code != 0) {
      s := sprintf("Error: symxlate terminated");
      printf("symxlate terminated\n\n");
      error(s, nil);
   } else {
      if (cmd_count == 3)
         printf("\nsymxlate completed\n\n");
   }
}


/* Function to generate the synopsys.script file */
function gen_synopsys_script(orig_outname_str, techfile_list)
{
   fp := fopen("synopsys.script","w");
   fputline(fp,"read_lib "+orig_outname_str+".edif -format edif -symbol "
                          +orig_outname_str+".slib");
   fputchar(fp,'\n');
   fputline(fp,"write_lib "+orig_outname_str+"_LIB -output "
                           +orig_outname_str+".sdb");
   fputchar(fp,'\n');

   /* Loop to compare this lib against each tech file */
   if (arg_count >= 1) {   /* Do compare_lib only if a techfile was specified */
      foreach(i; vscript.list_iter(techfile_list)) {
         if (i != nil) {
            fputline(fp,"read "+i+".db");
            fputchar(fp,'\n');
            fputline(fp,"compare_lib "+orig_outname_str+"_LIB "+i);
            fputchar(fp,'\n');
            /* Flush the lib file here if possible */
         }
      }
   }
   fputline(fp,"quit");
   fputchar(fp,'\n');
   fclose(fp); 
}


/* Function for actions to take upon clicking accept */
/* It checks that all necessary info was added, then performs syms2sch, symbol
   library transfer, writes out a Synopsys script and executes it */
function accept(d,e)
{
   declare token;
   declare alias_list;
   declare outname_list;
   declare techfile_list;
   cmd_count = 0;   /* Must be reset each time accept is entered */
   primdir := FALSE;
   aliasdir := FALSE;

   /******** Get library alias info ********/
   libtype_reg := vcTable.Get(d.FILES, 0, 1);
   if (vcPicklist.IsSet(libtype_reg, 0))
      primdir = TRUE;
   if (vcPicklist.IsSet(libtype_reg, 1))
      aliasdir = TRUE;
   if ((!primdir) && (!aliasdir))
      error("You must choose Primary Directory and/or Aliased Library.", nil);

   if (aliasdir) {
      alias := vcRegion.Get(vcTable.Get(d.FILES, 1, 1), vlStringVT);
      orig_alias_str := alias;   /* Save the original */
      while(alias != nil) {  /* Builds a list of the aliases in reverse order */
         alias := string_token(alias, &token, " ");
         alias_list := cons(token, alias_list);
      }
      alias_list := reverse(alias_list);  /* Put back original order */
      /* printf("The value of alias_list is %s\n", alias_list); */

      /* Check that there is at least one alias specified */
      arg_count = count_args(alias_list);
      if (arg_count < 1)
         error("You must specify at least one library alias.", nil);
   }

   /******** Get outname info ********/
   outname := vcRegion.Get(vcTable.Get(d.FILES, 2, 1), vlStringVT);
   orig_outname_str := outname;   /* Save the original */
   while(outname != nil) {   /* Builds a list of outnames in reverse order */
      outname := string_token(outname, &token, " ");
      outname_list := cons(token, outname_list);
   }
   outname_list := reverse(outname_list);  /* Put back original order */
   /* printf("The value of outname_list is %s\n", outname_list); */

   /* Check that there is exactly one outname specified */
   arg_count = count_args(outname_list);
   if (arg_count < 1)
      error("You must specify the output library name.", nil);
   if (arg_count > 1)
      error("You must specify only one output library name.", nil);

   /******** Get technology library info ********/
   techfile := vcRegion.Get(vcTable.Get(d.FILES, 3, 1), vlStringVT);
   orig_techfile_str := techfile;   /* Save the original */
   while(techfile != nil) {  /* Builds list of the techfiles in reverse order */
      techfile := string_token(techfile, &token, " ");
      techfile_list := cons(token, techfile_list);
   }
   techfile_list := reverse(techfile_list);  /* Put back original order */
   /* printf("The value of techfile_list is %s\n", techfile_list); */

   /* Check that there is at least one techfile specified */
   arg_count = count_args(techfile_list);
   if (arg_count < 1)
      printf("No technology library specified - will skip compare_lib in dc_shell\n\n");

   /******** Build and execute the command lines ********/
   /* Run syms2sch on primary and/or aliased libraries */
   if (primdir && aliasdir)
      syms2sch_cmd := "syms2sch -primary -alias " +orig_alias_str
                                         +" -outname "+orig_outname_str;
   else if (primdir)
      syms2sch_cmd := "syms2sch -primary -outname "+orig_outname_str;
   else
      syms2sch_cmd := "syms2sch -alias " +orig_alias_str
                                         +" -outname "+orig_outname_str;

   /* printf("%s\n", syms2sch_cmd); */

   /* Run vl2edif2 on the symbol library*/
   vl2edif2_cmd := "vl2edif2 -p -up "+orig_outname_str+" "
                                     +orig_outname_str+".edif";
   /* printf("%s\n", vl2edif2_cmd); */

   /* Run synopsys.script file */
   synopsys_cmd := "dc_shell -f synopsys.script ";
   /* printf("%s\n", synopsys_cmd); */

   vcWv.CommandSequence (list (syms2sch_cmd,
                               vl2edif2_cmd,
                               func() gen_synopsys_script(orig_outname_str,
                                                          techfile_list);,
                               synopsys_cmd),
                               nil, out_handler, nil, exit_handler, nil);

}  /* end function accept */
accept_cmd := vcWv.Cmd(fac, accept, "accept");


/* Function for action upon picklist click */
function toggle_grey(files)
   func (val, on_off) {
      if (val == 2) {
         /* printf ("Val=%d and on_off=%s\n", val, on_off); */
         aliasrgn0 := vcTable.Get(files, 1, 0);   /* label */
         aliasrgn1 := vcTable.Get(files, 1, 1);   /* command */
         if (on_off == vlOnOff.On) {
            vcRegion.Enable(aliasrgn0, vlOnOff.On, TRUE);
            vcRegion.Enable(aliasrgn1, vlOnOff.On, TRUE);
         } else {
            vcRegion.Enable(aliasrgn0, vlOnOff.Off, TRUE);
            vcRegion.Enable(aliasrgn1, vlOnOff.Off, TRUE);
         }
      }
   };   /* end of func */

/************************ main window setup ****************************/

function _Create(data, region)
{
   d := struct { TABLE; FILES; BUTTONS };

   /* Create the main table */
   d.TABLE = vcTable.Create("symxlate", 2, 1);

   /* Create the file entry subtable section, fill in the labels first */
   d.FILES = vcTable.Create("files", 4, 2);
   vcTable.Set(d.FILES, 0, 0, label("Translate symbols from:"));
   vcTable.Set(d.FILES, 1, 0, label("Specify library alias(es):"));
   vcTable.Set(d.FILES, 2, 0, label("New symbol library name:"));
   vcTable.Set(d.FILES, 3, 0, label("Synopsys technology library:")); 
   vcTable.Col(d.FILES, 0, vcTable.Border.Invisible,1,1,1, vgAlign.MiddleRight);

   /* Create the library type picklist for row 0 of the subtable */
   libtypes := list( list (cons ("Primary Directory", 1),
                           cons ("Aliased Library", 2)));
   liblist := vcRegion.Create("picklist", vcPicklist.fac, libtypes,
                              exclusive <- FALSE);
   vcPicklist.SetPick(liblist, 1);   /* Default is aliased dir */
   vcPicklist.Notify(liblist, toggle_grey (d.FILES));

   /* Now fill in the "command" side */
   vcTable.Set(d.FILES, 0, 1, liblist);
   vcTable.Set(d.FILES, 1, 1, cmdline("", "", 20));
   vcTable.Set(d.FILES, 2, 1, cmdline("", "", 20)); 
   vcTable.Set(d.FILES, 3, 1, cmdline("", "", 20)); 

   vcTable.Add(d.TABLE, d.FILES);

   /* Create buttons section */
   d.BUTTONS = vcRegion.Create("Buttons", vcButnRow.fac,
   list (list ("yes", "Accept", accept_cmd, list (close_window <- FALSE)),
         list ("no",  "Cancel", vcWin.QuitCmd, list (close_window <- FALSE))));

   vcTable.Add(d.TABLE, d.BUTTONS);

   /* Add this table to original region */
   vcRegion.AddChild(region, d.TABLE);

   /* d is data returned/passed to each function */
   d;
}
