/*
 * $Revision: 1.31 $
 *
 * Viewscript GUI encapsulation of the Vantage analyzer.  All known options
 * are represented in the command dialog box.  The output of the analyzer
 * is piped to a vcShell, which can be scrolled and the errors can be
 * traversed, highlighting appropriate lines in original VHDL source
 * files, in the editor of the user's choice.
 */


/*
 * All the boolean flags are lumped together in one command parameter
 * as a single flag.  This facility, vcFlagEdit, is good at editing
 * those flags, providing individual one-line help strings on each
 * field.  vcPickEnum doesn't provide that last feature.
 */
vcFac.Create ("vcFlagEdit", vscmVT);


/*
 * non-boolean options to the analyzer require enumerations to be created.
 */

define_enum ("vssOverwriteVT", list ("Only specified design unit",
                                     "Any file",
                                     "No files"));
overwrite_flags := ["overwriteself", "overwrite", "nooverwrite"];
default_overwrite := 0;

define_enum ("vssShowBuildVT", list ("Compile only",
                                     "Show design units first, then compile",
                                     "Show design units only"));
show_build_flags := ["noshowbuild", "showbuild", "showbuildonly"];
default_show_build := 0;

/*
 * C OPTIMIZER FLAGS
 *
 * define_enum ("vssCOptLevelVT", list ("0", "1", "2", "3", "4"));
 * default_c_opt := 0;
 */


/*
 * We want to have only one analyzer log window open at once, and we
 * want to manage serialization of multiple analyze commands, rather
 * than running them all in parallel.  So the user can quickly fire
 * off multiple analyze commands and we'll chew through them one at
 * a time.  The variable `the_analyzer_window' keeps track of this for us.
 * If we don't do this, the Vantage software intermittenly fails, as
 * the parallel analyzers try to work out of the same directory.
 */
the_analyzer_window := nil;
the_analyzer := nil;


/*
 * The following function brings up a text file at a particular line.
 * It is made very complex to allow us to run in a pre-5.3 environment.
 * This turns into a one-line function after that's no longer a concern.
 */
if (!isbound (`TextEdit, vcWin))
    load ("vanText", vcWin);

/*
 * We want to keep track of when the cockpit changes design directories,
 * but we also want sticky defaults to work so the user can analyze
 * several files from a different directory.  So what we store both
 * the current project and the default path.  We modify the default path
 * whenever someone executes an analyze command on a file.  We use that
 * default path as the default value unless the current project changes,
 * which we find out by looking at the environment variable WPROJECT.
 * When that changes from our previous notion of the current project,
 * we use that directory, with no filename, as the default.
 */
current_project := "";
default_path := vcWv.NoDefault;

function default_pathname(fac_data) {
    this_project := getenv ("WPROJECT");
    if ((this_project != nil) && !string_eq (this_project, current_project)) {
        current_project = this_project;

        /* ensure trailing slash in a portable way */
        default_path = vsMerge ("", this_project, "", "");
    } /* if */
    default_path;
} /* function default_pathname */


/*
 * shell_prefs controls the behavior of the shell we spawn to
 * run the Vantage commands.  We want it to get the environment
 * SYS_ENV, which is created either by vansetup.vs, or rvananal.vs,
 * depending on how the analyzer window was invoked.  SYS_ENV
 * has VANTAGE_VSS set up, as well as the path that includes
 * VANTAGE_VSS/pgm/dir.  kill_on_close means that when the user
 * dismisses the shell window, vcShell will kill the subprocess.
 */
function shell_prefs() {
    if (!isbound (`SYS_ENV, current_package ()))
        SYS_ENV := vansetup.SYS_ENV;
    list (env <- SYS_ENV, kill_on_close <- TRUE);
} /* function shell_prefs */

function _Create(cmd_src, region) {
    shell := vcRegion.Create1 ("", vcShell.fac, car (cmd_src),
                               shell_prefs ());
    buttons := vcRegion.Create ("", vcButnRow.fac,
                                list (list ("dismiss", "Dismiss",
                                            vcWin.wclose, nil),
                                      list ("interrupt", "Interrupt",
                                            vcShell.InterruptCmd, nil),
                                      list ("next_error", "Next Error",
                                            next_error_cmd, nil),
                                      list ("prev_error", "Previous Error",
                                            prev_error_cmd, nil),
                                      list ("show",
                                            "Show errors in small font",
                                            show_errors_cmd, nil)));

    t := vcTable.Make ("Vantage Analyzer", 2, 1, shell, buttons);
    analyzer := struct {
        shell = shell;
        dismiss_button = vcButnRow.Get (buttons, 0);
        interrupt_button = vcButnRow.Get (buttons, 1);
        next_error_button = vcButnRow.Get (buttons, 2);
        prev_error_button = vcButnRow.Get (buttons, 3);
        error_buttons_enabled = FALSE;
        start_line = 1;
        pending = list (cmd_src);
        vsi = nil
    };

    vcRegion.Enable (analyzer.interrupt_button, FALSE, FALSE);
    vcRegion.Enable (analyzer.next_error_button, FALSE, FALSE);
    vcRegion.Enable (analyzer.prev_error_button, FALSE, FALSE);
    vcRegion.AddChild (region, t);
    setup_subprocess (analyzer);
    analyzer;
} /* function _Create */

/*
 * exit_handler is called whenever the subprocess exits, at which
 * point we'll grab the next analyze command in the queue, run it,
 * and update the window, etc.  We'll also update the browser in
 * the VHDL Manager (vansetup.vs) if that was passed into run_analyzer.
 */
function exit_handler(analyzer, mode, code) {
    if (!vcRegion.Destroyed (analyzer.shell)) {
        vcRegion.Enable (analyzer.interrupt_button, FALSE, TRUE);
        analyzer.pending = cdr (analyzer.pending);
        if (analyzer.pending != nil) {
            cmd_src := car (analyzer.pending);
            vcShell.Restart (analyzer.shell, car (cmd_src),
                             shell_prefs ());
            setup_subprocess (analyzer);
        } /* if */
        else {
            display_queue (analyzer);
            vcRegion.Enable (analyzer.dismiss_button, TRUE, TRUE);
            if ((analyzer.vsi != nil) &&
                !vcRegion.Destroyed (analyzer.vsi.files))
                (vansetup.update_browser (analyzer.vsi)) (mode, code);
        } /* else */
    } /* if */
} /* function exit_handler */


/*
 * Show the queue of analyze commands in the status window, and update
 * the title of the window to indicate what's being analyzed right now.
 */
function display_queue(analyzer) {
    w := vcRegion.Window (analyzer.shell);
    if (w != nil) {
        vcWin.ClearStatus (w);
        first := TRUE;
        title := "Vantage Analyzer (idle)";
        foreach (cmd_src; vscript.list_iter (analyzer.pending)) {
            src := convert (vlStringVT, cdr (cmd_src));
            if (src[0] != '-') {
                parsed := vscript.vsParse (src);
                src = vscript.vsMerge ("", "", caddr (parsed),
                                       cadddr (parsed));
            } /* if */
            if (first) {
                first = FALSE;
                title = "Vantage Analyzer: " + src;
                vcWin.AddStatus (w, "Pending compiles: ");
            } /* if */
            else
                vcWin.AddStatus (w, ", ");
            vcWin.AddStatus (w, src);
        } /* foreach */
        vcWin.SetTitle (w, title);
    } /* if */
} /* function display_queue */

/*
 * Given a line of output from the analyzer, what's the character
 * index of the line number in the source file?  If this line
 * doesn't point to an error message, return nil.
 *
 * The expected format of a Vantage analyzer error is:
 *  "**<Severity>: LINE <LineNumber> <text of message>"
 *
 * An example is:
**Error: LINE 12 *** Symbol >>VLBIT_VECTOR<< is not known in this context   (compiler/analyzer/3)
 *
 * Here <Severity>==Error, <LineNumber>==12.
 *
 * What this routine returns is the character index of <LineNumber>, or
 * nil if not found.  The reason it returns this is that it's used
 * twice, once to determine whether there's an error at all within
 * a routine that monitors the output of the subprocess, just for the
 * purpose of ungreying the buttons, and once for extracting the line
 * number.  I didn't want to pay the extraction penalty while just
 * scanning the line.
 */
function error_index(line) {
    declare index;

    if ((string_length (line) > 2) &&   /* substring("",0,2)->ERROR */
        string_eq (substring (line, 0, 2), "**") &&
        ((index = string_search (line, ": LINE ")) != nil))
        index + 7;          /* 7 == string_length (": LINE ") */
    else
        nil;
} /* function error_index */

function _Modified(analyzer, region)
    analyzer.pending != nil;


/*
 * After an analyzer has been spawned, set up the exit handlers,
 * output handlers, and display the queue.  Also note the starting
 * line in the text window (.start_line) for this analysis...we'll
 * need it to bound the wrapping search in next-error.
 */
function setup_subprocess(analyzer) {
    tx := vcRegion.FindFac (analyzer.shell, vtx.fac);
    vtx.EndOfBuffer (tx);
    analyzer.start_line = vtx.CurrentLine (tx);

    vcRegion.Enable (analyzer.interrupt_button, TRUE, TRUE);
    vcRegion.Enable (analyzer.dismiss_button, FALSE, TRUE);
    sys := vcShell.Sys (analyzer.shell);
    vcSys.CallOnExit (sys, func(mode, code) {
        exit_handler (analyzer, mode, code);
    });

    /*
     * Initially, the window is brought up with the next-error
     * and previous-error buttons greyed out.  Once any errors
     * appear, the buttons will be ungreyed, and left that way
     * permanently, cause the user can always scroll back in the
     * log and hit next-error.  So we don't even bother to line-buffer
     * & monitor the analyzer output once buttons are enabled.
     */
    if (!analyzer.error_buttons_enabled) {
        vcSys.LineBuffer (sys);
        vcSys.CallOnOutput (sys, func(line) {
            if (!vcRegion.Destroyed (analyzer.shell)) {
                if (!analyzer.error_buttons_enabled &&
                    (error_index (line) != nil))
                {
                    analyzer.error_buttons_enabled = TRUE;
                    vcRegion.Enable (analyzer.next_error_button,
                                     TRUE, TRUE);
                    vcRegion.Enable (analyzer.prev_error_button,
                                     TRUE, TRUE);
                } /* if */
            } /* if */
        });
    } /* if */

    if (vcMac.IsReplayingOn ())
        vcMac.SubprocessHold (sys, 0);
    display_queue (analyzer);
} /* function setup_subprocess */

/*
 * The large-fixed-font Viewtext window can make it hard to see
 * the long lines Vantage writes for error messages.  This embarassing
 * little feature is provided in lieu of horizontal scrollbars, which
 * haven't been put in Viewtext yet.
 */
function show_errors(analyzer) {
    stattext := vcRegion.Create ("", vcStaticText.fac,
                                 vtx.GetBuffer (analyzer.shell));
    vcRegion.Font (stattext, vgFont.small);
    buttons := vcRegion.Create ("", vcButnRow.fac,
                                list (list ("dismiss", "Dismiss",
                                            vcWin.wclose, nil)));
    scroll := vcRegion.Create ("Error Log", vcScroll.fac, stattext,
                               size <- vgSize.Create (100, 300,
                                                      700, 500,
                                                      1000, 1000));
    table := vcTable.Make ("", 2, 1, scroll, buttons);
    vcWin.Create (vcRegion.Window (analyzer.shell), table);
} /* function show_errors */

/*
 * Find the next error (direction==1) or previous error (direction==-1)
 * in the analyzer log window.  Bring up the users favorite text editor
 * on the line in the original source file.
 */
function next_error(analyzer, direction) {
    declare line, index, token, source_line_number, source_filename;

    tx := vcRegion.FindFac (analyzer.shell, vtx.fac);
    window := vcRegion.Window (analyzer.shell);
    l := vtx.CurrentLine (tx);
    wrapped := FALSE;

    /*
     * Look forward for an error or warning.  Actually, look for a line
     * beginning with "**", that also includes the string ": LINE".
     * If the cursor is not at the beginning
     * of a line, go to the beginning of the next line.  This will help
     * us allow the user to warp to specific errors with the mouse or
     * whatever, but still walk forward in the errors by just hitting
     * the next-error button.
     */
    if (vtx.CurrentCol (tx) > 0)
        l += direction;
    while (source_line_number == nil) {
        line = vtx.GetLine (tx, l);

        /*
         * If we fall off the end, wrap around and continue searching,
         * but only do this once!  Otherwise this would infinite-loop
         * if there really were no errors, which can occur if there were
         * errors in a previous analysis, direction==1, and so we're
         * wrapping back to the beginning of the current analysis.  It
         * can also happen if we exceed the line-limit of vcTxShell
         * and errors are truncated away.
         */
        if (line == nil) {
            if (wrapped) {
                vcWin.ClearStatus (window);
                vcWin.AddStatus (window, "No more errors.");
                return (nil);
            } /* if */
            else {
                wrapped = TRUE;
                if (direction == 1)
                    l = analyzer.start_line;
                else {
                    vtx.EndOfBuffer (tx);
                    l = vtx.CurrentLine (tx);
                } /* else */
            } /* else */
        } /* if */

        /*
         * If there's an error on this line, extract it from the
         * line and convert it to an integer.  If it fails it'll
         * just raise an exception, but I don't expect it to.
         */
        else if (((index = error_index (line)) != nil) &&
            (string_token (substring (line, index, 0),
                           &token, " ") != nil))
            source_line_number = string2number (token);
        else
            l += direction;
    } /* while */

    vtx.Move (tx, l, 2);    /* Move past the "**" */
    vtx.Update (tx);

    /*
     * Now search backwards for the line that identifies which file
     * we are compiling
     */
    --l;
    while (source_filename == nil) {
        line = vtx.GetLine (tx, l);
        if ((line == nil) || (l < analyzer.start_line))
            error ("Cannot find filename for line", source_line_number);

        /*
         * Look for source filename
         */
        if ((index = string_search (line, "Compiling \"")) != nil)
            string_token (substring (line, index + 11, 0),
                          &source_filename, "\"");

        /*
         * Look for VHDL design unit name.  We expect the line to look
         * like this:
         *    "   Compiling VHDL for architecture FFT.ADD17(BEHAVIOR)..."
         * We want to extract "FFT.ADD17(BEHAVIOR)" and hand it to
         * the browser for conversion to a source filename like
         *      "fft.lib/add17.edr/behavior.adr/vantage.vdl"
         */
        else if (((index = string_search (line, "Compiling VHDL for ")) != nil)
                 && (analyzer.vsi != nil))
        {
            declare s, type;

            s = string_token (substring (line, index + 19, 0), &type, " ");
            if ((string_length (s) > 3) &&
                string_eq (substring (s, -3, 0), "..."))
                s = substring (s, 0, -3);
            source_filename =
                vansetup.browse.symbolic_to_pathname (analyzer.vsi.files, s);
        } /* else if */
        else
            --l;
    } /* while */

    vcWin.TextEdit (window, source_filename, source_line_number, nil,
                    TRUE, FALSE);
/*
    vcWin.ClearStatus (window);
    vcWin.AddStatus (window, line);
*/
} /* function next_error */

function queue_cmd(analyzer, cmd, src) {
    if (analyzer.pending == nil) {
        analyzer.pending = list (cons (cmd, src));
        vcShell.Restart (analyzer.shell, cmd, shell_prefs ());
        setup_subprocess (analyzer);
    } /* if */
    else {
        for (p := analyzer.pending; cdr (p) != nil; p = cdr (p))
            ;
        set_cdr (p, list (cons (cmd, src)));
        display_queue (analyzer);
    } /* else */
} /* function queue_cmd */

next_error_cmd := vcWv.Cmd (fac, func(a, e) next_error (a, 1);, "next_error");
prev_error_cmd := vcWv.Cmd (fac, func(a, e) next_error (a, -1);, "prev_error");
show_errors_cmd := vcWv.Cmd (fac, func(a, e) show_errors (a);, "show_errors");

/* 
 * Once all the parameters have been assembled to one final command line,
 * run_analyzer gets called.  `vsi' is the data associated with the
 * vantage setup window, or is nil if this was called directly from the
 * cockpit or Viewsim.  It's used to update the browser tree when the
 * analyzer finishes.  `cmd' is the command string were going to send
 * to Unix.  `src' is the name of the source file, or design unit,
 * attached to this command, and is used to display the "queue" of
 * analysis commands that is formed when the user fires of many at once.
 */
function run_analyzer(vsi, cmd, src) {
    if (the_analyzer_window == nil) {
        r := vcRegion.Create ("Vantage Analyzer", fac, cons (cmd, src));
        the_analyzer_window = vcWin.Create (r);
        the_analyzer = vcRegion.GetData (r);
        vcWin.DestroyNotify (the_analyzer_window, func(w) {
            the_analyzer = nil;
            the_analyzer_window = nil;
        });
        vcWin.SetConfirmMessage (the_analyzer_window, "

 If you close the analyzer window, the analyzer will be
 non-gracefully terminated, and all of the queued analyze
 commands will be discarded.

");
    } /* if */

    /*
     * If there already was an analyzer, run the new command if it was
     * idle, or queue it up if it wasn't.
     */
    else {
        vcWin.Show (the_analyzer_window);
        queue_cmd (the_analyzer, cmd, src);
    } /* else */

    if (vsi != nil)
        the_analyzer.vsi = vsi;
} /* function run_analyzer */

function build_command_line(libs, overwrite, showbuild, fromline,
                            maxerr, maxdesunits, flags, listing /*, c_opt */)
{
    cmd := "";

    /*  Its possible that there may be two "-dbg 2" strings sitting in "flag"
     *  So, check for such a case and remove the duplicate guy
     */
    debug_str   := "";
    profile_str := "";
    profile     := FALSE;

    foreach (flag; string_loop (flags, "\n"))
    {
	str := substring (flag, 0, 4);
	if (string_eq (str, "-dbg"))
	    debug_str = flag;
	else if (string_eq (flag, "-profile -dbg 2"))
	{
	    profile = TRUE;
	    profile_str = flag;
	}
	else
	    cmd += " " + flag;
    }

    if (profile == TRUE)
	cmd += " " + profile_str;
    else
	cmd += " " + debug_str;


    overwrite = convert (vlIntVT, overwrite);
    if (overwrite != default_overwrite)
        cmd += " -" + overwrite_flags[overwrite + 0];
    showbuild = convert (vlIntVT, showbuild);
    if (showbuild != default_show_build)
        cmd += " -" + show_build_flags[showbuild + 0];
/*
 * C OPTIMIZER FLAGS
 *
 *   c_opt = convert (vlIntVT, c_opt);
 *   if (c_opt != default_c_opt)
 *       cmd += " -ccoptlevel " + convert (vlStringVT, c_opt);
 *
 */
    if (fromline != 1)  cmd += sprintf (" -fromline %d", fromline);
    if (!string_eq (maxerr, "")) cmd += " -maxerr " + maxerr;
    if (!string_eq (maxdesunits, "")) cmd += " -maxdesunits " + maxdesunits;
    if (!string_eq (listing, "")) cmd += " -lst " + listing;

    cmd + " " + libs;
} /* function build_command_line */

function analyze_without_setup(w, e, src, flags, overwrite, showbuild,
                               fromline, maxerr, maxdesunits, listing /*,
                               c_opt*/)
{
    default_path = src;
    cmd := "analyze -src " + src +
        build_command_line ("-libfile vsslib.ini", overwrite, showbuild,
                            fromline, maxerr, maxdesunits, flags, listing /*,
                            c_opt */);

    run_analyzer (nil, cmd, src);
} /* function analyze_without_setup */

common_flags :=
    list (list ("Debug", "Enable VHDL source level debugging",
                "-dbg 2", "-dbg 0"),
          list ("Code Coverage", "Enable Profiling of VHDL source code",
                "-profile -dbg 2", nil),
          list ("Range Check", "Produce code for runtime range checking",
                nil, "-nobndchk"),
          list ("Dependencies", "Compile out-of-date dependencies",
                nil, "-nodep"),
          list ("Halt on error",
"Stop compiling design units as soon as one is found to contain syntax errors",
                nil, "-nohaltonerr"),
          list ("Fail on warning", 
                "Compiler emit a nonzero return code if a warning is found",
                nil, "-nowarnrc"));

check_flag := 
    list ("Check only", "Do not write any new files to the design library",
          "-chkonly", nil);

force_flag :=
    list ("Force", "Force recompilation even if time stamps are okay",
          "-force", nil);

flags_with_check :=
    reverse_destructive (cons (check_flag, reverse (common_flags)));
flags_with_force :=
    reverse_destructive (cons (force_flag, reverse (common_flags)));

/*
 * There are two analyze commands, most of whose options are the same,
 * except for the first couple.  We use this list and the non-var-args
 * form of vcWv.Cmd: vcCmd.Create, where it's easier to form the list
 * of args with a call to append.
 */
common_params :=
    list (vcWv.Param ("Overwrite", vssOverwriteVT, vcWv.Sticky,
                      default_overwrite,
                      vcWv.Secondary,
                      "Specify whether compiler should overwrite existing design library files"),
          vcWv.Param ("Show Build", vssShowBuildVT, vcWv.Sticky,
                      default_show_build,
                      vcWv.Secondary,
                      "Specify whether analyzer should compile and/or show which units require compilation"),
          vcWv.Param ("First line", vlPositiveIntVT, vcWv.Sticky, 1,
                      vcWv.Secondary,
                      "Specifies the line in the source file where compilation is to commence."),
          vcWv.Param ("Maximum Errors", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies the number of syntax errors at which the compiler will terminate."),
          vcWv.Param ("Maximum Design Units", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies the maximum number of design units to be compiled."),
          vcWv.Param ("Listing Filename", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies a compilation listing file.")
/* ,
 * C OPTIMIZER FLAGS
 *         vcWv.Param ("C Optimization Level", vssCOptLevelVT, vcWv.Sticky,
 *                      default_c_opt, vcWv.Secondary,
 *                      "Specifies the optimization flag passed to the C compiler")
 */
);

/*
 * This little bit of magic does this:
 *      cons2 (a, b, c, list (d, e, f)) -> list (a, b, c, d, e, f)
 */
function cons2(args...) {
    if (args == nil)
        nil;
    else if (cdr (args) == nil)
        car (args);
    else
        cons (car (args), apply (cons2, cdr (args)));
} /* function cons2 */

flg_str := "flags_with_check";

lib_flag_table := nil;

function get_obj_libs(vsi) {
    if (lib_flag_table == nil) {
        lib_flag_table = vlSym.StrCreate (3);
        lib_flag_table[vsMerge ("", vansetup.SYS_PATH, "ieee", "lib")]
            = "-libieee";
        lib_flag_table[vsMerge ("", vansetup.SYS_PATH, "math", "lib")]
            = "-libmath";
        lib_flag_table[vsMerge ("", vansetup.SYS_PATH, "vantage_logic", "lib")]
            = "-libvantage_logic";
    } /* if */

    function lib_cmd_spec(obj) {
        p := vansetup.browse.obj_pathname (obj);
        flag := lib_flag_table[p];
        if (flag == nil)
            "-lib " + vsRelPath (p);
        else
            flag;
    } /* function lib_cmd_spec */

    this_lib := vansetup.browse.obj_lib (vansetup.current_obj (vsi));
    lib_str := lib_cmd_spec (this_lib);
    foreach (lib; vansetup.current_library_objs (vsi))
        if (this_lib != lib)
            lib_str += " " + lib_cmd_spec (lib);
    lib_str;
} /* function get_obj_libs */

function analyze_src(vsi, e, src, flags, overwrite, showbuild, fromline,
                     maxerr, maxdesunits, listing/*, c_opt*/)
{
    default_path = src;

    if (vsi.is_modified)
        vansetup.save_ini (vsi, e);

    if (isbound (`vsRelPath, nil))
        src = vsRelPath (src);

    cmd := "analyze -src " + src +
        build_command_line ("-libfile " + vsi.init_filename, overwrite,
                            showbuild, fromline, maxerr, maxdesunits,
                            flags, listing/*, c_opt*/);
    run_analyzer (vsi, cmd, src);
} /* function analyze_src */

function analyze_obj(vsi, e, flags, overwrite, showbuild,
                     fromline, maxerr, maxdesunits, listing/*, c_opt*/)
{
    if (vsi.is_modified)
        vansetup.save_ini (vsi, e);

    src_info := vansetup.browse.design_unit_string (vansetup.current_obj (vsi));
    cmd := "analyze " + src_info +
        build_command_line (get_obj_libs (vsi), overwrite, showbuild,
                            fromline, maxerr, maxdesunits, flags,
                            listing/*, c_opt*/);

    run_analyzer (vsi, cmd, src_info);
} /* function analyze_obj */

function confirm_analyze(args...) {
    vsi := car (args);
    if (vsi.is_modified)
"The library initialization file, " + vsi.init_filename + ",
has been modified.  It will be automatically saved before
analysis begins.

Save " + vsi.init_filename + " and run the analyzer?";
    else
        FALSE;
} /* function confirm_analyze */

analyze_src_cmd :=
    apply (vcWv.CmdWithConfirm,
           cons2 (vansetup.fac, analyze_src, confirm_analyze, TRUE,
                  "analyze_src", "Analyze VHDL Source File",
                  vcWv.Param ("VHDL Source File", vlStringVT,
                              vcWv.Dynamic, default_pathname,
                              vcFileSelect.fac, filter <- "*.vhd*"),
                  vcWv.Param ("Flags", vlStringVT,
                              vcWv.Sticky, list ("-dbg 2"),
                              vcWv.Secondary, vcFlagEdit.fac,
                              flags <- "vananalz.flags_with_check"),
                  common_params));
vcCmd.SetHelp (analyze_src_cmd,
            "Analyze a VHDL source file into the current working library");
vcCmd.EnableApply (analyze_src_cmd, TRUE);

analyze_obj_cmd :=
    apply (vcWv.CmdWithConfirm,
           cons2 (vansetup.fac, analyze_obj, confirm_analyze,
                  TRUE, "analyze_obj", "Analyze VHDL Design Unit",
                  vcWv.Param ("Flags", vlStringVT,
                              vcWv.Sticky, list ("-dbg 2"),
                              vcWv.Secondary, vcFlagEdit.fac,
                              flags <- "vananalz.flags_with_force"),
                  common_params));
vcCmd.SetHelp (analyze_obj_cmd,
               "Re-analyze the selected VHDL design object");
vcCmd.EnableApply (analyze_obj_cmd, TRUE);
flg_str := "vananalz.flags_with_check";

analyze_without_setup_cmd :=
    vcCmd.Create (vcWin.fac, analyze_without_setup, "analyze_src",
                  "Analyze VHDL Source File",
                  cons2 (vcWv.Param ("VHDL Source File",
                                     vlStringVT, vcWv.Dynamic,
                                     default_pathname,
                                     vcFileSelect.fac,
                                     filter <- "*.vhd*",
                                     read_now <- TRUE),
                         vcWv.Param ("Flags", vlStringVT,
                                     vcWv.Sticky, "-dbg 2",
                                     vcWv.Secondary, vcFlagEdit.fac,
                                     flags <- flg_str),
                         common_params));
vcCmd.SetHelp (analyze_without_setup_cmd,
               "Analyze a VHDL source file into the current working library");
vcCmd.EnableApply (analyze_without_setup_cmd, TRUE);

function errors_from_file(vsi, event, filename) {
    cmd := ((vcWv.OS (wv) == vcOs.Unix) ? "cat " : "type ") + filename;
    run_analyzer (vsi, cmd, filename);
} /* function errors_from_file */

errors_from_file_cmd := vcWv.Cmd (vansetup.fac, errors_from_file,
                                  "analyzer_errors_from_file",
                                  "Read Analyzer Errors From File",
                                  vcWv.Param ("", vlStringVT, vcWv.Sticky,
                                              "vhdl_compiler.msg",
                                              vcFileSelect.fac));
vcCmd.SetHelp (errors_from_file_cmd,
               "Read and track errors from a previous analysis using the log file");


                                  
