/*
 * $Id: vansetup.vs,v 1.54 1995/03/03 20:47:21 dahan Stab $
 *
 * Copyright (c) 1993 by Viewlogic Systems, Inc.
 */

/*
 * This facility implements a browser for the Vantage database.
 * It is implemented in two pieces, which are probably too closely
 * coupled.  This file, vansetup.vs, implements the top level of
 * the user interface, laying out a file-selection box, a picklist,
 * a button row, some other stray buttons, and some help text.  It
 * manages the interaction between these.
 *
 * Generally, the functions in this module take something called "vsi"
 * as an argument (Vantage Setup Info).  This is a Viewscript structure
 * that mainly contains the region IDs for some of the UI components.
 * The best place to look for what it contains is its definition, at
 * the end of function _Create.
 *
 * The other module, vanbrowse.vs, manages just the picklist.
 * It has detailed knowledge of the layout of Vantage libraries, and
 * uses Viewscript's TraverseDir iterator to figure out what's in
 * them.  This file provides a package "browse", to implement some
 * measure of data hiding, or whatever.  All the calls to that module
 * look like "browse.something".
 *
 * A picklist is used to hold the library search order.  This is a
 * reasonably obvious choice.  I'm not so fond of vcListEdit recently
 * because it's visually big and clunky.  The picklist can be
 * programmatically manipulated with vcPicklist.DeleteRows and InsertRows
 * (which were created for vcListEdit), and the user probably doesn't
 * need to do as much more than is supplied here.
 *
 * A decision that was less obvious was to introduce hierarchical
 * browsing into the picklist, where each sub-object gets a new row.
 * This seems to work well, although there is probably too much software
 * labor required to have other junk interspersed with the libraries,
 * to keep related objects together, etc.  If there were a good tree
 * widget, I'd probably prefer to use it instead.
 */

define_mask ("vanFieldsVT",
             list ("SourceTime", "VIFTime", "BinTime", "SourceFile",
                   "RangeCheck", "C_Opt", "VHDL_Opt",
                   "VSSVersion", "Debug", "SrcStartLine", "SrcEndLine"));

vlPref.Replace (prefs, "fields", vanFieldsVT,
                "Fields to show in the library search order list",
                vanFields.Debug + vanFields.RangeCheck);

vlPref.Replace (prefs, "show_error_detail", vlTrueFalseVT,
                "~~Expound on why vansetup thinks a library is corrupt",
                FALSE);

vlPref.Replace (prefs, "show_lib_browser", vlTrueFalseVT,
                "~~Show the library directory browser to the left of the data browser",
                TRUE);

vlPref.Replace (prefs, "rows", vlPositiveIntVT,
                "~~Number of rows to show by default in the library browser",
                15);

load ("vanbrowse.vs");
vcWv.RequireFac (wv, "vananalz");

/*
 * This iterator reads a file that has pathnaames of Vantage libraries,
 * iterating on each pathname.
 */
iterator ini_file_pathnames(lib_list_filename) {
    if (fexists (lib_list_filename)) {
        f := fopen (lib_list_filename, "r");
        declare s;
        while ((s = fgetline (f)) != nil) {
            len := string_length (s);
            if (len > 1) {
                if (s[len-1] == '\n')
                    s = substring (s, 0, -1);
                iterate (s);
            } /* if */
        } /* while */
        fclose (f);
    } /* if */
} /* iterator ini_file_pathnames */

/*
 * Test to see if a library exsits by trying to open "foo.lib/vantage.lts".
 * Note the usage of vsMerge to provide portability.
 */
function library_exists(filename)
    fexists (vsMerge ("", filename, "vantage", "lts"));

function working_lib_is_read_only(vsi) {
    declare len;

    pathname := browse.obj_pathname (pick_obj (vsi, 0));
    ((SYS_PATH != nil) &&
     (string_length (pathname) > (len = string_length (SYS_PATH))) &&
     string_eq (substring (pathname, 0, len), SYS_PATH));
} /* function working_lib_is_read_only */

/*
 * Given whether or not anything is selected, or whether libraries are
 * selected, or whether the modified flag is set, grey out the appropriate
 * buttons.
 */
function grey_buttons(vsi) {
    w := vcRegion.Window (vsi.region);
    foreach (cmd; vscript.list_iter (list (purge_lib_cmd, rem_libs_cmd)))
        vcCmd.Enable (cmd, vsi.n_libs_selected > 0);
    vcRegion.Enable (vsi.rem_lib_button, vsi.n_libs_selected > 0, TRUE);
    foreach (cmd; vscript.list_iter (list (del_obj_cmd, set_working_cmd)))
        vcCmd.Enable (cmd, vsi.n_selected > 0);
    vcCmd.Enable (save_ini_cmd, vsi.is_modified);

    if ((n_entries (vsi) == 0) || working_lib_is_read_only (vsi)) {
        vcCmd.Enable (vananalz.analyze_obj_cmd, FALSE);
        vcCmd.Enable (vananalz.analyze_src_cmd, FALSE);
        msg := "Create a working library (File Menu) before analyzing VHDL source";
        vcCmd.SetDisabledMsg (vananalz.analyze_obj_cmd, msg);
        vcCmd.SetDisabledMsg (vananalz.analyze_src_cmd, msg);
    } /* if */
    else {
        if ((vsi.n_selected == 0) || (vsi.n_libs_selected > 0)) {
            vcCmd.Enable (vananalz.analyze_obj_cmd, FALSE);
            vcCmd.SetDisabledMsg (vananalz.analyze_obj_cmd,
                                  "Select a design object in Library Search Order area for analysis");
        } /* if */
        else
            vcCmd.Enable (vananalz.analyze_obj_cmd, TRUE);

        vcCmd.Enable (vananalz.analyze_src_cmd, TRUE);
    } /* else */
        
    obj := vcRegion.Get (vsi.files, vscmVT);

    vcCmd.Enable (toggle_expand_cmd,
                  (obj != nil) && (browse.has_children (obj) ||
                                   browse.has_view (obj)));
    vcCmd.Enable (view_obj_cmd,
                  (obj != nil) && browse.has_view (obj));
    
    vcWin.UpdateMenus (w);
} /* function grey_buttons */

/*
 * Convenience function for getting the data for a particular picklist
 * entry.  vcPicklist.Get returns a cons cell: (visual . obj).  We
 * only want the obj, so we take the cdr.
 */
function pick_obj(vsi, row)
    cdr (vcPicklist.Get (vsi.files, row, 0));

function _Modified(vsi, region)
    vsi.is_modified;

/*
 * Set the modification flag for the window.
 */
function modify(vsi, on_off) {
    w := vcRegion.Window (vsi.region);
    vsi.is_modified = on_off;
    if (w != nil)
        grey_buttons (vsi);
} /* function modify */

/*
 * Dynamic defaulting function that grabs the current pathname out
 * of the file-selection box.  It's used by create_lib.
 */
function default_pathname(vsi) {
    filename := "user.lib";
    catch (vlEx.FAILURE) {
        f := vcRegion.Get (vsi.fs, vlFileWriteVT);
        if ((string_length (f) > string_length (SYS_PATH)) &&
            (string_eq (substring (f, 0, string_length (SYS_PATH)), SYS_PATH)))
            filename = f;
    } /* catch */
    exception
        ;
    filename;
} /* function default_pathname */

/*
 * Helper command for making add_lib macro-recordable
 */
function do_add_lib(vsi, event, lib_pathname) {
    browse.add_lib (vsi.files, nil, convert (vlStringVT, lib_pathname));
    modify (vsi, TRUE);
} /* function do_add_lib */
do_add_lib_cmd := vcWv.Cmd (fac, do_add_lib, "add_lib", "Add Library",
                            vcWv.Param ("", vlFileNameVT,
                                        vcWv.Constant, vcWv.NoDefault));

/*
 * Command-function to add a library.  We get the library pathname
 * from the file-selection region (vsi.fs).  This can fail if there
 * is no file specified, but it's difficult to detect that up front
 * and keep the button greyed properly.
 */
function add_lib(vsi, event) {
    declare filename;
    catch (vlEx.FAILURE) {
        filename = vcRegion.Get (vsi.fs, vlStringVT);
        vcCmd.Run (do_add_lib_cmd, vsi, event,
                   list (convert (vlFileNameVT, filename)), TRUE);
    } /* catch */
    exception {
        w := vcEvent.Window (event);
        vcErr.Report (w, nil, vlErr.CurrentError ());
        vcWin.ClearStatus (w);
        if (filename == nil)
            vcWin.AddStatus (w, "Cannot add library -- none specified");
        else
            vcWin.AddStatus (w, "Cannot find library `" + filename + "'");
    } /* exception */
} /* function add_lib */

/*
 * Note that add-lib is not macro recorded.  It calls do_add_lib,
 * which is macro recorded, once it grabs the filename out of the
 * file selection box.  This is necessary because vcFileSelect doesn't
 * record macros right now.
 */
add_lib_cmd := vcWv.Cmd (fac, add_lib);
vcCmd.SetHelp (add_lib_cmd, "Append the currently selected library filename to the library search path");

function n_entries(vsi) {
    declare n_rows, n_cols;
    vcPicklist.Dimensions (vsi.files, &n_rows, &n_cols);
    n_rows;
} /* function n_entries */

/*
 * Command-function to remove a library from the master-list.  This
 * function is a little convoluted just because it was originally
 * written to deal with a non-exclusive picklist, allowing many items
 * to be selected at once.  I haven't been motivated to change the
 * logic around to take advantage of the simpler situation offered
 * by exclusive picklists.
 */
function rem_libs(vsi, event) {
    n_rows := n_entries (vsi);

    /* Search through the picklist for libraries that are set */
    for (r := 0; r < n_rows; ++r) {
        if (vcPicklist.IsSet (vsi.files, r) &&
            browse.is_library (cdr (vcPicklist.Get (vsi.files, r, 0))))
        {
            /* Make sure we're not browsing anything under the library */
            browse.collapse (vsi.files, pick_obj (vsi, r), r);
            vcPicklist.DeleteRows (vsi.files, r, 1);
            n_rows := n_entries (vsi);
            modify (vsi, TRUE);
            if ((r == 0) && (n_rows > 0))
                browse.set_working (vsi.files, 0, TRUE);
        } /* if */
    } /* for */
    grey_buttons (vsi);
    if (vsi.n_libs_selected != 0)
        error ("Assertion after rem_libs, vsi.n_libs_selected != 0",
               vsi.n_libs_selected);
} /* function rem_libs */

rem_libs_cmd := vcWv.Cmd (fac, rem_libs, "rem_lib");
vcCmd.SetHelp (rem_libs_cmd, "Remove the currently selected symbolic library name from the library search path.");

vcBind.Set (bindings, 'R', rem_libs_cmd);
vcBind.Set (bindings, 'r', rem_libs_cmd);

/*
 * Create a new design library.  This is done by running the program
 * vanlibcreate, which sets up the right directory structure, timestamp
 * file, etc.  When its done, we look at the exit status to make
 * sure it succeeded, and either add it to the search path, or raise
 * an exception saying why it failed.
 * 
 * This function is *not* run by the Create Library button directly.
 * 
 */
function create_lib(vsi, event, pathname, symbolic_name) {
    buf := "";
    function error_proc(str)
        buf += str;
    pathname = convert (vlStringVT, pathname);
    symbolic_name = convert (vlStringVT, symbolic_name);
    function exit_proc(mode, code) {
        w := vcEvent.Window (event);
        vcWin.ClearStatus (w);
        if ((mode == vcSysMode.Exited) && (code == 0) &&
            (vsInfo (pathname) != nil))
        {
            browse.add_lib (vsi.files, symbolic_name, pathname);
            vcWin.ClearStatus (w);
            vcWin.AddStatus (w, sprintf ("Library `%s' created in %s",
                                         symbolic_name, pathname));
            vcFileSelect.Reset (vsi.fs);
            modify (vsi, TRUE);
        } /* if */
        else

            /*
             * Raise an exception to notify the user of the fact that
             * vanlibcreate failed, but don't raise an exception from
             * the exit handler, since that prevents other exit notifications
             * from running (the vcQuit.active_subprocesses thing and maybe
             * some macro stuff as well).  Instead set up a timer to do
             * the exception once we get back to "top level".
             */
            vcWv.Delay (wv, func()
                        vlEx.Throw2 (vlEx.FAILURE, "vssSetup.create_lib",
                                     vlErrSev.Error, buf););
    } /* function exit_proc */
            
    vcWin.System (vcEvent.Window (event),
                  "vanlibcreate " + pathname + " " + symbolic_name,
                  SYS_ENV, TRUE, nil, error_proc,
                  exit_proc, nil, nil);
} /* function create_lib */

/*
 * Give the user a break and let him know ASAP if he tries to pick
 * a VHDL keyword as the symbolic name, which vanlibcreate doesn't like.
 * This list of keywords is from the Export 1076 source code.
 */
vhdl_keywords := vlSym.StrCreate (83);
foreach (kw; vscript.list_iter (list
    ("ABS", "ACCESS", "AFTER", "ALIAS", "ALL", "AND",
    "ARCHITECTURE", "ARRAY", "ASSERT", "ATTRIBUTE",
    "BEGIN", "BLOCK", "BODY", "BUFFER", "BUS",
    "CASE", "COMPONENT", "CONFIGURATION", "CONSTANT",
    "DISCONNECT", "DOWNTO",
    "ELSE", "ELSIF", "END", "ENTITY", "EXIT",
    "FILE", "FOR", "FUNCTION",
    "GENERATE", "GENERIC", "GUARDED",
    "IF", "IN", "INOUT", "IS",
    "LABEL", "LIBRARY", "LINKAGE", "LOOP",
    "MAP", "MOD",
    "NAND", "NEW", "NEXT", "NOR", "NOT", "NULL",
    "OF", "ON", "OPEN", "OR", "OTHERS", "OUT",
    "PACKAGE", "PORT", "PROCEDURE", "PROCESS",
    "RANGE", "RECORD", "REGISTER", "REM", "REPORT", "RETURN",
    "SELECT", "SEVERITY", "SIGNAL", "SUBTYPE",
    "THEN", "TO", "TRANSPORT", "TYPE",
    "UNITS", "UNTIL", "USE",
    "VARIABLE",
    "WAIT", "WHEN", "WHILE", "WITH",
    "XOR")))
    vlSym.Store (vhdl_keywords, kw, TRUE);

legal_vhdl := vscript.re_compile ("^[_A-Z][_A-Z0-9]*$");

/*
 * This function verifies as much as reasonable that the args
 * to vanlibcreate are hip, so that the user can edit them on
 * the spot.
 */
function verify_create(vsi, event, pathname, symbolic_name) {
    pathname = convert (vlStringVT, pathname);
    symbolic_name = convert (vlStringVT, symbolic_name);

    if (library_exists (pathname))
        vlEx.Throw2 (vlEx.FAILURE, "vssSetup.create_lib", vlErrSev.Error,
                     sprintf ("Library pathname `%s' already exists",
                              pathname));
    if (string_length (string_trim (symbolic_name)) == 0)
        vlEx.Throw2 (vlEx.FAILURE, "vssSetup.create_lib", vlErrSev.Error,
                     "Library symbolic name must be supplied");
    if (!vscript.re_exec (legal_vhdl, symbolic_name))
        vlEx.Throw2 (vlEx.FAILURE, "vssSetup.create_lib", vlErrSev.Error,
                     "Invalid VHDL syntax: " + symbolic_name);
    if ((vlSym.Fetch (vhdl_keywords, symbolic_name) != nil) ||
        string_eqi (symbolic_name, "WORK"))
        vlEx.Throw2 (vlEx.FAILURE, "vssSetup.create_lib", vlErrSev.Error,
                     "VHDL Keywords cannot be used as the symbolic name");
} /* function verify_create */

create_lib_cmd := vcWv.Cmd (fac, create_lib, "create_lib",
                            "Create VSS Library",
    vcWv.Param ("Library Pathname", vlFileWriteVT,
                vcWv.Dynamic, default_pathname,
                "Physical pathname in which to create the library",
                filter <- "*.lib",
                edit_filter <- FALSE,
                traverse_matching_dirs <- FALSE,
                select_type <- vcFileSelect.Style.Directories,
                filename_label <- "Library",
                files_label <- "Available Libraries"),
    vcWv.Param ("Symbolic Name", vlStringUpperVT, vcWv.Sticky, "",
                "Name with which you can refer to this library in your VHDL",
                case <- vcCmdlineCase.Upper));
vcCmd.VerifyParams (create_lib_cmd, verify_create);
vcCmd.SetHelp (create_lib_cmd, "Create a new VSS library");
vcBind.Set (bindings, 'C', create_lib_cmd);
vcBind.Set (bindings, 'c', create_lib_cmd);

function update_browser(vsi)
    func(mode, code)
        vcWin.WithBusy (vcRegion.Window (vsi.files),
                        func() browse.update (vsi.files););
    ;

function purge_lib(vsi, event) {
    pathname := browse.obj_pathname (vcRegion.Get (vsi.files, vscmVT));
    vcWin.System (vcEvent.Window (event),
                  "vanlibpurge " + pathname,
                  SYS_ENV, TRUE, vcSysOutput.Viewtext, nil,
                  update_browser (vsi), nil, nil);
} /* function purge_lib */

function lib_to_purge(vsi, event)
"This operation purges a design library by removing
all of the compiled VHDL data from the library.  It
leaves the source (schematics, vhdl, etc), which may
then be recompiled.

Purge " + browse.obj_pathname (vcRegion.Get (vsi.files, vscmVT)) + "?";

purge_lib_cmd := vcWv.CmdWithConfirm (fac, purge_lib, lib_to_purge, TRUE,
                                      "purge_lib");
vcCmd.SetHelp (purge_lib_cmd, "Delete all the compiled objects in the selected library from the disk (the VHDL source remains intact).");

/*
 * Command-function to delete selected objects from the disk.  At
 * first, I implemented this with TraverseDir and vsDelete, but
 * there are files in the vantage libraries that are write-protected,
 * and there is no Viewscript primitive for chmod.  So instead, I'm
 * going with running the command "rm -rf object-directory", and I'll
 * have to deal with the PC when the time comes.
 *
 * This function makes the assumption that every vantage object is
 * represented by a directory, and that all the subdirectories should
 * go away when the object does.  For example, when deleting an entity,
 * all its architectures go away, and all the configurations for those
 * architectures disappear as well.
 *
 * This function also takes care of removing the objects from the browser,
 * or, if it's a library, removing the entire library by calling rem_libs.
 * In that case, we must also update the FileSelection box to reflect
 * the fact that the specified library is no longer available.
 */
function del_obj(vsi, event) {
    w := vcEvent.Window (event);
    obj := vcRegion.Get (vsi.files, vscmVT);
    obj_name := browse.obj_name (obj);
    pathname := browse.obj_pathname (obj);

    /*
     * Make a list of the configurations dependent on this object.
     * Any configurations that include an architecture or entity
     * being deleted will automatically be removed as well, due to
     * the vantage directory structure.  But we need to make a list
     * of them to remove references later that appear in the "top-level"
     * directory vantage.cds.
     */
    configs := nil;
    foreach (cfg_sym, cfg_path; browse.configurations (obj))
        configs = cons (cons (cfg_sym, cfg_path), configs);

    if (browse.is_deletable (obj)) {
        error_str := "";
        function error_proc(str)
            error_str += str;
        function exit_proc(mode, code) {

            /*
             * Now delete all the references to the deleted configurations
             * in $LIB/vantage.cds/$CONFIG_NAME.lea
             */
            lib_pathname := browse.obj_pathname (browse.obj_lib (obj));
            vantage_cds_dir := vsDnpath (lib_pathname, "vantage.cds");

            foreach (sym_path; vscript.list_iter (configs))
                if (vsInfo (cdr (sym_path)) == nil) {
                    lea := vsMerge ("", vantage_cds_dir, car (sym_path),
                                    "lea");
                    if (vsInfo (lea) != nil)
                        vscript.vsDelete (lea, vsAttr.File);
                } /* if */

            /* Test the status code and the directory to ensure success */
            if ((mode != vcSysMode.Exited) || (code != 0))
                vcWv.Delay (wv, func()
                            vlEx.Throw2 (vlEx.FAILURE, "vssSetup.del_obj",
                                         vlErrSev.Error, error_str););
            else if (vsInfo (pathname) != nil)
                vcWv.Delay (wv, func()
                            vlEx.Throw2 (vlEx.FAILURE, "vssSetup.del_obj",
                                         vlErrSev.Error,
                                         "Directory deletion failed: " +
                                         pathname););

            /*
             * Success...update the file-selection box and remove the
             * deleted object and its decendents from the picklist.
             */
            else
                vcWin.WithBusy (w, func() {
                    if (browse.is_library (obj)) {
                        vcFileSelect.Reset (vsi.fs);
                        rem_libs (vsi, event);
                    } /* if */
                    else {
                        browse.reset_parent (vsi.files,
                                             pick_obj (vsi,
                                                        vsi.cur_selection),
                                             vsi.cur_selection);
                        vsi.cur_selection = -1;
                        vsi.n_selected = 0;
                        vsi.n_libs_selected = 0;
                        grey_buttons (vsi);
                    } /* else */
                    vcWin.ClearStatus (w);
                    vcWin.AddStatus (w, "Deleted " + obj_name);
                });
        } /* function exit_proc */

        vcWin.System (w, "rm -rf " + pathname,
                      SYS_ENV, TRUE, nil, error_proc, exit_proc, nil, nil);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, "Deleting " + obj_name + "...");
    } /* if */
} /* function del_obj */

function check_del_obj(vsi, event) {
    obj := vcRegion.Get (vsi.files, vscmVT);
    obj_name := browse.obj_name (obj);
    pathname := browse.obj_pathname (obj);

    configs := "";

    if (browse.is_deletable (obj))
"This deletes the directory hierarchy,
" + pathname + ",
associated with the selected object, " + obj_name + ".
All objects dependent on this one will be deleted as well.
For example, all  entities in a library will be deleted,
and all architectures within an entity will be deleted.


Delete " + obj_name + " from the file system?";
    else
        error ("Object cannot be deleted", obj_name);
} /* function check_del_obj */

del_obj_cmd := vcWv.CmdWithConfirm (fac, del_obj, check_del_obj, TRUE,
                                    "del_obj");
vcCmd.SetHelp (del_obj_cmd, "Permanently delete the selected object, and all its decendents from the file system.");

/*
 * This function ensures that the selected item in the files list
 * is a library, and that none of its elements are being browsed.
 * This makes it much easier to move it around.
 */
function collapse_to_library(vsi, row, set_pick) {
    while (!browse.is_library (pick_obj (vsi, row)))
        --row;
    browse.collapse (vsi.files, pick_obj (vsi, row), row);
    if (set_pick)
        vcPicklist.SetPick (vsi.files, row);
    row;
} /* function collapse_to_library */

function set_working(vsi, event) {
    if (vsi.cur_selection != 0) {
        collapse_to_library (vsi, vsi.cur_selection, TRUE);
        obj := cdr (vcPicklist.Get (vsi.files, vsi.cur_selection, 0));
        vcPicklist.DeleteRows (vsi.files, vsi.cur_selection, 1);
        vcPicklist.InsertRows (vsi.files, 0, 1);
        vcPicklist.Add (vsi.files, browse.find_visual (vsi.files, obj), obj);
        vcPicklist.SetPick (vsi.files, 0);
        modify (vsi, TRUE);
        browse.set_working (vsi.files, 1, FALSE);
        browse.set_working (vsi.files, 0, TRUE);
    } /* if */
} /* function set_working */

set_working_cmd := vcWv.Cmd (fac, set_working, "set_working");
vcCmd.SetHelp (set_working_cmd, "Move the selected library to the front of search path, thereby making it the working library");
vcBind.Set (bindings, '\^\enter', set_working_cmd);

/*
 * Dynamic defaulting routine used to set the default for the "save as..."
 * dialog.
 */
function current_init_filename(vsi)
    vsi.init_filename;

/*
 * Reset the icon and window name, and the confirm dialog, to match the
 * new init_filename.
 */
function setup_window(vsi, event) {
    w := vcEvent.Window (event);
    vcWin.SetTitle (w, "Vantage VHDL Library Manager:  " + vsi.init_filename);
    parsed := vsParse (vsi.init_filename);
    vcWin.SetIconName (w, caddr (parsed) + "." + cadddr (parsed));

    vcWin.SetConfirmMessage (w, 
"The initialization file, " + vsi.init_filename + ", has not been saved.
To install the changes to the library search path, use
the `File -> Save Ini File' menu sequence.");
} /* function setup_window */

/*
 * This loops through all the objects in the picklist, and calls the
 * iteration body on each one that's a library.
 */
iterator current_library_objs(vsi) {
    declare row, col;
    n_rows := n_entries (vsi);
    for (r := 0; r < n_rows; ++r) {
        obj := pick_obj (vsi, r);
        if (browse.is_library (obj))
            iterate (obj);
    } /* for */
} /* iterator current_library_objs */

/*
 * Save the current initialization file. no questions asked.
 */
function save_ini(vsi, event) {
    f := fopen (vsi.init_filename, "w");
    foreach (obj; current_library_objs (vsi)) {
        fputline (f, browse.obj_pathname (obj));
        fputline (f, "\n");
    } /* if */
    fclose (f);
    modify (vsi, FALSE);
    vcWin.ClearStatus (vcEvent.Window (event));
    vcWin.AddStatus (vcEvent.Window (event),
                     "Wrote initialization file " +
                     vsAbsPath (vsi.init_filename));
} /* function save_ini */

save_ini_cmd := vcWv.Cmd (fac, save_ini, "save_ini");
vcCmd.SetHelp (save_ini_cmd, "Save the library search order in the initialization file.");

vcBind.Set (bindings, 'S', save_ini_cmd);
vcBind.Set (bindings, 's', save_ini_cmd);


/*
 * Save the init file with a new filename.
 */
function save_as(vsi, event, filename) {
    filename = convert (vlStringVT, filename);
    if (!string_eq (vsi.init_filename, filename)) {
        vsi.init_filename = filename;
        setup_window (vsi, event);
    } /* if */
    save_ini (vsi, event);
} /* function save_as */

save_as_cmd := vcWv.Cmd (fac, save_as, "save_as", "Save Library Init File",
                         vcWv.Param ("", vlFileWriteVT, vcWv.Dynamic,
                                     current_init_filename));
vcCmd.SetHelp (save_as_cmd, "Save the library search order as a new initialization file.");

vcBind.Set (bindings, 'A', save_as_cmd);
vcBind.Set (bindings, 'a', save_as_cmd);

/*
 * Show the .lib files in the current directory in the file selection box.
 */
function set_user(vsi, event) {
    vcFileSelect.SetFilename (vsi.fs, "");
    vcFileSelect.SetPath (vsi.fs, ".");
    vcFileSelect.SetExclude (vsi.fs, nil);
    select_pick_in_fs (vsi, vcRegion.Get (vsi.files, vscmVT));
} /* function set_user */

set_user_cmd := vcWv.Cmd (fac, set_user, "set_user");
vcCmd.SetHelp (set_user_cmd, "Show libraries in current directory in the File Selection Box");

/*
 * For cosimulation, VANTAGE_VSS is set to $WDIR/van_vss.  However,
 * if there is no van_vss underneath one of the directories listed in
 * $WDIR, we use the environment variable VANTAGE_VSS, which the user
 * must set to run VSS 4.2 from Vantage proper (no cosim).  Note that
 * even if the user sets this, if there is a $WDIR/van_vss, we'll use
 * that to do cosimulation.
 *
 * These variables are "global" to this module rather than being part
 * of the "vsi" structure because the analyzer encapsulation needs them
 * as well, and you can invoke the analyzer without invoking vansetup
 * so we couldn't make it a routine that takes a "vsi" as an argument.
 * It probably would be better to make it a routine that takes *no*
 * arguments than to use these global variables, but it's not worth
 * changing right now.
 */
VANTAGE_VSS := vsSrchdirs ("van_vss", vsAttr.Dir);
if (VANTAGE_VSS == nil)
    VANTAGE_VSS = getenv ("VANTAGE_VSS");

/*
 * Likewise, we look for system libraries in VANTAGE_VSS/pgm/libs,
 * however it was we got VANTAGE_VSS set up (see above).
 */
if (VANTAGE_VSS != nil)
    SYS_PATH := vsDnpath (vsDnpath (VANTAGE_VSS, "pgm"), "libs");

/*
 * Finally, set up a new PATH environment variable, and in fact a complete
 * environment variable list for spawning vantage programs.
 */
PATH := vsDnpath (vsDnpath (VANTAGE_VSS, "pgm"), "dir") + ":" +
    getenv ("PATH");
SYS_ENV := list ("PATH=" + PATH);
if (VANTAGE_VSS != nil)
    SYS_ENV = cons ("VANTAGE_VSS=" + VANTAGE_VSS, SYS_ENV);

/*
 * Show the .lib files in the system directory in the file selection box.
 */
function set_system(vsi, event) {
    vcFileSelect.SetFilename (vsi.fs, "");
    vcFileSelect.SetPath (vsi.fs, SYS_PATH);
    vcFileSelect.SetExclude (vsi.fs, list ("styxsynch.lib", "standard.lib",
                                           "vantage.lib", "std.lib"));
    select_pick_in_fs (vsi, vcRegion.Get (vsi.files, vscmVT));
} /* function set_system */

set_system_cmd := vcWv.Cmd (fac, set_system, "set_system");
vcCmd.SetHelp (set_system_cmd, "Show libraries in current directory in the File Selection Box");

if (SYS_PATH == nil)
    vcCmd.Enable (set_system_cmd, FALSE);


function toggle_expand(vsi, event)
    vcWin.WithBusy (vcEvent.Window (event), func()
                    browse.toggle_expand (vsi.files,
                                          vcRegion.Get (vsi.files, vscmVT),
                                          nil););
toggle_expand_cmd := vcWv.Cmd (fac, toggle_expand, "toggle_expand");
vcCmd.SetHelp (toggle_expand_cmd,
               "Toggle expansion of selected object");
vcBind.Set (bindings, '\F5', toggle_expand_cmd);

function view_obj(vsi, event)
    vcWin.WithBusy (vcEvent.Window (event), func()
                    browse.view_obj (vsi.files,
                                     vcRegion.Get (vsi.files, vscmVT)););
view_obj_cmd := vcWv.Cmd (fac, view_obj, "view_obj");
vcCmd.SetHelp (view_obj_cmd, "View the VHDL source for the selected object");
vcBind.Set (bindings, '\F6', view_obj_cmd);

function current_obj(vsi)
    pick_obj (vsi, vsi.cur_selection);

/*
 * Try to select the corresponding path in the file selection
 * box, but only if it's in the same directory that's already
 * displayed -- it takes too long to switch directories.
 */
function select_pick_in_fs(vsi, obj) {
    if ((obj != nil) && (vsi.fs != nil)) {
        obj_pathname := browse.obj_pathname (obj);

        /* DPNE == (Device Path Name Extension) */
        obj_DPNE := vsParse (obj_pathname);
        obj_path := vsMerge (car (obj_DPNE), cadr (obj_DPNE), "", "");

        if (string_eq (vcFileSelect.GetPath (vsi.fs), obj_path))
            vcFileSelect.SetFilename (vsi.fs,
                                      vsMerge ("", "",
                                               caddr (obj_DPNE),
                                               cadddr (obj_DPNE)));
        else
            vcFileSelect.SetFilename (vsi.fs, "");
    } /* if */
} /* function select_pick_in_fs */

/*
 * Core calls _Initial the first time the window is mapped.  We'll
 * take advantage of this to set up things that we need the Window ID
 * for, which we don't have inside _Create -- setting the ConfirmMessage.
 * We also grey the buttons, although we could have done that when we
 * created them.
 */
function _Initial(vsi, event) {
    w := vcEvent.Window (event);

    if (!string_eq (vsi.corrupt_libs, "")) {
        vcWv.Dialog (w, "Corrupt Libraries", 
                     "The following libraries in " + vsi.init_filename +
                     " are corrupt.",
                     vsi.corrupt_libs,
                     vcRegion.Create ("", vcButnRow.fac,
                                      list (list ("dismiss", "Dismiss",
                                                  vcWin.wclose, nil))));
        modify (vsi, TRUE);
    } /* if */
    else
        grey_buttons (vsi);

    setup_window (vsi, event);
} /* function _Initial */

function _ResetPrefs(vsi, region, prefs, modified)
    browse.set_fields (vsi.files, vcRegion.PrefGet (region, "fields"));

function border(r) vcRegion.Create ("", vcBorder.fac, r, rounded <- FALSE);
function margin(r) vcRegion.Create ("", vcMargin.fac, r);

function _Create(lib_filename, region) {
    lib_filename = convert (vlStringVT, lib_filename);
    fs := nil;

    /* Create the file selection box and the picklist */
    if (vcRegion.PrefGet (region, "show_lib_browser"))
        fs = vcRegion.Create ("files", vcFileSelect.fac, "",
                              macro_on <- FALSE,
                              read_now <- TRUE,
                              filter <- "*.lib",
                              edit_filter <- FALSE,
                              traverse_matching_dirs <- FALSE,
                              select_type <- vcFileSelect.Style.Directories,
                              filename_label <- "Library",
                              files_label <- "Available Libraries");
    files := vcRegion.Create ("list", vcPicklist.fac, nil,
                              macro_on <- FALSE,
                              columns <- 1,
                              range <- TRUE,
                              exclusive <- TRUE,
                              will_scroll <- TRUE,
                              enhanced_notify <- TRUE);

    /*
     * Add the pathnames in the ini file to the picklist.  We do a "catch"
     * here because browse.add_lib will raise an exception if something
     * goes wrong in verifying that the library exists.  In a user
     * environment, the only reason an exception should be raised is
     * if a library is missing from the disk, or is corrupt.  When hacking
     * on the viewscript code, however, if there's a programmer error
     * that will raise the same kind of exception, unfortunately, and
     * so the programmer-preference "show_error_detail" controls the
     * addition of the possibly ugly error message with the "corrupt library"
     * alert box.
     */
    corrupt_libs := "";
    foreach (pathname; ini_file_pathnames (lib_filename)) {
        catch (vlEx.FAILURE)
            browse.add_lib (files, nil, pathname);
        exception  {
            corrupt_libs += "\n   " + pathname;
            if (vcRegion.PrefGet (region, "show_error_detail"))
                corrupt_libs += "(" +
                    vlErr.String (vlErr.CurrentError (), vlErrCntx.StatusLine)
                        + ")";
        } /* exception */
    } /* foreach */
    vcPicklist.SetPick (files, vcPicklist.NotSet);

    /* Scroll the picklist -- it's better to do this after it's filled */
    scr := vcPicklist.Scroll3 ("", files, 3,
                               vcRegion.PrefGet (region, "rows"),
                               0,
                               50, 400, 800,
                               vertical <- vcScroll.Presence.Always,
                               horizontal <- vcScroll.Presence.Sometimes);

    /*
     * The picklist and file-select keep each other up to date through
     * notify functions.  But we don't want this to be mutually recursive.
     * There's no danger of crashing thru an infinite loop because the
     * notify functions will stop the infinite recursion, but we don't
     * even want to let it propogate once.  Here's an example of a
     * potential problem.  Suppose my directory is /home/josh/project,
     * but an NFS synonym for that directory is /u/concept/josh/project.
     * If my current directory was /home/josh/project, but I had vsslib.ini
     * entries that looked like /u/concept/josh/project/project.lib, then
     * when I select project.lib in the file-selection box, it will correctly
     * propogate to the picklist, but when it propoages back,
     * select_pick_in_fs will think the pathnames don't match, even though
     * they do, and the system will erase the file-selection pick we just
     * made.  Note the assumption that the notify routine will take 2
     * args.  If this ceases to be true, we'll have to use func().
     */
    propogate_notify := TRUE;
    function notify_call(f, a, b) {
        if (propogate_notify) {
            propogate_notify = FALSE;
            unwind_protect
                f (a, b);
            cleanup
                propogate_notify = TRUE;
        } /* if */
    } /* function notify_call */

    if (fs != nil)
        vcFileSelect.CallOnChange (fs, func(new_path) {
            dev_path_name_ext := vsParse (new_path);
            if (!string_eq (caddr (dev_path_name_ext), "") &&
                string_eq (cadddr (dev_path_name_ext), "lib"))
                notify_call (browse.select_pathname, files, new_path);
        });

    /*
     * Keep track of what's selected in the picklist.  This still
     * has some remnants of the non-exclusive picklist implementation
     */
    vcPicklist.Notify (files, func(obj, on_off, index) {
        if (browse.is_deletable (obj))
            vsi.n_selected = (on_off ? 1 : 0);
        if (browse.is_library (obj))
            vsi.n_libs_selected = (on_off ? 1 : 0);
        if (on_off) {
            vsi.cur_selection = index;
            w := vcRegion.Window (files);
            vcWin.ClearStatus (w);
            vcWin.AddStatus (w, browse.obj_pathname (obj));
            notify_call (select_pick_in_fs, vsi, obj);
        } /* if */
        else
            vsi.cur_selection = -1;
        grey_buttons (vsi);
        foreach (f; vscript.list_iter (vsi.call_on_select))
            f (browse.obj_pathname (obj), on_off);
    });
                                
    /* Set up the double-click handler to the magic browse.toggle_expand */
    vcPicklist.ActivateNotify (files, func(obj, index) {
        event := vcEvent.Create (0, 0, '\0', "", vcRegion.Window (files),
                                 files, nil);
        vcCmd.Run (toggle_expand_cmd, vsi, event, nil, TRUE);
    });

    /* Set up the button row */
    buttons := vcRegion.Create ("buttons", vcButnRow.fac,
        list (list ("add", "Add lib", add_lib_cmd, nil),
              list ("remove", "Remove lib", rem_libs_cmd, nil),
              list ("user", "List user libs", set_user_cmd, nil),
              list ("system", "List system libs", set_system_cmd, nil)));

    if (VANTAGE_VSS == nil)
        vcRegion.Enable (vcButnRow.Get (buttons, 3), FALSE, FALSE);

    if (fs != nil) {
        fs_label := vcRegion.Create ("", vcLabel.fac,
                                     "Double-click on available libraries to add to search order");
        vcRegion.Font (fs_label, vgFont.smallb);

        fs_table := vcTable.Make ("", 2, 1,
                               margin (fs),
                               margin (buttons));

        lib_label := vcRegion.Create ("", vcLabel.fac, "Library Search Order (the working library is boldfaced)");
        vcRegion.Font (lib_label, vgFont.smallb);

        t := vcTable.Make ("", 2, 2,
                           border (margin (fs_label)),
                           border (margin (lib_label)),
                           border (margin (fs_table)),
                           border (margin (scr)));
        vcTable.Col (t, 1, vcTable.Border.Adjustable, 7, 7, 7,
                     vgAlign.MiddleCenter);
        vcRegion.AddChild (region, t);
    } /* if */
    else
        vcRegion.AddChild (region, scr);

    vcRegion.FocusInhibit (region);

    vsi := struct {
        fs = fs;
        n_libs_selected = 0;
        n_selected = 0;
        cur_selection = -1;
        files = files;
        region = region;
        is_modified = FALSE;
        corrupt_libs = corrupt_libs;
        init_filename = lib_filename;
        add_lib_button = vcButnRow.Get (buttons, 0);
        rem_lib_button = vcButnRow.Get (buttons, 1);
        call_on_select = nil
    };
} /* function _Create */
       
function Run() {
    if (!isbound (`RunOnce, vitc))
        load ("vitcUtil.vs", vitc);
    vitc.RunOnce ("vansetup",
                  vsAbsPath ("vsslib.ini"),
                  func() vcWin.Create (vcRegion.Create ("vsslib.ini",
                                                        fac, "vsslib.ini")););
} /* function Run */

/*\Function vansetup.GetSimulationString -- get a design unit spec suitable for passing to Viewsim

DESCRIPTION
     Returns a string "lib:entity(arch)" or "lib:configuration.
     If the library is the working library, "lib:" is eliminated.
     The double-quotes are included in the string so that viewsim's
     and csh's argument parsing won't choke on the parens.

     If the user has selected an entity without an architecture, the
     most recently compiled architecture for the entity is chosen and
     included in the string.
*/
function GetSimulationString(vsi_region) {
    vsi := vcRegion.GetData (vsi_region);
    obj := current_obj (vsi);
    if (obj == nil)
        "";
    else
        browse.simulation_string (vsi.files, obj);
} /* function GetSimulationString */

function CallOnSelect(vsi_region, f) {
    vsi := vcRegion.GetData (vsi_region);
    vsi.call_on_select = cons (f, vsi.call_on_select);
} /* function CallOnSelect */

file_menu := vcWv.MakeMenu ("Files",
                            list ("Save INI File", save_ini_cmd),
                            list ("Save INI As", save_as_cmd),
                            list ("Create Library", create_lib_cmd),
                            list ("Purge library", purge_lib_cmd),
                            list ("Delete design object", del_obj_cmd));

edit_menu := vcWv.MakeMenu ("Edit",
                            vcWv.Pin (),
                            list ("Add library  Enter", add_lib_cmd),
                            list ("Remove library", rem_libs_cmd),
                            list ("Set Working", set_working_cmd));

view_menu := vcWv.MakeMenu ("View",
                            vcWv.Pin (),
                            list ("Show/Hide Entries", toggle_expand_cmd),
                            list ("View Object", view_obj_cmd),
                            list ("User Libraries", set_user_cmd),
                            list ("System Libraries", set_system_cmd));

vcBind.Set (bindings, 'F', vananalz.analyze_src_cmd);
vcBind.Set (bindings, 'f', vananalz.analyze_src_cmd);
vcBind.Set (bindings, 'O', vananalz.analyze_obj_cmd);
vcBind.Set (bindings, 'o', vananalz.analyze_obj_cmd);

analyze_menu := vcWv.MakeMenu ("Analyze",
                               vcWv.Pin (),
                               list ("Analyze File",
                                     vananalz.analyze_src_cmd),
                               list ("Analyze Object",
                                     vananalz.analyze_obj_cmd),
                               list ("Read Errors From File",
                                     vananalz.errors_from_file_cmd));

main_menu := vcWv.MakeMenu ("VanSetup",
                            vcWin.RedSquareEntryNoPlotNoQuit,
                            list ("File", file_menu),
                            list ("Edit", edit_menu),
                            list ("View", view_menu),
                            list ("Analyze", analyze_menu),
                            list ("Preferences", vcWin.PrefsCmd));
vcFac.SetMenus (fac, main_menu);
