/*
 * $Revision: 1.1 $
 *
 * Facility to prompt for design & scope for DRC.  The facility
 * data is:
 *      cons (scope_string, design_spec)
 * where scope_string is one of:
 *      "design", "level", "sheet"
 * and design_spec is either "name.sheet", if scope is "sheet", or
 * "name" otherwise.
 *
 * The design_spec default is dynamically changed based on the scope,
 * and whether the user has edited the name field manually.  We use
 * the environment variables WDESIGN, WBLOCK, and WSHEET to determine
 * what the defaults should be, rather than calling viewdraw API functions,
 * because DRC may be invoked from the cockpit.
 */

radio_buttons :=
    list (list ("Full Design", "design"),
          list ("All sheets on this level, no hierarchy", "level"),
          list ("One sheet, plus all hierarchy underneath", "sheet"));

/*
 * Figures out what the name default should be, given a scope, and
 * an old name, and whether that name has been edited.  If the name
 * has been edited, then this function will return what was passed
 * in, maybe modified to include or not include a sheet specification
 * (e.g. (.5).  If the name has never been edited, this will ingore
 * the name passed in and return a new name based on default sheet,
 * default block, or whatever, depending on the scope.
 */
function recalc_name_from_scope(scope, name, name_has_been_edited) {
    dot := lindex (name, '.');
    new_name := name;

    function getenv_with_default(env_name, default_val) {
        v := getenv (env_name);
        if (v == nil)
            default_val;
        else
            v;
    } /* function getenv_with_default */

    default_sheet := getenv_with_default ("WSHEET", "1");
    current_block := getenv_with_default ("WBLOCK", name);
    current_design := getenv_with_default ("WDESIGN", name);

    /*
     * If it's the sheet, then stick the current sheet number
     * as the default name.
     */
    if (string_eq (scope, "sheet")) {
        if (!dot) {
            if (!name_has_been_edited)
                new_name = current_block + "." + default_sheet;
            else
                new_name = name + "." + default_sheet;
        } /* if */
    } /* if */
    else if (string_eq (scope, "design")) {
        if (name_has_been_edited) {
            if (dot)
                new_name = substring (name, 0, dot);
        } /* if */
        else
            new_name = current_design;
    } /* else if */
    else {              /* must be the "level" */
        if (name_has_been_edited) {
            if (dot)
                new_name = substring (name, 0, dot);
        } /* if */
        else
            new_name = current_block;
    } /* else */
    new_name;
} /* function recalc_name_from_scope */

function _Create(scope_and_name, region) {
    scope := car (scope_and_name);
    name := cdr (scope_and_name);

    scope_region := vcRegion.Create ("scope", vcRadioBox.fac, scope,
                                     data <- radio_buttons);
    name_region := vcRegion.Create ("name", vcCmdline.fac,
                                    recalc_name_from_scope (scope,
                                                            name, FALSE));
    table := vcTable.Make ("", 2, 1, name_region, scope_region);
    vcRegion.AddChild (region, table);


    name_has_been_edited := FALSE;
    vcCmdline.CallOnChange (name_region,
                            func(str) name_has_been_edited = TRUE;);

    vcRadioBox.NotifyOnChange(scope_region, func(event, value) {
        name := vcRegion.Get (name_region, vlStringVT);
        new_name := recalc_name_from_scope (value, name,
                                            name_has_been_edited);
        if (new_name != name) {
            save_edit_flag := name_has_been_edited;
            vcRegion.Set (name_region, new_name);
            name_has_been_edited = save_edit_flag;
        } /* if */
    });
    cons (scope_region, name_region);
} /* function _Create */

function _Get(scope_name_regions, region, ts)
    cons (vcRegion.Get (car (scope_name_regions), vlStringVT),
          vcRegion.Get (cdr (scope_name_regions), vlStringVT));
