vqALL := vqFlag.All;
vqSELECTED := vqAllOrSel.Selected;

ObjInitMoveSelObj := move_center;

/*\Docpage vdraw.DisplayGetViewport -- Return viewport from a display.

DESCRIPTION
    This function returns the viewport associated with the
    display.

    display specifies the input display from which the
    viewport is associated.

EXAMPLE
    vp := DisplayGetViewport(display);

RETURNS
    vgUViewportVT viewport;

SYNOPSIS
    vgUViewportVT DisplayGetViewport(display)
        vdrawVT display;
*/
function DisplayGetViewport(display) {
    declare vp;
    vp = getviewport(display);

    vp;
} /* function DisplayGetViewport */


/*\Docpage vdraw.CompFindAttr -- Locate component attribute with precedence.

DESCRIPTION
    This function locates the named attribute and returns
    the component value if it is located on the component
    or returns the symbol value if found on the symbol but
    not on the component.

    block specifies the block in which the component is
    located.

    comp specifies the component to search.

    attr_name specifies the attribute name string to be
    located.

EXAMPLE
    attr := CompFindAttr(block, comp, attr_name);
    if (attr != nil)
      printf("Found attribute: %s\n", attr);

RETURNS
    vqAttributeVT attribute if found or nil if not found.

SYNOPSIS
    vqAttributeVT CompFindAttr(block, comp, attr_name)
        vqBlockVT block;
	vqComponentVT comp;
	vlStringVT attr_name;
*/
function CompFindAttr(block, comp, attr_name) {
    declare attr;

    attr = ObjFindAttr(block, comp, attr_name);
    if (attr == nil)
	attr = ObjFindAttr(block, CompGetBlock(block, comp), attr_name);
    
    attr;
} /* function CompFindAttr */

/*\Docpage vdraw.OrdinateToGrid -- Snap an ordinate to the current grid

DESCRIPTION
    This function returns the ordinate value which snaps
    an ordinate to the nearest grid space.  The grid is
    obtained from the block and the resultant new
    value is returned.

EXAMPLE
    loc := ObjGetLoc(block, component);
    new_x := OrdinateToGrid(block, LocGetX(loc));
    new_y := OrdinateToGrid(block, LocGetY(loc));
    ObjSetLoc(block, component, new_x, new_y);

RETURNS
    vlIntVT new_ordinate

SYNOPSIS
    vlIntVT OrdinateToGrid(block, ordinate)
        vqBlockVT block;
	vlCoordinateT ordinate;
*/
function OrdinateToGrid(block, ordinate) {
    declare rem,grid,diff;
    grid = vdraw.ParamGetValue(block,vdraw.vqValue.Grid);
    diff = 0;
    
    rem =  ordinate % grid;
    if(rem > 0) {
	diff = grid - rem;
	return(ordinate + diff);
    }
    if(rem < 0){
	diff = grid + rem;
	return(ordinate - diff);
    }
    if (rem == 0) 
	return(ordinate); 
}

/*\Docpage vdraw.BlockSaveSelected -- Save the current selected list

DESCRIPTION
    This function is provided as a convenience for saving
    the list of currently selected objects on a block.
    Since many API function clear or change the selected
    list this insures that the initial list is preserved.

EXAMPLE
    selected_objects := BlockSaveSelected(block);
    foreach(item; vscript.list_iter(selected_objects))
       printf("item: %s\n", item);

RETURNS
    vscmVT list of objects which are currently selected
    on the input block.

SEE ALSO
    BlockRestoreSelected()

SYNOPSIS
    vscmVT BlockSaveSelected(block)
        vqBlockVT block;
*/
function BlockSaveSelected(block) {
    declare selected_list;

    foreach(item; BlockIterObj(block, vqFlag.All, vqAllOrSel.Selected))
	selected_list = cons(item, selected_list);

    selected_list;
} /* function BlockSaveSelected */

/*\Docpage vdraw.BlockRestoreSelected -- Restore the selected list

DESCRIPTION
    This function is provided as a convenience for restoring
    the list of currently selected objects that resulted
    from a call to BlockSaveSelected().

    selected_list specifies a list of objects to be placed
    on the selected objects list for block.

EXAMPLE
    selected_objects := BlockSaveSelected(block);

    foreach(item; vscript.list_iter(selected_objects))
       printf("item: %s\n", item);

    BlockRestoreSelected(block, selected_objects);

RETURNS
    void

SEE ALSO
    BlockSaveSelected()

SYNOPSIS
    void BlockRestoreSelected(block, selected_list)
        vqBlockVT block;
	vscmVT selected_list;
*/
function BlockRestoreSelected(block, slist) {
    BlockDeselectAll(block);

    foreach(item; vscript.list_iter(slist))
	BlockSelectObj(block, item);

} /* function BlockRestoreSelected */


/*\Docpage vdraw.DesignIterBlock -- Design block iterator

DESCRIPTION
    This iterator is provided as a convienence for
    iterating through all blocks in the hierarchy
    down to level(s) or to simply iterate all sheets
    at the same level of hierarchy.

    The flag recurse is used to indicate whether or
    not to traverse down the hierarchy.  If recurse
    is TRUE then the levels string is tested against
    each component instance before traversing down.
    If the flag is FALSE then only sheets at the
    same level of hierarchy are traversed.

    library specifies the library alias to load the design
    from and may be nil.

    name specifies the initial schematic name to start from.

    sheet specifies the sheet number to start with.

    levels specifies a space-delimited string of levels which
    cause the design to stop loading other than at the
    primitive level.  e.g.  levels := "STD VHDL"; would stop
    at the primitive, STD and VHDL levels of hierarchy.
    The levels parameter is ignored if recurse is FALSE.
    
    recurse is a boolean flag indicates whether or not to
    traverse down the hierarchy.    

EXAMPLE (traverse the hierarchy)
    foreach(block; DesignIterBlock(lib, name, sheet, levels, TRUE)) 
      printf("Block: %s\n", block);

    (traverse only sheets, e.g. foo.1 foo.2 .. foo.n

    foreach(block; DesignIterBlock(lib, name, sheet, nil, FALSE)) 
      printf("Block: %s\n", block);
    
ITERATES
    vqBlockVT block;

SYNOPSIS
    foreach(block; DesignIterBlock(library,name,sheet,levels,recurse))
        vqLibBlockNameVT library;
	vqLibBlockNameVT name;
	vqSheetNumVT sheet;
	vlStringVT levels;
	vlTrueFalseVT recurse;
*/
iterator DesignIterBlock(lib, name, sheet, levels, recurse) {
    declare namelist,blocklist;
  
    namelist = nil;
    blocklist = nil;

    function loadsch(lib, name, sheet, levels) {
	declare str, token, level,blist;

	function inlist(name) {
	    foreach(item; vscript.list_iter(namelist)) {
		if (string_eq(name, item))
		    return(TRUE);
	    } /* foreach */
	    return(FALSE);
	} /* function inlist */
  
	function FindAttr(block, comp, attrname) {
	    declare attr;

	    attr = ObjFindAttr(block, comp, attrname);
	    if (attr == nil)
		attr = ObjFindAttr(block, CompGetBlock(block, comp), attrname);
  
	    attr;
	} /* function FindAttr */

	fullname := BlockMakeName(lib, name, sheet);

	if (inlist(fullname) == FALSE) {
	    namelist = cons(fullname, namelist);
	    for(i:=1; i<1000; ++i) {
		if (BlockExists(lib, name, i, "Schematic")) {
		    block := BlockOpen(lib, name, i, "Schematic");

		    blocklist = cons(block, blocklist); /* Stick block into list */

		    if (recurse) {
			foreach(comp; BlockIterObj(block, "Component", "All")) {
			    NewBlock := CompGetBlock(block, comp);

			    stop := FALSE;

			    if (levels != nil) {
				if ((level = FindAttr(block, comp, "LEVEL")) != nil) {
				    str = levels;
				    value := AttrGetValue(block, level);
				    while ((str = string_token(str, &token, " ")) != nil) {
					if (string_eq(value, string_trim(str))) {
					    stop = TRUE;
					    break;
					} /* if */
				    } /* while */
				} /* if */
			    } /* if */
			    
			    if (BlockGetType(NewBlock) == vqBlockType.Composite && stop == FALSE) {
				loadsch(BlockGetLibName(NewBlock), ObjGetName(NewBlock, NewBlock, "TRUE"),
					BlockGetSheet(NewBlock), levels);
			    } /* if */
			} /* foreach */
		    } /* if */
		} else
		    break;
	    } /* for */
	} /* if */
	blist;
    } /* function loadsch */
    
    loadsch(lib,name,sheet,levels);
    
    foreach(block; vscript.list_iter(blocklist))
	iterate(block);
    
    blocklist = nil;
    
} /* iterator DesignIterBlock */


/*\Docpage vdraw.DisplayPushSheet -- Push display to sheet N of schematic

DESCRIPTION
    This function pushes the display to block.N, use
    vdraw.BlockPop to get back.

    display is the input display to be pushed from.

    sheet_number is a positive integer of the sheet number to
    push to.

EXAMPLE
    newblock := BlockPushSheet (display, 2);

RETURNS
    vqBlockVT block;

SYNOPSIS
    vqBlockVT DisplayPushSheet(display, sheet_number)
        vdrawVT display;
        vlPositiveIntVT sheet_number;
*/
function DisplayPushSheet(display, sheet_number) {
   
    win := getwindow (display);
    psheetnoref (display, vcEvent.Create (0, 0, '\0', nil,
                                    win, vcWin.Region (win), nil),
                 sheet_number);
   
    DisplayGetBlock (display);
} /* function DisplayPushSheet */

/*\Docpage vdraw.DesignIterComps -- Design component iterator

DESCRIPTION
    This iterator is provided as a convienence for
    iterating through all components in the hierarchy
    down to level(s) or to simply iterate all components
    across one level of hierarchy.

    The flag recurse is used to indicate whether or
    not to traverse down the hierarchy.  If recurse
    is TRUE then the levels string is tested against
    each component instance before traversing down.
    If the flag is FALSE then only sheets at the
    same level of hierarchy are traversed.

    library specifies the library alias to load the design
    from and may be nil.

    name specifies the initial schematic name to start from.

    sheet specifies the sheet number to start with.

    levels specifies a space-delimited string of levels which
    cause the design to stop loading other than at the
    primitive level.  e.g.  levels := "STD VHDL"; would stop
    at the primitive, STD and VHDL levels of hierarchy.
    The levels parameter is ignored if recurse is FALSE.
    
    recurse is a boolean flag indicates whether or not to
    traverse down the hierarchy.    

EXAMPLE (traverse the hierarchy down to levels VHDL or STD)
    levels := "VHDL STD";
    
    foreach(block, comp; DesignIterComps(display, lib, name, sheet, 
                                             levels, TRUE)) 
       printf("block: %s, comp: %s\n",
         ObjGetName (block, block, FALSE), ObjGetName (block, comp, FALSE));

    (traverse only sheets, e.g. foo.1 foo.2 .. foo.n

    foreach(block, comp; DesignIterComps(display, lib, name, sheet,
                                          nil, FALSE)) 
       printf("block: %s, comp: %s\n",
         ObjGetName (block, block, FALSE), ObjGetName (block, comp, FALSE));
    
ITERATES
    vqBlockVT block;
    vqComponentVT comp;

SYNOPSIS
    foreach(block; DesignIterComps(display, lib, name, sheet, levels, recurse)
        vdrawVT display;
        vqLibBlockNameVT library;
	vqLibBlockNameVT name;
	vqSheetNumVT sheet;
	vlStringVT levels;
	vlTrueFalseVT recurse;
*/
iterator DesignIterComps(display, lib, name, sheet, levels, recurse) {
    function dopaths(display, lib, name, sheet, recurse, levels, depth) {
        for(i:=sheet; i<1000; ++i) {
            if (BlockExists(lib, name, i, vqDataType.Schematic)) {
                if (i == 1 && depth == 0) 
                    block := BlockOpen(lib, name, i, vqDataType.Schematic);
                else 
                    block := DisplayPushSheet (display, i);
                
                foreach(c; BlockIterObj (block, vqFlag.Component,
                                         vqAllOrSel.All)) 
                {   
                    stop_at_level := FALSE;
                    
                    if (!null (levels)) {
                        declare level;
                        declare token;
                        
                        if (!null ((level =
                                    CompFindAttr (block, c, "LEVEL")))) {
                            declare str;

                            value := AttrGetValue (block, level);
                            str = levels;
                            while (!null ((str =
                                           string_token (str, &token, " ")))) {
                                if (string_eqi (value, string_trim (token))) {
                                    stop_at_level = TRUE;
                                    break;
                                } /* if */
                            } /* while */
                        } /* if */
                    } /* if */
                    
                    btype := BlockGetType (CompGetBlock (block, c));
                    if (recurse && !stop_at_level &&
                        btype == vqBlockType.Composite) 
                    {    
                        newblock := BlockPushComp (block, c);
                        dopaths (display,
                                 BlockGetLibName (newblock),
                                 ObjGetName (newblock, newblock, TRUE),
                                 sheet,
                                 recurse,
                                 levels,
                                 ++depth);
                        BlockPop (newblock, 1);
                    } else 
                        iterate (block, c);
                } /* foreach */
                
                if (i > 1)
                    BlockPop (block, 1);
            } else
                break;
        } /* for */
    }

    if (sheet < 1)
        sheet = 1;
    
    dopaths (display, lib, name, sheet, recurse, levels, 0);
} /* iterator DesignIterComps */
