/*
 * $Revision: 1.1 $
 *
 * vflMaster file
 */

/*\vscmuserSection vflMaster. - vfl Master file
 *
 * The vflMaster.vs file contains all of the predefined
 * stuff, such as task classes, and success functions.
 *
 * When this file is loaded (which will be done automatically
 * by any application using the vfl library), the customization
 * files vflus.vs and vflum.vs will also be loaded. This 
 * is where users may add their own classes and functions,
 * and generally have a good time.
 */

/* vflVscript task class expressions are evaluated within this package */
package vflVscriptPkg(Set, in, out) {
    declare in;
    declare out;
    function Set (i, o) {
        in = i;
        out = o;
    }
}

package vflMaster (System, CheckCode, vflVscript, AlwaysPass, PassOnRun,
                   AlwaysFail, PrepareTaskOutput, PrepareTaskError,
                   Subflow,
                   SetLogFunction, SetSearchForFlow, SetAddFlow) {


/* PREDEFINED SUCCESS FUNCTIONS */
/*\vscmuserDocpage vflMaster.CheckCode - success function to check code
 *
 * DESCRIPTION
 *     This success function, registered under the name
 *     'CheckCode', uses the mode and code produced by the task
 *     to determine success. It works like this:
 *
 *                               MODE
 *    CODE                 Exited                 Signaled
 *    ----                 -------                ---------
 *      0                   Completed             error "signalled 0"
 *      other               error "returned ?"     error "signalled ?"
 *      string                              string   
 *      error object                     error object
 *
 * On an Update, the state of the task remains unchanged.
 */
function CheckCode (task, fl, mode, code, in, out, flag) {
    declare ret;

    if (flag == vflTaskCircumstance.Update)
        ret = vflTask.GetState2 (task);
    else if (istype (code, vlStringVT))
        ret = code;
    else if (typeof (code) == vlErrVT)
        ret = vlErr.String (code, vlErrCntx.Stderr);
    else if (mode == vcSysMode.Exited)
        ret = (code == 0 || code == vflTaskState.Completed) ? 
                                vflTaskState.Completed : 
                                sprintf ("Return code: %s", code);
    else /* signalled */
        ret = sprintf ("Signal: %s", code);
    ret;
}
vfl.AddSuccessFunction ("CheckCode", CheckCode); 

/*\vscmuserDocpage vflMaster.AlwaysPass - always green success function
 *
 * DESCRIPTION
 *    This success function, registered under the name 'AlwaysPass'
 *    always succeeds.
 */
function AlwaysPass(task, fl, mode, code, in, out, flag)
    vflTaskState.Completed;
vfl.AddSuccessFunction ("AlwaysPass", AlwaysPass);

/*\vscmuserDocpage vflMaster.PassOnRun -  pass-on-run success function
 *
 * DESCRIPTION
 *    This success function, registered under the name 'AlwaysPass'
 *    always succeeds when the function has been run; it has no
 *    effect on an update.
 */
function PassOnRun(task, fl, mode, code, in, out, flag)
    if (flag == vflTaskCircumstance.Update)
        vflTask.GetState2 (task);
    else
        vflTaskState.Completed;
vfl.AddSuccessFunction ("PassOnRun", PassOnRun);

/*\vscmuserDocpage vflMaster.AlwaysFails - always fail success function
 *
 * DESCRIPTION
 *    This success function, registered under the name 'AlwaysFails'
 *    always fails.
 */
function AlwaysFail(task, fl, mode, code, in, out, flag)
    "This task always fails.";
vfl.AddSuccessFunction ("AlwaysFail", AlwaysFail);

/* used by success function Timestamp to timestamp collections */
define_enum ("vflCollectionTime", list ("AsFile", "Earliest", "Latest"));
function collection_timestamp(obj, coltime) {
    /* coltime != AsFile here */
    declare extreme;
    error := FALSE;
    foreach (mem; vdm.CollLoopMember (obj)) {
        if (!error) {
            t := vdm.ObjCallGeneric (mem, "Timestamp", coltime);
            if (t == nil || t == -1) {
                error = TRUE;
                extreme = t;
            } else if (extreme == nil || 
                    (coltime == vflCollectionTime.Earliest && t < extreme) ||
                    (coltime == vflCollectionTime.Latest && t > extreme))
                extreme = t;
        }
    }
    extreme;
}

/* a function similar to this will have to be registered for each
 * vdm encapsulation for which timestamping is desired.
 *
 * The Timestamp encapsulation should work like this:
 * Return -1 if the file doesn't exist.
 * Return the time if it does exist.
 * It should call collection_timestamp for any collections.
 *
 * This one is used for vdmFile.
 */
function file_timestamp (obj, end, endarg, dir, key, coltime) {
    f := dir + vdm.ObjGetPhysicalName (obj);
    if (vdm.ObjIsCollection (obj) && coltime != vflCollectionTime.AsFile)
        ret := collection_timestamp (obj, coltime);
    else if (vscript.vsExists (f, vsAttr.Both))
        ret := vscript.vsFtime (f);
    else
        ret := -1;
    end (obj, endarg, ret);
}
if (isbound (`vdmFile, nil))
    vdm.EncapCallOnGeneric (vdmFile.Encap, "Timestamp", file_timestamp);

/* timestamp method for vdmRCS */
function rcs_timestamp (obj, end, endarg, rcs, key, coltime) {
    f := rcs.working_area + vdm.ObjGetPhysicalName (obj);
    if (vdm.ObjIsCollection (obj) && coltime != vflCollectionTime.AsFile)
        ret := collection_timestamp (obj, coltime);
    else if (vscript.vsExists (f, vsAttr.Both))
        ret := vscript.vsFtime (f);
    else
        ret := -1;
    end (obj, endarg, ret);
}
if (isbound (`vdmRCS, nil))
    vdm.EncapCallOnGeneric (vdmRCS.Encap, "Timestamp", rcs_timestamp);

/* timestamp method for vdmCVS */
function cvs_timestamp (obj, end, endarg, cvs, key, coltime) {
    f := cvs.working_dir + vdm.ObjGetPhysicalName (obj);
    if (vdm.ObjIsCollection (obj) && coltime != vflCollectionTime.AsFile)
        ret := collection_timestamp (obj, coltime);
    else if (vscript.vsExists (f, vsAttr.Both))
        ret := vscript.vsFtime (f);
    else
        ret := -1;
    end (obj, endarg, ret);
}
if (isbound (`vdmCVS, nil))
    vdm.EncapCallOnGeneric (vdmCVS.Encap, "Timestamp", cvs_timestamp);

/* does all the appropriate timestamp comparsions, and then 
 * returns either Completed or a string explaining failure.
 */
function compare_timestamps(flow, input, inflag, output, outflag) {
    error := FALSE;
    declare input_extreme;
    declare input_obj;
    str := "";
    foreach (data; vscript.iter (input)) {
        elem := vflData.GetVdmObject (data, flow);
        t := vdm.ObjCallGeneric (elem, "Timestamp", inflag); 
        if (t == nil) {
            error = TRUE;
            input_extreme = nil;
            str += sprintf (
                     "Input object %s does not have a timestamp method.\n", 
                                     vdm.ObjGetPhysicalName  (elem));
        } else if (t == -1) {
            error = TRUE;
            input_extreme = -1;
            str += sprintf ("Input object %s does not exist.", 
                                     vdm.ObjGetPhysicalName  (elem));
        } else if (!error && (input_extreme == nil ||
                              t > input_extreme)) {
            input_extreme = t;
            input_obj = elem;
        }
    }
    declare output_extreme;
    declare output_obj;
    foreach (data; vscript.iter (output)) {
        elem := vflData.GetVdmObject (data, flow);
        t := vdm.ObjCallGeneric (elem, "Timestamp", outflag);
        if (t == nil) {
            error = TRUE;
            output_extreme = nil;
            str += sprintf (
                     "Output object %s does not have a timestamp method.\n", 
                                     vdm.ObjGetPhysicalName  (elem));
        } else if (t == -1) {
            error = TRUE;
            output_extreme = -1;
            str += sprintf ("Output object %s does not exist.\n", 
                                     vdm.ObjGetPhysicalName  (elem));
        } else if (!error && (output_extreme == nil ||
                              t < output_extreme)) {
            output_extreme = t;
            output_obj = elem;
        }
    }
    if (!error && input_extreme != nil && output_extreme != nil &&
                  input_extreme > output_extreme) {
        error = TRUE;
        str += sprintf ("Output object %s is older than input object %s",
                           vdm.ObjGetPhysicalName (output_obj),
                           vdm.ObjGetPhysicalName (input_obj));
    }
    (error) ? str : vflTaskState.Completed;
}    
    
/*\vscmuserDocpage vflMaster.Timestamp - determine success with timestamps
 *
 * DESCRIPTION
 *     This success function, registered under the name "Timestamp",
 *     compares the timestamps of the input and output objects. The
 *     task passes if:
 *         1. All input objects exist.
 *         2. All output objects exist.
 *         3. Timestamping is available for all input/output objects.
 *         4. All input objects are older than all output objects.
 *
 *     Timestamping is available for an object if it is stored in a
 *     location whose encapsulation has been given a Timestamp method.
 *     By default, Timestamp methods have been provided for the vdmFile,
 *     vdmRCS, and vdmCVS encapsulations.
 *
 *     This success function handles collections in a deep manner, i.e.,
 *     all members of the collection are timestamped as well, and must
 *     have timestamping available on their encapsulations.
 *
 * SEE ALSO
 *     vflMaster.TimestampShallow
 */
function Timestamp(task, fl, mode, code, in, out, flag)
    compare_timestamps (fl, in,  vflCollectionTime.Latest, 
                            out, vflCollectionTime.Earliest);
vfl.AddSuccessFunction ("Timestamp", Timestamp);

/*\vscmuserDocpage vflMaster.TimestampShallow - determine success with timestamps
 *
 * DESCRIPTION
 *     This function has the same behavior and restrictions as 
 *     vflMaster.Timestamp, except for the way in which collections
 *     are handled. This function timestamps the collection object
 *     itself, and does not examine the collection members.
 *
 *     This success function is registered under the name 
 *     "TimestampShallow".
 *
 * SEE ALSO
 *     vflMaster.Timestamp
 */
function TimestampShallow(task, fl, mode, code, in, out, flag)
    compare_timestamps (fl, in,  vflCollectionTime.AsFile, 
                            out, vflCollectionTime.AsFile);
vfl.AddSuccessFunction ("TimestampShallow", TimestampShallow);


/* PREDEFINED TASK CLASSES */

define_enum ("vflTaskOutputVT", 
     list ("Log", "Vtext", "Terminal", "TermNoPause", "Stdout", "None"));

/*\vscmuserDocpage vflMaster.System - system task class
 *
 * DESCRIPTION
 *     This task class allows system commands to be executed.
 *     Thus, if the task command string is "ls", executing
 *     the task will produce a directory listing.
 *
 *     In addition to the usual global symbol substutions,
 *     the following special symbols are available:
 *         $*  - logical names of inputs
 *         $`* - physical names of inputs
 *         &*  - logical names of outputs
 *         &`* - phyiscal names of outputs
 *
 *     This class is registered with the name 'System.'
 *
 * ATTRIBUTES
 *     HOST - the host on which to execute
 *     OUTPUT - where to send the output to
 *                (default is the Log Window)
 */
function system(task, flow, in, out, complete) {
    /* prepare what we do with the output */
    function stdout(str)
        printf ("%s\n", str);
    function log(str)
        vflFlow.Output (flow, task, str);
    outp := vflTask.GetAttr (task, "OUTPUT");

    /* are we in a core GUI environment? */
    win := isbound (`vcWin, nil);

    declare outproc;
    if (outp == vflTaskOutput.Log)
        outproc = log;
    else if ((!win && outp != vflTaskOutput.None) || 
                     outp == vflTaskOutput.Stdout)
        outproc = stdout;
    else if (outp == vflTaskOutput.Vtext)
        outproc = vcSysOutput.Viewtext;
    else if (outp == vflTaskOutput.Terminal)
        outproc = vcSysOutput.TerminalPause;
    else if (outp == vflTaskOutput.TermNoPause)
        outproc = vcSysOutput.TerminalNoPause;


    str := vflFlow.ReplaceSymbols2 (flow, vflTask.Command (task), in, out);
    host := vflFlow.ReplaceSymbols (flow, vflTask.GetAttr (task, "HOST"));
    if (vfl.Verbose()) {
        e := vlErr.Create (nil, "vfl", 2550, vlErrSev.Note,
                          sprintf ("Running system string: `%s'", str));
        vflFlow.OutputError (flow, task, e);
    }
    if (win)
        vcWin.System (nil, str, nil, FALSE, outproc, log, complete,
                               complete, host);
    else
        vcSys.Command (str, nil, outproc, outproc, complete, host);
}
System := vflTaskClass.Create ("System", system, "CheckCode");
vflTaskClass.AddAttr (System, "HOST", vlStringVT, "");
vflTaskClass.AddAttr (System, "OUTPUT", vflTaskOutputVT, vflTaskOutput.Log);


define_enum ("vflVscriptTypeVT", list ("Function", "Expression"));
/*\vscmuserDocpage vflMaster.vflVscript - viewscript function task class
 *
 * DESCRIPTION
 *     This task class runs a viewscript function. The command
 *     string is expected to be the name under which a vscript
 *     function has been registered with vfl.AddVscriptFunction(man).
 *     Alternately, the string itself can be a vscript
 *     expression. See Attributes.
 *
 *     If the viewscript function returns, the mode passed to the
 *     success function is 'Exited,' and the code is the returned
 *     value. If the viewscript function throws an exception,
 *     the mode is 'Signaled' and the code is the error object.
 *     Some viewscript functions may take a completion
 *     function to which the mode and code are passed by
 *     that function; see vfl.AddVscriptFunction(man).
 *
 *     This class is registered with the name 'vflVscript'.
 *
 * ATTRIBUTES
 *     REPORT_ERR - if function throws an exception, report it
 *                     in addition to using it as Failure code?
 *     TYPE - an alias of a viewscript function, or 
 *              viewscript expression?
 */
function run_vscript(task, flow, in, out, complete) {
    command := vflTask.Command (task);
    declare mode, code;
    is_function := vflTask.GetAttr (task, "TYPE") == vflVscriptType.Function;
    if (is_function) {
        proc := vfl.GetVscriptFunctionEx (vflTask.Command (task));
        nargs := Nargs (proc);
    }

    catch (vlEx.FAILURE) {
        if (!is_function) {
            vflVscriptPkg.Set (in, out);
            code = eval_string (vflTask.Command (task), vflVscriptPkg);
        } else if (nargs == 4)
            code = proc (task, flow, in, out);
        else
            proc (task, flow, in, out, complete);
        mode = vcSysMode.Exited;
    } exception {
        mode = vcSysMode.Signaled;
        code = vlErr.CurrentError ();
        if (vflTask.GetAttr (task, "REPORT_ERR"))
            vflFlow.OutputError (flow, task, code);
    }
    if (!is_function || nargs == 4)
        complete (mode, code);
    /* else rely on the vscript function to complete the task itself */
}
vflVscript := vflTaskClass.Create ("vflVscript", run_vscript, "CheckCode");
vflTaskClass.AddAttr (vflVscript, "TYPE", vflVscriptTypeVT, 
                                         vflVscriptType.Function);
vflTaskClass.AddAttr (vflVscript, "REPORT_ERR", vlYesNoVT, TRUE);

define_enum ("vflSubflowOutputVT", 
               list ("Default", "LogWindow", "Stdout"));
define_enum ("vflSubflowDisplayVT",
               list ("Default", "Switch", "None"));

/* these are used to label output & errors with the task that caused
 * them. They are also used (for similar purpose) by guis.
 */
function PrepareTaskOutput(t, str)
    if (t == nil)
        str;
    else
        sprintf ("Task %s: %s\n", vflTask.Name (t), str);
function PrepareTaskError(t, e) {
    if (t != nil){
        e2 := vfl.ErrorInTask (t, e);
        vlErr.Append (e2, e);
        e2;
    } else
        e;
}

declare LogFunction;
function SetLogFunction(proc)
    LogFunction = proc;

declare SearchForFlow;
function SetSearchForFlow(proc)
    SearchForFlow = proc;
declare AddFlow;
function SetAddFlow(proc)
    AddFlow = proc;

/*\vscmuserDocpage vflMaster.Subflow - subflow task class
 *
 * DESCRIPTION
 *    This task class runs as a task a flow created with ViewFlow.
 *    The command is the vdm object name or file path
 *    to the flow.
 *
 * ATTRIBUTES
 *    LOCATION - vdm location for flow (optional)
 *    OUTPUT - where to output flow
 *               default- use single log window ewntry
 *               logwindow - create a new log window
 *               stdout - write to stdout
 *    DISPLAY - how to display the graphical image 
 *               default - load it
 *               switch - switch to it when flow runs
 *               none - no graphical display
 *    SYMBOLS - graphical symbols to assign subflow
 *                DESIGN=${TOPLEVEL} assigns the
 *                DESIGN symbol to the value of the parent
 *                flow's TOPLEVEL symbol.
 *      
 */
function subflow(task, flow, in, out, complete) {
    name := vflTask.Command (task);
    dm := vflFlow.VdmCntx  (flow);
    class := vdm.CntxFindClass (dm, "vflFlow");
    loc := vfl.Root;

    declare o;
    declare f;
    if (SearchForFlow != nil)
       f = SearchForFlow (name);
    if (f == nil) {
        location := vflTask.GetAttr (task, "LOCATION");
        if (length (location) > 0)
            loc = vpm.MapFindLocation (vdm.CntxGetMap (dm), location);
        if (loc == nil) {
            vflFlow.OutputError (flow, task,
                      e := vlErr.Create (nil, "vfl", 0, vlErrSev.Error,
                           sprintf ("No location defined for alias `%s'", 
                                       location)));
             complete (vcSysMode.Signaled, e);
             return nil;
        }
        o = vdm.ObjCreate (name, vdm.VersLatest, 
                                           class, loc, dm, nil);
        if (length (location) == 0)
            vdm.ObjFind (o, nil, nil);
        f = vflFlow.Read (o, dm);

        output := vflTask.GetAttr (task, "OUTPUT");
        if (output == vflSubflowOutput.Default) {
            vflFlow.CallOnOutput (f, func (t, str)
                             vflFlow.Output (flow, task, 
                                      PrepareTaskOutput (t, str)););
            vflFlow.CallOnOutputError (f, func (t, e)
                             vflFlow.OutputError (flow, task, 
                                            PrepareTaskError (t,e)););
        } else if (output == vflSubflowOutput.LogWindow &&
                    LogFunction != nil)
            LogFunction (flow, task, f);
    } 
    
    vflFlow.CallOnMakeDone (f, 
              func (f, mode, str) 
                      complete (mode,
                                   (str == nil) ? vflTaskState.Completed :
                                                  str););
    if (AddFlow != nil)
        AddFlow (f, vflTask.GetAttr (task, "DISPLAY"), o);

    /* get the global symbol list, doing parent-flow replacement,
     * and then setting the child-flow from that 
     */
    syms := vflFlow.ReplaceSymbols2 (flow, vflTask.GetAttr (task, "SYMBOLS"), in, out);
    vflFlow.ParseGsymString (f, syms);

    /* reset everything in the flow */
    foreach (t; vflFlow.Tasks (f))
        vflTask.Reset (t, f);
    foreach (d; vflFlow.Datas (f))
        vflData.Reset (d);

    vflFlow.Make (f);
}

Subflow := vflTaskClass.Create ("Subflow", subflow, "CheckCode");
vflTaskClass.AddAttr (Subflow, "LOCATION", vlStringVT, "");
vflTaskClass.AddAttr (Subflow, "OUTPUT", vflSubflowOutputVT, 
                                            vflSubflowOutput.Default);
vflTaskClass.AddAttr (Subflow, "DISPLAY", vflSubflowDisplayVT, 
                                            vflSubflowOutput.Default);
define_string ("vflSymbolVT");
vflTaskClass.AddAttr (Subflow, "SYMBOLS", vflSymbolVT, "");

/* load the us and um files */
vcsa.LoadCustomizations ("vfl", vfl);


/********************************************************************/
/* don't modify this stuff below unless you know what you are doing */
/********************************************************************/
function CreateCompleteFunc(t, fl, in, out)
    func (mode, code)
        vflTask.CompleteExec (t, fl, mode, code, in, out);;
vflTaskClass.RegisterCompleteFunc (CreateCompleteFunc);

}







