load("vlibutil.vs");

/*
 * These strings are used as drag data, so that they may be easily
 * differentiated.
 */
define_string ("vllmDragDbVT");

function Update(d, update_obj) {
    w := vcRegion.Window(d.rgnBrowser);
    vcWin.WithBusy(w, func() {
        if (null(update_obj)) {
            obj := d.cntx;
            /* Clear the selected object list */
            vlSym.Clear(d.symSelectedObjs);
            d.n_selected_libs = 0;
            d.n_selected_dbs = 0;
        }
        else
            obj := update_obj;
        vcBrowse.ObjRecalcChildren(d.rgnBrowser, obj, FALSE); 
        enable_cmds(d);
    });
}

function SetContentsRegionDirty(d) {
    rgnViewlib := vcWin.OwnerRegion(vcRegion.Window(d.rgnBrowser));
    vlibtop.SetRegionDirty(rgnViewlib, "Contents");
}


function GetSelectedLib(d, alert) {
    /* Need to find either a db or a lib selected */
    liblist := GetSelectedObjects(d, vllmLibVT);
    dblist := GetSelectedObjects(d, vllmDbVT);
    lib := nil;
    if (length(dblist) > 0) 
        lib = vllm.DbGetLib(car(dblist));
    else if (length(liblist) > 0)
        lib = car(liblist);
    else if (alert)
        vlibThrow(1002, "Either a database or a library must be selected");
    lib;
}

function FillDatabaseHash(d) {
    foreach (lib; vllm.CntxLoopLib(d.cntx)) {
        foreach(db; vllm.LibLoopDb(lib)) {
            dbinfo := struct {
                name     = vllm.DbGetName(db);
                type     = vllm.DbGetType(db);
                location = vllm.DbGetLocation(db);
                filename = vllm.DbGetFileName(db)
            };
            vlSym.Store(d.symDbs, dbinfo.name, dbinfo);
        }
    }
}

function GetDatabaseList(d) {
    dblist := nil;
    foreach (dbname, dbinfo; vlSym.Loop(d.symDbs))
        dblist = cons(cons(dbname, dbname), dblist); 
    dblist;
}

/********************************************************************/
/****************  Database Command Definitions *********************/
/********************************************************************/

cmdDatabaseAdd :=
   vcWv.Cmd (fac, func(d,e,n,l,t,f) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           lib := GetSelectedLib(d, TRUE);
           db := vllm.LibAddDb(lib, n, l, t, f, FALSE);
           oid := vllm.DbGetVdmObj(db);
           <wa, sm> := vdm.ObjGetStatus(oid);
           if (wa == vdm.WorkStatus.Empty) {
               vdm.ObjOpen(oid, vdm.IOMode.Write);
               vdm.ObjClose(oid);
               vdm.ObjPurgeStatus(oid);
           }

           catch (vlEx.FAILURE) {
               vllm.LoadDbFile(d.cntx, db);
           }
           exception {
               vllm.LibDeleteDb(lib, db);
               vllm.LibReload(lib);
               Update(d, lib); 
               vcBrowse.ObjExpand(d.rgnBrowser, lib);
               vlEx.Throw (vlEx.FAILURE, vlErr.CurrentError());
           }           
           dbinfo := struct {
               name     = n;
               type     = t;
               location = l;
               filename = f
           };
           vlSym.Store(d.symDbs, dbinfo.name, dbinfo);
           Update(d, lib); 
           vcBrowse.ObjExpand(d.rgnBrowser, lib);
           SetContentsRegionDirty(d); 
       });
   },  "DatabaseAdd", "Add Database", 
             vcWv.Param("Name", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- "",
                        structure <- GetDatabaseList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Location", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- "",
                        structure <- GetLocationList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Type", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- "",
                        structure <- GetDbClassList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Filename", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- "",
                        structure <- nil,
                        label_fac <- vcCmdline.fac));
vcCmd.EnableApply(cmdDatabaseAdd, TRUE);
vcCmd.CallOnDialogCreate(cmdDatabaseAdd, func(d,e,rgnName,rgnLoc,rgnType,rgnFiles) {
    function reset_picklist (new_region) {
        if (!string_eq(vcRegion.Name(new_region), "FileNames")) {
            class := vcRegion.Get(rgnType, vlStringVT); 
            loc := vcRegion.Get(rgnLoc, vlStringVT);    
            newlist := GetDatabaseFiles(d,class,loc);
            vcRegion.PrefSet(rgnFiles, list (structure <- newlist));
        }
    }
    vcRegion.CallOnFocusChange(rgnType, reset_picklist);
    vcRegion.CallOnFocusChange(rgnLoc, reset_picklist);
});

vcCmd.VerifyParams(cmdDatabaseAdd, func(d,e,n,l,t,f) {
    if (string_eq(string_trim(n), ""))
        vlibThrow(1008, "Database name must not be empty");  
    lib := GetSelectedLib(d, TRUE);
    if (!null(vllm.LibFindDb(lib, n)))
        vlibThrow(1009, "Database `" + n + "' already exists"); 
}); 

function confirmDatabaseDelete(d,e) {
    BuildConfirmRegion(GetSelectedObjects(d, vllmDbVT), 
                       "The following databases will be deleted, OK?",
                       vllm.DbGetName);
}

cmdDatabaseDelete := 
   vcWv.CmdWithConfirm (fac, func(d,e) {
       symLibsAffected := vlSym.StrCreate(3);
       foreach(db; vscript.list_iter(GetSelectedObjects(d, vllmDbVT))) {
           lib := vllm.DbGetLib(db);
           SelectObject(d, db, database_class, FALSE);
           vlSym.Store(symLibsAffected, vllm.LibGetName(lib), lib);
           vlSym.Remove(d.symDbs, vllm.DbGetName(db));
           vllm.LibDeleteDb(lib, db); 
       }
       foreach (libname, lib; vlSym.Loop(symLibsAffected)) {
           vllm.LibReload(lib);
           Update(d, lib); 
       }
       SetContentsRegionDirty(d); },
                        confirmDatabaseDelete, TRUE,
                        "DatabaseDelete", "Database Delete");


function GetSelectedDbName(d) {
    db := GetOneSelectedObject(d, vllmDbVT);
    vllm.DbGetName(db);
}

function GetSelectedDbType(d) {
    db := GetOneSelectedObject(d, vllmDbVT);
    vllm.DbGetType(db);
}

function GetSelectedDbLocation(d) {
    db := GetOneSelectedObject(d, vllmDbVT);
    vllm.DbGetLocation(db);
}

function GetSelectedDbFileName(d) {
    db := GetOneSelectedObject(d, vllmDbVT);
    vllm.DbGetFileName(db);
}

function GetSelectedDbFiles(d) {
    db := GetOneSelectedObject(d, vllmDbVT);
    GetDatabaseFiles(d, vllm.DbGetType(db), vllm.DbGetLocation(db));
}

cmdDatabaseProps :=
   vcWv.Cmd (fac, func(d,e,n,l,t,f,vtypes) {
       db := GetOneSelectedObject(d, vllmDbVT);
       lib := vllm.DbGetLib(db);
       
       /* Remove old db from hash and from lib */
       SelectObject(d, db, database_class, FALSE);
       vlSym.Remove(d.symDbs, vllm.DbGetName(db));
       vllm.LibDeleteDb(lib, db);

       /* Create a new db, add it to hash and lib */
       db := vllm.LibAddDb(lib, n, l, t, f, FALSE);
       dbinfo := struct {
           name     = n;
           type     = t;
           location = l;
           filename = f
       };
       vlSym.Store(d.symDbs, dbinfo.name, dbinfo);

       /* Create the file if it doesn't exist */
       oid := vllm.DbGetVdmObj(db);
       <wa, sm> := vdm.ObjGetStatus(oid);
       if (wa == vdm.WorkStatus.Empty) {
           vdm.ObjOpen(oid, vdm.IOMode.Write);
           vdm.ObjClose(oid);
           vdm.ObjPurgeStatus(oid);
       }

       foreach(vtn; vscript.list_iter(vcExclude.GetExcludeList(vtypes)))
           vllm.DbExcludeVtype(db, vtn);
       vllm.LibReload(lib); 
       Update(d, lib);
       SetContentsRegionDirty(d); },
             "DatabaseProperties", "Database Properties", 
             vcWv.Param("Name", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- GetSelectedDbName,
                        structure <- GetDatabaseList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Location", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- GetSelectedDbLocation,
                        structure <- GetLocationList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Type", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- GetSelectedDbType,
                        structure <- GetDbClassList,
                        label_fac <- vcCmdline.fac),
             vcWv.Param("Filename", vscmVT, vcWv.Dynamic, 
                        func(d) d;, vcPullDown.fac,
                        initial <- GetSelectedDbFileName,
                        structure <- GetSelectedDbFiles,
                        label_fac <- vcCmdline.fac),
             vcWv.Param ("Attributes", vscmVT, vcWv.Dynamic, 
                         func(d) {
                             db := GetOneSelectedObject(d, vllmDbVT);
                             exclist := nil;
                             foreach (n; vllm.DbLoopExcludedVtypeName(db)) 
                                 exclist = cons(n, exclist);
                             inclist := nil;
                             foreach (vtype; vllm.CntxLoopVtype(d.cntx)) {
                                 vtypename := vllm.VtypeGetName(vtype);
                                 if (!member(vtypename, exclist))
                                     inclist = cons(vtypename, inclist); 
                             }
                             inclist = vscript.Sort(inclist, string_gt); 
                             cons(inclist, exclist); },
                         vcExclude.fac));

vcCmd.CallOnDialogCreate(cmdDatabaseProps, func(d,e,rgnName,rgnLoc,rgnType,rgnFiles,rgnVtypes) {
    function reset_picklist (new_region) {
        if (!string_eq(vcRegion.Name(new_region), "FileNames")) {
            class := vcRegion.Get(rgnType, vlStringVT); 
            loc := vcRegion.Get(rgnLoc, vlStringVT);    
            newlist := GetDatabaseFiles(d,class,loc);
            vcRegion.PrefSet(rgnFiles, list (structure <- newlist));
        }
    }
    vcRegion.CallOnFocusChange(rgnType, reset_picklist);
    vcRegion.CallOnFocusChange(rgnLoc, reset_picklist);
});

function MoveDb(d, direction) {
    thedb := GetOneSelectedObject(d, vllmDbVT);
    SelectObject(d, thedb, database_class, FALSE);
    lib := vllm.DbGetLib(thedb);
    db_index := nil; dblist := nil;
    foreach (db; vllm.LibLoopDb(lib)) {
        if (db == thedb)
            db_index = length(dblist);
        dbinfo := struct {
            name     = vllm.DbGetName(db);
            type     = vllm.DbGetType(db);
            location = vllm.DbGetLocation(db);
            filename = vllm.DbGetFileName(db)
        };
        dblist = cons(dbinfo, dblist);
    }
    new_index := db_index + direction;
    dblist = reverse_destructive(dblist);
    if ((new_index >= 0) && (new_index < length (dblist))) {
        the_dbinfo := list_ref(dblist, db_index);
        list_set(dblist, db_index, list_ref(dblist, new_index));
        list_set(dblist, new_index, the_dbinfo);
    }
    vllm.LibClearDbs(lib);
    foreach(dbinfo; vscript.list_iter(dblist))
        db := vllm.LibAddDb(lib, dbinfo.name, dbinfo.location,
                            dbinfo.type, dbinfo.filename, FALSE);
    vllm.LibReload(lib);
    vcBrowse.ObjRecalcChildren(d.rgnBrowser, d.cntx, FALSE);
    /* Expand the library, since recalc on root collapsed everything */
    vcBrowse.ObjExpand(d.rgnBrowser, lib);
    SetContentsRegionDirty(d);
}

cmdDatabaseMoveUp :=
   vcWv.Cmd(fac, func(d,e) {
       MoveDb(d, -1);
   }, "DatabaseMoveUp", "Move Database Up One Entry");
       
cmdDatabaseMoveDown :=
   vcWv.Cmd(fac, func(d,e) {
       MoveDb(d, 1);
   }, "DatabaseMoveDown", "Move Database Down One Entry");
       
cmdDatabaseManage :=
   vcWv.Cmd (fac, func(d,e) {
       db := GetOneSelectedObject(d, vllmDbVT);
       oid := vllm.DbGetVdmObj(db);
       rgnViewlib := vcWin.OwnerRegion(vcRegion.Window(d.rgnBrowser));
       vlibtop.SelectTabAndLocate(rgnViewlib, "Management", oid);
   },
             "DatabaseManage", "Database Manage");

menuDatabase :=
   vcWv.MakeMenu ("Database",
                  vcWv.Pin (),
                  list ("&Add",        cmdDatabaseAdd),
                  list ("D&elete",     cmdDatabaseDelete),
                  list ("&Properties", cmdDatabaseProps),
                  list ("&Manage",     cmdDatabaseManage),
                  vcWv.Separator (),
                  list ("Move&Up",     cmdDatabaseMoveUp),
                  list ("Move&Down",   cmdDatabaseMoveDown));


/* This function enables each command based on which items are selected */
function enable_cmds(d) {

    /* Enable these commands if only one library is selected: */
    vcCmd.Enable (cmdDatabaseAdd,      ((d.n_selected_libs == 1) ||
                                        (d.n_selected_dbs >= 1)));  

    /* Enable these commands if only one database is selected: */
    vcCmd.Enable (cmdDatabaseProps,    d.n_selected_dbs == 1);
    vcCmd.Enable (cmdDatabaseManage,   d.n_selected_dbs == 1);
    vcCmd.Enable (cmdDatabaseMoveUp,   d.n_selected_dbs == 1);
    vcCmd.Enable (cmdDatabaseMoveDown, d.n_selected_dbs == 1);

    /* Enable these commands if at least one database is selected: */
    vcCmd.Enable (cmdDatabaseDelete,   d.n_selected_dbs >= 1);

    /* Update the menus */
    win := vcRegion.Window(d.rgnBrowser);
    if (!null(win)) vcWin.UpdateMenus (win);
}

/********************************************************************/
/****************  Browser Class Definitions ************************/
/********************************************************************/

/*  
 *  Copy of library class defined in vlibBrowse.vs, but it's
 *  children are dbs.
 */
function drag_lib (lib)
    convert (vllmDragLibVT, vllm.LibGetName (lib));

iterator library_children(lib, sub_class) {
    foreach (db; vllm.LibLoopDb (lib))
        iterate (db, database_class);
}

library_class := 
   vcBrowse.ClassCreate ("Library",
                         children      <- library_children,  
                         get_name      <- vllm.LibGetName,
                         drag_data     <- drag_lib,
                         icon          <- "vllmlib",
                         vertex_prefs  <-
                         list (raster_font <- vgFont.small,
                               text_color  <- vgColor.Cyan));

/*  
 *  Database Class
 */
function drag_db (db)
    convert (vllmDragDbVT, vllm.DbGetName (db));

function database_edit(db, notify, class) {
    oid := vllm.DbGetVdmObj(db);
    if (!null(oid)) {
        <wk_status, sm_status> := vdm.ObjGetStatus (oid);
        if ((sm_status != vdm.SMStatus.Uncontrolled) &&
            (wk_status == vdm.WorkStatus.Empty))
            vdm.ObjCheckoutLock(oid);
        vdm.ObjInvokeEditor(oid, nil, nil, func(m,c) notify(); );
    }
}

database_class := 
   vcBrowse.ClassCreate ("Database",
                         get_name      <- vllm.DbGetName,
                         menu          <- menuDatabase,
                         icon          <- "vllmdb",
                         drag_data     <- drag_db,
                         edit          <- database_edit,
                         vertex_prefs  <-
                         list (raster_font <- vgFont.small,
                               text_color  <- vgColor.Red));

function SelectObject(this, obj, class, on_off) {
    add := vlSym.Size(this.symSelectedObjs);
    if (on_off) 
        vlSym.Store (this.symSelectedObjs, obj, class);
    else 
        vlSym.Remove (this.symSelectedObjs, obj);
    add = vlSym.Size(this.symSelectedObjs) - add;
    
    if (class == library_class) 
        this.n_selected_libs = this.n_selected_libs + add;
    else if (class == database_class) 
        this.n_selected_dbs = this.n_selected_dbs + add;
    
    enable_cmds(this);
}

function _Create(vllmcntx, region) {
    this := struct {
        cntx = vllmcntx;
        symSelectedObjs = vlSym.ObjCreate (11); /* all selected objects      */
        n_selected_libs = 0;                    /* number of selected libs   */
        n_selected_dbs  = 0;                    /* number of selected dbs    */
        symDbs          = vlSym.StrCreate (11); /* all databases             */
        rgnBrowser = region
    };
    FillDatabaseHash(this);

    /*  
     *  Context (Search Space) Class
     */
    iterator cntx_children(cntx, sub_class) {
        foreach (lib; vllm.CntxLoopLib (cntx))
            iterate (lib, library_class);
    }
    
    cntx_class :=
        vcBrowse.ClassCreate ("Cntx",
                              parse    <- (func(name) this.cntx;), 
                              get_name <- vllm.CntxGetName,
                              children <- cntx_children);

    this.rgnBrowser = vcRegion.Create("lib_db", vcBrowse.fac,
                                      cons (this.cntx, "Cntx"), 
                                      hide_root <- TRUE, 
                                      exclusive <- FALSE,
                                      drags     <- TRUE,
                                      show_info <- FALSE,
                                      size <- vgSize.Create(200, 100, 200, 60, 
                                                            1000, 1000));
    vcBrowse.CallOnSelect (this.rgnBrowser, func(obj, class, e, on_off) {
        SelectObject(this, obj, class, on_off); });
    vcRegion.AddChild(region, this.rgnBrowser);
    enable_cmds(this);
    this;
}

vcCmd.SetHelp(cmdDatabaseAdd,      "Create database for selected library");
vcCmd.SetHelp(cmdDatabaseDelete,   "Delete selected databases");
vcCmd.SetHelp(cmdDatabaseProps,    "Edit properties of selected database");
vcCmd.SetHelp(cmdDatabaseManage,   "Find this object in Management region");
vcCmd.SetHelp(cmdDatabaseMoveUp,   "Move database earlier in search order");
vcCmd.SetHelp(cmdDatabaseMoveDown, "Move database later in search order");
