/*  vsimColor.vs -- preference sheet for selecting color/icon for navigator
 */


set_value_from_item_picklist := FALSE;

colors := list ( vgColor.Black, vgColor.Blue, vgColor.Green, vgColor.Cyan,
                 vgColor.Red, vgColor.Magenta, vgColor.Brown, vgColor.LightGrey,
                 vgColor.DarkGrey, vgColor.LightBlue, vgColor.LightGreen,
                 vgColor.LightCyan, vgColor.LightRed, vgColor.LightMagenta,
                 vgColor.Yellow, vgColor.White);

icons := list ("vstree", "vsleaf");


function register_color_action_func (region, picklist)
{
    function color_action_func (pair, on_or_off)
	if (equal (on_or_off, vlOnOff.On))
	{
	    this := vcRegion.GetData (region);

	    /*  update item_picklist's selection with the new color
	     */
	    if (! set_value_from_item_picklist)
		change_icon_in_item_picklist (this.item_picklist, nil, pair, this);
	}

    vcPicklist.Notify (picklist, color_action_func);
}


function set_color_picklist_selection (picklist, color_sel)
{
    declare	rows, columns;

    vcPicklist.Dimensions (picklist, &rows, &columns);

    index := 0;

    for (i := 0;  i < rows;  ++i)
	for (j := 0;  j < columns;  ++j)
	{
	    /*  foreach entry in the picklist
	     *  get the color entry
	     *  if the color matches color_sel, set that entry
	     */
	    pair := vcPicklist.Get (picklist, i, j);
	    if (equal (color_sel, cdr (pair)))
	    {
		vcPicklist.SetPick (picklist, index);
		return (TRUE);
	    }
	    ++index;
	}

    FALSE;
}


function create_picklist_from_colors (colors)
{
    picklist := vcRegion.Create ("color-picklist", vcPicklist.fac, nil,
                                exclusive <- TRUE, columns <- length (colors)/2,
                                rows <- 2);

    /*  create an entry in picklist for each color in "colors" list
     */
    foreach (color; vscript.iter (colors))
    {
        color_reg := vcRegion.Create ("x", vcColor.fac, color);
        vcPicklist.Add (picklist, color_reg, color);
    }

    picklist;
}


function register_icon_action_func (region, picklist)
{
    function icon_action_func (pair, on_or_off)
	if (equal (on_or_off, vlOnOff.On))
	{
	    this := vcRegion.GetData (region);

	    /*  update item_picklist's selection with the new icon
	     */
	    if (! set_value_from_item_picklist)
		change_icon_in_item_picklist (this.item_picklist, pair, nil, this);
	}

    vcPicklist.Notify (picklist, icon_action_func);
}


function set_icon_picklist_selection (picklist, icon_sel)
{
    declare	rows, cols;

    vcPicklist.Dimensions (picklist, &rows, &cols);

    index := 0;

    for (i := 0;  i < cols;  ++i)
    {
	/*  foreach entry in the picklist
	 *  get the icon
	 *  if the icon matches icon_sel, set that entry
	 */
	pair := vcPicklist.Get (picklist, 0, i);
	if (string_eq (icon_sel, cdr (pair)))
	{
	    vcPicklist.SetPick (picklist, index);
	    return (TRUE);
	}
	++index;
    }

    FALSE;
}


function create_picklist_from_icons (icons)
{
    picklist := vcRegion.Create ("icon-picklist", vcPicklist.fac, nil,
                                exclusive <- TRUE,
				columns   <- length (icons),
                                rows      <- 1);

    /*  create an entry in picklist for each icon in "icons" list
     */
    foreach (icon; vscript.iter (icons))
    {
        icon_reg := vcRegion.Create ("i", vsimIcon.fac,
				    list (icon, vgColor.White, vgColor.Black));
        vcPicklist.Add (picklist, icon_reg, icon);
    }

    picklist; 
}


function change_icon_in_item_picklist (picklist, icon, color, this)
{
    declare	rows, cols;

    vcPicklist.Dimensions (picklist, &rows, &cols);

    /*  foreach entry in the picklist
     */
    for (--rows;  rows >= 0;  --rows)
    {
	/*  check if the entry is a pick (selection)
	 */ 
	if (vcPicklist.IsSet (picklist, rows))
	{
	    /*  change its color and/or icon of the vsimIcon region.
	     */
	    pair := vcPicklist.Get (picklist, rows, 0);
	    tab  := car (pair);

	    icon_info := vcRegion.Get (vcTable.Get (tab, 0, 1), vscmVT);

	    if (icon  == nil) icon  = icon_info[0];
	    if (color == nil) color = icon_info[1];

	    vcTable.Set (tab, 0, 1, vcRegion.Create ("i", vsimIcon.fac,
			 list (icon, color, vgColor.Black)));
	    
	    vcTable.Col (tab, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
			 vgAlign.MiddleCenter);
	    vcTable.Row (tab, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
			 vgAlign.MiddleCenter);

	    vcRegion.Refresh (tab, vcRegion.Area (tab));


	    /*  call the change_func, when the color value changes
	     */
	    if (this.change_func != nil)
		(this.change_func) ("value");

	    return (TRUE);
	}
    }

    FALSE;
}


function register_item_action_func (region, picklist)
{
    function item_action_func (pair, on_or_off)
	if (equal (on_or_off, vlOnOff.On))
	{
	    this := vcRegion.GetData (region);
	    tab  := pair;

	    icon_info := vcRegion.Get (vcTable.Get (tab, 0, 1), vscmVT);

	    set_value_from_item_picklist = TRUE;
	    /*  update color in color_picklist
	     */
	    set_color_picklist_selection (this.color_picklist, icon_info[1]);

	    /*  update icon in icon_picklist
	     */
	    set_icon_picklist_selection (this.icon_picklist, icon_info[0]);
	    set_value_from_item_picklist = FALSE;
	}

    vcPicklist.Notify (picklist, item_action_func);
}


function add_entries_to_item_picklist (picklist, item_list)
{

    tab_list := nil;

    /*  for each "item" in the picklist
     */
    foreach (item;  vscript.iter (item_list))
    {
	/*  create an entry in the picklist which shows 
	 *  the item-name (item[0]) + an icon with a color (item[2], item[1])
	 */
	tab := vcTable.Create ("tab", 1, 2);
	lab := vcRegion.Create (item[0], vcLabel.fac, item[0]);
	ico := vcRegion.Create (item[0] + "ico", vsimIcon.fac,
				    list (item[2], item[1], vgColor.Black));
	vcTable.Add (tab, lab);
	vcTable.Add (tab, ico);

	vcTable.Col (tab, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
		     vgAlign.MiddleCenter);
	vcTable.Row (tab, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
		     vgAlign.MiddleCenter);

	tab_list = cons (tab, tab_list);
	vcPicklist.Add (picklist, tab, tab);
    }

    vcTable.AlignColumns (tab_list);
}


function create_picklist_from_item_list (item_list)
{
    len := length (item_list);
    p := vcRegion.Create ("ItemPick", vcPicklist.fac, nil,
			  rows		<- len,
			  columns	<- 1,
			  macro_on	<- TRUE,
			  exclusive	<- TRUE);

    add_entries_to_item_picklist (p, item_list);

    p;
}


function _Set (this, item_list)
{
    declare	rows, cols;

    picklist := this.item_picklist;

    vcPicklist.Dimensions (picklist, &rows, &cols);
    vcPicklist.DeleteRows (picklist, 0, rows);
    add_entries_to_item_picklist (picklist, item_list);
    set_value_from_item_picklist = TRUE;
    vcPicklist.SetPick (picklist, 0);
    set_value_from_item_picklist = FALSE;
}


function create_item_list_from_picklist (p)
{
    declare	rows, cols;

    vcPicklist.Dimensions (p, &rows, &cols);

    out_list := nil;
    for (--rows;  rows >= 0;  --rows)
    {
	/*  foreach entry in the picklist
	 *  get the label, from vcLabel region
	 *  get the icon, color, list from vsimIcon region
	 */
	pair := vcPicklist.Get (p, rows, 0);
	label := vcRegion.Get (vcTable.Get (car (pair), 0, 0), vscmVT);
	mylist := vcRegion.Get (vcTable.Get (car (pair), 0, 1), vscmVT);


	/*  add list (label, color, icon) to out_list
	 */
	out_list = cons (list (label, mylist[1], mylist[0]), out_list);
    }
    out_list;
}


vlPref.Add (prefs, "color_list",    vscmVT, "~~list of colors", colors);
vlPref.Add (prefs, "icon_list",     vscmVT, "list of icons",  icons);

vlPref.Parse ("powerview.ini", wv);

function CallOnChange (region, change_func)
{
    if (change_func != nil)
    {
	this := vcRegion.GetData (region);
	this.change_func = change_func;
    }
}


function _Get (this, region, data_type)
    convert (data_type, create_item_list_from_picklist (this.item_picklist));



function _Create (init_data, region)
{
    this := struct {
	main_table;
	item_picklist;
	change_func;
	color_picklist;
	icon_picklist;
	data
    };

    this.data	= init_data;

    main_table := vcTable.Create ("Design Browser Pref Sheet", 1, 1);
    vcTable.Row (main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);
    vcTable.Col (main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);

    color_list		:= vcRegion.PrefGet (region, "color_list");
    icon_list		:= vcRegion.PrefGet (region, "icon_list");

    v_tab := vcTable.Create ("v tab", 3, 1);

    /*  create the color picklist
     */
    outer_tab := vcTable.Create ("ct", 1, 2);
    colors_lab := vcRegion.Create ("x", vcLabel.fac, "Colors: ");
    vcRegion.Font (colors_lab, vgFont.mediumb);
    vcTable.Add (outer_tab, colors_lab);

    this.color_picklist = create_picklist_from_colors (color_list);
    register_color_action_func (region, this.color_picklist);
    vcTable.Add (outer_tab, this.color_picklist);
    b := vcRegion.Create ("a", vcBorder.fac, outer_tab);
    vcTable.Add (v_tab, b);

    /*  create the icon picklist
     */
    outer_tab := vcTable.Create ("ct", 1, 2);
    icons_lab := vcRegion.Create ("x", vcLabel.fac, "Icons: ");
    vcRegion.Font (icons_lab, vgFont.mediumb);
    vcTable.Add (outer_tab, icons_lab);

    this.icon_picklist = create_picklist_from_icons (icon_list);
    register_icon_action_func (region, this.icon_picklist);
    vcTable.Add (outer_tab, this.icon_picklist);
    b := vcRegion.Create ("b", vcBorder.fac, outer_tab);
    vcTable.Add (v_tab, b);

    /*  add a rubber for nicer layout
     */
    b := vcRubber.Create ("r", 0, 0);
    vcTable.Add (v_tab, b);

    vcTable.Row (v_tab, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);
    vcTable.Col (v_tab, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);

    h_tab := vcTable.Create ("h tab", 1, 2);

    /*  create the item picklist
     */
    outer_tab := vcTable.Create ("ct", 2, 1);
    nodes_lab := vcRegion.Create ("x", vcLabel.fac, "Design Browser Nodes: ");
    vcRegion.Font (nodes_lab, vgFont.mediumb);
    vcTable.Add (outer_tab, nodes_lab);

    this.item_picklist = create_picklist_from_item_list (init_data);
    register_item_action_func (region, this.item_picklist);
    vcTable.Add (outer_tab, this.item_picklist);
    b := vcRegion.Create ("c", vcBorder.fac, outer_tab);

    vcTable.Add (h_tab, b);
    vcTable.Add (h_tab, v_tab);

    vcTable.Add (main_table, h_tab);

    vcRegion.AddChild (region, main_table);


    this.main_table = main_table;

    this;
}

