/* ---------------------------------------------------------------------- *\
 * Module -- vtime.vs
 *
 *      Facility:      Vtime
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Mani Alagappan,  Dec 1, 1993
 *                     Jeff Hare
 *
 *      Description:   vtime facility vs file
 *       
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section Commands -- Command interface to the vtime facility
*/

load ("vtvutils.vs");

vcFac.Create("tracedlg", vscmVT);
vcFac.Create("vtimevtm", vscmVT);
vcFac.Create("visiterr", vscmVT);
vcFac.Create("paramtrs", vscmVT);
vcFac.Create("path",vscmVT);
vcFac.Create("vtimemcp",vscmVT);

vlPref.Replace (prefs, "checkstm", vlOnOffVT,
            "Enable Auto-Correction of .stm filename parameters",
            vlOnOff.On);

vlPref.Replace (prefs, "capslock", vcCmdlineCaseVT,
            "Fold character case to Upper, Lower or Either (mixed)",
            vcCmdlineCase.Upper);

vlPref.Parse("",wv);

/*===========================================================================*/
/*==                      E r r o r M e s s a g e s                        ==*/
/*===========================================================================*/
/*
   The error messages listed here are stored in a cons cell with the car() 
   being the error number and the cdr() being the help string.  Remember,
   do not renumber any of these messages!  The numbers will be tied to both
   the documentation and the vtime-err.vhb help text file.
*/
msg_CmdFileNExist := cons(2000, "Unable to find specified command file [%s]");
msg_DspRgnNExist  := cons(2001, "Unable to locate the display region named [%s]");
msg_InfoRgnNExist := cons(2002, "Unable to locate the Info display region named [%s]");
msg_BadVrepName   := cons(2003, "Unable to determine the Verification Report filename");
msg_BadTrepName   := cons(2004, "Unable to determine the Trace Report filename");
msg_BadVarFname   := cons(2005, "Unable to determine the %s filename.");
msg_SameCaseName  := cons(2006, "New case name '%s' must be different from current case '%s'.");
msg_CaseDirNCreated := cons (2007, "Unable to create Case Directory '%s'.");
msg_FileNOpened   := cons (2008, "Unable to open file '%s' for '%s'.");
msg_FileNCopied   := cons (2009, "Unable to copy file '%s' to file '%s'.");
msg_NoDesignLoaded:= cons (2010, "Issue Verify, Trace or DesignLoad command first.");
msg_NoVfyReport   := cons (2011, "No previous Verification Report Exists.\n   Searching for: [%s]\n   Issue a Verify command first.");
msg_MustRestartFirst:= cons (2012, "A design is currently loaded, Issue RestartMtv first.");
msg_VsFileNExist := cons(2013, "Unable to open specified ViewScript file [%s]");
msg_VMpfUnavailable := cons(2014, "Must perform a verify first.");
msg_NoIddFileName := cons(2015, "IDD option is selected, but no IDD file name value was specified.");
msg_VMpfNoErrsReported := cons(2016, "No Timing Errors Were Reported.");
msg_CannotCreateMcp := cons(2017, "Unable to create an MCP file. Check permissions.");

/*===========================================================================*/
/*==                      A l e r t B o x ( )                              ==*/
/*===========================================================================*/
/*\Function AlertBox  -- Post an alert box for the specified condition      

DESCRIPTION
     This function throws an exception posting the specified message,
     severity and error number.  Since this throws an exception, no
     code will execute after calling this function.

     Note that this function also posts the error to the transcript via
     the "remark" command for trackability.

EXAMPLE
     function foo(d,e)
     {
        ...
        
        AlertBox(d, e, 1234, vlErrSev.Error, "Foo error detected");
     }

SYNOPSIS
     AlertBox( data,       ; vtime facility data                
               event,      ; core event object                  
               msgnum,     ; message number for help navigation 
               severity,   ; vlErrSev. severity                 
               message     ; Text of message to post            
             );
*/
function AlertBox (d, e, msgnum, severity, message) 
{
   vtq.CmdRun(GetQueue(d),e,"remark **************");
   vtq.CmdRun(GetQueue(d),e,"remark " + vlErr.SeverityToString (severity) +"-" +
              convert (vlStringVT, msgnum)+ ":" + message);
   vtq.CmdRun(GetQueue(d),e,"remark **************");
   errobj := vlErr.Create(nil, "vtime", msgnum, severity, message);
   vlEx.Throw(vlEx.FAIL, errobj);
}



/*===========================================================================*/
/*==                      C m d C r e a t e ( )                            ==*/
/*===========================================================================*/
/*\Function CmdCreate -- Create a command execution queue compatible command

DESCRIPTION
     This function must be used to create commands instead of vcWv.Cmd().
     The commands created by this function are guaranteed to execute in
     the proper sequence when external executables such as Motive are 
     involved.  This also ensures that Macros and command files are properly
     executed.

EXAMPLE

     The following call creates a command named "checkstm" which takes 
     no arguments.

     <cmdCheckStm,prvChkStm> := CmdCreate(vtime.fac,
                                     ckstm_function,
                                     "checkstm",
                                     "Checks the validity of a .stm file", nil);

     The following call creates a command named "execute" which takes a 
     single filename argument.  If the argument is not specified, then
     a dialog box with a browse function is provided.

     <cmdExecute,prvCmdExec> := CmdCreate(vtime.fac,
                                  ex_function,
                                  "Execute",
                                  "Execute a ViewTime command file.", 
                                  list (vcWv.Param ("Command Filename", 
                                                    vlStringVT, 
                                                    vcWv.Sticky, "./tim/states/dynamic",
                                                    vcFilePop.fac,
                                                    filter <- "*.cmd",
                                                    file_columns <- 1,
                                                    dir_columns <- 1)));
RETURNS

     The return value from CmdCreate is a list of the two new core command
     elements which were created as a result of this command.  The first 
     element is the public command object which can be bound to menus and
     keypresses.  The second command element is the internal private core
     command which is normally not used.

     Use the syntax:  <pubcmd,privatecmd> := CmdCreate(...);

*/
function CmdCreate(Facility, ActionRoutine, PublicName, Description, CmdParams)
{
   
   PrivateName   := "_" + PublicName;

   /*--------------------------------------------------------*/
   /* vtmSubmit is the action routine invoked by ALL public  */
   /* commands.  It adds the full command line to the Queue  */
   /* pending execution.                                     */
   /*--------------------------------------------------------*/
   function vtmSubmit (arglist...) {
      declare a;
      
      data   := car(arglist);
      arglist = cdr(arglist);
      event  := car(arglist);
      arglist = cdr(arglist);

      command := PublicName;
      foreach (arg; vscript.list_iter(arglist)) 
         command += " " + convert (vlStringVT,arg);

      vtq.Add(GetQueue (data), event, command);
   }
   
   if (CmdParams != nil) {
       public  :=  vcCmd.Create (Facility,
                          vtmSubmit,
                          PublicName,
                          Description,
                          CmdParams);
   
       private := vcCmd.Create(Facility,
                          ActionRoutine,
                          PrivateName,
                          Description,
                          CmdParams);
   }
   else {
      public  := vcWv.Cmd(Facility,
                          vtmSubmit,
                          PublicName,
                          Description);
   
      private := vcWv.Cmd(Facility,
                          ActionRoutine,
                          PrivateName,
                          Description);
   }      
   
   vcCmd.EnableMacro(public, vlOnOff.On);
   vcCmd.SetHelp    (public, Description);

   vcCmd.EnableMacro(private, vlOnOff.Off);
   vcCmd.SetHelp    (private, Description);

   cmds := list(public, private);
   cmds;
}



/*===========================================================================*/
/*==                      C m d C r e a t e N o Q u e u e ( )              ==*/
/*===========================================================================*/
/*\Function CmdCreateNoQueue -- Create a command which does not get queued

DESCRIPTION
     This function must be used to create commands instead of vcWv.Cmd().
     The commands created by this function have macro recording enabled
     and command help text assigned.  This is a convenience function.

EXAMPLE

     The following call creates a command named "stop" which takes 
     no arguments.

     cmdCheckStm := CmdCreateNoQueue(vtime.fac,
                                     stop_function,
                                     "stop",
                                     "Stops something", 
                                     nil);

     The following call creates a command named "stop_it" which takes a 
     single filename argument.  If the argument is not specified, then
     a dialog box with a browse function is provided.

     cmdExecute := CmdCreateNoQueue(vtime.fac,
                                  stop_function,
                                  "stop_it",
                                  "Stop something specific"", 
                                  list (vcWv.Param ("Its Filename", 
                                                    vlStringVT, 
                                                    vcWv.Sticky, "./its/directory/",
                                                    vcFilePop.fac,
                                                    filter <- "*.cmd",
                                                    file_columns <- 1,
                                                    dir_columns <- 1)));
RETURNS

     The return value from CmdCreateNoQueue is a single core command object just
     like vcWv.Cmd

*/
function CmdCreateNoQueue(Facility, ActionRoutine, CmdName, Description, CmdParams)
{
   if (CmdParams != nil) {
       public  :=  vcCmd.Create (Facility,
                          ActionRoutine,
                          CmdName,
                          Description,
                          CmdParams);
   }
   else {
      public  := vcWv.Cmd(Facility,
                          ActionRoutine,
                          CmdName,
                          Description);
   }      
   
   vcCmd.EnableMacro(public, vlOnOff.On);
   vcCmd.SetHelp    (public, Description);

   public;
}


/*===========================================================================*/
/*==                      M t v                                            ==*/
/*===========================================================================*/
/*\Docpage Mtv -- Directly invoke a Motive command

DESCRIPTION
     This command is way important.  It is the sole command interface to
     the Motive engine.  All communication with the the Motive engine must 
     go through this command.  Null Motive commands are ignored. If a
     "sv" or "scv" command is detected it is followed by a "gv" command
     for important internal reasons.  Do not attempt to bypass this.

*/
function mtv(d, event, cmd)  SendCmd (d, event, cmd);
<cmdMtv,prvCmdMtv> := CmdCreate(vtime.fac,
                                mtv,
                                "Mtv",
                                "Send a command directly to Motive",
                                list (vcWv.Param ("Motive Command",
                                                  vlStringRestOfLineVT, 
                                                  vcWv.Constant, "")));


/*===========================================================================*/
/*==                    R e s t a r t M t v                                ==*/
/*===========================================================================*/
/*\Docpage RestartMtv -- Shut down and restart the Motive timing engine.

DESCRIPTION
     This function is used to reinitialize the Motive timing engine. The 
     command "restartmtv" accepts one argument which indicates whether a new 
     parameter file (stm file) should be rewritten.  If the argument is the 
     enumerated type Yes,  then the new stm file will be written.
     If the argument is the enumerated type No, then the stm file is NOT 
     rewritten regardless of whether the environment has changed.  If No
     arguments are specified then a dialog box will prompt the user whether
     to save any changes regardless of whether the environment has changed 
     since the design was loaded.

NOTES
     This is a recursive command.  If the caller requests that the parameter
     file be saved prior to restarting,  then save command is run, followed
     by the an invocation of this command again except with the "no" option
     to actually shut down and restart the engine.

RETURN
     notta..

*/
function restartmotive(d, e, arg) 
{
   if (arg == vlYesNo.Yes) {
      vtq.CmdRun(GetQueue(d), e, "Save");
      vtq.CmdRun(GetQueue(d), e, "RestartMtv No");
   }
   else {
      MtvStart(d);
      ResetConsList (d);
      vtq.CmdRun(GetQueue(d), e, "current Transcript");
   }
}
<cmdRestart,prvCmdRestart> := vtime.CmdCreate(vtime.fac,
                                              restartmotive,
                                              "RestartMtv",
                                              "Start/Restart Motive",
                                              list (vcWv.Param ("Save Current Parameters?",
                                                                vlYesNoVT, 
                                                                vcWv.Sticky,
                                                                vlYesNo.Yes)));

/*===========================================================================*/
/*==                       S a v e                                         ==*/
/*===========================================================================*/
/*\Docpage Save -- Save all important data pertaining to this case

DESCRIPTION
     This command is used to write all 'interesting' case specific data to 
     disk.  It eventually will provide a vcNotify hook which will allow      
     customer written functions to be called when a save command is invoked.

RETURN
     notta..

*/
function savecase(d, e) 
{
   vtq.CmdRun (GetQueue (d), e, "current Transcript");
   vtq.CmdRun(GetQueue(d), e, "ConsSave");
   vtq.CmdRun(GetQueue(d), e, "McpSave");
   if (IsMtvSysAlive (d) == vlTrueFalse.True)
      vtq.CmdRun(GetQueue(d), e, "mtv save");
}
<cmdSave,prvCmdSave> := vtime.CmdCreate(vtime.fac,
                                        savecase,
                                        "Save",
                                        "Save current state of this case to disk",
                                        nil);

/*===========================================================================*/
/*==                    E x e c u t e                                      ==*/
/*===========================================================================*/
/*\Docpage Execute -- Execute a viewtime batch file.

DESCRIPTION
     This function parses and executes commands contained in a Viewtime
     command file. The verbose option allows each and every command in the
     batch file to be printed to the transcript window as comments.

CAUTIONS
     Blank lines are ignored.
     Lines beginning with '|' are echoed to the transcript but not executed.
     No recursion protection is in place yet so DON'T execute a command file
     within a command file which could cause recursion...

RETURN
     notta..

*/

function execute(d,e, execfile, verbose) {
   declare commandline;
   declare cmd;
   declare len;

   file := FndFileInHierarchy(d,execfile);
   if (fexists(file)) {
      fp := fopen(file, "r");
      
      while ((commandline=fgetline(fp)) != nil) {
         string_token(commandline, &cmd, " \n\t");
         if (cmd != nil) {
            if (string_ref(cmd, 0) != '|') {
               if (string_eqi (verbose, "") || string_eqi (verbose, "yes"))
                  vtq.CmdRun(GetQueue(d), e, "remark --> " + commandline);
               vtq.CmdRun(GetQueue(d), e, commandline);
            }
         }
      }
      fclose(fp);
   }
   else {
      msg := sprintf(cdr(msg_CmdFileNExist),execfile);
      AlertBox(d,e,car(msg_CmdFileNExist),vlErrSev.Error,msg);
   }
} /* execute */

<cmdExecute,prvCmdExec> := CmdCreate(vtime.fac,
                                  execute,
                                  "Execute",
                                  "Execute a ViewTime command file.", 
                                  list (vcWv.Param ("Command Filename", 
                                                    vlStringVT, 
                                                    vcWv.Sticky, "./tim/states/dynamic",
                                                    vcFilePop.fac,
                                                    filter <- "*.cmd",
                                                    file_columns <- 1,
                                                    dir_columns <- 1),
                                        vcWv.Param ("Verbose Output",
                                                    vlStringRestOfLineVT,
                                                    vcWv.Sticky,
                                                    "yes")));




/*===========================================================================*/
/*==                C h e c k S t m                                        ==*/
/*===========================================================================*/
/*\Docpage CheckStm   -- Check for valid input/output filenames in Motive. 

DESCRIPTION
     This command verifies that the filenames in the .stm file are valid and
     that output filenames are writable.  Since this command operates on the
     Vtime var table, it assumes that a gv_all will have been done prior to 
     update the table.  If the table has not been updated,  then this command
     will submit a "mtv gv_all *" followed by a "checkstm" command.

RETURN
     notta..

*/
function checkstm( d, e, doit ) {

   ok := TRUE;

   if (doit != nil && string_eqi(doit,"Yes") && vlPref.Val(prefs,"checkstm") == vlOnOff.Off)
      ok = FALSE;

   /* Engine must be running and with no design currently loaded */
   if (IsMtvSysAlive(d)) {
      Ready := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
      if (string_eqi (Ready, "TRUE")) {
         ok = FALSE;
         msg := cdr(vtime.msg_MustRestartFirst);
         vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
      }
   }
   else
      ok = FALSE;

   
   if (ok) {

      Block := GetBlockName (d);
      Case  := GetCaseName (d);
      Rpath := vtime.GetCaseRelPath(d);
      Queue := vtime.GetQueue (d);
      
      declare Key;
      declare NewVal;
      declare Val;
      declare Var;
      declare Ext;
      declare x;
      declare pdevice;
      declare ppath;
      declare pname;
      declare pext;
      declare dot;
      declare Fnm;

      McpFile := "";

      changes := 0;
      
      MustBeValid := list(list(MtvVar.MV_file_bsi,nil,"bsi"),
                          list(MtvVar.MV_file_rct,nil,"rct"),
                          list(MtvVar.MV_file_idd,nil,"idd")
                          );
      
      Required := list(list(MtvVar.MV_file_alg,nil,"alg"),
                       list(MtvVar.MV_file_ctl,nil,"ctl"),
                       list(MtvVar.MV_file_gsr,nil,"gsr"),
                       list(MtvVar.MV_file_hst,nil,"hst"),
                       list(MtvVar.MV_file_llg,nil,"llg"),
                       list(MtvVar.MV_file_mcp,nil,"mcp"),
                       list(MtvVar.MV_file_mod,nil,"mod"),
                       list(MtvVar.MV_file_net,nil,"pin"),
                       list(MtvVar.MV_file_ref,nil,"ref"),
                       list(MtvVar.MV_file_rep,nil,"rep"),
                       list(MtvVar.MV_file_stm,nil,"stm"),
                       list(MtvVar.MV_file_tim,nil,"tim"),
                       list(MtvVar.MV_file_trc,nil,"trc"),
                       list(MtvVar.MV_file_tmp_info,"info","tmp"),
                       list(MtvVar.MV_file_tmp_int,"mtv_int","tmp"),
                       list(MtvVar.MV_file_tmp_trace,"trace","tmp"),
                       list(MtvVar.MV_file_tmp_wave,"wave","tmp"),
                       list(MtvVar.MV_file_tmp_mpf_report,"report","mpf"),
                       list(MtvVar.MV_file_tmp_mpf_trace,"trace","mpf"),
                       list(MtvVar.MV_iam_capp_file_loc,nil,"loc"),
                       list(MtvVar.MV_iam_cappf_file_loc,nil,"loc"),
                       list(MtvVar.MV_iam_xline_file_loc,nil,"loc")
                       );
      
      vtq.CmdRun (Queue, e, "Current Transcript");
      vtq.CmdRun (Queue, e, "Remark Verifying Filenames loaded from STM file.");

      foreach (x ; vscript.list_iter(Required)) {
         Key = list_ref(x,0);
         Fnm = list_ref(x,1);
         Ext = list_ref(x,2);
         Val = GetMtvVar(d, Key, MtvValue.VALUE);
         Var = GetMtvVar(d, Key, MtvValue.NAME);
         
         if (FileIsInAnotherCase(d,Block,Case,Val) || Fnm != nil) {
            
            /* Make the path point to the current case.*/
            <pdevice,ppath,pname,pext> = vscript.vsParse(Val);
            
            
            /*-------------------------------------------*/
            /* If a filename is specified in the "list", */
            /* then this must be used.                   */
            /*-------------------------------------------*/
            if (Fnm != nil) {
               pname = Fnm;
               if (Ext == nil)
                  pext = "";
               else
                  pext  = Ext;
            }
            
            /* if No extension, don't add trailing "." to the filename" */
            if (string_eqi(pext,""))
               dot = "";
            else
               dot = ".";
            
            NewVal = ResolveFilePathName(d, pname + dot + pext, Ext);
         }
         else 
            NewVal = ResolveFilePathName(d, Val, Ext);
         
         if (Key == MtvVar.MV_file_mcp) 
            McpFile = NewVal;
         
         if (!string_eqi(Val,NewVal)) {
            vtq.CmdRun (Queue, e, "Remark STM Parameter: " + Var + "     New Value: " + NewVal);
            vtq.CmdRun (Queue, e, "SV " + Var + " " + NewVal);
            changes = changes + 1;
         }
      }
      if (changes > 0)
         vtq.CmdRun (Queue, e, "Remark CheckStm Finished.  Issue Save to preserve these changes.");
      else
         vtq.CmdRun (Queue, e, "Remark CheckStm Finished.  No parameter changes made.");
      

      if (!vtime.vsExists(McpFile,vsFILE)) {

/*         catch (vlEx.FAILURE) { */
            mcpfp   := fopen (McpFile, "w");
            fclose(mcpfp);
/*         }
           exception {
            msg := sprintf(cdr(vtime.msg_CannotCreateMcp));
            vtime.AlertBox(d,e,car(vtime.msg_CannotCreateMcp),vlErrSev.Error,msg);
           }
*/
      } /* end if */

   }
}
<cmdCheckStm,prvChkStm> := CmdCreate(vtime.fac,
                                     checkstm,
                                     "Checkstm",
                                     "Checks the validity of filenames in the .stm file", 
                                     list(vcWv.Param ("Use checkstm preference?",
                                                 vlStringRestOfLineVT,
                                                 vcWv.Constant,
                                                 "")));


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function ThrowIfNoDesignLoaded -- Throw if no design is loaded
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function ThrowIfNoDesignLoaded(d, event)
{
   TblStateVal := GetMtvVar(d, MtvVar.MV_mtv_version, MtvValue.VALUE); 
   StmStateVar := GetMtvVar(d, MtvVar.VT_file_status, MtvValue.NAME);

   if (TblStateVal == nil) {  /* No gv_all has been run yet... */
      AlertBox(d,event,car(msg_NoDesignLoaded),vlErrSev.Warning,cdr(msg_NoDesignLoaded));
   }
} /* ThrowIfNoDesignLoaded */


/*===========================================================================*/
/*==                          V e r i f y                                  ==*/
/*===========================================================================*/
/*\Docpage Verify -- Perform a complete circuit verification in Motive

DESCRIPTION
     This command invokes a Motive "verify" command to perform a complete
     circuit timing verification.

     The command does the following:
     ===============================
        * determine whether Motive is ready to verify a design.
        * mtv verify
        * update report file display.

*/
function verify(d, event) {
   Queue := GetQueue(d);
   vtq.CmdRun (Queue, event, "mtv verify");
   vtq.CmdRun (Queue, event, "vrepfile");
}
<cmdVerify,prvCmdVfy> := CmdCreate(vtime.fac,
                                   verify,
                                   "Verify",
                                   "Perform a complete circuit constraints verification",
                                   nil);


/*===========================================================================*/
/*==               D i s p l a y C m d C r e a t e                         ==*/
/*===========================================================================*/
/*\Function DisplayCmdCreate -- Create a display command for a display region **OBSOLETE???

DESCRIPTION
     This function is called when a new display region is registered with
     vtime.  Its purpose is to create a "show_<rgn_name>" command which
     simply makes the specified region visible in the display region if
     it is not resident in its own window. (ie: not peeled off) 

CAUTION
     It should not be casually called or repeatedly called by any other user
     command since it creates a new command which cannot be deleted.  This
     new command will be visible to the user.  This routine must only be
     called by the the display region registration functions.

EXAMPLE

     DisplayCmdCreate("MyTestResults");

*/
function DisplayCmdCreate(name)
   <cmdpub, cmdpriv> := CmdCreate(vtime.fac,
                                  func(d, event){
                                     vtq.CmdRun (GetQueue(d), event, 
                                                 "display " +  name);
                                     /* printf ("Adding command 'show_%s'\n", name); */
                                  },
                                  "show_"+name,
                                  "Make the " + name + " visible in the display window",
                                  nil);

/*===========================================================================*/
/*==                      D i s p l a y                                    ==*/
/*===========================================================================*/
/*\Docpage Display -- Show the specified region as the current display region

DESCRIPTION
     This command is used in conjunction with the results display region.
     Any regions registered with the vtime display facility are candidates
     regions for this command.

BUGS
     This requires that you know the Registration Name of the region you
     want to view.  These names are the ones presented in the pulldown
     button above the results display region.

CAUTION
     Since the display() function (which is the command's action routine)
     is also used by the DisplayCmdCreate() function,  do not change the
     argument requirements without first checking with that function.
 
EXAMPLE

     display "Transcript"

*/
function display(d, event, display_name)
{
   if (string_eq(display_name,"")) display_name = "transcript";
   r := DisplayGetRgnByName (d, display_name);
   if (r != nil)
      DisplaySetCur (d, r);
   else {
      msg := sprintf(cdr(msg_DspRgnNExist),display_name);
      AlertBox(d,event,car(msg_DspRgnNExist),vlErrSev.Warning,msg);
   }
}
<cmdDisplay,prvCmdDisplay> := CmdCreate(vtime.fac,
                                  display,
                                  "Display",
                                  "Select what to view in the results display region", 
                                        list (vcWv.Param ("Display Name", vlStringRestOfLineVT, 
                                                          vcWv.Constant, "transcript")));

/*===========================================================================*/
/*==                      U p d a t e                                      ==*/
/*===========================================================================*/
/*\Docpage Update -- Command to call a particular display region's update method

DESCRIPTION
     This command calls the particular display region's update function. 
     This routine was registered when the display region was handed to vtime
     to manage by the results display facility.

EXAMPLE
     update "MyTestResults"

*/
function update(d, event, display_name)
{
   r := DisplayGetRgnByName (d, display_name);
   if (r != nil)
      DisplayUpdate (d, display_name);
   else {
      msg := sprintf(cdr(msg_DspRgnNExist),display_name);
      AlertBox(d,event,car(msg_DspRgnNExist),vlErrSev.Warning,msg);
   }
}
<cmdUpdate,prvCmdUpdate> := CmdCreate(vtime.fac,
                                  update,
                                  "Update",
                                  "Update a display region's contents", 
                                   list (vcWv.Param ("Display name", vlStringVT, 
                                                     vcWv.Constant, "")));

/*===========================================================================*/
/*==                      V r e p f i l e                                  ==*/
/*===========================================================================*/
/*\Docpage Vrepfile -- Display the current verification report

DESCRIPTION
     This command causes the latest verification report to be loaded and
     displayed to the user.  If the VReport display region is not peeled
     off in a separate window, then it is made the currently visible 
     region in the display window.

EXAMPLE

     vrepfile

*/
function vrepfile(d, event)
{
   VrepDspName := "VReport";

   repfile    := GetMtvVar(d, MtvVar.MV_file_rep, MtvValue.VALUE);
   repfileRgn := DisplayGetRgnByName(d, VrepDspName);
   if (repfile != nil && repfileRgn != nil) {
      viewtxt := vcRegion.Data(repfileRgn, vtxVT);
      if (viewtxt != nil) {
         /* Note that we must make the buffer writable to reload */
         vtx.ReadOnly(viewtxt,FALSE); 
         vtx.Load(viewtxt, repfile);
         vtx.ReadOnly(viewtxt,TRUE);
         vcRegion.Refresh (repfileRgn, vcRegion.Area (repfileRgn));
         vtq.CmdRun (GetQueue (d), event, "current " + VrepDspName);
      }
   }
   else {
      ThrowIfNoDesignLoaded(d, event);
      if (repfile == nil) {
         AlertBox(d,event,car(msg_BadVrepName),vlErrSev.Warning,cdr(msg_BadVrepName));
      }
      else {
         msg := sprintf(cdr(msg_DspRgnNExist),VrepDspName);
         AlertBox(d,event,car(msg_DspRgnNExist),vlErrSev.Warning,msg);
      }
   }
}
<cmdVrepfile,prvCmdVrepfile> := CmdCreate(vtime.fac,
                                          vrepfile,
                                          "vrepfile",
                                          "Show the latest Verification Results", 
                                          nil);


/*===========================================================================*/
/*==                      P e e l                                          ==*/
/*===========================================================================*/
/*\Docpage Peel -- Move the Specified display(s) out of the display region   

DESCRIPTION
     This command will cause the top (currently visible) display region to
     be removed from the results display and displayed in its own window
     if no arguments are specified.  If one or more display names are 
     specified, then those display regions are moved to their own windows.
     Upon closure of any window containing a peeled display,  the display 
     region is returned to the results display overlay.  Note that if a
     peeled display is selected in the display picklist,  its window is 
     closed and it returns to the display overlay.
*/
function peel(d, event, names) DisplayPeel (d, event, names);
<cmdPeel,prvCmdPeel> := CmdCreate(vtime.fac,
                                  peel,
                                  "Peel",
                                  "Peel the currently displayed result into a separate window.", 
                                  list (vcWv.Param ("Region(s) to be peeled:",
                                                    vlStringRestOfLineVT, 
                                                    vcWv.Constant, "")));



/*===========================================================================*/
/*==                      M e r g e                                        ==*/
/*===========================================================================*/
/*\Docpage Merge -- Move all peeled display regions back to the results display

DESCRIPTION
     This command is the opposite of peel except that it returns ALL regions
     previously peeled to the results display.  This has the affect of closing
     all results display windows.  This then makes them available in the 
     results display picklist.

*/
function merge(d, event, names) DisplayMerge (d, event, names);
<cmdMerge,prvCmdMerge> := CmdCreate(vtime.fac,
                                    merge,
                                    "Merge",
                                    "Close all peeled regions, returning them to the main window.", 
                                    list (vcWv.Param ("Region(s) to be merged:",
                                                      vlStringRestOfLineVT, 
                                                      vcWv.Constant, "")));



/*===========================================================================*/
/*==                       V i e w F i l e                                 ==*/
/*===========================================================================*/
/*\Docpage ViewFile -- Display a file in a readonly ViewText window

DESCRIPTION
     This command causes the specified file to be loaded into its own
     ViewText window for viewing.


NOT COMPLETED...
     Should use a preference to bring a file up in the user specified editor.
*/
function ViewFile (d,e,str)  {
   fname := FndFileInHierarchy (d, str);
   if (string_eqi (fname,"") == FALSE){
      vt := vtx.CreateWindow (fname, TRUE);
      vtx.ReadOnly(vt, vlTrueFalse.True);
      msg := sprintf ("Note: Viewing file '%s'...", fname);
      status_msg (e,msg);
      vtq.CmdRun (GetQueue (d), e, "Remark " + msg);
      /*Log(d,msg,vlTrueFalse.True);*/
   }
   else
      status_msg (e,sprintf ("Error: File '%s' does not exist in search hierarchy",str));
}
<cmdViewFile,prvCmdViewFile> := CmdCreate(vtime.fac,
                                   ViewFile,
                                   "ViewFile",
                                   "File Viewer",
                                   list (vcWv.Param ("", 
                                                     string, 
                                                     STICKY, 
                                                     "",
                                                     vcFileSelect.fac)));


/*===========================================================================*/
/*==                       E d i t F i l e                                 ==*/
/*===========================================================================*/
/*\Docpage EditFile -- Display a file in a editable ViewText window

DESCRIPTION
     This command causes the specified file to be loaded into its own
     ViewText window for viewing/editing.


NOT COMPLETED...
     Should use a preference to bring a file up in the user specified editor.
*/
function EditFile (d,e,str)  {
   fname := GetFileInHierarchy (d, str);
   vtx.CreateWindow (fname, TRUE);
   msg := sprintf ("Note: Editing file '%s'...", fname);
   status_msg (e,msg);
   vtq.CmdRun (GetQueue (d), e, "Remark " + msg);
   /*Log(d,msg,vlTrueFalse.True);*/

}
<cmdEditFile,prvCmdEditFile> := CmdCreate(vtime.fac,
                                   EditFile,
                                   "EditFile",
                                   "File Editor",
                                   list (vcWv.Param ("", 
                                                     string, 
                                                     STICKY, 
                                                     "",
                                                     vcFileSelect.fac)));


/*===========================================================================*/
/*==                      A l g f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Algfile -- Display the current Aborted Paths Log File

DESCRIPTION
     This command causes the latest Aborted Paths Log File to be displayed
     in its own window.

EXAMPLE

     algfile

*/
function algfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_alg, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE)
      ViewFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Aborted Paths (.alg)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdAlgfile,prvCmdAlgfile> := CmdCreate(vtime.fac,
                                          algfile,
                                          "Algfile",
                                          "Show the latest Aborted Paths Log File", 
                                          nil);



/*===========================================================================*/
/*==                      B s i f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Bsifile -- Edit the current Boolean States file

DESCRIPTION
     This command causes the latest Boolean States File to be displayed
     in its own window.

EXAMPLE

     bsifile

*/
function bsifile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_bsi, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Boolean States (.bsi)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdBsifile,prvCmdBsifile> := CmdCreate(vtime.fac,
                                          bsifile,
                                          "Bsifile",
                                          "Edit the latest Boolean States File", 
                                          nil);



/*===========================================================================*/
/*==                      C t l f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Ctlfile -- Edit the MMP Control File

DESCRIPTION
     This command causes the latest MMP Control file to displayed
     in its own window.

EXAMPLE

     ctlfile

*/
function ctlfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_ctl, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE)
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"MMP Control File (.ctl)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdCtlfile,prvCmdCtlfile> := CmdCreate(vtime.fac,
                                          ctlfile,
                                          "Ctlfile",
                                          "Edit the latest MMP Control File", 
                                          nil);


/*===========================================================================*/
/*==                      G s r f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Gsrfile -- Edit the latest Global Slack Report

DESCRIPTION
     This command causes the latest Global Slack Report file to displayed
     in its own window.

EXAMPLE

     gsrfile

*/
function gsrfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_gsr, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Global Slack Report File (.gsr)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdGsrfile,prvCmdGsrfile> := CmdCreate(vtime.fac,
                                          gsrfile,
                                          "Gsrfile",
                                          "Edit the Global Slack Report File", 
                                          nil);


/*===========================================================================*/
/*==                      I d d f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Iddfile -- Edit the Interconnect Delay Data File

DESCRIPTION
     This command causes the latest Interconnect Delay Data file to displayed
     in its own window.

EXAMPLE

     iddfile

*/
function iddfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_idd, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Interconnect Delay Data (.idd)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdIddfile,prvCmdIddfile> := CmdCreate(vtime.fac,
                                          iddfile,
                                          "Iddfile",
                                          "Edit the latest Interconnect Delay Data File", 
                                          nil);


/*===========================================================================*/
/*==                      M c p f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Mcpfile -- Edit the Multi-Cycle Path (MCP) File

DESCRIPTION
     This command causes the latest Multi-Cycle Path (MCP) file to displayed
     in its own window.

EXAMPLE

     mcpfile

*/
function mcpfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_mcp, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Multi-Cycle Paths (.mcp)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdMcpfile,prvCmdMcpfile> := CmdCreate(vtime.fac,
                                          mcpfile,
                                          "Mcpfile",
                                          "Edit the latest Multi-Cycle Paths File", 
                                          nil);


/*===========================================================================*/
/*==                      L l g f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Llgfile -- Edit the Logged Loops (llg) File

DESCRIPTION
     This command causes the latest Logged Loops file to displayed
     in its own window.

EXAMPLE

     llgfile

*/
function llgfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_llg, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Logged Loops (.llg)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdLlgfile,prvCmdLllfile> := CmdCreate(vtime.fac,
                                          llgfile,
                                          "Llgfile",
                                          "Edit the latest Logged Loops File", 
                                          nil);



/*===========================================================================*/
/*==                      M o d f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Modfile -- Edit the Compiled Models File

DESCRIPTION
     This command causes the Compiled Models file to displayed
     in its own window.

EXAMPLE

     modfile

*/
function modfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_mod, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Compiled Models (.mod)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdModfile,prvCmdModfile> := CmdCreate(vtime.fac,
                                          modfile,
                                          "Modfile",
                                          "Edit the latest MMP compiled Models File", 
                                          nil);



/*===========================================================================*/
/*==                      P i n f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Pinfile -- View the Design Pinlist File

DESCRIPTION
     This command causes the latest Design Pinlist file to displayed
     in its own window.

EXAMPLE

     pinfile

*/
function pinfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_net, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      ViewFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Design Pinlist (.pin)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdPinfile,prvCmdPinfile> := CmdCreate(vtime.fac,
                                          pinfile,
                                          "Pinfile",
                                          "Show the latest Design Pinlist File", 
                                          nil);


/*===========================================================================*/
/*==                      H s t f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Hstfile -- View the Session History Log file

DESCRIPTION
     This command causes the latest Session History log file to be shown
     in its own window.

EXAMPLE

     hstfile

*/
function hstfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_hst, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      ViewFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Session History Log (.hst)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdHstfile,prvCmdHstfile> := CmdCreate(vtime.fac,
                                          hstfile,
                                          "Hstfile",
                                          "Show the Session History Log File", 
                                          nil);



/*===========================================================================*/
/*==                      R c f f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Rcffile -- Edit the MMP Revised Control File

DESCRIPTION
     This command causes the latest MMP Revised Control file to displayed
     in its own window.

EXAMPLE

     rcffile

*/
function rcffile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_rcf, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"MMP Revised Control File (.rcf)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdRcffile,prvCmdRcffile> := CmdCreate(vtime.fac,
                                          rcffile,
                                          "Rcffile",
                                          "Edit the latest MMP Revised Control File", 
                                          nil);



/*===========================================================================*/
/*==                      R e f f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Reffile -- Edit the Design's Clock Reference File

DESCRIPTION
     This command causes the latest Design Clock Reference file to displayed
     in its own window.

EXAMPLE

     reffile

*/
function reffile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_ref, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Circuit Reference (.ref)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdReffile,prvCmdReffile> := CmdCreate(vtime.fac,
                                          reffile,
                                          "Reffile",
                                          "Edit the Design's Clock Reference File", 
                                          nil);


/*===========================================================================*/
/*==                      R e p f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Repfile -- Edit the Latest Verification Report File

DESCRIPTION
     This command causes the latest Design Verification Report file to displayed
     in its own window.

EXAMPLE

     repfile

*/
function repfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_rep, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE)
      if (fexists(file))
         EditFile(d,e,file);
      else {
         msg := sprintf(cdr(msg_NoVfyReport),file);
         AlertBox(d,e,car(msg_NoVfyReport),vlErrSev.Warning,msg);
      }
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Design Verification Report (.rep)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdRepfile,prvCmdRepfile> := CmdCreate(vtime.fac,
                                          repfile,
                                          "Repfile",
                                          "Edit the Latest Design Verification Report File", 
                                          nil);


/*===========================================================================*/
/*==                      S t m f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Stmfile -- Edit the Motive Parameter File (use caution!)

DESCRIPTION
     This command causes the latest Motive Parameter file to displayed
     in its own window.

EXAMPLE

     stmfile

*/
function stmfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_stm, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Circuit Parameters (.stm)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdStmfile,prvCmdStmfile> := CmdCreate(vtime.fac,
                                          stmfile,
                                          "Stmfile",
                                          "Edit the latest MMP Control File", 
                                          nil);



/*===========================================================================*/
/*==                      T i m f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Timfile -- Edit the Motive Generated Design Timing File

DESCRIPTION
     This command causes the latest Motive Generated Timing file to displayed
     in its own window.

EXAMPLE

     timfile

*/
function timfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_tim, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Generated Timing Model (.tim)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdTimfile,prvCmdTimfile> := CmdCreate(vtime.fac,
                                          timfile,
                                          "Timfile",
                                          "Edit the latest Motive Generated Design Timing File", 
                                          nil);



/*===========================================================================*/
/*==                      T r c f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Trcfile -- View the Current Trace Log File

DESCRIPTION
     This command causes the latest Trace Log file to displayed
     in its own window.

EXAMPLE

     trcfile

*/
function trcfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_trc, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      ViewFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"Trace Log (.trc)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdTrcfile,prvCmdTrcfile> := CmdCreate(vtime.fac,
                                          trcfile,
                                          "Trcfile",
                                          "Edit the latest Trace Log File", 
                                          nil);



/*===========================================================================*/
/*==                      L o c f i l e                                    ==*/
/*===========================================================================*/
/*\Docpage Locfile -- Edit the current X,Y location File

DESCRIPTION
     This command causes the latest Component X,Y location file to displayed
     in its own window.

EXAMPLE

     locfile

*/
function locfile(d, e)
{
   file    := GetMtvVar(d, MtvVar.MV_file_loc, MtvValue.VALUE);

   if (file != nil && string_eqi("",file) != TRUE) 
      EditFile(d,e,file);
   else {
      ThrowIfNoDesignLoaded(d, e);
      msg := sprintf(cdr(msg_BadVarFname),"X-Y Location (.loc)");
      AlertBox(d,e,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdLocfile,prvCmdLocfile> := CmdCreate(vtime.fac,
                                      locfile,
                                      "Locfile",
                                      "Edit the current component X,Y location File", 
                                      nil);


/*===========================================================================*/
/*==                         D s t a t                                     ==*/
/*===========================================================================*/
function dstat(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_monitor_dstat, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   if (val == vtimeEnableDisable.Enable)
      vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                  " Verbose Runtime Reporting (Caution! Very Verbose)");
   else
      vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                  " Detailed Runtime Reporting.");

}
<cmdDstat,prvCmdDstat> := CmdCreate(vtime.fac,
                                    dstat,
                                    "Dstat",
                                    "Enable/Disable Detailed Execution Status",
                                    list (vcWv.Param ("Detailed Status",
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Sticky, 
                                                      vtimeEnableDisable.Disable)));


/*===========================================================================*/
/*==                          S t e p S i z e                              ==*/
/*===========================================================================*/
/*\Docpage StepSize -- Set the resolution of the timing calculations performed

DESCRIPTION
     This command invokes a Motive "sv" command to set the resolution of 
     the timing verification engine in (ns/analysis unit).  A typical unix
     default setting is 0.0001ns/step.  This is normally a platform specific
     setting.


*/
function stepsize(d, event, step) {
   Queue := GetQueue(d);
   
	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);

	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,event,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {
      res    := GetMtvVar(d, MtvVar.MV_timebase_resolution, MtvValue.NAME);

      vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,step));
      vtq.CmdRun (Queue, event, "remark Stepsize resolution set to " 
                  + convert(vlStringVT,step)+"ns/analysis unit");
   }
}
<cmdStepSize,prvCmdStepSize> := CmdCreate(vtime.fac,
                                          stepsize,
                                          "StepSize",
                                          "Set the resolution of the timing calculations (ns)",
                                          list (vcWv.Param ("Motive Timebase Resolution",
                                                            vlPositiveFloatVT,
                                                            vcWv.Constant, "0.0001")));


/*===========================================================================*/
/*==                          C y c l e                                    ==*/
/*===========================================================================*/
/*\Docpage Cycle -- Set the major clock period cycle time for the analysis  

DESCRIPTION
     This command invokes a Motive "sv" command to set the clock period of
     the timing verification cycle time in nanoseconds.  This is a design
     specific setting.


*/
function cycle(d, event, val) {
   Queue := GetQueue(d);
   
	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);

	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,event,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {
      res    := GetMtvVar(d, MtvVar.MV_timebase_cycle_time, MtvValue.NAME);

      vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
      vtq.CmdRun (Queue, event, "remark Analysis Cycle Time set to " 
                  + convert(vlStringVT,val) + "ns/cycle");
   }
}
<cmdCycle,prvCmdCycle> := CmdCreate(vtime.fac,
                                    cycle,
                                    "Cycle",
                                    "Set the Major Cycle Time for Timing Verification (ns)",
                                    list (vcWv.Param ("Major Cycle Time",
                                                      vlPositiveIntVT,
                                                      vcWv.Sticky, 1)));


/*===========================================================================*/
/*==                          S l a c k                                    ==*/
/*===========================================================================*/
/*\Docpage Slack -- Enable/Disable slack information in the Trace report      

DESCRIPTION
     This command invokes a Motive "sv" command to enable or disable the 
     slack time reporting information in a path trace report.  Slack info
     is not yet available in the graphical path trace display. Motive 4.1


*/
function slack(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_trace_slack, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Slack Information in the Trace Report");

}
<cmdSlack,prvCmdSlack> := CmdCreate(vtime.fac,
                                    slack,
                                    "Slack",
                                    "Enable/Disable Slack reporting in the Trace Report",
                                    list (vcWv.Param ("Trace File Slack Reporting",
                                                      vtimeEnableDisableVT,
                                                      vcWv.Constant, vtimeEnableDisable.Enable)));


/*===========================================================================*/
/*==                              Scount                                   ==*/
/*===========================================================================*/
/*\Docpage Scount -- Set the maximum slack pin count for slack command

  DESCRIPTION
       This number is the number of addtional component path to be displayed
       when the slack report is tunred on during trace analysis.
*/
function scount(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_trace_slack_max, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, 
               sprintf ("remark Slack Pin Count Information set to %d",val));
}
<cmdScount,prvCmdScount> := CmdCreate(vtime.fac,
                                    scount,
                                    "Scount",
                                      "Slack Pin Count",
                                      list (vcWv.Param ("Number",
                                                        vlPositiveIntVT,
                                                        vcWv.Sticky, 5)));

/*===========================================================================*/
/*==                          D e t a i l                                  ==*/
/*===========================================================================*/
/*\Docpage Detail -- Enable/Disable Detailed IDD effects in the Trace Report

DESCRIPTION
     This command invokes a Motive "sv" command to enable or disable the 
     Detailed Electrical Interconnect Delay information in the Trace Report.


*/
function detail(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_trace_detailed_elec, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Detailed electrical mode (for interconnect) in the Trace Report");

}
<cmdDetail,prvCmdDetail> := CmdCreate(vtime.fac,
                                      detail,
                                      "Detail",
                                      "Enable/Disable Detailed interconnect info in the Trace Report",
                                      list (vcWv.Param ("Trace File Detailed Interconnect Reporting",
                                                        vtimeEnableDisableVT,
                                                        vcWv.Constant, vtimeEnableDisable.Enable)));


/*===========================================================================*/
/*==                       E x p l a n a t i o n s                         ==*/
/*===========================================================================*/
/*\Docpage Explanations -- Enable/Disable Trace Report Explanation Mode

DESCRIPTION
     This command invokes a Motive "sv" command to enable or disable the 
     Explanation Mode in the Trace Report.


*/
function expl(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_trace_expl_mode, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Explanation mode in the Trace Report");

}
<cmdExpl,prvCmdExpl> := CmdCreate(vtime.fac,
                                    expl,
                                    "Explanation",
                                    "Enable/Disable Explanation Mode in Trace Report",
                                    list (vcWv.Param ("Verbose Trace Report", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Enable)));


/*===========================================================================*/
/*==                          L o o p l o g g i n g                        ==*/
/*===========================================================================*/
/*\Docpage LoopLogging -- Enable the writing of the design.llg loop log file 

DESCRIPTION
     This command invokes a Motive "sv" command to enable or disable the 
     creation of a log file of detected loops.

*/
function looplogging(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_loop_log, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
               " Logging of Detected Signal Loops"); 
}
<cmdLoopLogging,prvCmdLoopLogging> := CmdCreate(vtime.fac,
                                                looplogging,
                                                "LoopLogging",
                                                "Enable/Disable the logging of detected signal loops",
                                                list (vcWv.Param ("Log Signal Loops",
                                                                  vtimeEnableDisableVT,
                                                                  vcWv.Constant, vtimeEnableDisable.Enable)));


/*===========================================================================*/
/*==                       C l k g a t i n g                               ==*/
/*===========================================================================*/
/*\Docpage ClockGating -- Enable/Disable timing effects of gating terms

DESCRIPTION
     This command invokes a Motive "sv" command to enable or disable the 
     gated clocks mode.  When enabled, timing effects all gating terms are
     included in the analysis, and clocks are propagated only on delays
     having "A" or "O" modifiers specified (in the model).  Enabling this
     mode can lead to pessimism.

*/
function clkgat(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_clock_gating, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   if (val == vtimeEnableDisable.Enable)
      vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                  " Clock Gating effects (may cause pessimism)");
   else
      vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                  " Clock Gating effects");

}
<cmdClkgat,prvCmdClkgat> := CmdCreate(vtime.fac,
                                    clkgat,
                                    "ClockGating",
                                    "Enable/Disable Clockgating",
                                    list (vcWv.Param ("Clockgating", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                         M t v q u i t                                 ==*/
/*===========================================================================*/
/*\Docpage Mtvquit -- Shutdown the Motive Engine

DESCRIPTION
   This command issues a "quit" command to Motive.  The default action when 
   Motive shuts down is to restart itself.  This command differs from the
   "restart" command in that restart provides the option of saving data prior
   to shutting down Motive.
   

*/
function mtvquit(d,event)	vtq.CmdRun (GetQueue (d),event,"mtv quit");
<cmdMtvquit,prvCmdMtvquit> := CmdCreate(vtime.fac,
                                        mtvquit,
                                        "mtvquit",
                                        "Shutdown Motive",nil);

/*===========================================================================*/
/*==                    Queue Commands                                     ==*/
/*===========================================================================*/
/*\Docpage Queue -- ViewTime commands for controlling the command queue

DESCRIPTION
   The command execution queue manages the sequential execution of 
   ViewTime facility commands.  Some of these commands are always
   available and run immediately while others are queued and only
   function after any preceeding commands have completed.

   Immediate Queue Control Commands:
   =================================
   print    -- Print pending commands to stdout
   stop     -- Stop executing commands
   continue -- Resume executing commands
   step     -- Single Step execution of commands
   clear    -- Clear all pending commands in the queue
   hold     -- Wait for Motive command to finish
   release  -- Clear the hold status and resume executing commands

   Deferred Queue Control Commands:
   ================================
   pause    -- Like "stop" except that it can be used in batch files.
               Use "continue" to resume execution.
*/


/*===========================================================================*/
/*==                        P a u s e                                      ==*/
/*===========================================================================*/
/*\Docpage Pause    -- Stop executing commands when encountered in a batch file

DESCRIPTION
     This command is useful in batch files to stop the execution and wait
     for the user to issue a continue. Continue, Step and Clear are valid
     operations after the queue is stopped with the Pause command.
*/
function Qpause(d,event)      vtq.Stop(GetQueue (d),"Stopped via the Pause command");
<cmdPause,cmdPrvPause> := CmdCreate(fac,
                    Qpause,
                    "pause",
                    "Stop the Command Execution Queue",nil);


/*===========================================================================*/
/*==                        P r i n t                                      ==*/
/*===========================================================================*/
/*\Docpage Print    -- Print out the current state of the queue (stdout)        

DESCRIPTION
     This command prints the contents of the command execution queue to
     stdout.  The current command, pending commands, and the Hold/Stop
     status are also included.
*/
function Qprint(d,event)     print(GetQueue(d));
cmdPrint:= CmdCreateNoQueue (fac, 
                             Qprint,
                             "print",
                             "Print the contents of the Queue",
                             nil);


/*===========================================================================*/
/*==                        S t o p                                        ==*/
/*===========================================================================*/
/*\Docpage Stop     -- Stop executing commands after the current one finishes   

DESCRIPTION
     This command is useful for interrupting batch files or long sequences of
     commands.  This command takes effect immediately after the currently
     executing command has completed.  Continue, Step or Clear are valid
     operations when the queue is stopped.
*/
function Qstop(d,event)      vtq.Stop(GetQueue (d),"Stopped Interactively"); 
cmdStop := CmdCreateNoQueue(fac,
                    Qstop,
                    "stop",
                    "Stop the Command Execution Queue",nil);


/*===========================================================================*/
/*==                        H o l d                                        ==*/
/*===========================================================================*/
/*\Docpage Hold     -- Stop executing commands & wait for a release            

DESCRIPTION
     This command is not used interactively.  Rather it is used only by  
     other commands or operations which must wait for an external process
     or event to signal or complete.  In practice it is similar to "stop"
     except that Continue, Step and Clear are not valid operations on a
     queue with a hold condition set. Whenever a command is sent to Motive,
     a "hold" status is set.  When the result comes back,  a "release" is
     issued to clear the hold status and allow the next command to run
     provided that the queue is not also "stopped".
*/
function Qhold(d,event)      vtq.Hold(GetQueue (d),"Held Interactively"); 
cmdHold := CmdCreateNoQueue(fac,
                    Qhold,
                    "hold",
                    "Place a Hold condition on the Queue (debug only)",nil);


/*===========================================================================*/
/*==                        S t e p                                        ==*/
/*===========================================================================*/
/*\Docpage Step     -- Single step through pending commands                    

DESCRIPTION
     This command is used to single step through commands in the command  
     execution queue.  For step to work,  the queue must be currently
     stopped and must not have a "hold" condition in effect. If a command
     invokes several other commands,  each of these subcommands must be
     stepped through also.
*/
function Qstep(d,event)      vtq.Step(GetQueue (d)); 
cmdStep := CmdCreateNoQueue(fac,
                    Qstep,
                    "step",
                    "Single Step pending commands",nil);


/*===========================================================================*/
/*==                        C l e a r                                      ==*/
/*===========================================================================*/
/*\Docpage Clear    -- Erase all pending commands in the queue

DESCRIPTION
     This command will erase all pending commands in the command queue
     (if any).  For this command to function, the queue must be stopped
     and must not have a "hold" condition in effect.
*/
function Qclear(d,event)     vtq.Clear(GetQueue (d));
cmdClear := CmdCreateNoQueue(fac,
                     Qclear,
                     "clear",
                     "Cancel all pending commands (Queue must be stopped first)",nil);


/*===========================================================================*/
/*==                        R e l e a s e                                  ==*/
/*===========================================================================*/
/*\Docpage Release  -- Remove a "hold" condition from the queue

DESCRIPTION
     This command is normally not used interactevly.  Since a hold is placed
     on the queue to block commands until an external event signals or ends,
     the release is normally called wherever this signal or 'ready' detection
     occurs.  In some rare situations, the user may need to issue a release
     manually, but these situations should be considered bugs and not normal
     operating practice.
*/
function Qrelease(d,event)   vtq.Release(GetQueue (d));
cmdQrelease := CmdCreateNoQueue(fac,
                                Qrelease,
                                "Qrelease",
                                "Remove a Hold Condition on the Queue",nil);


/*===========================================================================*/
/*==                        C o n t i n u e                                ==*/
/*===========================================================================*/
/*\Docpage Continue -- Resume executing commands if no "hold" condition exists

DESCRIPTION
     This command is used to resume the free-running execution of commands 
     following a "stop" provided that the queue does not currently have a 
     "hold" condition in effect.
*/
function Qcontinue(d,event)  vtq.Continue(GetQueue (d)); 
cmdContinue := CmdCreateNoQueue(fac,
                        Qcontinue,
                        "continue",
                        "Continue executing pending commands",nil);
 

/*===========================================================================*/
/*==                        Q d e b u g                                    ==*/
/*===========================================================================*/
/*\Docpage Qdebug   -- Enable the command queue's Verbose Debugging mode.

DESCRIPTION
     This command is not useful to end users.  It allows additional debug
     messages to be printed to stdout.
*/
function QDebug(d,event,f)   vtq.CmdDebug(GetQueue(d),f);
cmdDebug := CmdCreateNoQueue(fac,
                        QDebug,
                        "QDebug",
                        "Enable Debugging Mode on the Queue",
                             list(vcWv.Param("Debug?", vlTrueFalseVT,
                                             vcWv.Sticky, vlTrueFalse.True)));
 

/*===========================================================================*/
/*==                      M o d l i b r a r i e s                          ==*/
/*===========================================================================*/
/*\Docpage Modlibraries -- Command to change the Motive library search order

DESCRIPTION
     This command posts a dialog box which allows the user to change the
     Motive model library search order used by the ViewTime netlister (vtm).

*/
function chgsrch(d,event)
{
   rpath := GetCaseRelPath(d);
   inifile := vscript.vsMerge("",rpath, "library", "lst");
   ChgSrch (d,event, inifile);
}
cmdChgSrch := CmdCreateNoQueue(vtime.fac,
                               chgsrch,
                               "modlibraries",
                               "Change timing model library search order", nil);


/*===========================================================================*/
/*==                        A u t o l o a d                                ==*/
/*===========================================================================*/
/*\Docpage Autoload -- Load a file into File vtx region

DESCRIPTION
     This command loads the given file into Files display region.
     circuit timing verification.
*/
function autoload(d,event,file) {
   InfoDspName := "Info";

   infofileRgn := DisplayGetRgnByName(d, InfoDspName);
   if (infofileRgn != nil){
      viewtxt := vcRegion.Data(infofileRgn, vtxVT);
      vtx.Load(viewtxt, file);
      vcRegion.Refresh (infofileRgn, vcRegion.Area (infofileRgn));
   }
   else {
      msg := sprintf(cdr(msg_InfoRgnNExist),InfoDspName);
      AlertBox(d,event,car(msg_InfoRgnNExist),vlErrSev.Warning,msg);
   }
}
<cmdAutoload,prvCmdAutoload> := CmdCreate(vtime.fac,
                                          autoload,
                                          "_autoload",
                                          "Load file into vtx", 
                                          list (vcWv.Param ("File Name", vlStringVT, 
                                                            vcWv.Constant, "")));

/*===========================================================================*/
/*==                        I n f o                                        ==*/
/*===========================================================================*/
/*\Docpage Info -- Extract specific design information type

DESCRIPTION
     This command invokes a Motive "info .. <type? " command to extract a complete
     circuit information, 
             where <type> is - mod, net, loc, mcp, wireload

     The command should do the following:
     ======================================
        * Extract the value of info file
        * Execute an "mtv info filename <type>" command
        * update info file display.
        * if user preference to autodisplay verification report is
          enabled then submit vrepfile command also
*/
function info(d, event, type, pattern) {
   infofile    := GetMtvVar(d, MtvVar.MV_file_tmp_info, MtvValue.VALUE);
   if (infofile != nil) {
      Queue := GetQueue(d);

      cmd := "mtv info " + infofile + " " + type + " " + pattern;

      vtq.CmdRun (Queue, event, cmd);
      vtq.CmdRun (Queue, event, "_autoload " + infofile);
      vtq.CmdRun (GetQueue (d), event, "current info");  /* pop the files to the top */
   }
   else {
      msg := sprintf(cdr(msg_BadVarFname),"Temporary Info");
      AlertBox(d,event,car(msg_BadVarFname),vlErrSev.Warning,msg);
   }
}
<cmdInfo,prvCmdInfo> := CmdCreate(vtime.fac,
                                  info,
                                  "Info",
                                  "Show Design Information",
                                  list (vcWv.Param ("Motive Info Keyword:  ",
                                                    vlStringVT, 
                                                    vcWv.Constant, ""),
                                        vcWv.Param ("Pattern ",
                                                    vlStringRestOfLineVT, 
                                                    vcWv.Constant, ".*")));


/*===========================================================================*/
/*==                       M o d I n f o                                   ==*/
/*===========================================================================*/
/*\Docpage Modinfo -- Extract design model information 

DESCRIPTION
     This command invokes "info" command with type "mod" to produce a
     report showing all the models which match the specification pattern.

DEFAULT
     The default pattern ".*" matches all models.
     
*/
function modinfo(d, event, pattern) {
   Queue := GetQueue(d);
   info(d, event, "mod", pattern); 
}
<cmdModinfo,prvCmdModinfo> := CmdCreate(vtime.fac,
                                        modinfo,
                                        "Modinfo",
                                        "Show model information",
                                        list (vcWv.Param ("Pattern ",
                                                          vlStringVT, 
                                                          vcWv.Constant, ".*")));

/*===========================================================================*/
/*==                       L o c I n f o                                   ==*/
/*===========================================================================*/
/*\Docpage Locinfo -- Extract design componenet information 

DESCRIPTION
     This command invokes "info" command with type "loc" to produce a 
     report of all component instances which match the specified pattern.

DEFAULT
     The default pattern ".*" matches all component instances.
     
*/
function locinfo(d, event, pattern) {
   Queue := GetQueue(d);
   info(d, event, "loc", pattern); 
}
<cmdLocinfo,prvCmdLocinfo> := CmdCreate(vtime.fac,
                                        locinfo,
                                        "Locinfo",
                                        "Show component instance information",
                                        list (vcWv.Param ("Pattern ",
                                                          vlStringVT, 
                                                          vcWv.Constant, ".*")));

/*===========================================================================*/
/*==                       N e t I n f o                                   ==*/
/*===========================================================================*/
/*\Docpage Netinfo -- Extract design net information 

DESCRIPTION
     This command invokes "info" command with type "net" to produce a
     report of all the nets which match the specified pattern.

DEFAULT
     The default pattern ".*" matches all net names.
     
*/
function netinfo(d, event, pattern) {
   Queue := GetQueue(d);
   info(d, event, "net", pattern);
}
<cmdNetinfo,prvCmdNetinfo> := CmdCreate(vtime.fac,
                                        netinfo,
                                        "Netinfo",
                                        "Show net information",
                                        list (vcWv.Param ("Pattern ",
                                                          vlStringVT, 
                                                          vcWv.Constant, ".*")));

/*===========================================================================*/
/*==                       M c p I n f o                                   ==*/
/*===========================================================================*/
/*\Docpage Mcpinfo -- Extract circuit MCP information 

DESCRIPTION
     This command invokes "info" command with type "mcp" to produce a
     report of all the multi-cycle paths which match the specified pattern.

DEFAULT
     The default pattern ".*" matches all mcp definitions.
     
*/
function mcpinfo(d, event) {
   Queue := GetQueue(d);
   pattern := "";
   info(d, event, "mcp", pattern); 
}
<cmdMcpinfo,prvCmdMcpinfo> := CmdCreate(vtime.fac,
                                        mcpinfo,
                                        "Mcpinfo",
                                        "Show mcp information", nil);


/*===========================================================================*/
/*==                  W i r e l o a d I n f o                              ==*/
/*===========================================================================*/
/*\Docpage Wireloadinfo -- Extract design wireload information 

DESCRIPTION
     This command invokes "info" command with type "wireload" to produce
     a report of the currently active wireload table from the model library.
     It is quite possible for the report to indicate that no wireload table
     is in use since not all libraries make use of this feature.


*/
function wireloadinfo(d, event) {
   Queue := GetQueue(d);
   pattern := "";
   info(d, event, "wireload", pattern);
}
<cmdWireloadinfo,prvCmdWireloadinfo> := CmdCreate(vtime.fac,
                                        wireloadinfo,
                                        "Wireloadinfo",
                                        "Show wireload information",nil);

/*===========================================================================*/
/*==                  S t a t i s t i c s                                  ==*/
/*===========================================================================*/
/*\Docpage Statistics -- Extract general design statistics

DESCRIPTION
     This command invokes mtv command "stats general" command to produce
     a report showing the Motive runtime and cache usage statistics. The
     statistics are echoed to the Vtime transcript.

*/
function stats(d, event) {
   Queue := GetQueue(d);
   vtq.CmdRun (Queue, event, "mtv stats general");
   vtq.CmdRun (Queue, event, "current transcript");
}
<cmdStats,prvCmdStats> := CmdCreate(vtime.fac,
                                    stats,
                                    "Statistics",
                                    "General Design Statistics",nil);

/*===========================================================================*/
/*==                          H p i n s                                    ==*/
/*===========================================================================*/
/*\Docpage Hpins -- Show list of most frequently recalculated component pins.

DESCRIPTION
     This command invokes mtv command "stats hpins #"
*/
function hpins(d, event, pins) {
   Queue := GetQueue(d);
   vtq.CmdRun (Queue, event, sprintf ("mtv stats hpins %d",pins));
   vtq.CmdRun (Queue, event, "current transcript");
}
<cmdHpins,prvCmdHpins> := CmdCreate(vtime.fac,
                                    hpins,
                                    "Hpins",
                                    "List the most frequently recalculated component pins",
                                    list (vcWv.Param ("Number of Hpins to display ",
                                                      vlPositiveIntVT, 
                                                      vcWv.Constant, "100",
                                                      vcScalebar.fac,
                                                      increment <- 1,
                                                      min <- 1,
                                                      max <- 500,
                                                      ticks <- 10,
                                                      clip <- FALSE)));


/*===========================================================================*/
/*==                          S i g P p i n s                              ==*/
/*===========================================================================*/
/*\Docpage SigPpins -- Show list of pins most frequently sig(p) loop-broken

DESCRIPTION
     This command invokes mtv command "stats sigps #"
*/
function sigppins(d, event, pins) {
   Queue := GetQueue(d);
   vtq.CmdRun (Queue, event, sprintf ("mtv stats sigps %d",pins));
   vtq.CmdRun (Queue, event, "current transcript");
}
<cmdHpins,prvCmdHpins> := CmdCreate(vtime.fac,
                                    sigppins,
                                    "Sigppins",
                                    "List the most frequently sig(p) loop-broken component pins",
                                    list (vcWv.Param ("Number of Sig(P) pins to display ",
                                                      vlPositiveIntVT, 
                                                      vcWv.Constant, "100",
                                                      vcScalebar.fac,
                                                      increment <- 1,
                                                      min <- 1,
                                                      max <- 500,
                                                      ticks <- 10,
                                                      clip <- FALSE)));

/*===========================================================================*/
/*==                         L n e t s                                     ==*/
/*===========================================================================*/
/*\Docpage lnets -- Extract loop nets information

DESCRIPTION
     This command invokes mtv command "stats lnets <pin_count>"
*/
function lnets(d, event, pins) {
   Queue := GetQueue(d);
   vtq.CmdRun (Queue, event, sprintf ("mtv stats lnets %d",pins));
   vtq.CmdRun (Queue, event, "current transcript");
}
<cmdLnets,prvCmdLnets> := CmdCreate(vtime.fac,
                                    lnets,
                                    "Lnets",
                                    "Display nets most frequently encountered when breaking logic loops",
                                    list (vcWv.Param ("Number of Lnets to display ",
                                                      vlPositiveIntVT, 
                                                      vcWv.Constant, "100",
                                                      vcScalebar.fac,
                                                      increment <- 1,
                                                      min <- 1,
                                                      max <- 500,
                                                      ticks <- 10,
                                                      clip <- FALSE)));

/*===========================================================================*/
/*==                      T r e p f i l e                                  ==*/
/*===========================================================================*/
/*\Docpage Trepfile -- Display the current trace analysis report

DESCRIPTION
     This command causes the latest trace analysis report to be loaded and
     displayed to the user.  If the Treport display region is not peeled
     off in a separate window, then it is made the currently visible 
     region in the display window.

EXAMPLE

     trepfile

*/
function trepfile(d, event)
{
   TrepDspName := "TReport";

   repfile    := GetMtvVar(d, MtvVar.MV_file_tmp_trace, MtvValue.VALUE);
   repfileRgn := DisplayGetRgnByName(d, TrepDspName);
   if (repfile != nil && repfileRgn != nil) {
      viewtxt := vcRegion.Data(repfileRgn, vtxVT);
      if (viewtxt != nil) {
         /* Note that we must make the buffer writable to reload */
         vtx.ReadOnly(viewtxt,FALSE);
         vtx.Load(viewtxt, repfile);
         vtx.ReadOnly(viewtxt,TRUE);
         vcRegion.Refresh (repfileRgn, vcRegion.Area (repfileRgn));
         vtq.CmdRun (GetQueue (d), event, "current " + TrepDspName);
      }
   }
   else {
      ThrowIfNoDesignLoaded(d, event);
      if (repfile == nil) {
         AlertBox(d,event,car(msg_BadTrepName),vlErrSev.Warning,cdr(msg_BadTrepName));
      }
      else {
         msg := sprintf(cdr(msg_DspRgnNExist),TrepDspName);
         AlertBox(d,event,car(msg_DspRgnNExist),vlErrSev.Warning,msg);
      }
   }
}
<cmdTrepfile,prvCmdTrepfile> := CmdCreate(vtime.fac,
                                          trepfile,
                                          "Trepfile",
                                          "Show the latest Trace Analysis Results", 
                                          nil);



/*===========================================================================*/
/*==                             |  (comment)                              ==*/
/*===========================================================================*/
/*\Docpage |   -- Adds a comment to transcript for interactive purposes

DESCRIPTION
     This command is used interactively to add a commented message to the
     transcript.  This command is essentially a no-op and since it is 
     commented with a "|" character, it will not reappear if the transcript
     is saved and reexecuted.  Its usefulness is simply in being able to 
     make some comments about what happened or what you might be doing for
     reference purposes.

RETURN
     notta..

*/
<cmdComment,prvCmdComment> := vtime.CmdCreate(vtime.fac,
                                  func(d,e,a);,
                                  "|",
                                  "Echo a commented message to the transcript (interactive)",
                                  list (vcWv.Param ("Message? ", 
                                                    vlStringRestOfLineVT, 
                                                    vcWv.Constant, 
                                                    "")));


/*===========================================================================*/
/*==                         R e m a r k                                   ==*/
/*===========================================================================*/
/*\Docpage Remark  -- Adds a comment to the transcript for batch file purposes

DESCRIPTION
     This command is used in command files to add a commented message to the
     transcript.  Since the remark command itself is echoed to the transcript
     when run interactively, the "|" command should be used for interactive
     purposes.   Its usefulness is simply in being able to make some comments 
     which will be visible in the transcript for future reference purposes.
     This command is especially useful for echoing status messages to the
     transcript from within command action routines since the transcript can
     always be safely re-executed since this comment will not automatically
     be present in the new transcript.

CAUTION
     If a command file contains "remark" commands, the transcript will contain
     only the commented remarks.  Therefore,  if you save and re-execute the
     transcript,  the comments will not be present in the new transcript.

RETURN
     notta..

*/
<cmdRem,prvCmdRem> := vtime.CmdCreate(vtime.fac,
                                  func(d,e,message) vtime.Log(d, message, vlTrueFalse.True);,
                                  "Remark",
                                  "Echo a commented message to the transcript (command files)",
                                  list (vcWv.Param ("Message? ", 
                                                    vlStringRestOfLineVT, 
                                                    vcWv.Constant, 
                                                    "")));

/*===========================================================================*/
/*==                   C u r r e n t                                       ==*/
/*===========================================================================*/
/*\Docpage Current -- Make a region as the current display region, if in overlay

DESCRIPTION
     This command brings up the region identified by name to the top of the
     display region, if it is in overlay. If the specified region is in a
     peeled window, no action is performed.
*/
function current(d, event, name)
{
   Rgn := DisplayGetRgnByName(d, name);
   if (Rgn != nil){
      RgnWin := vcRegion.Window (Rgn);
      FacWin := vcEvent.Window (event);
      if (FacWin == RgnWin)
         vtq.CmdRun (GetQueue (d), event, "display " + name);
   }
}
<cmdCurrent,prvCmdCurrent> := CmdCreate(vtime.fac,
                                        current,
                                        "Current",
                                        "Make the specified display item current", 
                                        list (vcWv.Param ("Display Item Name", vlStringVT, 
                                                          vcWv.Constant, "")));

/*===========================================================================*/
/*==                     E x i t C m d                                     ==*/
/*===========================================================================*/
/*\Docpage Exit -- Exit the timing analysis session

DESCRIPTION
     This command closes the current timing analysis session window.
     The exit command will eventually shutsdown motive engine and destroys
     the current viewtime facility window.

     The options are No, Yes, Force where,
     
     No    - No exit is performed,
     Yes   - A "save" command is executed before exiting the window,
     Force - The window is closed without saving the modified data/results     
*/
function exit(d, event, flag) 
{
   if (flag == vtimeExit.Yes){
		r := vcEvent.Region (event);
		SetMtvRestartFlg (d, FALSE);
      vtq.CmdRun (GetQueue (d), event, "mtvquit");
      vtq.CmdRun (GetQueue (d), event, "exit force");
	}
	else if (flag == vtimeExit.Force)
      vcWin.Destroy (vcEvent.Window (event));

}
<cmdExit,prvCmdExit> := vtime.CmdCreate(vtime.fac,
													 exit,
													 "Exit",
                                        "Exit Timing Analysis",
                                        list (vcWv.Param ("Exit Timing Analysis?",
                                                          vtimeExitVT, 
                                                          vcWv.Sticky,
                                                          vtimeExit.No)));

function vtime_quit() ; /*printf ("vtime_quit: Called\n");*/
vcQuit.AddFunction (vtime_quit);



/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function findhistogram  -- Locate the histogram in the verification report
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function findhistogram(d, e) 
{
   vxid := vcRegion.Data(rgn:=vtime.DisplayGetRgnByName(d,"VReport"),vtxVT);
   if (vxid != nil) {
      vtx.BeginningOfBuffer(vxid);
      vtx.SearchForward(vxid ,"MOTIVE DELAY HISTOGRAM");
      vcRegion.Refresh(rgn, vcRegion.Area(rgn));
   }
} /* findhistogram */
<cmdFindHst,prvCmdFindHst> := vtime.CmdCreate(vtime.fac,
                                        findhistogram,
                                        "_findhistogram",
                                        "Find the verification histogram.", nil);



/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function histogram  -- Display the verification report histogram
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function histogram(d, e) 
{
   vtq.CmdRun(vtime.GetQueue(d),e,"vrepfile");
   vtq.CmdRun(vtime.GetQueue(d),e,"_findhistogram");
   
} /* histogram */
<cmdHst,prvCmdHst> := vtime.CmdCreate(vtime.fac,
                                        histogram,
                                        "Histogram",
                                        "Display the verification histogram.", nil);


/*===========================================================================*/
/*==                              X                                        ==*/
/*===========================================================================*/
/*\Docpage X -- Add a net constant with don't care signal state.

  DESCRIPTION
       This command allows a net to be defined as CONSTANT in Motive thus
       and the value of the signal state to Don't Care. Once a signal is added
       as constant, it is ignored from critical path and trace analysis since
       it does not change in state.
*/
<cmdX,prvCmdX> := CmdCreate(vtime.fac,
                            func (d,e,net) {
                               if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Upper)
                                  net = string_upper(net);
                               else if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Lower)
                                  net = string_lower(net);
                               cmd := sprintf ("X %s", net);
                               conslist := GetConsList (d);
                               conslist  = cons (cmd, conslist);
                               SetConsList (d,conslist);
                               vtq.CmdRun(GetQueue(d), e, 
                                          sprintf ("mtv con add %s X", net));
                            },
                            "X",
                            "Set Net to the constant don't care state",
                            list (vcWv.Param ("Net Name",
                                              vlStringVT, 
                                              vcWv.Sticky, "")));
vcCmd.EnableApply(cmdX, vlOnOff.On);
/*===========================================================================*/
/*==                              L                                        ==*/
/*===========================================================================*/
/*\Docpage L -- Add a net constant with LOW signal state.

  DESCRIPTION
       This command allows a net to be defined as CONSTANT in Motive thus
       and the value of the signal state to LOW (0). Once a signal is added
       as constant, it is ignored from critical path and trace analysis since
       it does not change in state.
*/
<cmdL,prvCmdL> := CmdCreate(vtime.fac,
                            func (d,e,net){
                               if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Upper)
                                  net = string_upper(net);
                               else if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Lower)
                                  net = string_lower(net);
                               cmd := sprintf ("L %s", net);
                               conslist := GetConsList (d);
                               conslist  = cons (cmd, conslist);
                               SetConsList (d,conslist);
                               vtq.CmdRun(GetQueue(d), e, 
                                          sprintf ("mtv con add %s 0 ",net));
                               
                            },
                            "L",
                            "Net Constant to Low state",
                            list (vcWv.Param ("Net Name",
                                              vlStringVT, 
                                              vcWv.Sticky, "")));
vcCmd.EnableApply(cmdL, vlOnOff.On);
/*===========================================================================*/
/*==                              H                                        ==*/
/*===========================================================================*/
/*\Docpage H -- Add a net constant with HIGH(1) signal state.

  DESCRIPTION
       This command allows a net to be defined as CONSTANT in Motive thus
        and the value of the signal state to HIGH(1). Once a signal is added
       as constant, it is ignored from critical path and trace analysis since
       it does not change in state.
*/
<cmdH,prvCmdH> := CmdCreate(vtime.fac,
                            func (d,e,net){
                               if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Upper)
                                  net = string_upper(net);
                               else if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Lower)
                                  net = string_lower(net);
                               cmd := sprintf ("H %s", net);
                               conslist := GetConsList (d);
                               conslist  = cons (cmd, conslist);
                               SetConsList (d,conslist);
                               vtq.CmdRun(GetQueue(d), e, 
                                          sprintf ("mtv con add %s 1 ",net));
                            },                            
                            "H",
                            "Net Constant to High state",
                            list (vcWv.Param ("Net Name",
                                              vlStringVT, 
                                              vcWv.Sticky, "")));
vcCmd.EnableApply(cmdH, vlOnOff.On);

/*===========================================================================*/
/*==                           R e l e a s e                               ==*/
/*===========================================================================*/
/*\Docpage Release - Release a net constant 

  DESCRIPTION
       This command allows a net constant that was added using H,L or X command
       to be deleted thus allowing the net to be included in the anlaysis.
*/
<cmdRelease,prvCmdRelease> := CmdCreate(vtime.fac,
                                        func (d,e,net){
                                           if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Upper)
                                              net = string_upper(net);
                                           else if (vlPref.Val(prefs,"capslock") == vcCmdlineCase.Lower)
                                              net = string_lower(net);
                                           cmd := sprintf ("Release %s", net);
                                           conslist := GetConsList (d);
                                           conslist  = cons (cmd, conslist);
                                           SetConsList (d,conslist);
                                           vtq.CmdRun(GetQueue(d), e,
                                                      sprintf ("mtv con del %s",
                                                               net));
                                        },
                                        "Release",
                                        "Net Constant to be released",
                                        list (vcWv.Param ("Net Name",
                                                          vlStringVT, 
                                                          vcWv.Sticky, "")));
vcCmd.EnableApply(cmdRelease, vlOnOff.On);

/*===========================================================================*/
/*==                         C o n s S a v e                               ==*/
/*===========================================================================*/
/*\Docpage ConsSave -- Save current net constants to design.con file.

  DESCRIPTION
       This command allows a the user to save all the current net constants
       created using H, L and X commands to be saved to design.con file in the
       case directory. During a new run of the same analysis, this file can be 
       played back as a batch file to set all those net constants.
*/
function WriteConsFile(d,e,entries)
{
   rpath := vtime.GetCaseRelPath(d);
   consfile := vscript.vsMerge("",rpath, GetBlockName (d), "con");
   BackupFile(d,consfile,"con_",3);
   fp := fopen (consfile, "w");
   fputline (fp, sprintf ("| Net constants for Design: %s, TimingCase: %s\n",
             GetBlockName (d), GetCaseName (d)));
   foreach (arg; vscript.list_iter(entries))
      fputline (fp, arg + "\n");
   fclose (fp);
   vtime.Log (d,"Net Constants file " + consfile + " written", 
              vlTrueFalse.True);
}
function Conssave (d,e){
   conslist := GetConsList (d);
   newlist := reverse (conslist);
   WriteConsFile (d,e, newlist);
   /*ResetConsList (d);*/
}
<cmdConssave,prvCmdConssave> := CmdCreate(vtime.fac,
                                          Conssave,
                                          "Conssave",
                                          "Save Net Constatns",nil);

/*===========================================================================*/
/*==                         M c p S a v e                                 ==*/
/*===========================================================================*/
/*\Docpage McpSave -- Save current Mcps to design.mcp file.

  DESCRIPTION
       This command allows a the user to save all the current MCP settings
       currently defined.
*/
function WriteMcpFile(d,e)
{
   
   Running := IsMtvSysAlive(d);
   if (Running) {
      Ready := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
      if (string_eqi (Ready, "TRUE")) {
         
         mcpfile    := GetMtvVar(d, MtvVar.MV_file_mcp, MtvValue.VALUE);
         mcpfilevar := GetMtvVar(d, MtvVar.MV_file_mcp, MtvValue.NAME);
         
         mcpfile = ResolveFilePathName(d, mcpfile, "mcp");
         
         if (vtime.vsExists(mcpfile,vsFILE)) {
            vtime.BackupFile(d,mcpfile,"mcp_",3);
         }

         vtq.CmdRun (vtime.GetQueue (d), e, "mtv info " + mcpfile + " mcp");
         vtq.CmdRun (vtime.GetQueue (d), e, "mtv sv " + mcpfilevar + " " + mcpfile);
         vtq.CmdRun (vtime.GetQueue (d), e, "remark MCP File " + mcpfile + " Written");
      }
      else {
         vtq.CmdRun (vtime.GetQueue (d), e,"remark Note: Not necessary to save MCPs, they have not been changed...");
         vtq.CmdRun (vtime.GetQueue (d), e, "remark       Preserving existing MCP File.");
      }
   }      
   else {
      vtq.CmdRun (vtime.GetQueue (d), e,"remark Note: Motive is not currently running. Unable to save MCPs.");
      vtq.CmdRun (vtime.GetQueue (d), e, "remark       Preserving existing MCP file.");
   }
}


/*===========================================================================*/
/*==              B a c k u p F i l e                                      ==*/
/*===========================================================================*/
/*\Function BackupFile  -- Create Backup file for specified file
  
  DESCRIPTION
       This function manages a set of backup files for a given file.  This
       backup set includes incrementally numbered backup files of a given
       filename extension.  This extension is built by combining the given
       backup extension with a numeric index.  
       
  RETURNS
       A boolean status code indicating whether the backup was successful.

  EXAMPLE
       BackupFile(d,"file.ext","ex",3);

    Will move "file.ex2" to "file.ex3"  (if both .ex2 & .ex3 exist, .ex3 is erased)
    and  move "file.ex1" to "file.ex2"  (if it exists)
    and  copy "file.ext" to "file.ex1"
       
*/
function BackupFile(d,file,back_ext,limit)
{
   declare ok;
   <device,path,name,ext> := vscript.vsParse(file);

   bakfile := vscript.vsMerge("",path, name, back_ext + convert(vlStringVT,limit));
   
   for (i:=limit-1; i > 0; i=i-1) {
      tstfile := vscript.vsMerge("",path, name, back_ext + convert(vlStringVT,i));
      if (vtime.vsExists(tstfile,vsFILE)) {
         ok = vscript.vsRename(tstfile,bakfile);
      }

      bakfile = tstfile;
   }
   ok = vscript.vsCopy(file,bakfile);

   ok;
} /* BackupFile */


<cmdMcpSave,prvCmdMcpSave> := CmdCreate(vtime.fac,
                                          WriteMcpFile,
                                          "McpSave",
                                          "Save Current MCP Definitions",nil);


/*===========================================================================*/
/*==                       S a v e A s                                     ==*/
/*===========================================================================*/
/*\Docpage SaveAs -- Save all important data creating a new case.

DESCRIPTION
     This command is used to write all 'interesting' case specific data to 
     disk.  It eventually will provide a vcNotify hook which will allow      
     customer written functions to be called when a save command is invoked.

RETURN
     notta..

*/
function savecaseas(d, e, case) 
{
	oldcase := vtime.GetCaseName(d);
   if (string_eqi (oldcase, case) != TRUE) {
      vtq.CmdRun (GetQueue (d), e, "current Transcript");
      vtq.CmdRun(vtime.GetQueue(d), e, "CopyCase " + case);
   }
   else{
      msg := sprintf(cdr(vtime.msg_SameCaseName), case, oldcase);
      vtime.AlertBox(d,e,car(vtime.msg_SameCaseName),vlErrSev.Error,msg);
   }      
}
<cmdSaveAs,prvCmdSaveAs> := vtime.CmdCreate(vtime.fac,
                                            savecaseas,
                                            "SaveAs",
                                            "Save Current Case as a new one",
                                            list (vcWv.Param ("case", 
                                                              vlStringVT, 
                                                              vcWv.Sticky, 
                                                              "")));
/*===========================================================================*/
/*==                      C r e a t e C a s e                              ==*/
/*===========================================================================*/
/*\Docpage CreateCase -- Create a new case directory.
*/
function createcase(d, e, design, case) 
{
   if ((design != nil) && (case != nil)) {
      rpath := vtime.BuildCaseRelPath(design,case);
      print(rpath);
      if (vtime.vsExists(rpath,vsDIR))
         printf("Case Directory %s Exists...\n",rpath);
      else {
         printf("Creating Case: %s  ",rpath);
         if (vtime.CreateDir(rpath)) 
            printf("... Done\n");
         else {
            msg := sprintf(cdr(vtime.msg_CaseDirNCreated), rpath);
            vtime.AlertBox(d,e,car(vtime.msg_CaseDirNCreated),vlErrSev.Error,msg);
         }
      }
   }
}
<cmdCreateCase,prvCmdCreateCase> := vtime.CmdCreate(vtime.fac,
                                                    createcase,
                                                    "_createcase",
                                                    "Create A new Case directory",
                                                    list (vcWv.Param ("Design", 
                                                                      vlStringVT, 
                                                                      vcWv.Sticky, 
                                                                      ""),
                                                          vcWv.Param ("case", 
                                                                      vlStringVT, 
                                                                      vcWv.Sticky, 
                                                                      "")));


/*===========================================================================*/
/*==                          C o p y C a s e                              ==*/
/*===========================================================================*/
/*\Docpage CopyCase -- Copies a current case data files to a new case directory

  DESCRIPTION
       This command copies a new timing case directory from the current case
       directory for the current design block. Then the current parameter
       setting is saved to the new case directory. Then the pathnames of design
       deata files from the new design stimulus parameter file (.stm) are modified 
       to point to the new case directory.
*/
function CopyCase (d,e,case) 
{
   declare device;
   declare path;
   declare name;
   declare ext;

   design := vtime.GetBlockName(d);
	oldcase := vtime.GetCaseName(d);

   rpath := vtime.BuildCaseRelPath(design,case);

   if (vtime.vsExists(rpath,vsDIR)){
      foreach(filename; vscript.TraverseDir(vtime.GetCaseRelPath(d) + "*.*", vsAttr.File)) {
         filename = vscript.vsRelPath(filename);
         <device,path,name,ext> = vscript.vsParse(filename);
         vtime.Log(d,"Copy: " + filename + " to " + rpath + name + "." + ext, vlTrueFalse.True);
         vscript.vsCopy(filename,rpath+name+"."+ext);
      }
      stmfile := vscript.vsMerge("",rpath, design, "stm");

      vtq.CmdRun (vtime.GetQueue (d), e, "mtv saveas " + stmfile);
		cmd := sprintf ("_modstm %s %s", stmfile, case);
      vtq.CmdRun (vtime.GetQueue (d), e, cmd);
   }
   else {
      vtq.CmdRun(vtime.GetQueue(d), e, "_CreateCase " + design + " " + case);
      vtq.CmdRun(vtime.GetQueue(d), e, "CopyCase " + case);
   }
}
<cmdCopyCase,prvCmdCopyCase> := vtime.CmdCreate(vtime.fac,
                                                CopyCase,
                                                "CopyCase",
                                                "Copy current case to a new or existing case",
                                                list (vcWv.Param ("Destination Case Name", 
                                                                  vlStringVT, 
                                                                  vcWv.Sticky, 
                                                                  "")));

/*===========================================================================*/
/*==                          M o d S t m                                  ==*/
/*===========================================================================*/
/*\Docpage ModStm -- Modify stmfile pathnames to point to a new case.

  DESCRIPTION
       This command opens up the stm file from the new case. It then parses
       the whole file looking for pathnames that point to the current case
       directory, changing them to point to the new case directory.
*/
function ModStm(d,e, stm, newcase)
{
   declare count, index, substr, newline, temp;

   design  := vtime.GetBlockName(d);
	oldcase := vtime.GetCaseName(d);
	srch    := sprintf ("tim/%s/%s", design, oldcase);

   rpath   := vtime.BuildCaseRelPath(design, newcase);
   temp    := vscript.vsMerge("",rpath, "temp", "stm");
/*   catch (vlEx.FAILURE){*/
      stmfp   := fopen (stm, "r");
/*   }
   exception {
      msg := sprintf(cdr(vtime.msg_FileNOpened), stm, "reading");
      vtime.AlertBox(d,e,car(vtime.msg_FileNOpened),vlErrSev.Error,msg);
   }

   catch (vlEx.FAILURE){
*/
      tempfp  := fopen (temp, "w");
/*   }
   exception {
      msg := sprintf(cdr(vtime.msg_FileNOpened), temp, "writing");
      vtime.AlertBox(d,e,car(vtime.msg_FileNOpened),vlErrSev.Error,msg);
   }
*/
	for (s := fgetline (stmfp); s != nil; s = fgetline (stmfp)) {
      newline = s;
      count = 1;

      while (s != nil &&  count != 3){
         ++count;
         s = string_token (s, &token, " \t\n");
      }

      if (count != 3)
         fputline (tempfp, newline);
      
      else{
         /* find a match and replace old case name with new name */

         index = string_search (newline, srch);
         if (index != nil){
            path := sprintf ("%stim/%s/%s%s",
                             substring (newline, 0, index),
                             design,
                             newcase,
                             substring (newline, index+string_length (srch), 0));
            fputline (tempfp, path);
         }
         else
            fputline (tempfp, newline);      
      }
	}
   fclose (stmfp);
	fclose (tempfp);
	if (vscript.vsCopy (temp, stm) == FALSE){
      msg := sprintf(cdr(vtime.msg_FileNCopied), temp, stm);
      vtime.AlertBox(d,e,car(vtime.msg_FileNCopied),vlErrSev.Error,msg);
   }
      
}
<cmdModStm,prvCmdModStm> := vtime.CmdCreate(vtime.fac,
                                            ModStm,
                                            "_modstm",
                                            "Modify stm file",
                                            list (vcWv.Param ("Stmfile Name", 
                                                              vlStringVT, 
                                                              vcWv.Sticky,""),
                                                  vcWv.Param ("Destination Case Name", 
                                                              vlStringVT, 
                                                              vcWv.Sticky, 
                                                              "")));

/*===========================================================================*/
/*==                    C r e a t e S t m f i l e                          ==*/
/*===========================================================================*/
/*\Docpage CreateStmfile -- Create a stmfile with the design, case name
*/
function CreateStmfile(d)
{
   declare line;

   rpath   := GetCaseRelPath (d);
   design := GetBlockName (d);

   stmfile := vscript.vsMerge("",rpath, design, "stm");

   /*catch (vlEx.FAILURE){*/
      fp  := fopen (stmfile, "w");
/*}
   exception {
      msg := sprintf(cdr(vtime.msg_FileNOpened), stmfile, "writing");
      vtime.AlertBox(d,e,car(vtime.msg_FileNOpened),vlErrSev.Error,msg);
   }*/


   file := vscript.vsMerge("",rpath, design, "stm");
   line = sprintf ("stm_begin       4       \"%s\"\n", file);
   fputline (fp, line);
   fputline (fp, "#\n");

   file := vscript.vsMerge ("", rpath, design, "gsr");
   line = sprintf ("sv   /file/gsr                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "mod");
   line = sprintf ("sv   /file/mod                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "pin");
   line = sprintf ("sv   /file/net                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "ref");
   line = sprintf ("sv   /file/ref                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "rep");
   line = sprintf ("sv   /file/rep                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "stm");
   line = sprintf ("sv   /file/stm                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "tim");
   line = sprintf ("sv   /file/tim                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "trc");
   line = sprintf ("sv   /file/trc                 \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "info", "tmp");
   line = sprintf ("sv   /file/tmp/info            \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "mtv_intr", "tmp");
   line = sprintf ("sv   /file/tmp/int             \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "trace", "tmp");
   line = sprintf ("sv   /file/tmp/trace           \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "z@@wave", "tmp");
   line = sprintf ("sv   /file/tmp/wave            \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "report", "mpf");
   line = sprintf ("sv   /file/tmp/mpf/report      \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, "trace", "mpf");
   line = sprintf ("sv   /file/tmp/mpf/trace       \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "loc");
   line = sprintf ("sv   /iam/capp/file/loc        \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "loc");
   line = sprintf ("sv   /iam/cappf/file/loc       \"%s\"\n",file);
   fputline (fp, line);
   file := vscript.vsMerge ("", rpath, design, "loc");
   line = sprintf ("sv   /iam/xline/file/loc       \"%s\"\n",file);
   fputline (fp, line);

   /*fputline (fp, "sv   /trace/waveform/ascii     disable\n");
     fputline (fp, "sv   /analysis/reconvergence   disable\n"); 
     fputline (fp, "sv   /msg/occ_missing          enable\n");
   */
   
   file := vscript.vsMerge ("", rpath, design, "stm");
   line = sprintf ("# end-of-file \"%s\"\n", file);
   fputline (fp, line);
   fputline (fp, "stm_end\n");
   
   fclose (fp);
   
}


/*===========================================================================*/
/*==                    W a v e f o r m                                    ==*/
/*===========================================================================*/
/*\Function waveform -- Display trace results in viewtrace as waveform 
  
  DESCRIPTION
      This command invokes a viewtrace window displaying the rsults of a trace
      analysis. If a viewtrace context is not started, one is started.
       
  CAUTION
      This command first translates the Quad waveform to a generic wfm for
      viewtrace. Then viewtrace is started on this wfm file.
*/

function waveform(d,e)
{
   wavefile := GetMtvVar(d, MtvVar.MV_file_tmp_wave, MtvValue.VALUE);  
   rpath := GetCaseRelPath(d);
   wfmfile := vscript.vsMerge("",rpath, "vtrace", "wfm");

   if (Quad2WFM (d, wavefile, wfmfile) == vlTrueFalse.True){
      OpenWaveWindow (d,wfmfile);
      status_msg (e,"Note: Waveform display started...");
   }
   else
      status_msg (e, "Error: Unable to create wfm file.");
}
<cmdWaveform,prvCmdWaveform> := CmdCreate(vtime.fac,waveform,"Waveform",
                                          "Display trace results as waveform in ViewTrace window",
                                          nil);

function Cr8FileInCaseDir(d,name,ext)
   vscript.vsMerge("",GetCaseRelPath (d), name, ext);

/*===========================================================================*/
/*==                 _ s a v e _ t r e p o r t                             ==*/
/*===========================================================================*/
/*\Function _save_report  -- Saves the current report information into a file
  
  DESCRIPTION
      This command saves the current trace report and mpf information into
      the files provided.

  CAUTION
      Currently no parsing mechanism is added to the appended file.
*/
function save_treport(d,e,app_repfile, app_mpffile)
{   
   repfile    := GetMtvVar(d, MtvVar.MV_file_tmp_trace, MtvValue.VALUE);
   mpffile    := GetMtvVar(d, MtvVar.MV_file_tmp_mpf_trace, MtvValue.VALUE);
   
   fp := fopen (app_repfile, "a");
   fputline (fp, "|---------------------------------------");
   fputline (fp, "---------------------------------------|\n");
   fclose (fp);
   if (vsAppend (repfile,app_repfile) == FALSE)
      status_msg (e, sprintf ("Error: Unable to append file %s to %s",
                              repfile, app_repfile));      

   fp = fopen (app_mpffile, "a");
   fputline (fp, "|----------------------------------------");
   fputline (fp, "----------------------------------------|\n");
   fclose (fp);
   if (vsAppend (mpffile,app_mpffile) == FALSE)
      status_msg (e, sprintf ("Error: Unable to append file %s to %s",
                              mpffile, app_mpffile));      
}
<cmdSave_Treport,prvCmdSave_Treport> := CmdCreate(vtime.fac,
                                                  save_treport,
                                                  "_save_treport",
                                                  "Save Verification Report",
                                                  list (vcWv.Param ("Report File Name", vlStringVT, 
                                                                    vcWv.Constant, ""),
                                                        vcWv.Param ("MPF File Name", vlStringVT, 
                                                                    vcWv.Constant, "")));

/*===========================================================================*/
/*==                 _ s a v e _ t r e p o r t                             ==*/
/*===========================================================================*/
/*\Function _copy_report  -- Copies the saved report and mpf files to originals.
  
  DESCRIPTION
      This command copies the currently saved  trace report and mpf information 
      into the original report and mpf files respectively.

*/
function copy_treport(d,e, app_repfile, app_mpffile)
{
   repfile    := GetMtvVar(d, MtvVar.MV_file_tmp_trace, MtvValue.VALUE);
   mpffile    := GetMtvVar(d, MtvVar.MV_file_tmp_mpf_trace, MtvValue.VALUE);

   if (vsCopy (app_repfile, repfile) == FALSE)
      status_msg (e, sprintf ("Error: Unable to copy temporary file %s to %s",
                              app_repfile, repfile));      
   if (vsCopy (app_mpffile, mpffile) == FALSE)
      status_msg (e, sprintf ("Error: Unable to copy temporary file %s to %s",
                              app_mpffile, mpffile));      
}

<cmdCopy_Treport,prvCmdCopy_Treport> := CmdCreate(vtime.fac,
                                          copy_treport,
                                                  "_copy_treport",
                                                  "Copy Verification Report",
                                                  list (vcWv.Param ("Report File Name", vlStringVT, 
                                                                    vcWv.Constant, ""),
                                                        vcWv.Param ("MPF File Name", vlStringVT, 
                                                                    vcWv.Constant, "")));

/*===========================================================================*/
/*==                 T r a c e                                             ==*/
/*===========================================================================*/
/*\Function Trace  -- Initiate a Motive path trace analysis
  
  DESCRIPTION
      This command invokes a trace of the specified type.  The tracedlg
      facility is used to gather arguments if no arguments are specified.
       
  CAUTION
      This command does not syntax check the arguments.  If any arguments
      are specified then the command is assumed to be complete.  This is
      not typical of Viewlogic Core style commands.  Since Motive will 
      syntax check the command it will be safer to provide this checking 
      there rather than duplicating it here.
       
*/
function trace(d,e, str)
{
   constraint := nil;
   token      := nil;
   cmd        := "";
   cmdlist    := nil;
   q          := GetQueue (d);
   all_cons   := FALSE;

   foreach(token;string_loop(str," ")) {
      if (string_eqi("setup",token) || string_eqi("hold",token) || string_eqi("pulse",token)){
         constraint = token;
      }
      else if (string_eqi ("all", token)) {
         all_cons = TRUE;
         cmdlist  = cons( "mtv "+ cmd + " hold_falling", cmdlist);
         cmdlist  = cons( "mtv "+ cmd + " hold_rising", cmdlist);
         cmdlist  = cons( "mtv "+ cmd + " setup_falling", cmdlist);
         cmdlist  = cons( "mtv "+ cmd + " setup_rising", cmdlist);
      }
      else if ((constraint != nil) && 
               (string_eqi("rising",token) || string_eqi("falling",token) ||
                string_eqi("high",token)   || string_eqi("low",token))) 
         cmd = cmd + " " + constraint + "_" + token;

      else if (string_eqi("slack_on",token))
         vtq.CmdRun (q,e, "slack enable");
      else if (string_eqi("slack_off",token))
         vtq.CmdRun (q,e,"slack disable");
      else if (string_eqi("detail_on",token))
         vtq.CmdRun (q,e, "detail enable");
      else if (string_eqi("detail_off",token))
         vtq.CmdRun (q,e,"detail disable");
      else if (string_eqi("expl_on",token))
         vtq.CmdRun (q,e,"explanation enable");
      else if (string_eqi("expl_off",token))
         vtq.CmdRun (q,e,"explanation disable");

      else {
         cmd = cmd + " " + token;
         constraint = nil;
      }
   }

   if (all_cons == TRUE){
      repfile := Cr8FileInCaseDir (d,"tmp","trc");
      mpffile := Cr8FileInCaseDir (d,"tmp","mpf");
      vscript.vsDelete (repfile, vscript.vsFILE);
      vscript.vsDelete (mpffile, vscript.vsFILE);

      foreach(cmd;vscript.list_iter(cmdlist)){
         vtq.CmdRun (q,e,cmd); 
         vtq.CmdRun (q,e, "_save_treport " + repfile + " " + mpffile); 
      }
      vtq.CmdRun (q,e, "_copy_treport " + repfile + " " + mpffile ); 
   }
   else
      vtq.CmdRun(q,e,"mtv "+cmd);

   vtq.CmdRun(q,e,"trepfile");

   StoreTraceArgs(d,str);
} /* trace */

function StoreTraceArgs(d,str)
{
   trclist := vtime.GetTraceList(d);
   trclist  = cons(str,trclist);
   vtime.SetTraceList(d,trclist);
}

function GetTraceArgs(d)
{
   trccmd := "tw";
   trclist := GetTraceList(d);
   if (trclist != nil)
      trccmd = car(trclist);

   trccmd;
}

<cmdTrace,prvCmdTrace> := vtime.CmdCreate(vtime.fac,
                                          trace,
                                          "Trace", "Trace Command Editor", 
                                          list (vcWv.Param ("",
                                                            vlStringRestOfLine1VT, 
                                                            vcWv.Dynamic, GetTraceArgs,
                                                            tracedlg.fac,
                                                            case <- vlPref.Val(prefs,"capslock"))));
                                                            /*vlPref.Val (prefs, "capslock"))));*/

vcCmd.EnableApply(cmdTrace, vlOnOff.On);


/*===========================================================================*/
/*==                      v t m                                            ==*/
/*===========================================================================*/
/*\Function vtm -- Command line editor for VTM netlister
  
  DESCRIPTION
      This command invokes the editor for the netlister. The vtimevtm
      facility is the editor used for this purpose.
       
  CAUTION
*/
function vtm(d, e, str)
{
   StoreVtmArgs (d,str);
   RunVtm(d,e,str);
/*
   cmd := "Runvtm " + str;
   vtq.CmdRun (GetQueue (d), e, cmd);
   vtime.Log (d,cmd, vlTrueFalse.False);
*/
}

function StoreVtmArgs(d,str)
   SetVtmList (d,str);

function GetVtmArgs(d){
   if (GetVtmList (d) == nil)
      StoreVtmArgs (d,GetBlockName(d) + " -ca  " + GetCaseName(d) + " -li library.lst");
   GetVtmList (d);
}
cmdVtm := CmdCreateNoQueue(vtime.fac,
                           vtm,
                           "Vtm", "VTM Netlist Command Editor", 
                           list (vcWv.Param ("",
                                             vlStringRestOfLine1VT, 
                                             vcWv.Dynamic, GetVtmArgs,
                                             vtimevtm.fac)));

vcCmd.EnableApply(cmdVtm, vlOnOff.On);

                                      
/*===========================================================================*/
/*==                      RunVtmcmd                                        ==*/
/*===========================================================================*/
function RunVtm(d,e, vtm_args) {

   /* Emulate the "current" command before running vtm. */
   Rgn := DisplayGetRgnByName(d, "Vtm");
   if (Rgn != nil){
      RgnWin := vcRegion.Window (Rgn);
      FacWin := vcEvent.Window (e);
      if (FacWin == RgnWin)
         DisplaySetCur(d,Rgn);
   }
   
   cmd := "RunVtm " + vtm_args;
   vtime.Log (d,cmd, vlTrueFalse.False);
   VtmRun (d, vtm_args);
}
<cmdRunVtm,prvCmdRunVtm> := CmdCreate(vtime.fac,
                                RunVtm,
                                "RunVtm",
                                "Run Vtm with arguments",
                                list (vcWv.Param ("Motive Command", 
                                                  vlStringRestOfLineVT, 
                                                  vcWv.Constant, "")));



/*===========================================================================*/
/*==             V i s i t E r r o r s                                     ==*/
/*===========================================================================*/
/*\Function visiterrors  -- 
  
  DESCRIPTION
       
*/
function visiterrors(d,e)
{
   win := vcEvent.Window(e);
   
   rval := ParseVfyMpf(d,e);

   if (rval != nil) {

      Vmpf := car(rval);
      args := cdr(rval);

      setups := list_ref(args,2);
      holds  := list_ref(args,3);
      pulses := list_ref(args,4);
      skews  := list_ref(args,5);

      su_cnt := length(setups);
      hd_cnt := length(holds);
      pw_cnt := length(pulses);
      sk_cnt := length(skews);

      if (su_cnt > 0 || hd_cnt >0 || pw_cnt > 0  || sk_cnt > 0 ) {
         vcWin.ClearStatus(win);
         vcWin.AddStatus(win,"Building Verification Errors Summary Dialog... Please Wait...");
      
         rgn := vcRegion.Create(sprintf("Verification Run # %s",car(Vmpf.RunNumber)), 
                                visiterr.fac,
                                args);
      
         vcWin.Popup (vcEvent.Window (e), rgn, rgn,vcWin.NoMenus);
      }
      else {
         msg := sprintf(cdr(vtime.msg_VMpfNoErrsReported));
         vtime.AlertBox(d,e,car(msg_VMpfNoErrsReported),vlErrSev.Note,msg);
      }

      vcWin.ClearStatus(win);
      vcWin.AddStatus(win,
                      sprintf("  Summary:   %d Setup,   %d Hold,   %d Pulsewidth,   %d Skew  Violations.",
                              length(setups), length(holds), length(pulses),length(skews)));
      
   }
   else {
      msg := sprintf(cdr(vtime.msg_VMpfUnavailable));
      vtime.AlertBox(d,e,car(msg_VMpfUnavailable),vlErrSev.Error,msg);
   }
} /* visiterrors */ 

<cmdVisiterrors,
cmdPrvVisiterrors> := vtime.CmdCreate(vtime.fac,
                                      visiterrors,
                                      "Visiterrors", "List Setup/Hold/Pulsewidth/Skew Errors",
                                      nil);



/*===========================================================================*/
/*==                      P a r a m e t e r s                             ==*/
/*===========================================================================*/
/*\Function parameters -- Invoke timing parameters GUI
  
  DESCRIPTION
      This command brings up the timing parameters GUI. From this GUI
      the user will be able to set/reset various design parameters
      including all report, trace, latch modes of operation, and
      interconnect analysis data.
       
  CAUTION
       This GUI will queue up all user input. This will also prompt
       the user whether the engine has to be restarted, if necessary.
       
*/
function parameters(d,e)
{
   function IsVtimeReady()
   {
      Queue := GetQueue (d);
      value := GetMtvVar(d, MtvVar.MV_design_loaded,
                               MtvValue.VALUE);
      
      if (vtq.IsHeld (Queue) == vlTrueFalse.False ||
          (value != nil && string_eqi (value, "true") == TRUE))
         return (TRUE);
      
      else
         status_msg(e,"ViewTime is not yet ready. Try again");

      FALSE;
   } /* IsVtimeReady */



   /* Check whether MtvVarTable is already initialized */
   if (IsVtimeReady() == FALSE)
      return (FALSE);

   vcWin.Busy (vcEvent.Window (e), TRUE);
   Params := vcRegion.Create("Timing Parameters", 
                            paramtrs.fac,
                               list (d,e),
                            macro_on <- TRUE);

   w := vcWin.Popup (vcEvent.Window (e),
                     Params, Params,vcWin.NoMenus);
   vcWin.Busy (vcEvent.Window (e), FALSE);
   vcWin.Forward (vcEvent.Window (e), w);
} 

cmdParameters := vtime.CmdCreateNoQueue(vtime.fac,
                                        parameters,
                                        "Parameters", "Setup Motive Parameters",
                                        nil);


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
function Smargin(d,e,value)
{
   cmd := sprintf ("mtv sv %s ",
                      GetMtvVar(d, MtvVar.MV_report_margin_setup, MtvValue.NAME));
   vtq.CmdRun (GetQueue (d),e,cmd + convert (vlStringVT, value));
}
<cmdSmargin,
prvCmdSmargin> := vtime.CmdCreate(vtime.fac,
                                 Smargin,
                                 "Smargin",
                                 "Setup Margin",
                                 list (vcWv.Param("Time (ns)",
                                                  vlFloatVT,
                                                  vcWv.Sticky,0.0)));

function Hmargin(d,e,value)
{
   cmd := sprintf ("mtv sv %s ",
                      GetMtvVar(d, MtvVar.MV_report_margin_hold, MtvValue.NAME));
   vtq.CmdRun (GetQueue (d),e,cmd + convert (vlStringVT, value));
}
<cmdHmargin,
prvCmdHmargin> := vtime.CmdCreate(vtime.fac,
                                 Hmargin,
                                 "Hmargin",
                                 "Hold Margin",
                                 list (vcWv.Param("Time (ns)",
                                                  vlFloatVT,
                                                  vcWv.Sticky,0.0)));

function Skmargin(d,e,value)
{
   cmd := sprintf ("mtv sv %s ",
                      GetMtvVar(d, MtvVar.MV_report_margin_skew, MtvValue.NAME));
   vtq.CmdRun (GetQueue (d),e,cmd + convert (vlStringVT, value));
}
<cmdSkmargin,
prvCmdSkmargin> := vtime.CmdCreate(vtime.fac,
                                 Skmargin,
                                 "Skmargin",
                                 "Skew Margin",
                                 list (vcWv.Param("Time (ns)",
                                                  vlFloatVT,
                                                  vcWv.Sticky,0.0)));


/*===========================================================================*/
/*==                          ForceD2q                                     ==*/
/*===========================================================================*/
/*\Docpage ForceD2q -- Enable/Disable Force latch D to Q interactively

  DESCRIPTION
       This command may be used interactively when examining latch traces.
       It forces MOTIVE's worst case trace to display the path through
       D-to-Q of the latch even if it is NOT in the worst case path.This 
       can be useful in seeing why MOTIVE did not consider the D-to-Q path
       to be the wrost case path.

       A warning is issued in the trace report when this is enabled.
*/
function forced2q(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_trace_force_d2q, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Force Latch D-to-Q in the Trace Report");

}
<cmdForced2q,prvCmdForced2q> := CmdCreate(vtime.fac,
                                    forced2q,
                                    "Forced2q",
                                    "Force_Latch D-to-Q",
                                    list (vcWv.Param ("Force D2Q", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Enable)));

/*===========================================================================*/
/*==                            Sigp                                       ==*/
/*===========================================================================*/
/*\Docpage Sigp - Enable/Disable Sigp Optimization algorithm

  DESCRIPTION
       Sig(p) is a loop breaking algorithmthat can greatly improve
       MOTIVE's computational efficiency for circuits with a high 
       degree of combinatorial latch loops. Sig(p) is a form of 
       automatic bus loop breaking which takes into account information
       contained within MOTIVE timing models. However not all designs
       should use Sig(p) -- refer to Viewtime `reference Manual.
*/
function sigp(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_sigp, MtvValue.NAME);
   
   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
               " Sig(p) optimization  mode in the Trace Report");
}
<cmdSigp,prvCmdSigp> := CmdCreate(vtime.fac,
                                  sigp,
                                  "sigp",
                                  "Sig(p) Optimization",
                                  list (vcWv.Param ("Loop Breaking", 
                                                    vtimeEnableDisableVT, 
                                                    vcWv.Constant, 
                                                    vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                            Reconvergence                              ==*/
/*===========================================================================*/
/*\Docpage -- Enable/Disable Common Path Ambiguity elimination

  DESCRIPTION
       This command allows the user to eliminate "reconvergent fanouts" or
       "common path ambiguity". 
*/
function reconvergence(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_reconvergence, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Reconvergence  optimization  mode in the Trace Report");
}
<cmdReconvergence,
prvCmdReconvergence> := CmdCreate(vtime.fac,
                                  reconvergence,
                                  "reconvergence",
                                  "Reconvergence Fanout",
                                  list (vcWv.Param ("Allow Correction", 
                                                    vtimeEnableDisableVT, 
                                                    vcWv.Constant, 
                                                    vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                          bidiio                                       ==*/
/*===========================================================================*/
/*\Docpage bidiio -- Control bi-directional I/O tracing

  DESCRIPTION
*/
function bidiio(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_bi_io_trc, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Bi-directional I/O tracing  mode in the Trace Report");
}
<cmdBidiio,prvCmdBidiio> := CmdCreate(vtime.fac,
                                  bidiio,
                                  "bidiio",
                                    "Bidirectional I/O Tracing",
                                    list (vcWv.Param ("Allow Tracing", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));


/*===========================================================================*/
/*==                           bidipin                                     ==*/
/*===========================================================================*/
/*\Docpage bidipin -- Control bi-directional internal PIN tracing

  DESCRIPTION
       
*/
function bidipin(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_bi_pin_trc, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Bi-directional Internal Pin tracing  mode in the Trace Report");
}
<cmdBidipin,prvCmdBidipin> := CmdCreate(vtime.fac,
                                  bidipin,
                                  "bidipin",
                                    "Bidirection Internal Pin Tracing",
                                    list (vcWv.Param ("Allow Tracing", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                            Latchmode                                  ==*/
/*===========================================================================*/
/*\Docpage Latchmode -- Setup latch analysis mode

  DESCRIPTION
       
*/
vcWv.MakeEnum ("LatchTypeVT", list ("Never", "Setup", "Setup_Hold", "Always"));
function Latch(d,e,type)
{
   declare arg;

   res := sprintf ("mtv sv %s ",
                   GetMtvVar(d, MtvVar.MV_analysis_latch_mode, MtvValue.NAME));

   if (type == LatchType.Never)
      arg = "disable";
   else if (type == LatchType.Setup)
      arg = "s";
   else if (type == LatchType.Setup_Hold)
      arg = "sh";
   else
      arg = "all";

   vtq.CmdRun (GetQueue (d),e,res + arg);
}
<cmdLatch,
prvCmdLatch> := vtime.CmdCreate(vtime.fac,
                                 Latch,
                                 "Latchmode",
                                 "Latch Analysis Mode",
                                 list (vcWv.Param ("Select Mode",
                                                   LatchTypeVT, 
                                                   vcWv.Sticky,
                                                   LatchType.Never)));

/*===========================================================================*/
/*==                           L d e p t h                                 ==*/
/*===========================================================================*/
/*\Docpage Ldepth -- Latch depth for latch analsysi

  DESCRIPTION
       
*/
function ldepth(d, event, val) {
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_latch_depth, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
}
<cmdLdepth,prvCmdLdepth> := CmdCreate(vtime.fac,
                                    ldepth,
                                    "Ldepth",
                                      "Latch Depth",
                                      list (vcWv.Param ("Number",
                                                        vcWv.Sticky,5)));

/*===========================================================================*/
/*==                         L A R I A T                                   ==*/
/*===========================================================================*/
/*\Docpage Lariat -- Latch Asymptotic Reconvergence of Input Arrival Time

  DESCRIPTION
       
*/
function lariat(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_lariat, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " LARIAT    mode in the Trace Report");
}
<cmdLariat,prvCmdLariat> := CmdCreate(vtime.fac,
                                  lariat,
                                  "lariat",
                                    "LARIAT Mode",
                                    list (vcWv.Param ("Mode", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                        O c c m i s s i n g                            ==*/
/*===========================================================================*/
/*\Docpage Occmissing -- Report all occurrences that are missing timing models

  DESCRIPTION
       
*/
function occmissing(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_msg_occ_missing,MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
               " Missing Occurrence warnings in the Verification Report");
}
<cmdOccmissing,
prvCmdOccmissing> := CmdCreate(vtime.fac,
                               occmissing,
                               "occmissing",
                               "Missing Timing Models for Occurrences",
                               list (vcWv.Param ("Warning", 
                                                 vtimeEnableDisableVT, 
                                                 vcWv.Constant, 
                                                 vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                       Z c p w a r n                                   ==*/
/*===========================================================================*/
/*\Docpage Zcpwarn -- Warn about all zero-cycle paths.

  DESCRIPTION
       
*/
function zcpwarn(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_msg_zcp_warn, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Zero Cycle Path mode in the Verification Report");
}
<cmdZcpwarn,prvCmdZcpwarn> := CmdCreate(vtime.fac,
                                  zcpwarn,
                                  "zcpwarn",
                                    "Zero Cycle Path Warning",
                                    list (vcWv.Param ("Warning", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));

/*===========================================================================*/
/*==                       X l i n e                                       ==*/
/*===========================================================================*/
/*\Docpage Xline -- Command to set Tansmission line analysis parameters.

  DESCRIPTION
       This comand sets up parameters for Transmission Line type
		 of interconnect analysis.
       
*/
function Xline(d,e,pin_cap,rfactor,trc_cap, trc_vel, locfile)
{
   declare cmd;

	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);

	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {
      Queue := GetQueue (d);
      
      cmd = sprintf ("mtv sv %s xline", GetMtvVar(d, MtvVar.MV_iam, MtvValue.NAME));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_def_pin_cap, 
                                               MtvValue.NAME),
                     convert (vlStringVT, pin_cap));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_route_factor, 
                                               MtvValue.NAME),
                     convert (vlStringVT, rfactor));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_trc_cap, 
                                               MtvValue.NAME),
                     convert (vlStringVT, trc_cap));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_trc_vel, 
                                               MtvValue.NAME),
                     convert (vlStringVT, trc_vel));
      vtq.CmdRun (Queue,e,cmd);
      if (string_eqi (locfile, ""))
         cmd = sprintf ("mtv sv %s  \"\"", GetMtvVar(d, MtvVar.MV_iam_xline_file_loc, 
                                                     MtvValue.NAME));
      else
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_file_loc, 
                                                  MtvValue.NAME),							
                        locfile);
      vtq.CmdRun (Queue,e,cmd);
      
      vtq.CmdRun (Queue,e,"mtv pv /iam");
   }
}
<cmdXline,
prvCmdXline> := vtime.CmdCreate(vtime.fac,
                                 Xline,
                                 "Xline",
                                 "Transmission Line Analysis",
                                 list (vcWv.Param ("Default Pin Cap.",
                                                   vlFloatVT,
                                                   vcWv.Sticky,"5.0000"),
                                       vcWv.Param("Route Factor",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 1.5000),
                                       vcWv.Param("Trace Capacitance",
                                                  vlFloatVT,
                                                  vcWv.Sticky,3.0000),
                                       vcWv.Param("Trace Velocity",
                                                  vlFloatVT,
                                                  vcWv.Sticky,2.0000),
													vcWv.Param ("XY Location File", 
																	vlStringRestOfLineVT, 
																	vcWv.Sticky, "",
																	vcFilePop.fac,
																	filter <- "*.loc",
																	file_columns <- 1,
																	dir_columns <- 1)));

/*===========================================================================*/
/*==                         C a p p                                       ==*/
/*===========================================================================*/
/*\Docpage Capf -- Setup parameters for Capacitive from Placement analysis.

  DESCRIPTION
       This comand sets up parameters for capacitive from placement type
		 of interconnect analysis.
       
*/
function Capp(d,e,pin_cap,rfactor,trc_cap, locfile)
{
   declare cmd;

	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {

      Queue := GetQueue (d);
      
      cmd = sprintf ("mtv sv %s capp", GetMtvVar(d, MtvVar.MV_iam, MtvValue.NAME));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capp_def_pin_cap, 
                                               MtvValue.NAME),
                     convert (vlStringVT, pin_cap));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capp_route_factor, 
                                               MtvValue.NAME),
                     convert (vlStringVT, rfactor));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capp_trc_cap, 
                                               MtvValue.NAME),
                     convert (vlStringVT, trc_cap));
      vtq.CmdRun (Queue,e,cmd);
      if (string_eqi (locfile, ""))
         cmd = sprintf ("mtv sv %s  \"\"", GetMtvVar(d, MtvVar.MV_iam_xline_file_loc, 
                                                     MtvValue.NAME));
      else
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_xline_file_loc, 
                                                  MtvValue.NAME),							
                        locfile);
      vtq.CmdRun (Queue,e,cmd);
      
      vtq.CmdRun (Queue,e,"mtv pv /iam");
   }
}
<cmdCapp,
prvCmdCapp> := vtime.CmdCreate(vtime.fac,
                                 Capp,
                                 "Capp",
                                 "Capacitive From Placement Analysis",
                                 list (vcWv.Param ("Default Pin Cap.",
                                                   vlFloatVT,
                                                   vcWv.Sticky,"5.0000"),
                                       vcWv.Param("Route Factor",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 1.5000),
                                       vcWv.Param("Trace Capacitance",
                                                  vlFloatVT,
                                                  vcWv.Sticky,3.0000),
													vcWv.Param ("XY Location File", 
																	vlStringRestOfLineVT, 
																	vcWv.Sticky, "",
																	vcFilePop.fac,
																	filter <- "*.loc",
																	file_columns <- 1,
																	dir_columns <- 1)));

/*===========================================================================*/
/*==                            C a p f                                    ==*/
/*===========================================================================*/
/*\Docpage Capf -- Setup parameters for Capacitive from fanout analysis

  DESCRIPTION
       This comand sets up parameters for capacitive from fanout type
		 of interconnect analysis.
*/
vcWv.MakeEnum ("CalcTypeVT", list ("Linear", "Bilinear"));
function Capf(d,e,pin_cap,pf_fo,ctype,s1off, s1pf, brkpt, s2off, s2pf)
{
   declare cmd;
	
	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {

      Queue := GetQueue (d);
      
      cmd = sprintf ("mtv sv %s capf", GetMtvVar(d, MtvVar.MV_iam, MtvValue.NAME));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_def_pin_cap, 
                                               MtvValue.NAME),
                     convert (vlStringVT, pin_cap));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_lin_slope, 
                                               MtvValue.NAME),
                     convert (vlStringVT, pf_fo));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_cap_func, 
                                               MtvValue.NAME),
                     convert (vlStringVT, ctype));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_bi_seg1_offset, 
                                               MtvValue.NAME),
                     convert (vlStringVT, s1off));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_bi_seg1_slope, 
                                               MtvValue.NAME),
                     convert (vlStringVT, s1pf));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_bi_break_point, 
                                               MtvValue.NAME),
                     convert (vlStringVT, brkpt));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_bi_seg2_offset, 
                                               MtvValue.NAME),
                     convert (vlStringVT, s2off));
      vtq.CmdRun (Queue,e,cmd);
      cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_capf_bi_seg2_slope, 
                                               MtvValue.NAME),
                     convert (vlStringVT, s2pf));
      vtq.CmdRun (Queue,e,cmd);
      
      vtq.CmdRun (Queue,e,"mtv pv /iam");
   }
}
<cmdCapf,
prvCmdCapf> := vtime.CmdCreate(vtime.fac,
                                 Capf,
                                 "Capf",
                                 "Capacitive From Fanout Analysis",
                                 list (vcWv.Param ("Default Pin Cap.",
                                                   vlFloatVT,
                                                   vcWv.Sticky,"5.0000"),
                                       vcWv.Param("Picofarads/Fanout",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 0.1200),
													vcWv.Param ("Calculation Type",
                                                   CalcTypeVT, 
                                                   vcWv.Sticky,
                                                   CalcType.Linear),
                                       vcWv.Param("Seg 1 Offset",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 0.0000),
                                       vcWv.Param("Seg 1 Pf/fanout",
                                                  vlFloatVT,
                                                  vcWv.Sticky,0.0000),
                                       vcWv.Param("Break Point",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 0.0),
                                       vcWv.Param("Seg 2 Offset",
                                                  vlFloatVT,
                                                  vcWv.Sticky, 0.0000),
                                       vcWv.Param("Seg 2 Pf/fanout",
                                                  vlFloatVT,
                                                  vcWv.Sticky,0.0000)));


/*===========================================================================*/
/*==               I n t e r c o n n e c t                                 ==*/
/*===========================================================================*/
/*\Docpage Interconnect -- Select an interconnect analysis option 
  
  DESCRIPTION
  
*/
vcWv.MakeEnum ("IntconTypeVT", list ("None","Xline", "CapPlace", "CapFanout", "IDD"));
function Interconnect(d,e,type)
{
	declare var, arg;

	restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {
      var = GetMtvVar(d, MtvVar.MV_iam, MtvValue.NAME);
      if (type == IntconType.None)
         arg = "none";
      else if (type == IntconType.Xline)
         arg = "xline";
      else if (type == IntconType.CapPlace)
         arg = "capp";
      else if (type == IntconType.CapFanout)
         arg = "capf";
      else if (type == IntconType.IDD)
         arg = "idd";
         
      Queue := GetQueue (d);
      vtq.CmdRun (Queue,e,sprintf ("mtv sv %s %s", var, arg));
      vtq.CmdRun (Queue, e,sprintf ("Remark Interconnect analysis set to %s",
                                    convert(vlStringVT,type)));
      vtq.CmdRun (Queue,e,"mtv pv /iam");
   }
}	
<cmdInterconnect,
prvCmdInterconnect> := vtime.CmdCreate(vtime.fac,
													Interconnect,
													"Interconnect",
													"Interconnect Analysis",
													list (vcWv.Param ("Select Mode",
                                                         IntconTypeVT, 
                                                         vcWv.Sticky,
                                                         IntconType.None)));






/*===========================================================================*/
/*==                    v s l o a d                                        ==*/
/*===========================================================================*/
/*\Function vsload  -- Load the specified Viewscript file into a package
  
  DESCRIPTION
     This command loads the specified Viewscript file into the specified
     namespace (package).  If an invalid package name is specified, then
     an unbound variable message is posted.  If no package is specified
     then "vtime" is used as the package.
       
*/
function vsload(d, e, file, pkg) 
{
   pthname := nil;
   if (string_eq ((pkg = string_trim(pkg)),"")) 
      pkg = "vtime"; ;
   
   if (file = vscript.vsConvPath(file)) {
      
      if (fexists(file))
         pthname = file;
      else 
         pthname = vscript.vsGetini(file);
   }
   
   if (pthname == nil)
      pthname = vscript.vsAbsPath(file);

   if (pthname != nil) {
      vscript.load(pthname,eval_string(pkg+";",current_package()));
      vtq.CmdRun(vtime.GetQueue(d),e,"remark Loading: [ "+pthname+" ]  Namespace: ["+pkg+".xxx]");
   }
   else {
      msg := sprintf(cdr(msg_VsFileNExist),file);
      vtime.AlertBox(d,e,car(msg_VsFileNExist),vlErrSev.Error,msg);
   }
} /* vsload */
cmdVsload := vtime.CmdCreate(vtime.fac,
                                  vsload,
                                  "vsload",
                                  "Load specified viewscript module",
                                  list (vcWv.Param ("Viewscript File:", 
                                                    vlStringVT, 
                                                    vcWv.Sticky,"",
                                                    vcFilePop.fac,
                                                    filter <- "*.vs",
                                                    file_columns <- 1,
                                                    dir_columns <- 1),
                                        vcWv.Param ("Package:",
                                                    vlStringRestOfLineVT,
                                                    vcWv.Sticky,"vtime")));


/*===========================================================================*/
/*==                     ReportType                                        ==*/
/*===========================================================================*/
/*\Docpage ReportType -- Setup motive to report just the violations or all margins

  DESCRIPTION
       This command allows the user to setup motive to report just the 
       'violations' or 'all margins' while performing a 'verify' command.
       This report is displayed in the main window as soon as the verify
       command is finished.
*/
vcWv.MakeEnum ("ReportTypeVT", list ("AllMargins", "Violations"));
function Reporttype(d,e,flag)
{
   cmd := sprintf ("mtv sv %s ", GetMtvVar(d, MtvVar.MV_report_margin_report,
                                           MtvValue.NAME));
   if (flag == ReportType.Violations)
      cmd = cmd + "vio";
   else
      cmd = cmd + "all";
   
   Queue := GetQueue (d);
   vtq.CmdRun (Queue,e,cmd);
   vtq.CmdRun (Queue, e, sprintf ("remark Report '%s' during verification",
                                  convert (vlStringVT, flag)));
}
<cmdReportType,
prvCmdReportType> := vtime.CmdCreate(vtime.fac,
                                     Reporttype,
                                     "Reporttype",
                                     "Report All Margins or Violations Only",
                                     list (vcWv.Param ("Select Mode",
                                                       ReportTypeVT, 
                                                       vcWv.Sticky,
                                                       ReportType.Violations)));

/*===========================================================================*/
/*==                           Gsr                                         ==*/
/*===========================================================================*/
/*\Docpage Gsr -- Turn on or off the Global slack report during verification
*/
function gsr(d,event,val)
{
   Queue := GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_analysis_gslack, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,val));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,val) + 
                             " Global slack report during verification");
}
<cmdGsr,prvCmdGsr> := CmdCreate(vtime.fac,
                                  gsr,
                                  "gsr",
                                    "Global Slack Report",
                                    list (vcWv.Param ("Report", 
                                                      vtimeEnableDisableVT, 
                                                      vcWv.Constant, 
                                                      vtimeEnableDisable.Disable)));


/*===========================================================================*/
/*==                               Sv                                      ==*/
/*===========================================================================*/
/*\Docpage Sv -- set a motive variable
*/
function Sv(d,e, name, value) {
   vtq.CmdRun (GetQueue (d), e, "current Transcript");
   vtq.CmdRun (GetQueue (d), e, "mtv sv " + name + " " + value);
}
<cmdSv,prvCmdSv> := CmdCreate(vtime.fac,
                              Sv,
                              "SV",
                              "Set a Motive variable",
                              list (vcWv.Param ("Name", 
                                                vlStringVT, 
                                                vcWv.Sticky, ""),
                                    vcWv.Param ("Value", 
                                                vlStringRestOfLineVT, 
                                                vcWv.Sticky, "")));

/*===========================================================================*/
/*==                               Pv                                      ==*/
/*===========================================================================*/
/*\Docpage Pv -- Prints a motive variable
*/
function Pv(d,e, name) {
   vtq.CmdRun (GetQueue (d), e, "current Transcript");
   vtq.CmdRun (GetQueue (d), e, "mtv pv " + name );
}
<cmdPv,prvCmdPv> := CmdCreate(vtime.fac,
                              Pv,
                              "PV",
                              "Print a Motive variable",
                              list (vcWv.Param ("Variable Name", 
                                                vlStringVT, 
                                                vcWv.Sticky, "")));

vcWv.MakeEnum ("McpActionVT", list ("New", "Add", "Del"));
/*===========================================================================*/
/*==                   M C P                                                ==*/
/*===========================================================================*/
/*\Docpage mcp -- Set/Delete mcp's
*/
function mcp(d,e,command,arg) 
   vtq.CmdRun(GetQueue(d),e,"mtv mcp " + convert(vlStringVT,command) + " " + arg);

   <cmdMcp,prvCmdMcp> := CmdCreate(vtime.fac,
                                   mcp,
                                   "MCP",
                                   "Set/Delete Motive MCPs",
                                   list (vcWv.Param ("Action", 
                                                     McpActionVT, 
                                                     vcWv.Sticky, McpAction.Del),
                                         vcWv.Param ("Argument", 
                                                     vlStringVT, 
                                                     vcWv.Sticky, "")));



/*===========================================================================*/
/*==                   i g n o r e _ s u b p 1                              ==*/
/*===========================================================================*/
/*\Docpage ignore_subp1 -- NOT FUNCTIONAL  (TESTING ONLY)
*/
function ignore_subp1(cmdln)
{
   sys := nil;
    function my_output_handler(str) print (str);
    function my_error_handler(str) print (str);
    function my_exit_handler(mode, code) printf ("exit> %s %s", mode, code);

    env := nil;
    host := nil;
    sys = vcSys.Command (cmdln, env, my_output_handler, my_error_handler,
                         my_exit_handler, host);
    if (sys != nil)
       vcQuit.IgnoreSubprocess (sys);
    else
       printf ("AMTV not found.\n");

    printf ("Ignoring subp...%s\n",sys);
    vcQuit.IgnoreSubprocess (sys);
    sys;
}

/*===========================================================================*/
/*==                       I d d                                           ==*/
/*===========================================================================*/
/*\Docpage Idd -- Command to set interconnect delay data parameters

  DESCRIPTION
       This comand sets up parameters for Interconnect delay data
       option.       
*/
function Idd(d,e,useidd,summarize,lhmin, lhmax, hlmin, hlmax, iddfile)
{
   declare cmd;

   if ((iddfile == nil) || string_eqi (iddfile,""))
      iddfile = "\"\"";
   else 
      iddfile = GetFileInHierarchy(d,iddfile);

   restart := GetMtvVar(d, MtvVar.MV_design_loaded, MtvValue.VALUE);
	if (string_eqi (restart, "TRUE")) {
      msg := cdr(vtime.msg_MustRestartFirst);
      vtime.AlertBox(d,e,car(vtime.msg_MustRestartFirst),vlErrSev.Error,msg);
   }
   else {
	
      Queue := GetQueue (d);
      
      /* Set mode to IDD if necessary */ 
      if (useidd == vlYesNo.Yes){
         if (string_eqi(iddfile, "\"\"")) {
            vtime.AlertBox(d,e,car(msg_NoIddFileName),vlErrSev.Error,cdr(msg_NoIddFileName));
            Err := TRUE;  /* Hack!! This command should not be permitted without an idd file */
         }
         else {
            Err := FALSE;  /* Hack!!   */
            val := GetMtvVar(d, MtvVar.MV_iam, MtvValue.VALUE);
            if (string_eqi (val, "none")){
               cmd = sprintf ("mtv sv %s idd", GetMtvVar(d, MtvVar.MV_iam, 
                                                         MtvValue.NAME));
               vtq.CmdRun (Queue,e,cmd);
            }
            
            cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_file_idd, 
                                                     MtvValue.NAME),
                           iddfile);
            vtq.CmdRun (Queue,e,cmd);
         }
      }
      else{
         Err := vlTrueFalse.False; /* Hack!!   */
         val := GetMtvVar(d, MtvVar.MV_iam, MtvValue.VALUE);
         if (string_eqi (val, "idd"))
            vtq.CmdRun (Queue,e,sprintf ("mtv sv /iam none" ));
         
         cmd = sprintf ("mtv scv %s %s",   GetMtvVar(d, MtvVar.VT_file_idd, 
                                                     MtvValue.NAME),
                        iddfile);
         vtq.CmdRun (Queue,e,cmd);
         cmd = sprintf ("mtv sv %s \"\"", GetMtvVar(d, MtvVar.MV_file_idd, 
                                                    MtvValue.NAME));
         vtq.CmdRun (Queue,e,cmd);
      }
      
      if (!Err) {
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_idd_summary, 
                                                  MtvValue.NAME),
                        convert (vlStringVT, summarize));
         vtq.CmdRun (Queue,e,cmd);
         
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_idd_default_lhmin, 
                                                  MtvValue.NAME),
                        convert (vlStringVT, lhmin));
         vtq.CmdRun (Queue,e,cmd);
         
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_idd_default_lhmax, 
                                                  MtvValue.NAME),
                        convert (vlStringVT, lhmax));
         vtq.CmdRun (Queue,e,cmd);
         
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_idd_default_hlmin, 
                                                  MtvValue.NAME),
                        convert (vlStringVT, hlmin));
         vtq.CmdRun (Queue,e,cmd);
         
         cmd = sprintf ("mtv sv %s %s", GetMtvVar(d, MtvVar.MV_iam_idd_default_hlmax, 
                                                  MtvValue.NAME),
                        convert (vlStringVT, hlmax));
         vtq.CmdRun (Queue,e,cmd);
         vtq.CmdRun (Queue,e,"mtv pv /iam");
      }
   }
   
}
<cmdIdd,
prvCmdIdd> := vtime.CmdCreate(vtime.fac,
                              Idd,
                              "Idd",
                              "Use IDD file for delay calculations",
                              list (vcWv.Param ("Use IDD Only",
                                                vlYesNoVT, 
                                                vcWv.Sticky,
                                                vlYesNo.Yes),
                                    vcWv.Param("Summarize Missing IDDs",
                                               vtimeEnableDisableVT, 
                                               vcWv.Constant, 
                                               vtimeEnableDisable.Disable),
                                    vcWv.Param("Rising Min (ns)",
                                               vlFloatVT,
                                               vcWv.Sticky,3.0000),
                                    vcWv.Param("Rising Max (ns)",
                                               vlFloatVT,
                                               vcWv.Sticky,3.0000),
                                    vcWv.Param("Falling Min (ns)",
                                               vlFloatVT,
                                               vcWv.Sticky,3.0000),
                                    vcWv.Param("Falling Max (ns)",
                                               vlFloatVT,
                                               vcWv.Sticky,3.0000),
                                    vcWv.Param ("IDD file", 
                                                vlStringRestOfLineVT, 
                                                vcWv.Sticky, "",
                                                vcFilePop.fac,
                                                filter <- "*.idd",
                                                file_columns <- 1,
                                                dir_columns <- 1)));



/*===========================================================================*/
/*==                      I n t e r r u p t                                ==*/
/*===========================================================================*/
/*\Docpage  Interrupt -- Interrupt build, verification or trace analysis

  DESCRIPTION
       This comand allows the user to interactively interrupt the
       Motive build, verification or trace anlaysis. A popup window
       with the many options apper and the user has to select one of
       these. The choices provided are Abort, Next, Path, Status,
       Hpins, and Lnets. During design build, only Abort has any
       effect on the engine.
*/
cmdIntr := vcWv.Cmd(vtime.fac,
                    func(d,e){
                       Interrupt (d);
                    },
                    "Interrupt",
                    "Send an interrupt to Motive");


/*===========================================================================*/
/*==                          T i m g e n                                  ==*/
/*===========================================================================*/
/*\Docpage Timgen -- Generate timing model

*/
function GetTimFile (d)
   vtime.GetMtvVar(d, vtime.MtvVar.MV_file_tim,vtime.MtvValue.VALUE); 

function timgen(d,event,mode, timfile)
{
   Queue := vtime.GetQueue(d);
   
   res    := GetMtvVar(d, MtvVar.MV_timgen, MtvValue.NAME);

   vtq.CmdRun (Queue, event, "mtv sv " + res + " " + convert(vlStringVT,mode));
   vtq.CmdRun (Queue, event, "remark " + convert(vlStringVT,mode) + 
                             " Generation of timing model during verification");

   if (mode == vtimeEnableDisable.Enable){
      res    := GetMtvVar(d, MtvVar.MV_file_tim, MtvValue.NAME);
      timfile = string_trim (timfile);

      timfile = ResolveFilePathName(d,timfile,"tim");

      vtq.CmdRun (Queue, event, "mtv sv " + res + " " + timfile);
      vtq.CmdRun (Queue, event, "remark Timing Model for design '" + 
                  GetBlockName(d) + 
                  "' will be created in file '" +
                  timfile + "'");

   }
}
<cmdTimgen,prvCmdTimgen> := vtime.CmdCreate(vtime.fac,
                                  timgen,
                                  "Timgen",
                                    "Enable Motive Timing Model Generation Mode",
                                            list (vcWv.Param ("Model Generation", 
                                                        vtime.vtimeEnableDisableVT, 
                                                        vcWv.Constant, 
                                                        vtime.vtimeEnableDisable.Disable),
                                            vcWv.Param ("Timing Model File to Create", 
                                                        vlStringRestOfLineVT, 
                                                        vcWv.Dynamic, GetTimFile,
                                                        vcFilePop.fac,
                                                        filter <- "*.tim",
                                                        file_columns <- 1,
                                                        dir_columns <- 1)));



/*===========================================================================*/
/*==                      C r e a t e C a s e F i l e N a m e              ==*/
/*===========================================================================*/
/*\Docpage CreateCaseFileName -- Returns a file pathname in the case directory.

  DESCRIPTION
       Create a filename with "name" and "ext" in the timing case directory.
       The directory is determined by the current design and timing case. If the
       argument 'name' is nil, then the design name is used for the filename.

  EXAMPLE
       print (CreateCaseFileName (d,"foo", "stm");  --> "./tim/block/case/foo.stm"
       print (CreateCaseFileName (d, nil,  "stm");  --> "./tim/block/case/block.stm"

  CAUTION
       Do not call this directly since ResolveFilePathName() is a better
       general interface to this routine.
*/  
function CreateCaseFileName(d,name,ext)
{
   declare fname;
   rpath    := GetCaseRelPath(d);
   if ((name == nil) || string_eqi(name,""))
      fname = vscript.vsMerge("",rpath, GetBlockName(d), ext);
   else 
      fname = vscript.vsMerge("",rpath, name, ext);

   fname;
}

/*===========================================================================*/
/*==                 R e s o l v e F i l e P a t h N a m e                 ==*/
/*===========================================================================*/
/*\Docpage ResolveFilePathName -- Return a file name in case dir, if not a pathname.

  DESCRIPTION
       If the argument does not contain any path information, return the filename
       with the relative case directory path prefix. Otherwise return the
       original filename itself.

  EXAMPLE
       print (ResolveFilePathName(d,"foo.stm","stm"));   --> "./tim/design/case/foo.stm"
       print (ResolveFilePathName(d,"./foo.stm","stm")); --> "./foo.stm"
*/
function ResolveFilePathName(d,filename,pref_ext)
{
   declare device;
   declare path;
   declare name;
   declare ext;

   <device,path,name,ext> = vscript.vsParse(filename);

   /*-----------------------------------------------------*/
   /* In order for the pathname to be validated unchanged */
   /* it must contain a "path/file.ext", otherwise, it    */
   /* resolve to the current case directory as a fully    */
   /* defined filespec.                                   */
   /*-----------------------------------------------------*/

   if (string_eqi (path, "") == TRUE || 
       string_eqi (name, "") == TRUE || 
       string_eqi (ext, "")  == TRUE )
         filename = CreateCaseFileName(d, name, (string_eqi(ext,"") == TRUE ? pref_ext:ext));

   filename;
}

/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function FileIsInAnotherCase  -- 
  
  DESCRIPTION
       
  RETURNS
       
*/
function FileIsInAnotherCase(d,Block,Case,pathname) 
{
   declare device;
   declare path;
   declare name;
   declare ext;
   declare dir;
   declare str;

   needs_fixing := FALSE;

   <device,path,name,ext> = vscript.vsParse(pathname);
   
   Ipath := vscript.vsRelPath(path);
   Cpath := vscript.vsRelPath( vtime.GetCaseRelPath(d) );
   
   if (Ipath != nil && !string_eqi(Ipath, "")) {
      if (!string_eqi(Ipath,Cpath)) {
         
         str = string_token(Ipath,&dir," /");
         
         if (dir != nil && string_eqi(dir,"tim")) 
            if (str != nil) {
               str = string_token(str,&dir," /");
               if (dir != nil && string_eqi(dir,Block)) {
                  if (str != nil) {
                     str = string_token(str,&dir," /");
                     if (dir != nil && !string_eqi(dir,Case)) 
                        needs_fixing = TRUE;
                  }
               }
               else if (dir != nil) needs_fixing = TRUE;
            }
      }
   }
   needs_fixing;
} /* FileIsInAnotherCase */



load ("mpfload.vs");



